/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.ItemProxyClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

(function(){
    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    var checkNonNegative = net.jazz.ajax.toolbox.lang.checkNonNegative;
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;

    dojo.declare("com.ibm.team.workitem.web.cache.internal.ItemProxyClient", null, {
        /*
         * Construct the proxy passing the object
         */
//        constructor: function(){      
//        },
//        destroy: function(){
//        },
        isGuest: function(){
            if(!com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR) return true;
            return com.ibm.team.repository.web.client.session.isGuest();
        },
        createOperation: function(parms /* message, id, isModal, dialogTitle */){
            checkNotNull(parms.message, "message");
            checkNotNull(parms.id, "id-1");
			var op = {}
			if(false && parms.isModal) { // TEST
				op.isModal = parms.isModal;
			}
			op.message = parms.message;
			op.id = parms.id;
			op.dialogTitle = parms.dialogTitle;
            return op;
        },
		setOperationMessage: function(parms /* operation, message */) {
			try {
				if (parms.operation && parms.operation.dialog && parms.operation.dialog.contentNode) {
					var span = document.createElement("SPAN");
					span.style.paddingLeft = "5px";
					span.appendChild(document.createTextNode(parms.message));
					dojox.data.dom.replaceChildren(parms.operation.dialog.contentNode, span);
				}
			} catch(e) {
				// this will occur when the dialog was closed
				if(parms.operation) {
					parms.operation.dialog = null;
				}
			}
		},
		getPropertyMap: function(propertyObject, originalObject, propertyName) {
			var properties;
			propertyName = propertyName || "properties";
			if(originalObject) {
				properties = originalObject;
			} else {
				properties = {}
			}
			var propertyValues = propertyObject[propertyName];
			if(propertyValues) {
				if (propertyValues.length) {
					for (var i = 0; i < propertyValues.length; i++) {
						if (propertyValues[i] && propertyValues[i].key) {
							var key = propertyValues[i].key;
							var value = propertyValues[i].value
							properties[key] = value;
						}
						
					}
				} else {
					properties = propertyValues;				
				}
			}
			return properties;
		},
        isInOperation: function(){
            return this._operationCount > 0;
        },
        /*
         * May be called by code within the com.ibm.team.workitem.web bundle.
         */
        startOperation: function(operation){
        	if (PlatformUI && PlatformUI.getWorkbench())
            	PlatformUI.getWorkbench().showMessage(operation.message.replace(/\.\.\./, ""));
			if(this._currentOperation) {
				this.setOperationMessage({
					operation: this._currentOperation,
					message: operation.message
				})
				return;
			}
            this._operationCount++;
			if(operation.isModal) {
				var contentNode = document.createElement("DIV");
				var span = document.createElement("SPAN");
				span.style.paddingLeft = "5px";
				span.appendChild(document.createTextNode(operation.message));
				contentNode.appendChild(span);
				var parms = {
					isModel: true,
					contentNode: contentNode,
					width: "14em",
					isClosable: true,
					closeOnEsc: true
				}
				if(operation.dialogTitle) {
					parms.primaryTitle = operation.dialogTitle;
				}
				operation.dialog = new jazz.ui.Dialog(parms);
			}
			this._currentOperation = operation;
        },
        
        /*
         * May be called by code within the com.ibm.team.workitem.web bundle.
         */
        finishOperation: function(operation){
			if(operation.dialog) operation.dialog.close();
            this._operationCount--;
			this._currentOperation = null;
        	if (PlatformUI && PlatformUI.getWorkbench())
	            PlatformUI.getWorkbench().hideMessageArea();
        }
    });
    
})();
