/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.mixins.Identifier"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.mixins.PlanMode"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.mixins.WorkflowInfo"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$


(function() { // module container

var PlanningUtil			= com.ibm.team.apt.web.client.util;
var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

var SERVICE_URI				= "com.ibm.team.apt.internal.service.rest.IPlanRestService"; //$NON-NLS-1$

var Assert					= com.ibm.team.apt.web.ui.internal.common.Assert;
var ReferencedItemStore		= com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore;
var ITEM_TYPES				= com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore.ITEM_TYPES;

var SCRIPT_DELIMITER		= "\r\n\r\n--a1a1bf4aa1b34a2db01610882541df6b----\r\n\r\n"; //$NON-NLS-1$
var PROVIDED_UNIT_REGEXP	= /dojo\.provide\(\"([^\"]+)\"\)/;

var estimateConfigurationLoaded= false;
window.setTimeout(function() {
	// we need to wait until the initialization data is available before we can load the scripts
	if (com.ibm.team.process.web.projectArea) {
		com.ibm.team.apt.web.client.internal.PlanningClient.fetchEstimateConfiguration(new ServiceResponseHandler({
			success : function(estimateConfig) {
				var EstimateConfig= com.ibm.team.apt.web.client.util.EstimateConfig;
				EstimateConfig.estimateOutputFormat= estimateConfig.estimateOutputFormat;
				EstimateConfig.estimateMode= estimateConfig.estimateMode;
				EstimateConfig.projectWorkEnvironment.hoursPerDay= estimateConfig.hoursPerDay;
				EstimateConfig.projectWorkEnvironment.daysPerWeek= estimateConfig.daysPerWeek;
				estimateConfigurationLoaded= true;
			},
			error : function() {
				// let initializing estimate config silently fail
				estimateConfigurationLoaded= true;
			}
		}, "success", "error")); //$NON-NLS-1$ //$NON-NLS-2$
	} else {
		// do not use setInterval, the scripts might get loaded more than once (see defect 65892)
		window.setTimeout(arguments.callee, 100);
	}
}, 100);


var scriptsLoaded= false;
window.setTimeout(function() {
	// we need to wait until the initialization data is available before we can load the scripts
	if (com.ibm.team.process.web.projectArea) {
		var url= com.ibm.team.repository.web.transport.TeamServerClient._getServicePrefix();
		url+= "com.ibm.team.apt.internal.service.scriptprovider.IScriptProviderService"; //$NON-NLS-1$
		url+= "?scope=agilePlanning&projectAreaId=" +  com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId; //$NON-NLS-1$

		dojo.xhrGet({
	        url: url,
			handleAs: "text", //$NON-NLS-1$
			handle: function(response, ioArgs){
				var scripts= String(response).split(SCRIPT_DELIMITER);
				for (var i= 0; i < scripts.length; i++) {
					var match= PROVIDED_UNIT_REGEXP.exec(scripts[i]);
					if (match && dojo._loadedModules[match[1]]) {
						continue;
					}

					eval(scripts[i]);
				}					
				PlanningUtil.runWithCondition(function() {
					scriptsLoaded= true;
				}, function() { return estimateConfigurationLoaded; });
			},
			error: function() {
				console.error("Failed to load plan scripts"); //$NON-NLS-1$
				// let initializing the plan fail rather than keeping it in an endless wait
				scriptsLoaded= true;
			}
		});
	} else {
		// do not use setInterval, the scripts might get loaded more than once (see defect 65892)
		window.setTimeout(arguments.callee, 100);
	}
}, 100);

com.ibm.team.apt.web.client.internal.PlanningClient= {

	// ---- API ------------------------------------------------------------------------------------------------------------	
	
	getScriptsLoadedCondition: function() {
		return function() {
			return scriptsLoaded;
		};
	},
		
	fetchPlanScripts: function(responseHandler) {
		this._invokePlanService(responseHandler, "getPlanScripts", null, null, function() { return true; }); //$NON-NLS-1$
	},
	
	fetchEstimateConfiguration: function(responseHandler) {
		this._invokePlanService(responseHandler, "getEstimateConfiguration", null, null, function() { return true; }); //$NON-NLS-1$
	},
	
	validateRecentPlans: function(responseHandler, recentItems) {
		var serviceArguments= {};
		serviceArguments["jsonValue"]= dojo.toJson(recentItems); //$NON-NLS-1$
	
		this._invokePlanService(responseHandler, "postValidateRecentPlans", serviceArguments); //$NON-NLS-1$
	},
	
	searchIterationPlans: function(responseHandler, queryDescriptor, startIndex, maxResults, searchToken) {
		var serviceArguments= {};
		
		serviceArguments["keywords"]= queryDescriptor.getKeywords() || ""; //$NON-NLS-1$ //$NON-NLS-2$
		serviceArguments["relationshipType"]= queryDescriptor.getRelationshipType() || ""; //$NON-NLS-1$ //$NON-NLS-2$
		serviceArguments["relatedItemId"]= queryDescriptor.getRelatedItemId() || ""; //$NON-NLS-1$ //$NON-NLS-2$
		serviceArguments["currentOnly"]= queryDescriptor.isCurrentOnly(); //$NON-NLS-1$
		serviceArguments["includeBacklog"]= true; //$NON-NLS-1$
		serviceArguments["startIndex"]= startIndex; //$NON-NLS-1$
		serviceArguments["maxResults"]= maxResults; //$NON-NLS-1$
		serviceArguments["token"]= searchToken || ""; //$NON-NLS-1$ //$NON-NLS-2$

		this._invokePlanService(responseHandler, "getIterationPlanSearchResults", serviceArguments); //$NON-NLS-1$
	},
	
	postModifyPlan: function(responseHandler, plan){
		var serviceArguments= {};
		serviceArguments["planId"]= plan.itemId; //$NON-NLS-1$
		serviceArguments["name"]= plan.label; //$NON-NLS-1$
		serviceArguments["ownerId"]= plan.teamArea; //$NON-NLS-1$
		serviceArguments["iterationId"]= plan.iteration; //$NON-NLS-1$
		serviceArguments["planTypeId"]= plan.planTypeId; //$NON-NLS-1$
		serviceArguments["alwaysLoadAllExecutionItems"]= plan.alwaysLoadAllExecutionItems; //$NON-NLS-1$
		this._invokePlanService(responseHandler, "postModifyPlan", serviceArguments); //$NON-NLS-1$
	},
	
	deletePlan: function(responseHandler, plan){
		var serviceArguments= {};
		serviceArguments["planId"]= plan.itemId; //$NON-NLS-1$
			
		this._invokePlanService(responseHandler, "postDeletePlan", serviceArguments); //$NON-NLS-1$
	},
	
	postModifyPlanStore: function(responseHandler, plan, planMode){
		var serviceArguments= {};
		serviceArguments["planId"]= plan.getPlanId(); //$NON-NLS-1$
		serviceArguments["store"]= planMode.getStore().serialize(); //$NON-NLS-1$
		serviceArguments["planModeId"]= planMode.id; //$NON-NLS-1$
		this._invokePlanService(responseHandler, "postModifyPlanStore", serviceArguments); //$NON-NLS-1$
	},
	
	fetchNewPlan: function(responseHandler, owner, iteration) {
		var serviceArguments= {};
		serviceArguments["edit"]= true; //$NON-NLS-1$
		serviceArguments["owner"]= owner; //$NON-NLS-1$
		serviceArguments["iteration"]= iteration; //$NON-NLS-1$
		
		this._invokePlanService(responseHandler, "getIterationPlan", serviceArguments); //$NON-NLS-1$
	},
	
	fetchEditPlan: function(responseHandler, planId) {
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$
		serviceArguments["edit"]= true; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getIterationPlan", serviceArguments); //$NON-NLS-1$
	},
	
	fetchIterationPlan: function(responseHandler, planId) {
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getIterationPlan", serviceArguments); //$NON-NLS-1$
	},
	
	previewAttachedPage: function(responseHandler, planId, pageId, wikiSource){
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$
		serviceArguments["pageId"]= pageId; //$NON-NLS-1$
		serviceArguments["wikiSource"]= wikiSource; //$NON-NLS-1$
		
		this._invokePlanService(responseHandler, "postWikiSource", serviceArguments); //$NON-NLS-1$
	},
	
	postAttachedPage: function(responseHandler, planId, pageId, pageStateId, wikiSource){
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$
		serviceArguments["pageId"]= pageId; //$NON-NLS-1$
		serviceArguments["pageStateId"]= pageStateId; //$NON-NLS-1$
		serviceArguments["wikiSource"]= wikiSource; //$NON-NLS-1$
		
		this._invokePlanService(responseHandler, "postAttachedPage", serviceArguments); //$NON-NLS-1$
	},
	
	fetchAttachedPage: function(responseHandler, planId, pageId) {
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$
		serviceArguments["pageId"]= pageId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getAttachedPage", serviceArguments); //$NON-NLS-1$
	},
	
	deleteAttachedPage: function(responseHandler, planId, pageId) {
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$
		serviceArguments["pageId"]= pageId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "postDeleteAttachedPage", serviceArguments); //$NON-NLS-1$
	},
	
	fetchPlannedWorkItems: function(responseHandler, planId) {
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getPlannedWorkItems", serviceArguments); //$NON-NLS-1$
	},
	
	fetchTeamLoad: function(responseHandler, planIds) {
		var serviceArguments= {};
		serviceArguments["planIds"]= planIds; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getTeamLoad", serviceArguments); //$NON-NLS-1$
	},
	
	loadPlanItems: function(responseHandler, planId, itemIds, referencedItemsStore) {
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$
		serviceArguments["itemIds"]= itemIds; //$NON-NLS-1$
		
		var filterInterceptor= new ServiceResponseHandler({	
			_success: function(result) {
				referencedItemsStore.addItems(result.referencedItems);
				var items= new Object;
				dojo.forEach(itemIds, function(id) {
					var workItem= referencedItemsStore.getItem(ReferencedItemStore.ITEM_TYPES.WORKITEM, id);
					if (workItem)
						items[id]= workItem;
				});		
				responseHandler.doCallback(items);
			},
			_error: function(errorObj) {
				responseHandler.doErrorCallback(errorObj);
			}
		}, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$
		
		this._invokePlanService(filterInterceptor, "getWorkItems", serviceArguments); //$NON-NLS-1$
	},
	
	fetchConfigurationElements: function(responseHandler, planId){
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getConfigurationElements", serviceArguments); //$NON-NLS-1$
	},
	
	deletePlanMode: function(responseHandler, sharedPlanMode){
		var serviceArguments= {};
		serviceArguments["sharedPlanModeIds"]= new Array(sharedPlanMode); //$NON-NLS-1$
		
		this._invokePlanService(responseHandler, "postDeletePlanMode", serviceArguments); //$NON-NLS-1$
	},
	
	savePlanMode: function(responseHandler, planId, planMode){
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$
		if(planMode.sharedPlanModeId)
			serviceArguments["sharedPlanModeId"]= planMode.sharedPlanModeId; //$NON-NLS-1$
		// for new plan modes: unset the placeholder-id so that server can assign one
		if(planMode.id != "__draft_plan_mode") //$NON-NLS-1$
			serviceArguments["id"]= planMode.id; //$NON-NLS-1$
		serviceArguments["name"]= planMode.name; //$NON-NLS-1$
		serviceArguments["viewModeId"]= planMode.viewMode.id; //$NON-NLS-1$
		serviceArguments["groupProviderId"]= planMode.groupProvider.id; //$NON-NLS-1$
		serviceArguments["sortModeId"]= planMode.sortMode.id; //$NON-NLS-1$
		serviceArguments["filterIds"]= new Array(); //$NON-NLS-1$
		dojo.forEach(planMode.filters, function(filter){
			serviceArguments["filterIds"].push(filter.id); //$NON-NLS-1$
		});
		serviceArguments["barMode"]= planMode.barMode.id; //$NON-NLS-1$
		serviceArguments["columnIds"]= new Array(); //$NON-NLS-1$
		dojo.forEach(planMode.columns, function(column){
			serviceArguments["columnIds"].push(column); //$NON-NLS-1$
		});
		
		serviceArguments["colors"]= []; //$NON-NLS-1$
		dojo.forEach(planMode.colors, function(color){
			serviceArguments["colors"].push(dojo.toJson(color)); //$NON-NLS-1$
		});
		
		this._invokePlanService(responseHandler, "postPlanMode", serviceArguments); //$NON-NLS-1$
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	/**
	 * Invokes the requested method on the APT REST service.
	 */
	_invokePlanService: function(responseHandler, method, args, existingStore, responseCondition) {
		var _self= this;
		var dtoMassageInterceptor= new ServiceResponseHandler({
			_success: function(result) {
				PlanningUtil.runWithCondition(function() {
					responseHandler.doCallback(_doMassageDTO(result, existingStore));
				}, responseCondition || _self.getScriptsLoadedCondition());
			},
			_error: function(errorObj) {
				responseHandler.doErrorCallback(errorObj);
			}
		}, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$

		
		this._handleProgressMonitor(responseHandler);
		
		var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		var serviceArguments= dojo.mixin(args || {}, { projectAreaId: currentProjectArea.itemId });
		serviceArguments= this._massageParameterWrapper(serviceArguments);
		var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(SERVICE_URI, method, serviceArguments);

		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(serviceRequest, dtoMassageInterceptor);
	},
	
	/**
	 * Removes undefined form the service call args
	 */
	_massageParameterWrapper: function(args) {
		var property;
		for(property in args){
			if(!args[property] || args[property] == "undefined"){ //$NON-NLS-1$
				delete args[property];
			}
		}
		return args;
	},
	
	_handleProgressMonitor: function(responseHandler) {
		if (dojo.isFunction(responseHandler["doStart"])) { //$NON-NLS-1$
			responseHandler.doStart();
		}
	},

	__sentinel: null // terminates this class definition
};



// ---- DTO massage --------------------------------------------------------------------------------------------------------

var CLASSNAME_PROPERTY= "_eQualifiedClassName"; //$NON-NLS-1$
var CLASSNAME_EXPRESSION= /com\.ibm\.team\.apt\.rest\.dto:([a-zA-Z0-9_]*)/;

var itemTypeCondition= function(itemType) {
	return function(dto) { return dto.itemType.indexOf(itemType) ==  0; };
};


var MassageInfo= {
	"IterationItemDTO": { //$NON-NLS-1$
		items: [ 
		    { name: "children", type: ITEM_TYPES.ITERATION } //$NON-NLS-1$
		],
		wrappers: [{
			ctor: "com.ibm.team.process.common.Iteration" //$NON-NLS-1$
		}]
	}, 
	
	"DevelopmentLineDTO": { //$NON-NLS-1$
		items: [
		  		{ name: "owner", type: ITEM_TYPES.TEAMAREA }, //$NON-NLS-1$
		  		{ name: "iterations", type: ITEM_TYPES.ITERATION }, //$NON-NLS-1$
		  		{ name: "currentIteration", type: ITEM_TYPES.ITERATION } //$NON-NLS-1$
		],
		node: [ 
		       { name: "iterationHierarchy", type: ITEM_TYPES.ITERATION } //$NON-NLS-1$
	    ]
	},
	
	"EnumerationElementDTO": { //$NON-NLS-1$
		wrappers: [{
			ctor: "com.ibm.team.workitem.common.Literal" //$NON-NLS-1$
		}]
	}, 
	
	"ContributorDTO": { //$NON-NLS-1$
		wrappers: [{
			ctor: "com.ibm.team.repository.common.Contributor" //$NON-NLS-1$
		}]
	},
	
	"CategoryItemDTO": { //$NON-NLS-1$
		items: [ 
		    { name: "parent", type: ITEM_TYPES.CATEGORY } //$NON-NLS-1$
		],
		wrappers: [{
			ctor: "com.ibm.team.workitem.common.Category" //$NON-NLS-1$
		}]
	},
	
	"TeamAreaItemDTO": { //$NON-NLS-1$
		arrays: [ "children", "categories" ], //$NON-NLS-1$ //$NON-NLS-2$
		items: [ 
		    { name: "devLine", type: ITEM_TYPES.DEVLINE }, //$NON-NLS-1$
		    { name: "parent", type: ITEM_TYPES.PROCESSAREA }, //$NON-NLS-1$
		    { name: "children", type: ITEM_TYPES.PROCESSAREA }, //$NON-NLS-1$
		    { name: "categories", type: ITEM_TYPES.CATEGORY } //$NON-NLS-1$
		],
		wrappers: [{
			ctor: "com.ibm.team.process.common.TeamArea" //$NON-NLS-1$
		}]
	},
		
	"ProjectAreaItemDTO": { //$NON-NLS-1$
		arrays: [ "children", "categories", "devLines" ], //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		items: [ 
		    { name: "devLines", type: ITEM_TYPES.DEVLINE }, //$NON-NLS-1$
		    { name: "parent", type: ITEM_TYPES.PROCESSAREA }, //$NON-NLS-1$
		    { name: "children", type: ITEM_TYPES.PROCESSAREA }, //$NON-NLS-1$
		    { name: "categories", type: ITEM_TYPES.CATEGORY }, //$NON-NLS-1$
		    { name: "devLines", type: ITEM_TYPES.DEVLINE } //$NON-NLS-1$
		],
		wrappers: [{
			ctor: "com.ibm.team.process.common.ProjectArea" //$NON-NLS-1$
		}]
	},
	
	"IterationPlanDTO": { //$NON-NLS-1$
		arrays: [ "attachedPages", "includedIterations", "includedTeamMemberAreas" ], //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		items: [ 
			{ name: "teamArea", type: ITEM_TYPES.PROCESSAREA }, //$NON-NLS-1$
			{ name: "iteration", type: ITEM_TYPES.ITERATION } //$NON-NLS-1$
		]
	},

	"StateDTO": { //$NON-NLS-1$
		// TODO wrap it
		mixin: com.ibm.team.apt.web.client.internal.mixins.Identifier
	},
	
	"SearchIterationPlansResultDTO": { //$NON-NLS-1$
		arrays: [ "results", "referencedItems" ] //$NON-NLS-1$ //$NON-NLS-2$
	}, 
	
	"IterationPlanTeamLoadResultDTO": { //$NON-NLS-1$
		arrays: [ "loadInfos" ] //$NON-NLS-1$
	},
	
	"PlannedItemsPageDTO": { //$NON-NLS-1$
		arrays: [ "iterations", //$NON-NLS-1$
		          "categories", //$NON-NLS-1$
		          "workItemAttributes", //$NON-NLS-1$
		          "workItems", //$NON-NLS-1$
		          "workEnvironments", //$NON-NLS-1$
		          "planElementAttributes", //$NON-NLS-1$
		          "planModes", //$NON-NLS-1$
		          "workItemTypes", //$NON-NLS-1$
		          "planChecks", //$NON-NLS-1$
		          "requirementServiceDescriptors", //$NON-NLS-1$
		          "topLevelWorkItemTypes", //$NON-NLS-1$
		          "referencedPlans", //$NON-NLS-1$
		          "requirementServiceDescriptors", //$NON-NLS-1$
		          "referencedItems" ], //$NON-NLS-1$ 
  		items: [
  		    { name: "teamArea", type: ITEM_TYPES.PROCESSAREA } //$NON-NLS-1$
  		],
	  	node: [ 
	  	    { name: "iterationHierarchy", type: ITEM_TYPES.ITERATION } //$NON-NLS-1$
 	    ]
	}, 
	
	"CALMServiceDescriptorDTO": { //$NON-NLS-1$
		items: [
	        { name: "workItemType", type: ITEM_TYPES.WITYPE } //$NON-NLS-1$
		]
	},

	
	"WorkItemDTO": { //$NON-NLS-1$
		arrays: [ "attributeValues " ], //$NON-NLS-1$ 
		items: [
	        { name: "parent", type: ITEM_TYPES.WORKITEM }, //$NON-NLS-1$
	        { name: "workflow", type: ITEM_TYPES.WORKFLOW } //$NON-NLS-1$
		]
//		wrappers: [{
//			ctor: "com.ibm.team.workitem.common.WorkItem" //$NON-NLS-1$
//		}]
	},
	
	"TeamMemberWorkEnvironmentDTO": { //$NON-NLS-1$
		items: [
	        { name: "contributor", type: ITEM_TYPES.CONTRIBUTOR } //$NON-NLS-1$
		],
		arrays: [ "workDays", "absences" ] //$NON-NLS-1$ //$NON-NLS-2$
	},

	"WorkdayDTO": { //$NON-NLS-1$
		wrappers: [{
			ctor: "com.ibm.team.apt.client.internal.Workday" //$NON-NLS-1$
		}]
	},

	"ContibutorAbsenceDTO": { //$NON-NLS-1$
		wrappers: [{
			ctor: "com.ibm.team.apt.common.ContributorAbsence" //$NON-NLS-1$
		}]
	},

	"ConfigurationDTO": { //$NON-NLS-1$
		arrays: ["parameters"], //$NON-NLS-1$
		wrappers: [{
			ctor: "com.ibm.team.apt.common.ConfigurationElement" //$NON-NLS-1$
		}]
	},

	"PlanCheckDTO": { //$NON-NLS-1$
		arrays: ["parameters", "problemDefinitions"], //$NON-NLS-1$ //$NON-NLS-2$
		wrappers: [{
			ctor: "com.ibm.team.apt.common.PlanCheckDescription" //$NON-NLS-1$
		}]
	},

	"PlanAttributeDTO": { //$NON-NLS-1$
		arrays: ["parameters"], //$NON-NLS-1$
		wrappers: [{
			ctor: "com.ibm.team.apt.common.AttributeDescription" //$NON-NLS-1$
		}]
	},
	
	"WorkflowInfoDTO": { //$NON-NLS-1$
		arrays: ["todoStates", "progressStates", "doneStates", "workflowActions"], //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		mixin: com.ibm.team.apt.web.client.internal.mixins.WorkflowInfo,
		referencedItemStore: true
	},
	
	"WorkflowActionDTO": { //$NON-NLS-1$
		arrays: ["startStates"], //$NON-NLS-1$
		mixin: com.ibm.team.apt.web.client.internal.mixins.Identifier
	},

	"PlanModeDTO": { //$NON-NLS-1$
		arrays: [ "filters", "colors", "columns" ], //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		mixin: com.ibm.team.apt.web.client.internal.mixins.PlanMode
	},

	"WorkItemAttributeDTO": { //$NON-NLS-1$
		arrays: [ "valueSet" ], //$NON-NLS-1$
		wrappers: [{
			ctor: "com.ibm.team.workitem.common.Attribute" //$NON-NLS-1$
		}],
		referencedItemStore: true
	},
	
	"WorkItemTypeDTO": { //$NON-NLS-1$
		items: [
		  		{ name: "workflow", type: ITEM_TYPES.WORKFLOW } //$NON-NLS-1$
		]
	}

};

function _doMassageDTO(dto, existingStore) {
	if(!dto)
		return dto;
	
	dto._referencedItemStore= null;

	switch (_extractClassName(dto)) {
		case "LoadIterationPlanResultDTO": //$NON-NLS-1$
		case "CreateIterationPlanResultDTO": //$NON-NLS-1$
		case "SearchIterationPlansResultDTO": //$NON-NLS-1$
		case "PlannedItemsPageDTO": //$NON-NLS-1$
		case "PlannedWorkItemsDTO": //$NON-NLS-1$
		case "OperationReportDTO": //$NON-NLS-1$
			dto._referencedItemStore= existingStore || new ReferencedItemStore();
			break;
	}
	
	_applyMassage(dto, dto._referencedItemStore);

	if (dto._referencedItemStore)
		dto._referencedItemStore.addItems(dto["referencedItems"]); //$NON-NLS-1$

	return dto;
};

function _applyMassage(dto, referencedItemStore) {
	var result= dto;
	var i;
	var className= _extractClassName(dto);
	if (className) {
		for (var property in dto) {
			if (dto[property] != null && dojo.isObject(dto[property])) {
				dto[property]= _applyMassage(dto[property], referencedItemStore);
			}
		}

		var massageInfo= MassageInfo[className];
		if (massageInfo) {
			if (massageInfo.referencedItemStore) {
				dto._referencedItemStore= referencedItemStore;
			}

			// 1) ensure all arrays are created, so that we do not have to clutter the code later on with tons of ifs
			var arrays= massageInfo.arrays || [];
			for (i = 0; i < arrays .length; i++) {
				dto[arrays[i]]= dto[arrays[i]] || [];
			}
			
			// 2) add item accessors
			var items= massageInfo.items || [];
			Assert.isTrue(items.length == 0 || referencedItemStore != null, "Missing ReferencedItemStore"); //$NON-NLS-1$
			for (i = 0; i < items.length; i++) {
				if (dojo.isArray(dto[items[i].name])) {
					_addArrayItemAccessor(dto, items[i].name, items[i].type, referencedItemStore);
				} else {
					_addItemAccessor(dto, items[i].name, items[i].type, referencedItemStore);
				}
			}
			
			// 3) mixin additions
			if (massageInfo.wrappers) {
				for (i= 0; i < massageInfo.wrappers.length; i++) {
					if (!massageInfo.wrappers[i].condition || massageInfo.wrappers[i].condition(dto)) {
						result= new (dojo.getObject(massageInfo.wrappers[i].ctor))(dto, referencedItemStore);
						break;
					}
				}
			} else if (massageInfo.mixin) {
				dojo.mixin(dto, massageInfo.mixin);
			}
			
			// 4) node inline
			var json= massageInfo.node || [];
			for (i = 0; i < json.length; i++) {
				_addNode(dto, json[i].name, json[i].type, referencedItemStore);
			}
		}
	} else if (dojo.isArrayLike(dto)) {
		for (i= 0; i < dto.length; i++) {
			if (dto[i] != null && dojo.isObject(dto[i])) {
				dto[i]= _applyMassage(dto[i], referencedItemStore);
			}
		}
	}
	return result;
};

function _extractClassName(dto) {
	var result= CLASSNAME_EXPRESSION.exec(dto[CLASSNAME_PROPERTY]);
	return result ? result[1] : null;
};

function _addItemAccessor(dto, property, type, referencedItemStore) {
	var accessorName= "get" + property.substr(0, 1).toUpperCase() + property.substr(1); //$NON-NLS-1$
	dto[accessorName]= function() {
		return referencedItemStore.getItem(type, dto[property]);
	};
};

function _addArrayItemAccessor(dto, property, type, referencedItemStore) {
	var accessorName= "get" + property.substr(0, 1).toUpperCase() + property.substr(1); //$NON-NLS-1$
	var result= null;
	dto[accessorName]= function() {
		if(!result){
			result= new Array();
			for(var element in dto[property]){
				result[element]= referencedItemStore.getItem(type, dto[property][element]);
			}
		}
		return result;
	};
};

function _addNode(dto, property, type, referencedItemStore) {
	var accessorName= "get" + property.substr(0, 1).toUpperCase() + property.substr(1); //$NON-NLS-1$
	var root= null;
	dto[accessorName]= function() {
		if(!root){
			root= dojo.fromJson(dto[property]);
			__traverse(root, function(node){
				_addNodeInterface(root, node, type, referencedItemStore);
			});
		}
		return root;
	};
};

function _addNodeInterface(root, node, type, referencedItemStore){
	
	node["getElement"]= function(){ //$NON-NLS-1$
		return referencedItemStore.getItem(type, node._element);
	};
	
	node["getParent"]= function(){ //$NON-NLS-1$
		var parentOrdinal= node._parent;
		if(!parentOrdinal)
			return null;
		
		var parent= __traverse(root, function(node){
			if(node._ordinal == parentOrdinal)
				return node;
			return null;
		});
		return parent;
	};
	
	node["getChildren"]= function(){ //$NON-NLS-1$
		return node._children;
	};
	
	node["find"]= function(element) { //$NON-NLS-1$
		return __traverse(node, function(child){
			if(child._element == element.itemId)
				return child;
			return null;
		});
	};
	
	node["ordinal"]= function(){ //$NON-NLS-1$
		return node._ordinal;
	};
}

function __traverse(node, fn){
	var value= fn(node);
	if(value)
		return value;
	
	dojo.forEach(node._children, function(child){
		if(!value){
			value= __traverse(child, fn);
		}
	});
	
	return value;
};


})();