/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * @author hyassine
 */
dojo.provide("com.ibm.team.scm.web.client.internal.ScmClient");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("net.jazz.ajax.core.internal.util");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

var callContributedCode = net.jazz.ajax.core.internal.util.callContributedCode;
var messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");	

(function() {

var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

com.ibm.team.scm.web.client.internal.ScmClient= {

	SECURE_SERVICE_PREFIX: dojo.baseUrl + "../../resource/",
	SERVICE_URI: "com.ibm.team.scm.common.internal.rest.IScmRestService",
	SERVICE_URI_2: "com.ibm.team.scm.common.internal.rest.IScmRestService2",
	FILESYSTEM_SERVICE_URI: "com.ibm.team.filesystem.common.internal.rest.IFilesystemRestService",
	FILESYSTEM_SERVICE_URI_2: "com.ibm.team.filesystem.common.internal.rest.IFilesystemRestService2",
	FILESYSTEM_WORKITEM_SERVICE_URI: "com.ibm.team.filesystem.common.workitems.internal.rest.IFilesystemWorkItemRestService",
	WORKSPACE_PREFIX: "itemOid/com.ibm.team.scm.Workspace/",
	COMPONENT_PREFIX: "itemOid/com.ibm.team.scm.Component/",
	
	
	///////////////////////////////////////////////////////////////////////////////////
	// IScmRestService API
	///////////////////////////////////////////////////////////////////////////////////
	
	/**
	 * Retrieves the list of workspaces/streams in a specific project area depending on the arguments passed.
	 * @return WorkspaceListDTO
	 * 
	 * @see IScmRestService#getWorkspaces
	 */
	getWorkspaces: function(responseHandler, args) {
		//this._invokeScmService(responseHandler, "getWorkspaces", args);
		//Invoke the gateway resource based service.
		this._getResource(responseHandler, "virtualPath/com.ibm.team.scm.rest.dto.WorkspaceListDTO", args);
	},
	
	/**
	 * Retrieves the list of active change sets in a specific team area.
	 * @return ChangeSetDTO[]
	 * 
	 * @see IScmRestService#getTeamChanges
	 */
	getTeamChanges: function(responseHandler, args) {
		this._invokeScmService(responseHandler, "getTeamChanges", args);
	},
	
	/**
	 * Retrieves the list of components in a specific workspace/stream.
	 * @return IComponent[]
	 * 
	 * @see IScmRestService#getComponents
	 */
	getComponents: function(responseHandler, args) {
		this._invokeScmService(responseHandler, "getComponents", args);
	},
	
	/**
	 * Retrieves a WorkspaceDTO containing the list of components under a specific workspace/stream.
	 * @return WorkspaceDTO
	 * 
	 * @see IScmRestService#getWorkspace
	 */
	getWorkspace: function(responseHandler, args) {
		//this._invokeScmService(responseHandler, "getWorkspace", args);
		//Invoke the gateway resource based service.
		this._getResource(responseHandler, "virtualPath/com.ibm.team.scm.rest.dto.WorkspaceDTO/" + args.itemId);
	},
	
	/**
	 * Retrieves a versionable specified by the given path, workspace and component. If the versionable is a folder, 
	 * the result includes info about its children as well as itself.
	 * when isPath is true, pathOrOid should be a / separated list of URIComponent encoded segments (as a string)
	 * @return VersionableDTO
	 * 
	 * @see IScmRestService#getVersionable
	 */
	getVersionable: function(responseHandler, args, isPath, pathOrOid) {
		var path = "virtualPath/com.ibm.team.scm.rest2.dto.VersionablePlusDTO/";
		
		if (isPath) {	//by path
			//NOTE: Spaces in the path throws an exception
			path += "~";
			if (pathOrOid != "/") {
				path += encodeURI(pathOrOid.substring(1));
			}
		}
		else {	//by oid
			path += pathOrOid;
		}
		//Invoke the gateway resource based service.
		this._getResource(responseHandler, path, args);
	},
	
	/**
	 * Retrieves the history (change sets) of a given component.
	 * @return HistoryDTO
	 * 
	 * @see IScmRestService#getHistoryPlus
	 */
	getHistory: function(responseHandler, args /*, isComponentHistory, oid */) {
        this._invokeScmService2(responseHandler, "getHistoryPlus", args);
		/* 
		if (isComponentHistory) {
			//Invoke the gateway resource based service.
			this._getResource(responseHandler, "virtualPath/com.ibm.team.scm.rest.dto.HistoryDTO/", args);
		}
		else if(!isComponentHistory){	//use item oid
			//Invoke the gateway resource based service.
			this._getResource(responseHandler, "virtualPath/com.ibm.team.scm.rest.dto.HistoryDTO/" + oid, args);
		}*/
	},
	
	/**
	 * Update a stream
	 * @see IScmRestService2#postUpdateStream
	 */
	postUpdateStream: function(responseHandler, args) {
		this._invokeFilesystemService2(responseHandler, "postUpdateStream", args);
	},
	/** 
	 * Update locks
	 * @see IScmRestService2#postUpdateLocks
	 */
	postUpdateLocks: function(responseHandler, args) {
		this._invokeScmService2(responseHandler, "postUpdateLocks", args);
	},
	/** 
	 * Add links to a change set
	 * @see IFileSystemWorkItemRestService#postAddLinks
	 */
	postAddLinks: function(responseHandler, args) {
		this._invokeFilesystemWorkItemService(responseHandler, "postAddLinks", args);
	},
		
	/**
	 * Retrieves a specific change set DTO with the specified changeSetItemId.
	 * @return ChangeSetDTO
	 * 
	 * @see IScmRestService#getChangeSet
	 */
	getChangeSet: function(responseHandler, args /*, changeSetId*/) {
		this._invokeFilesystemService(responseHandler, "getChangeSet", args);
		
		/* 
		//Invoke the gateway resource based service.
		this._getResource(responseHandler, "virtualPath/com.ibm.team.scm.rest.dto.ChangeSetDTO/" + changeSetId, args);
		*/
	},
	
	/**
	 * Retrieves the list of baseline sets for a specific stream/workspace.
	 * @return IBaselineSet[]
	 * 
	 * @see IScmRestService#getBaselineSets
	 */
	getBaselineSets: function(responseHandler, args) {
		this._invokeScmService(responseHandler, "getBaselineSets", args);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Service invocation helpers
	///////////////////////////////////////////////////////////////////////////////////
	
	/*
	 * Retrieve a virtual resource.  If an args hash is given, it's converted to query args in the URI.
	 * For a workspaceDTO retrieval, a sample URI would be: 
	 * https://localhost:9443/jazz/service/com.ibm.team.repository.common.internal.IVirtualRestService/virtualPath/
			com.ibm.team.scm.rest.dto.WorkspaceDTO/_XaoeUK5XEdyxHMGNKFC0Jw
	 */
	_getResource: function(responseHandler, resource, args) {
		
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var bindArgs = this._getBindArgs(responseHandler, resource, args);
		var xhr = dojo.xhrGet(bindArgs);
	},
	
	_getBindArgs: function(responseHandler, resource, args) {
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var self = this;
		var bindArgs = {
			url: self._assembleURL(resource, args),
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(response, ioArgs) {
				if (response instanceof Error) {
					// For work item 18631 ('Propogate server exception details to
					// REST service clients'), we can access the error response's
					// body via xhr.responseText.  Will probably have to manually
					// demarshall this data here, since I don't believe Dojo
					// provides any auto-demarshalling on error paths.
	
					// temporary fix for Defect 32729
					// test data (what we expect WebSphere to send back on ITEs)
					//ioArgs.xhr = {
					//	status: 500,
					//	statusText: "Internal Server Error",
					//	responseText: "Error 500: -- &#39;Save Work Item&#39; was blocked by process.\nThe work item is not assigned to a category.\n"
					//}
					var errorObj = {
						status: ioArgs.xhr.status,
						message: null,
						exception: undefined
					};
					if (ioArgs.xhr.statusText === "Internal Server Error") {
						var errorResponse = ioArgs.xhr.responseText || "";
						errorResponse = dojo.fromJson(errorResponse);
						if (dojo.isObject(errorResponse)) {
							errorObj.message = "["+errorResponse.errorCode + "] : " + errorResponse.errorMessage;
							errorObj.exception = errorResponse;
						}
						else {
							errorObj.message = errorResponse;
							if (errorObj.message.indexOf("-- ") > 0 && errorObj.message.length > 3) {
								errorObj.message = errorObj.message.substring(errorObj.message.indexOf("-- ") + 3);
							}
							errorObj.message = errorObj.message.replace(/&#39;/g, "\"");
						}
					} else if (ioArgs.xhr.status == 401) {
						// Capture the scenario where the user cancels out of a Basic auth prompt and display a helpful message
						// rather than a blank "Loading..." screen
						var errMsg = messages.ScmClient_0;
						alert(errMsg);
						errorObj.message = errMsg; 					
					} else if (ioArgs.xhr.status == 403 && ioArgs.args.url.match(/^.*IWebUIInitializerRestService\/initializationData.*$/)) {
						// Capture the scenario where a user account does not belong to the groups required to access the Web UI
						// Provide a helpful error message rather than a blank "Loading..." screen.
						var errMsg = messages.ScmClient_1;
						alert(errMsg);
						errorObj.message = errMsg;
					} else {
						errorObj.message = ioArgs.xhr.statusText;
					}
					
					errorObj.message = dojo.string.trim(unescape(errorObj.message).replace(/\+/g, " ").replace(/^(-*)/g, ""));
						
					responseHandler.doErrorCallback(errorObj);
//				} else if (ioArgs.xhr.getResponseHeader(X_AUTH_MSG) == "authrequired") {
//				    // TODO: handle login
//					_loginPrompt(this, serviceResponseHandler, serviceRequest.getMethodType().toLowerCase());
				} else {
					var returnValue = response;
					callContributedCode(function(){
						responseHandler.doCallback(returnValue);
					});
				}
			}
		};		
		return bindArgs;
	},
	
	_assembleURL: function(resource, callArgs) {	
		var regex = /^https/;
		var servicePrefix = this.SECURE_SERVICE_PREFIX;
		
		// assemble basic url
		var url = servicePrefix + resource;
		
		// if this is a GET request, append the query string
		if (callArgs) {
			var tmpQueryString = dojo.objectToQuery(callArgs);
			var queryString = (tmpQueryString !== null) ? ("?" + tmpQueryString) : "";
			url += queryString;
		}
		return url;
	},
	
	/**
	 * Invokes the requested method on the SCM REST service.
	 */
	_invokeScmService: function(responseHandler, method, args) {
		
		//Test whether or not any arguments were specified.
		if (arguments.length == 2) {
			var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.SERVICE_URI,
					method);
		} else {
			serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.SERVICE_URI,
					method,
					args);
		}
		//Invoke the requested method.
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	},
	
	/**
	 * Invokes the requested method on the SCM REST service 2.
	 */
	_invokeScmService2: function(responseHandler, method, args) {
		
		//Test whether or not any arguments were specified.
		if (arguments.length == 2) {
			var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.SERVICE_URI_2,
					method);
		} else {
			serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.SERVICE_URI_2,
					method,
					args);
		}
		//Invoke the requested method.
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	},
		
	/**
	 * Invokes the requested method on the Repository Virtual REST service.
	 */
	_invokeRepositoryService: function(responseHandler, method, args) {
		
		//Test whether or not any arguments were specified.
		if (arguments.length == 2) {
			var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.REPOSITORY_SERVICE_URI,
					method);
		} else {
			serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.REPOSITORY_SERVICE_URI,
					method,
					args);
		}
		//Invoke the requested method.
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	},
	
	/**
	 * Invokes the requested method on the Filesystem REST service.
	 */
	_invokeFilesystemService: function(responseHandler, method, args) {
		
		//Test whether or not any arguments were specified.
		if (arguments.length == 2) {
			var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.FILESYSTEM_SERVICE_URI,
					method);
		} else {
			serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.FILESYSTEM_SERVICE_URI,
					method,
					args);
		}
		//Invoke the requested method.
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	},
	
	/**
	 * Invokes the requested method on the Filesystem REST service 2.
	 */
	_invokeFilesystemService2: function(responseHandler, method, args) {
		
		//Test whether or not any arguments were specified.
		if (arguments.length == 2) {
			var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.FILESYSTEM_SERVICE_URI_2,
					method);
		} else {
			serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.FILESYSTEM_SERVICE_URI_2,
					method,
					args);
		}
		//Invoke the requested method.
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	},
	
	/**
	 * Invokes the requested method on the FilesystemWorkItem REST service .
	 */
	_invokeFilesystemWorkItemService: function(responseHandler, method, args) {
		
		//Test whether or not any arguments were specified.
		if (arguments.length == 2) {
			var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.FILESYSTEM_WORKITEM_SERVICE_URI,
					method);
		} else {
			serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.FILESYSTEM_WORKITEM_SERVICE_URI,
					method,
					args);
		}
		//Invoke the requested method.
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	}	
};

})();