/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.client.internal.WorkItemSaveManager"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.SequenceSaveRunnable"); //$NON-NLS-1$

(function() {

var Assert= com.ibm.team.apt.web.ui.internal.common.Assert;
var SequenceSaveRunnable= com.ibm.team.apt.web.client.internal.SequenceSaveRunnable;

var __compoundSaveCounter= 0;
var __mergedSaveRunnables= null;


function __startSpinning(planItems, viewModel) {
	if (planItems.length == 0)
		return [];
	
	var result= [];
	viewModel.updateModel(function(updateAccessor) {
		var navigator= updateAccessor.getEntryNavigator(true);
		for (var i= 0;  i < planItems.length; i++) {
			var moveEntry= navigator.findPrimaryEntry(planItems[i]);
			if (moveEntry) {
				updateAccessor.setTag(moveEntry, com.ibm.team.apt.ui.model.SpinningTag.INSTANCE);
				result.push(moveEntry);
			}
		}
	});
	
	return result;
}

function __stopSpinning(moveEntries, viewModel) {
	if (moveEntries.length == 0)
		return;
	
	viewModel.updateModel(function(updateAccessor) {
		var navigator= updateAccessor.getEntryNavigator(true);
		for (var i= 0;  i < moveEntries.length; i++) {
			if (moveEntries[i] && !moveEntries[i].isDisposed()) {
				updateAccessor.clearTag(moveEntries[i], com.ibm.team.apt.ui.model.SpinningTag.INSTANCE);
			}
		}
	});
}


function doCompoundSave(code, viewModel, errorReporter, thisObj) {
	// for now do not allow nested save calls
	Assert.isTrue(__compoundSaveCounter == 0);
	if (__compoundSaveCounter++ == 0) {
		__mergedSaveRunnables= [];
	}

	try {
		return code.call(thisObj);
	} finally {
		var toSave= null;
		if (__compoundSaveCounter-- == 1) {
			toSave= __mergedSaveRunnables;
			__mergedSaveRunnables= null;
		}

		if (toSave != null && toSave.length > 0) {
			var planItems= [];
			dojo.forEach(toSave, function(saveRunner) {
				if (saveRunner._planItem) {
					planItems.push(saveRunner._planItem);
				}
			});
			var spinEntries= __startSpinning(planItems, viewModel);

			var saveRunner= toSave.length == 1 ? toSave[0] : new SequenceSaveRunnable(toSave);
			saveRunner.save(function() {
				__stopSpinning(spinEntries, viewModel);
				errorReporter.showErrorMessage(null);
			}, function(operationReport) {
				__stopSpinning(spinEntries, viewModel);

				var description;
		   		if (operationReport.infos && operationReport.infos.length > 0)
		   			description= operationReport.infos[0].description;
		   		errorReporter.showErrorMessage(operationReport.message, description);
			});
		}
	}
}



function doSave(saveRunnable) {
	// for now, require an enclosing doCompoundSave call
	Assert.isTrue(__compoundSaveCounter > 0);

//	doCompoundSave(function() {
		for (var i= 0; i < __mergedSaveRunnables.length; i++) {
			if (__mergedSaveRunnables[i].canMergeWith(saveRunnable)) {
				__mergedSaveRunnables[i].mergeWith(saveRunnable);
				return;
			}
		}
		__mergedSaveRunnables.push(saveRunnable);
//	});
}

com.ibm.team.apt.web.client.internal.WorkItemSaveManager= {
	compoundSave: doCompoundSave,
	save: doSave
};

})();
