/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

(function(){

dojo.provide("com.ibm.rational.cm.web.ui.ajaxModules.RecordModule");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.BorderContainer");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.CqRecordController");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");
dojo.require("com.ibm.rational.cm.web.ui.view.record.CqRecord");

/*
 * AJAX Module of CqRecord
 * query parameters:
 *    action: "new" or "view". Default is "view"
 *    resource: If action is "new", this must be OSLC/CM resource URL for a record type. 
 *    			If action is "view", this must be OSLC/CM resource URL for a record.
 * Examples:
 * View record: https://localhost:9443/jazz/_ajax-modules/com.ibm.rational.cm.Record?resource=http://hostname:port/jazz/cqrest/repo/Demo/db/SAMPL/record/16777224-33554440 
 * New record: https://localhost:9443/jazz/_ajax-modules/com.ibm.rational.cm.Record?action=new&resource=http://hostname:port/jazz/cqrest/repo/Demo/db/SAMPL/record-type/16777224 
 */

var cq = com.ibm.rational.cm.web;
var loginController = com.ibm.rational.cm.cqLoginController;

dojo.declare(
	"com.ibm.rational.cm.web.ui.ajaxModules.RecordModule",
	[dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/record/templates/CqRecordView.html"),
	
	postCreate: function(){
		this.inherited(arguments);

		this._subscriptions=[
			dojo.subscribe("CQ_RECORD_CREATED", this, "_onRecordCreated"),
    		dojo.subscribe("CQ_RECORD_SELECTED", this,"_onRecordSelected"),
    		dojo.subscribe("CQ_EDIT_CANCELED", this, "_onEditCanceled")
		];	
		
		var qryObj = {};
		var qstr = dojo.global.location.search.substr(1);
		if(qstr.length > 0){
			qryObj = dojo.queryToObject(qstr);
		}
		var action = qryObj.action || "view";
		var resourceUrl = qryObj.resource;
	
		if(resourceUrl){
			var urlRepo = cq._getResourcePart(resourceUrl, "repo");
			var urlDb = cq._getResourcePart(resourceUrl, "db");
			// try to get repository/userdb information
			loginController.process({
				noPrompt: true,
				callback: dojo.hitch(this, function(){
					// TODO: getLoginInfo always returns user db set in admin config even if user logins other repo/db. 
					//       because Ping service returns  the data...
					//       for now, we cannot use it...
					//var db = dojo.cookie(loginController.COOKIE_NAME_FOR_DATABASE);
					var db = loginController.getLoginInfo("userdb");  
					if(db){
						var index = db.lastIndexOf("/");
						var loginRepo = db.substring(0,index);
						var loginDb = db.substring(index+1);
					}
					if(loginRepo && loginRepo == urlRepo && loginDb == urlDb){
						this.runRecord(action, resourceUrl,qryObj.fields);
					} else {
						this._reloginToDB(urlRepo, urlDb, action, resourceUrl,qryObj.fields);
					}
				}),
				onCancel: dojo.hitch(this, function(){
					this._reloginToDB(urlRepo, urlDb, action, resourceUrl,qryObj.fields);
				})
			});
		}
	},
	
	uninitialize:function() {
		this.inherited(arguments);
		dojo.forEach(this._subscriptions, dojo.unsubscribe);
	},
		
	_reloginToDB: function(urlRepo, urlDb, action, resourceUrl,prefillFields){
		loginController.doLogout(dojo.hitch(this, function(){
			setTimeout(dojo.hitch(this, function(){
				loginController.setDBInfo(urlRepo + "/" + urlDb);
				this.runRecord(action, resourceUrl,prefillFields);
			}));
		}));
	},
	
	_onRecordCreated: function(msg) {
		var content = {};
		content["oslc_cm:message"] = "oslc_cm:create";
		var record = {};
		record["rdf:resource"] = cq.getResourceHref("RECORD", msg.id);
		record["oslc_cm:label"] = msg.label;
		content["oslc_cm:results"] = [record];
		
		cq.sendFrameResponse(dojo.toJson(content));
	},
	
	_onRecordSelected: function(msg) {
		this.recordController._onRecordSelected(msg);
	},
	
	_onEditCanceled: function(msg) {
		// Test to see if this record is the "original" record being submitted.
		// Its ID will end in "/0". (See CqRecordController._onRecordSubmit.)
		if (msg.state == "SUBMIT" && /^rcm_new_record:.*\/0$/.test(msg.recordId)) {
			var content = {};
			content["oslc_cm:message"] = "oslc_cm:create";
			content["oslc_cm:results"] = [];
		
			cq.sendFrameResponse(dojo.toJson(content));
		}
	},
	
	runRecord: function(action, resourceUrl,prefillFields){
		//
		if(action == "new"){
			var recordTypeId = cq.getResourceIdFromURL("RECORD-TYPE", resourceUrl);
			if(recordTypeId){
				if ( prefillFields != null){
					this.preFill(prefillFields);
				}	
				this.recordController.newRecord(recordTypeId);
			}
		} else {
			var recordId = cq.getResourceIdFromURL("RECORD", resourceUrl);
			if(recordId){
				this.recordController.openRecordView(recordId);
			}
		}
	},
	
	preFill: function(fillFields){
		var fldPayload = "";
		var fldArray = [];
		var updateFlag = false;
		// check if pre-fill field values are available
		if (fillFields !=null){
			fldPayload = dojo.fromJson(unescape(fillFields));
			dojo.forEach(fldPayload, function(r){			
				fldArray.push({"FieldName":r.FieldName,"ControlValue":r.ControlValue,"DataType":'String'});
				updateFlag = true;
				
			});
		}		
		if ( updateFlag){
			var onFormShown = dojo.subscribe("CQ_MODIFY_FORM_SHOWN", function(msg) {
				dojo.unsubscribe(onFormShown);				
				msg.view.updateControls(fldArray);
				
			});
		}	
	}
})	
})();