/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.BuildEngines");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.build.web.ui.internal.utils.Table");
dojo.require("com.ibm.team.build.web.ui.internal.EngineColumn");
dojo.require("com.ibm.team.build.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.build.web.client.internal.BuildClient");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("jazz.util.html");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function(){
	
var SimplePage = com.ibm.team.build.web.ui.internal.SimplePage;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var Table = com.ibm.team.build.web.ui.internal.utils.Table;
var BuildClient = com.ibm.team.build.web.client.internal.BuildClient;
var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;
var EngineIdColumn = com.ibm.team.build.web.ui.internal.EngineIdColumn;
var EngineStatusColumn = com.ibm.team.build.web.ui.internal.EngineStatusColumn;
var EngineStateColumn = com.ibm.team.build.web.ui.internal.EngineStateColumn;
var EngineLastContackTimeColumn = com.ibm.team.build.web.ui.internal.EngineLastContactTimeColumn;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");
var TABLE_SPECS = {
		headings: [{label: "", width:"2%"},
				   {label: messages.labelID, width:"25%"},	
				   {label: messages.labelStatus, width:"43%"},		   
				   {label: messages.labelLastContactTime, width:"30%"}
				  ],
		style: com.ibm.team.build.web.ui.internal.utils.table.Styles.GOLD
	};

dojo.declare("com.ibm.team.build.web.ui.internal.BuildEngines",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.build.web", "ui/internal/templates/ListPage.html"),
	
	widgetsInTemplate: true,
	
	constructor: function() {
		this.loading = messages.loading;
	},
	
	postCreate: function() {
		dojo.addClass(this.domNode, "BuildEngines");
		this.populateHeader();
		this._doQuery();
	},
	
	refresh: function(e) {
		if (e)
			e.preventDefault();
		PlatformUI.getWorkbench().showMessage(messages.loadingBuildEngines);
		this._refreshTime = new Date().getTime();
		this._doQuery();
	},
	
	populateHeader: function() {
		var args = {
			imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/metanav/icons/refresh.gif"),
			label: messages.refresh
		};
		this.metanav.toolbar.addButton(this, dojo.hitch(this,this.refresh), args);
		var heading = dojo.string.substitute(messages.buildEnginesHeading,[ getCurrentProjectArea().name]);
		this.metanav.setTitle(jazz.util.html.escape(heading));
		PlatformUI.getWorkbench().setTitle(dojo.string.substitute(messages.buildsTitle,[heading]));
	},
	
	_doQuery:function() {
		var callback = dojo.hitch(this,this._handleBuildEnginesCallback);
		var args = {projectAreaItemId: getCurrentProjectArea().itemId, errback: callback, callback: callback};
		if (this._refreshTime)
			args.refreshTime = this._refreshTime;
		BuildClient.getEngineStatusRecords(args);
	},
	
	_handleBuildEnginesCallback: function(response){
		if (!this.domNode)
			return;
		if (response instanceof Error) {
			PlatformUI.getWorkbench().hideMessageArea();
			ErrorUtil.handleErrorResponse(response);
		}
		else {
			this._removeChildren(this.content);

			if (!response || !response.engineStatusRecords ||!response.engineStatusRecords.length) {
				var div = document.createElement("div");
				div.className = "row none-text";
				div.innerHTML = messages.noEnginesFound;
				this.content.appendChild(div);
			}
			else {
				var table = new Table(TABLE_SPECS);
				var bodyContainer = table.tbody;
				if(!this._columns){ //Create column objects only once
					this._columns = [new EngineStateColumn(), 
									 new EngineIdColumn(),
									 new EngineStatusColumn(),
									 new EngineLastContackTimeColumn()
									];
				}
				for (var i = 0; i < response.engineStatusRecords.length; i++) {
					this._createResultTable(bodyContainer, response.engineStatusRecords[i]);
				}
				this.content.appendChild(table.domNode);
			}
			PlatformUI.getWorkbench().hideMessageArea();
		}
	},
		
	_createResultTable: function(parent, result){
		var resultRow = document.createElement("tr");
		for (var i = 0; i < this._columns.length; i++) {
			resultRow.appendChild(this._columns[i].getCellElement(result));
		}
		parent.appendChild(resultRow);
	},
	
	_removeChildren: function(parent) {
		while (parent.firstChild)
			parent.removeChild(parent.firstChild);
	}

});
	
})();
