/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.BuildResult");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.build.web.ui.internal.Constants");
dojo.require("com.ibm.team.build.web.ui.internal.utils.TypeUtil");
dojo.require("com.ibm.team.build.web.util.BuildUtil");
dojo.require("com.ibm.team.build.web.ui.internal.utils.SimpleTabContainer");
dojo.require("com.ibm.team.build.web.ui.internal.utils.Table");
dojo.require("com.ibm.team.build.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ResultUtil");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");

dojo.require("com.ibm.team.build.web.ui.internal.query.CompilationCellElementViewer");
dojo.require("com.ibm.team.build.web.ui.internal.query.CompilationRowElementViewer");
dojo.require("com.ibm.team.build.web.ui.internal.query.JUnitCellElementViewer");
dojo.require("com.ibm.team.build.web.ui.internal.query.JUnitRowElementViewer");
dojo.require("com.ibm.team.build.web.ui.BuildConstants");
dojo.require("com.ibm.team.build.web.client.internal.BuildClient");
dojo.require("com.ibm.team.build.web.client.internal.WorkItemClient");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("jazz.util.date");

dojo.require("com.ibm.team.build.web.ui.internal.utils.WorkItemPicker");
dojo.require("com.ibm.team.build.web.ui.internal.utils.UIHelper");
dojo.require("jazz.client.xhr");
dojo.require("jazz.util.html");
dojo.require("jazz.app.ResourceLink");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function() {

var Constants = com.ibm.team.build.web.ui.internal.Constants;
var BuildConstants = com.ibm.team.build.web.ui.BuildConstants;
var BuildClient = com.ibm.team.build.web.client.internal.BuildClient;
var WorkItemClient = com.ibm.team.build.web.client.internal.WorkItemClient;
var TypeUtil = com.ibm.team.build.web.ui.internal.utils.TypeUtil;
var BuildUtil = com.ibm.team.build.web.util.BuildUtil;
var Table = com.ibm.team.build.web.ui.internal.utils.Table;
var ResultUtil = com.ibm.team.build.web.ui.internal.utils.ResultUtil;
var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;
var UIHelper = com.ibm.team.build.web.ui.internal.utils.UIHelper;
var CompilationCellElementViewer = com.ibm.team.build.web.ui.internal.query.CompilationCellElementViewer;
var CompilationRowElementViewer = com.ibm.team.build.web.ui.internal.query.CompilationRowElementViewer;
var JUnitRowElementViewer = com.ibm.team.build.web.ui.internal.query.JUnitRowElementViewer;
var JUnitCellElementViewer = com.ibm.team.build.web.ui.internal.query.JUnitCellElementViewer;

var DateUtil = jazz.util.date;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var WorkItemPicker = com.ibm.team.build.web.ui.internal.utils.WorkItemPicker;
var buttonWidth, tableHeadingWidth, submitWidth, browseWidth;
var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");
var TABLE_SPECS = {headings: [{label: messages.fileName, width: "45%"},
							  {label: messages.description},
							  {label: messages.fileSize, className: "fileSizeHead"}]};
var COMPILATION_TABLE_SPECS = { headings: [{label: messages.labelName, width: "80%"},
										   {label: messages.labelErrors, width: "10%", className: "numbersColumn"},
										   {label: messages.labelWarnings, width: "10%", className: "numbersColumn"}],
								style: com.ibm.team.build.web.ui.internal.utils.table.Styles.BLUE,
								noHeadLine: true};
var JUNIT_TABLE_SPECS = { headings: [{label: messages.labelName, width: "70%"},
									   {label: messages.labelTests, width: "10%", className: "numbersColumn"},		
									   {label: messages.labelFailures, width: "10%", className: "numbersColumn"},
									   {label: messages.labelErrors, width: "10%", className: "numbersColumn"}],
								style: com.ibm.team.build.web.ui.internal.utils.table.Styles.BLUE,
								noHeadLine: true};				
var pickerDialogInfo = null;

dojo.declare("com.ibm.team.build.web.ui.internal.BuildResult", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.build.web", "ui/internal/templates/BuildResult.html"),
	
	widgetsInTemplate: true,
	
	constructor: function() {
		this.messages = messages;
	},
	
	postCreate: function() {
		this.metanav.populateBreadcrumbs();
		this._initButtonWidths();
		this.metanav.toolbar.addButton(this, dojo.hitch(this,this.requestRebuild), {
			imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/metanav/icons/req_rebuild_co.gif"),
			label: messages.requestRebuild
		});
		this.metanav.toolbar.addSeparator();
		this.metanav.toolbar.addButton(this, dojo.hitch(this,this.refresh), {
			imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/metanav/icons/refresh.gif"),
			label: messages.refresh
		});
		this.initialSaveButton();
		this.tabContainer = new com.ibm.team.build.web.ui.internal.utils.SimpleTabContainer({
						callbackTabSwitch: dojo.hitch(this,this._renderTab),
						tabOrder: [BuildConstants.RESULT_TAB_OVERVIEW, BuildConstants.RESULT_TAB_COMPILATION,
						BuildConstants.RESULT_TAB_JUNIT, BuildConstants.RESULT_TAB_DOWNLOADS, BuildConstants.RESULT_TAB_LINKS,
						BuildConstants.RESULT_TAB_LOGS, BuildConstants.RESULT_TAB_WORKITEMS]}, this.tabContainer);
		this._updatedTabs = {};
		this._renderedTabs = {};
		this.connect(this.createReported,"onclick",this._handleCreateReported);
		this.connect(this.associateReported,"onclick",this._handleAssociateReported);
		this._doQuery();
		this._registerNavigation();
	},
	
	//currently listen to the change for tags and deletion allowed.
	_inputChangeListener: function(){
		if (this.result.buildResult.deleteAllowed == this.deletionAllowed.checked && this.result.buildResult.tags == this.tags.firstChild.value){
			this._setClean();
			return;
		}		
		this._setDirty();
	},
	
	_setDirty: function(){
		var heading = ResultUtil.getLabel(this.result, true, true);
		this.browserTitle = dojo.string.substitute(messages.buildsTitle,[ heading ]);
		PlatformUI.getWorkbench().setTitle(this.browserTitle);
		this.metanav.setTitle(dojo.string.substitute(messages.buildHeading,[ jazz.util.html.escape(heading) ]) + "*");
		this.save.disabled = false;	
	},
	
	_setClean: function(){
		var heading = ResultUtil.getLabel(this.result, true, true);
		this.browserTitle = dojo.string.substitute(messages.buildsTitle,[ heading ]);
		PlatformUI.getWorkbench().setTitle(this.browserTitle);
		this.metanav.setTitle(dojo.string.substitute(messages.buildHeading,[ jazz.util.html.escape(heading) ]));
		this.save.disabled = true;	
	},
	
	revert: function() {
		this.deletionAllowed.checked = this.result.buildResult.deleteAllowed;
		this.tags.firstChild.value = this.result.buildResult.tags;
		this._setClean();
	},
	
	_initButtonWidths: function() {
		if (!buttonWidth) {
			var testNode = document.createElement("div");
			testNode.style.position = "relative";
			document.body.appendChild(testNode);
			tableHeadingWidth = UIHelper.convertToEms(testNode, UIHelper.findMaxStringDisplayWidth([messages.nameHeading, messages.descriptionHeading],testNode)) + "em";
			buttonWidth = UIHelper.convertToEms(testNode, UIHelper.findMaxStringDisplayWidth([messages.addLabel, messages.removeLabel, messages.submitLabel],testNode, 61) + 12) + "em";
			submitWidth = UIHelper.convertToEms(testNode, UIHelper.findMaxStringDisplayWidth([messages.submitLabel],testNode, 61) + 12) + "em";
			browseWidth = UIHelper.convertToEms(testNode, UIHelper.findMaxStringDisplayWidth([messages.browseLabel],testNode, 61) + 12) + "em";
			document.body.removeChild(testNode);
		}
	},
	
	initialSaveButton: function(){
		this.save = document.createElement("input");
		this.save.type = "button";
		this.save.className = "button";
		this.save.value = messages.save;
		this.save.style.width = buttonWidth;
		this.save.disabled = true;
		this.metanav.toolbar.addWidget({domNode: this.save});
		this.connect(this.save,"onclick",dojo.hitch(this,this.saveModification));
	},
	
	//save modification including tags and delete allowed changes.
	saveModification: function(){
		var tagsValue = this.tags.firstChild.value;
		var deleteAllowedValue = this.deletionAllowed.checked;
		var args = {
				resultItemId: this.resultId,
				tags:tagsValue,
				deleteAllowed:deleteAllowedValue,
				callback: dojo.hitch(this, this._onTagsChanged),
				errback: this._handleModificationErrorback
		};
		BuildClient.updateBuildProperties(args);
		PlatformUI.getWorkbench().showMessage(messages.saving);
	},
	
	_handleModificationErrorback: function(response) {
		if (response instanceof Error) {
			ErrorUtil.handleErrorResponse(response, messages.errorDuringSavingTags);
		}
		PlatformUI.getWorkbench().hideMessageArea();
	},
		
	_onTagsChanged: function(){
		BuildClient.flushResultRecordsByUUID(this.result.buildResult.itemId);
		BuildClient.flushQueue({projectAreaItemId: getCurrentProjectArea().itemId});
		this._setClean();
		if (this.domNode)
			this.refresh();
		PlatformUI.getWorkbench().hideMessageArea();
	},
	
	_registerNavigation: function() {
			if (!this.navigateAwayHandle) {
				this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
			}
			if (!this.onBeforeUnloadHandle) {
				this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this._handleNavigateAway), /*priority*/ 10000);
			}
	},
	
	/*
	 * This gets called when navigating away from the web ui, e.g., to www.jazz.net
	 */
	_handleNavigateAway: function(e) {
			if (this.save.disabled) return false;
			return messages.confirmNavigateAway;
	},
	
	_handleNavigate: function(targetURL){
		var currentLoc = new dojo._Url(window.location.href);
		var workbenchNavigation = false;
		var navigateAway;
		if (targetURL.host && targetURL.scheme && !(targetURL.host === currentLoc.host && targetURL.scheme === currentLoc.scheme)) {
			// different domain - allow onBeforeUnload to prompt
			navigateAway = true;
		}
		else {
			if (targetURL.fragment && ((!targetURL.host && !targetURL.path) || (targetURL.path && targetURL.path === currentLoc.path))) {
				// link within web UI
				if (currentLoc.fragment.indexOf("com.ibm.team.build") < 0) {
					return false;
				}
				if (targetURL.fragment.indexOf("com.ibm.team.build") < 0) {
					// we're navigating to another area of the web UI
					workbenchNavigation = true;
					if (this._stripTabParam(targetURL.fragment) == this._stripTabParam(currentLoc.fragment)) {
						// the link is to the current url (not including the tab param)
						navigateAway = true;
					}
					else {
						// same exact base URI - prompt for save/cancel
						navigateAway = this.navigateAwayEvent(workbenchNavigation);
						// revert the changes, so they are gone if the user comes back and so we don't trigger a second prompt in onbeforeunload
						if (navigateAway) {
							this.revert();
						}
					}
				}
				else if(this._stripTabParam(targetURL.fragment) === "action=jazz.viewPage&id=com.ibm.team.build" ||
					this._stripTabParam(currentLoc.fragment) === "action=jazz.viewPage&id=com.ibm.team.build") {
					navigateAway = true;
				}
				else if (this._stripTabParam(targetURL.fragment) != this._stripTabParam(currentLoc.fragment)) {
						// other pages in build.
						navigateAway = this.navigateAwayEvent(true);						
				}
				else{
					navigateAway = true;
				}
			}
			else {
				//external links and toolbar actions.
				navigateAway = true;
			}
		}
		return !navigateAway;
	},

	/*
	 * this gets called when navigating to another page (e.g. Dashboards)
	 */
	navigateAwayEvent: function(navigation) {
		if (!navigation) return true;
		if (!this.save.disabled) {
			//if we have unsaved modification for tags. ask for saving.
			if (confirm(messages.confirmNavigateAway)) {
				return true;
			}
			else {
				return false;
			}
		}
		return true;
	},

	_stripTabParam: function(string) {
		if (!string)
			return string;
		var index = string.indexOf("&tab=");
		if (index == -1)
			return string;
		var result = string.substring(0,index);
		var end = string.substring(index+1);
		index = end.indexOf("&");
		if (index == -1)
			return result;
		return result + end.substring(index);
	},
	
	requestBuild: function() {
		if(this._confirmExecute()){
			location.href = BuildUtil.getRequestBuildHref(this.result.buildDefinition.itemId);
		}
	},
	
	requestRebuild: function() {
		if(this._confirmExecute()){
			location.href = BuildUtil.getRequestRebuildHref(this.resultId);	
		}
		
	},
	
	refresh: function(e) {
		if (e)
			e.preventDefault();
		if (this._confirmExecute()){
			this._doRefresh();
		}
	},

	_doRefresh: function() {
		PlatformUI.getWorkbench().showMessage(messages.loadingBuild);
		this._updatedTabs = {};
		// update the current tab
		this._pendingTabUpdate = this.tabContainer.getSelection();
		this._doQuery();
		this._setClean();
	},
	
	deleteBuild: function(e) {
		if(this._confirmExecute()){
			if (e)
				e.preventDefault();
			if (!confirm(messages.confirmDelete))
				return;
			PlatformUI.getWorkbench().showMessage(messages.deletingBuild);
	
			var args = { resultItemId: this.resultId,
								errback: this._handleResultErrorResponseAlert,
								callback: dojo.hitch(this, this._handleDeleteCallback) };
			// make the request and add an error callback to hide the message if we are still on the page
			BuildClient.deleteBuildResult(args).addErrback(dojo.hitch(this,function(response) {
				if (this.domNode)
					PlatformUI.getWorkbench().hideMessageArea();
			}));
		}
	},
	
	stopBuild: function(e) {
		if (e)
			e.preventDefault();
		var state = this.result && this.result.buildResult && this.result.buildResult.buildState;
		if (state == "NOT_STARTED") {
			this.cancelPendingRequest();
		} else if (state == "IN_PROGRESS") {
			this.abandonBuild();
		}
	},
	
	cancelPendingRequest: function() {
		if (confirm(messages.confirmCancel)) {
			// TODO: show a progress message
			this.doStateChange("CANCELED");
		}
	},

	abandonBuild: function() {
		if(this._confirmExecute()){
			if (confirm(messages.confirmAbandon)) {
			// TODO: show a progress message
			this.doStateChange("INCOMPLETE");
			}
		}
	},
	
	doStateChange: function(newState) {
		// TODO: hide the progress message in the callbacks
		var args = {
			resultItemId: this.resultId,
			buildState: newState,
			callback: dojo.hitch(this, this._onStateChanged),
			errback: this._handleResultErrorResponseAlert
		};
		BuildClient.updateBuildState(args);
	},
	
	_confirmExecute: function(){
		if (!this.save.disabled){
			if (!confirm(messages.confirmNavigateAway)) {
				return false;
			}
		}
		return true;
	},
	
	_handleResultErrorResponse: function(response) { 
		ErrorUtil.handleErrorResponse(response, messages.resultNotFound); 
	},

	_handleResultErrorResponseAlert: function(response) { 
		ErrorUtil.handleErrorResponseAlert(response, messages.resultNotFound); 
	},
	
	_onStateChanged: function() {
		BuildClient.flushResultRecordsByUUID(this.result.buildResult.itemId);
		BuildClient.flushQueue({projectAreaItemId: getCurrentProjectArea().itemId});
		this._setClean();
		if (this.domNode)
			this.refresh();
	},
	
	getArtifactInfo: function(srh) {
		if (!this.result) {
			if (!this._pendingInfoRequests)
				this._pendingInfoRequests = [];
			this._pendingInfoRequests.push(srh);
		}
		else
			srh.doCallback(this._getInfo());
	},
	
	_getTabNode: function(tabId) {
		switch(tabId) {
			case BuildConstants.RESULT_TAB_OVERVIEW:
				return this.overviewTab;
			case BuildConstants.RESULT_TAB_COMPILATION:
				return this.compilationTab;
			case BuildConstants.RESULT_TAB_JUNIT:	
				return this.junitTab;
			case BuildConstants.RESULT_TAB_DOWNLOADS:
				return this.downloadsTab;
			case BuildConstants.RESULT_TAB_LINKS:
				return this.linksTab;
			case BuildConstants.RESULT_TAB_LOGS:
				return this.logsTab;
			case BuildConstants.RESULT_TAB_WORKITEMS:
				return this.workItemsTab;
		}
	},
	
	_renderTab: function(tabId) {
		window.scrollTo(0,0);
		if (this._updatedTabs[tabId])
			return;
		this._updatedTabs[tabId] = true;
		if (tabId == BuildConstants.RESULT_TAB_COMPILATION)
			this._doGetCompilationInfo();
		else if (tabId == BuildConstants.RESULT_TAB_JUNIT)
			this._doGetJUnitInfo();	
		// the rest of the tabs require the build result
		// if it's not ready, queue an update for later
		if (this._queryPending) {
			this._updatedTabs[tabId] = false;
			this._pendingTabUpdate = tabId;
			// if the tab has never been rendered, use old data for now, return otherwise
			if (this._renderedTabs[tabId])
				return;
		}
		switch(tabId) {
			case BuildConstants.RESULT_TAB_OVERVIEW:
				this.populateOverview();
				break;
			case BuildConstants.RESULT_TAB_DOWNLOADS:
				if (this.result.contributions.downloads.length === 0) 
					this._populateEmptySection(this.downloadsTab, messages.emptyDownloads);
				else
					this._populateSection(this.downloadsTab, this.result.contributions.downloads, true, true);
				break;
			case BuildConstants.RESULT_TAB_LINKS:
				if (this.result.contributions.links.length === 0)
					this._populateEmptySection(this.linksTab, messages.emptyLinks);
				else
					this._populateSection(this.linksTab, this.result.contributions.links, true, false, true);
				break;
			case BuildConstants.RESULT_TAB_LOGS:
				if (this.result.contributions.logs.length === 0)
					this._populateEmptySection(this.logsTab, messages.emptyLogs);
				else
					this._populateSection(this.logsTab, this.result.contributions.logs, true, true);
				break;
			case BuildConstants.RESULT_TAB_WORKITEMS:
				if (this.result.contributions.reported.length === 0)
					this._populateEmptySection(this.reported, messages.emptyReported);
				else
					this._populateSection(this.reported, this.result.contributions.reported, false, false, true);
				if (this.result.contributions.included.length === 0)
					this._populateEmptySection(this.included, messages.emptyIncluded);
				else
					this._populateSection(this.included, this.result.contributions.included, false, false, true);
				break;
		}
		this._renderedTabs[tabId] = true;
	},
	
	_doQuery: function() {
		var args = {	resultItemId: this.resultId,
						types: ["com.ibm.team.build.common.model.IBuildResultContribution.artifact",
								"com.ibm.team.build.common.model.IBuildResultContribution.log",
								"com.ibm.team.build.common.model.IBuildResultContribution.link",
								"com.ibm.team.build.common.model.ICompileContribution",
								"com.ibm.team.build.common.model.IJUnitTestSuiteContribution"],
						callback: dojo.hitch(this, this._handleBuildResultCallback),
						errback: this._handleResultErrorResponse };
		this._queryPending = true;
		// make the request and add an error handler to hide the message area if we are still on the page
		BuildClient.getResultPresentation(args).addErrback(dojo.hitch(this,function() {
			if (this.domNode)
				PlatformUI.getWorkbench().hideMessageArea();
		}));	
	},
	
	_handleCreateReported: function(e) {
		dojo.stopEvent(e);
		if (!this._confirmExecute()) {
			return;
		}

		PlatformUI.getWorkbench().showMessage(messages.createReportedMessage);

		var buildLocation  = "itemOid/com.ibm.team.build.BuildResult/" + this.result.buildResult.itemId;
		var ref = {};
		ref['oslc_cm:label'] = ResultUtil.getLabel(this.result, true);
		ref['rdf:resource'] = buildLocation; 

		var postBody = {};
		postBody['dc:type'] = "defect";
		postBody['dc:title'] = dojo.string.substitute(messages.createReportedSummary, [ResultUtil.getLabel(this.result)]);
		postBody['rtc_cm:com.ibm.team.build.linktype.reportedWorkItems.com.ibm.team.build.common.link.reportedAgainstBuilds'] = dojo.objectToQuery(ref);

		var url = net.jazz.ajax._contextRoot + "/oslc/contexts/" + getCurrentProjectArea().itemId + "/drafts/workitems";
		var xhrArgs = {
			url: url,
			headers: {
				"accept": "text/json"
			},
			postData: dojo.objectToQuery(postBody),
			handleAs: "json"
		};
		var deferred = jazz.client.xhrPost(xhrArgs);
		deferred.addCallback(dojo.hitch(this, function(response) {
			PlatformUI.getWorkbench().hideMessageArea();
			this._setClean();
		    var href = response['rdf:resource'];		    
			if (href) {
				jazz.app.currentApplication.handleResource(href);
			}
		}));		
		deferred.addErrback(ErrorUtil.handleErrorResponseAlert);
		deferred.addErrback(dojo.hitch(this,function(){
			if (this.domNode)
				PlatformUI.getWorkbench().hideMessageArea();
		}));
	},
	
	_handleAssociateReported: function(e) {
		dojo.stopEvent(e);
		if (!this._confirmExecute()) {
			return;
		}
		if (pickerDialogInfo) {
		    this._openWorkItemPicker();
		} else {
			PlatformUI.getWorkbench().showMessage(messages.loading);
			var documentURL= net.jazz.ajax._contextRoot + "/oslc/contexts/" + getCurrentProjectArea().itemId + "/workitems/services.xml";
		   	WorkItemClient.loadDocument(documentURL, dojo.hitch(this, this._loadItemPicker));
		}
	},
	
	_loadItemPicker: function(serviceXML) {
		PlatformUI.getWorkbench().hideMessageArea();
		
		// Cache dialog info obtained from parsing service.xml
		pickerDialogInfo = WorkItemClient.findRemoteItemPickerInfo(serviceXML);

		this._openWorkItemPicker();
	},
	
	_openWorkItemPicker: function() {
		var dialogInfo = pickerDialogInfo;
		if (!dialogInfo) {
			return;
		}
		var args = {
			dialogTitle: dialogInfo.title,
            secondaryTitle: "",
            parentElement: dojo.body(),
            width: dialogInfo.widthHint,
            height: dialogInfo.heightHint,
            namespace: dialogInfo.namespace,
            onOk: dojo.hitch(this, this._handleOnOk),
            pickerURL: dialogInfo.url
            //pickerURL: net.jazz.ajax._contextRoot + "/_ajax-modules/com.ibm.team.workitem.WIPicker?projectAreaName=" + getCurrentProjectArea().name
		};
		new WorkItemPicker(args);
	},
	
	_handleOnOk: function(result){
		PlatformUI.getWorkbench().showMessage(messages.associatingLink);
        var workItemId = result[0].url.split("/");		
		var url = net.jazz.ajax._contextRoot + "/oslc/workitems/" + workItemId[workItemId.length -1] + "/rtc_cm:com.ibm.team.build.linktype.reportedWorkItems.com.ibm.team.build.common.link.reportedAgainstBuilds";
		var resultLabel = ResultUtil.getLabel(this.result, true);
		var buildLocation  = "itemOid/com.ibm.team.build.BuildResult/" + this.result.buildResult.itemId;
		var xhrArgs = {
			url: url,
			headers: {
				"accept": "text/json"
			},
			handleAs: "json"
		};
		var deferredGet = jazz.client.xhrGet(xhrArgs);
		deferredGet.addCallback(dojo.hitch(this, function(responseGet) {
			for(var i = 0; i < responseGet.length; i++){
				if(responseGet[i]['oslc_cm:label'] == resultLabel){
					// return if the build result link already exists in select workitem
					if (this.domNode) {
						PlatformUI.getWorkbench().hideMessageArea();
					}
					return;
				}
			}
			// Add link to workitem
			var ref = {};
			ref['oslc_cm:label'] = resultLabel;
			ref['rdf:resource'] = buildLocation;
			var postBody = dojo.toJson(ref);
			// Post data
			var xhrArgsPost = {
				url: url,
				headers: {
					"accept": "text/json",
					"content-type": "text/json"
				},
				postData: postBody,
				handleAs: "json"
			};
			
			var deferredPost = jazz.client.xhrPost(xhrArgsPost);
			deferredPost.addCallback(dojo.hitch(this, function() {
				if (this.domNode) {
					this._doRefresh();
				}
			}));
					
			deferredPost.addErrback(ErrorUtil.handleErrorResponseAlert);
			deferredPost.addErrback(dojo.hitch(this, function() {
				if (this.domNode) {
					PlatformUI.getWorkbench().hideMessageArea();
				}
			}));					
		}));
		deferredGet.addErrback(ErrorUtil.handleErrorResponseAlert);
		deferredGet.addErrback(dojo.hitch(this, function() {
			if (this.domNode)
				PlatformUI.getWorkbench().hideMessageArea();
		}));
	},
	
	_handleCompileSummaryCallback: function(response){
		if (!this.domNode)
			return;
		var unknownLabel = dojo.string.substitute(messages.compileSummaryLabel,[messages.na, messages.na ]);
		this._buildCompileJUnitSummary(response, BuildConstants.RESULT_TAB_COMPILATION, unknownLabel, this._junitSummaryNode);	
	},
	
	_handleJUnitSummaryCallback: function(response){
		if (!this.domNode)
			return;
		var unknownLabel = dojo.string.substitute(messages.junitSummaryLabel,[messages.na, messages.na, messages.na]);
		this._junitSummaryNode = this._buildCompileJUnitSummary(response, BuildConstants.RESULT_TAB_JUNIT, unknownLabel);			
	},
		
	_buildCompileJUnitSummary: function(response, tabId, unknownLabel, refNode){
		var summaryNode = document.createElement("div");
		summaryNode.className = "link";
		if(response instanceof Error){
			var sumInfo  = document.createElement("span");
			sumInfo.innerHTML = unknownLabel;
			summaryNode.appendChild(sumInfo);
		}else{
			var a = document.createElement("a");
			a.href = BuildUtil.getResultHref(this.resultId, tabId);
			this.connect(a,"onclick",dojo.hitch(this,function(e) {
				this.tabContainer.selectTab(tabId);
			}));
			var errorCount = response.errorCount || 0;
			if(tabId == BuildConstants.RESULT_TAB_JUNIT){
				var testCount = response.testCount || 0;	
				var failureCount = response.failureCount || 0;	
				a.innerHTML = dojo.string.substitute(messages.junitSummaryLabel,[testCount, failureCount, errorCount]);	
			}
			if(tabId == BuildConstants.RESULT_TAB_COMPILATION){
				var warningCount = response.warningCount || 0;
				a.innerHTML = dojo.string.substitute(messages.compileSummaryLabel,[errorCount, warningCount ]);
			}
			summaryNode.appendChild(a);
		}
		if (refNode)
			this.contributions.insertBefore(summaryNode, refNode);
		else
			this.contributions.appendChild(summaryNode);
		return summaryNode;
	},
	
	// TODO: We should be showing a message in the message area here
	_doGetCompilationInfo: function(){
		var args = {
			resultItemId: this.resultId,
			callback: dojo.hitch(this, this._handleCompilationsCallback),
			errback: this._handleResultErrorResponse
		};
		BuildClient.getCompilationContributions(args);
	},

	// TODO: We should be showing a message in the message area here
	_doGetJUnitInfo: function(){	
		var args = {  resultItemId: this.resultId,
				      callback: dojo.hitch(this, this._handleJUnitCallback),
				      errback: this._handleResultErrorResponse };
		BuildClient.getJUnitInfo(args);				
	},
	
	_handleJUnitCallback: function(response){
		if (!this.domNode)
			return;
		this.junits = response;
		this.populateJUnits();
	},
	
	_handleCompilationsCallback: function(response){
		if (!this.domNode)
			return;
		this.compilations = response;
		this.parseCompilations();
		this.populateCompilations();			
	},
		
	_handleBuildResultCallback:function(response){
		if (!this.domNode)
			return;
		this.result = response;
		this._queryPending = false;
		this.populate();
		if (this._pendingTabUpdate)
			this._renderTab(this._pendingTabUpdate);
		this._flushPendingRequests();
		PlatformUI.getWorkbench().hideMessageArea();
	},

	_handleDeleteCallback:function(){
		// no need to hide the message area here
		// either the user has already left or we are about to redirect them (thus showing a new message in BuildPage)
		if (this.result && this.result.buildDefinition) {
			BuildClient.flushResults({definitionItemId: this.result.buildDefinition.itemId});
			if (this.buildPage)
				this.buildPage.buildDeleted(this.result.buildResult.itemId);
			// only forward to the definition page if the user has not already browsed away
			if (this.domNode)
				location.href = BuildUtil.getDefinitionHref(this.result.buildDefinition.itemId);
		}
	},
	
	_flushPendingRequests: function() {
		if (this._pendingInfoRequests) {
			var info = this._getInfo();
			for (var i = 0; i < this._pendingInfoRequests.length; i++)
				this._pendingInfoRequests[i].doCallback(info);
		}
		this._pendingInfoRequests = null;
	},
	
	_getInfo: function() {
		if (!this.result)
			return null;
		return {
			itemId: this.resultId,
			title: ResultUtil.getLabel(this.result,true,false),
			iconUrl: this.result.buildResult.personalBuild ? dojo.moduleUrl("com.ibm.team.build.web", "graphics/result/icons/bld_rslt_personal.gif") :
						dojo.moduleUrl("com.ibm.team.build.web", "graphics/result/icons/bld_rslt.gif"),
			href: BuildUtil.getResultHref(this.resultId)
		};
		
	},
	
	populate: function() {
		if ( !this.isRequestBuildExisted && this.result.buildDefinition) {
			this.metanav.populateBreadcrumbs(this.result.buildDefinition);
			this.metanav.toolbar.addButton(this, dojo.hitch(this,this.requestBuild), {
				imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/metanav/icons/rquebuild.gif"),
				label: messages.requestBuild, index: 0
			});
			this.isRequestBuildExisted = true;
		}
		var heading = ResultUtil.getLabel(this.result, true, true);
		this.browserTitle = dojo.string.substitute(messages.buildsTitle,[ heading ]);
		PlatformUI.getWorkbench().setTitle(this.browserTitle);
		this.metanav.setTitle(dojo.string.substitute(messages.buildHeading,[ jazz.util.html.escape(heading) ]));
		
		this.sortContributions(this.result.contributions);
		this.populateStatusArea();
		this.updateToolbar();
		
		if (!this.tabContainer.hasTab(BuildConstants.RESULT_TAB_OVERVIEW))
			this.tabContainer.addTab(BuildConstants.RESULT_TAB_OVERVIEW, messages.overview, dojo.hitch(this, this._getTabNode), BuildUtil.getResultHref(this.resultId, BuildConstants.RESULT_TAB_OVERVIEW));
		if (this._hasCompileContributions && !this.tabContainer.hasTab(BuildConstants.RESULT_TAB_COMPILATION))
			this.tabContainer.addTab(BuildConstants.RESULT_TAB_COMPILATION, messages.compilation, dojo.hitch(this, this._getTabNode), BuildUtil.getResultHref(this.resultId, BuildConstants.RESULT_TAB_COMPILATION));
		if (this._hasJUnitContributions && !this.tabContainer.hasTab(BuildConstants.RESULT_TAB_JUNIT))
			this.tabContainer.addTab(BuildConstants.RESULT_TAB_JUNIT, messages.junit, dojo.hitch(this, this._getTabNode), BuildUtil.getResultHref(this.resultId, BuildConstants.RESULT_TAB_JUNIT));	
		if (this.result.contributions.downloads.length > 0 && !this.tabContainer.hasTab(BuildConstants.RESULT_TAB_DOWNLOADS))
			this.tabContainer.addTab(BuildConstants.RESULT_TAB_DOWNLOADS, messages.downloads, dojo.hitch(this, this._getTabNode), BuildUtil.getResultHref(this.resultId, BuildConstants.RESULT_TAB_DOWNLOADS));					
		if (this.result.contributions.links.length > 0 && !this.tabContainer.hasTab(BuildConstants.RESULT_TAB_LINKS))
			this.tabContainer.addTab(BuildConstants.RESULT_TAB_LINKS, messages.externalLinks, dojo.hitch(this, this._getTabNode), BuildUtil.getResultHref(this.resultId, BuildConstants.RESULT_TAB_LINKS));
		if (this.result.contributions.logs.length > 0 && !this.tabContainer.hasTab(BuildConstants.RESULT_TAB_LOGS))
			this.tabContainer.addTab(BuildConstants.RESULT_TAB_LOGS, messages.logs, dojo.hitch(this, this._getTabNode), BuildUtil.getResultHref(this.resultId, BuildConstants.RESULT_TAB_LOGS));
		if (this.result.buildResult.buildState != "NOT_STARTED" && this.result.buildResult.buildState != "CANCELED" && !this.tabContainer.hasTab(BuildConstants.RESULT_TAB_WORKITEMS))
			this.tabContainer.addTab(BuildConstants.RESULT_TAB_WORKITEMS, messages.workItems, dojo.hitch(this, this._getTabNode), BuildUtil.getResultHref(this.resultId, BuildConstants.RESULT_TAB_WORKITEMS));
		if (!this._isInitialized) {
			this._isInitialized = true;
			var defaultTabId = this.tabId && this.tabContainer.hasTab(this.tabId) ? this.tabId : BuildConstants.RESULT_TAB_OVERVIEW;
			this.tabContainer.selectTab(defaultTabId);
		}
	},
	
	parseCompilations: function() {
		var result = [], indexMap = {};
		for (var i = 0; i < this.compilations.length; i++) {
			var c = this.compilations[i];
			var name = c.componentName;
			if (typeof indexMap[name] !== "number") {
				var current = {name: name, packageCount: 0, errorCount: 0, warningCount: 0, simpleContributions: []};
				current[Constants.REPOSITORY_TYPE] = Constants.COMPILE_COMPONENT_ROW; // mimic component dto
				indexMap[name] = result.length;
				result.push(current);
			}
			var index = indexMap[name];
			result[index].packageCount += c.packageCount;
			result[index].errorCount += c.errorCount;
			result[index].warningCount += c.warningCount;
			if (!c.packageCount) {
				c[Constants.REPOSITORY_TYPE] = Constants.COMPILE_SOURCEFILE_ROW;
				result[index].simpleContributions.push(c);
			}
		}
		result.sort(function(a,b) {
			if (!a.name) return 1;
			if (!b.name) return -1;
			return a.name.localeCompare(b.name);
		});
		this.compilations = result;
	},
	
	populateCompilations: function(){
		if (this.compilations.length === 0) 
			this._populateEmptySection(this.compilationTab, messages.emptyCompilation);
		else
			this._populateCompilationsSection(this.compilationTab, this.compilations);		
	},
	
	populateJUnits: function(){
		if (this.junits.length === 0) 
			this._populateEmptySection(this.junitTab, messages.emptyJunit);
		else
			this._populateJUnitSection(this.junitTab, this.junits);				
	},
	
	populateStatusArea: function() {
		var statusClass = this._getStatusClass(this.result);
		// set classname explicitly (instead of dojo.addClass) to ensure old status is removed
		this._statusArea.className = "status-area " + statusClass;
		this.state.innerHTML = "";
		this.state.appendChild(document.createTextNode(this._getStateText(this.result.buildResult.buildState)));
		this.start.innerHTML = "";
		this.end.innerHTML = "";
		this.duration.innerHTML = "";
		if (this.result.buildResult.buildState != "NOT_STARTED" && this.result.buildResult.buildState != "CANCELED") {
			var date = new Date(this.result.buildResult.buildStartTime);
			this.start.appendChild(document.createTextNode(DateUtil.formatDif(date, true)));
			if (this.result.buildResult.buildState != "IN_PROGRESS") {
				date.setTime(date.getTime() + this.result.buildResult.buildTimeTaken);
				this.end.appendChild(document.createTextNode(DateUtil.formatDif(date, true)));
				this.duration.appendChild(document.createTextNode(DateUtil.formatDuration(this.result.buildResult.buildTimeTaken)));
			} else {
				this.duration.appendChild(document.createTextNode(ResultUtil.getDuration(this.result)));
			}
		}
		var imgSrc;
		if (statusClass == "ok-status")
			imgSrc = this.result.buildResult.buildState == "NOT_STARTED" ?
				dojo.moduleUrl("com.ibm.team.build.web", "graphics/result/icons/pendingblds_obj_28.png") :
				dojo.moduleUrl("com.ibm.team.build.web", "graphics/result/icons/completedblds_obj_24.png");
		else if (statusClass == "error-status")
			imgSrc = dojo.moduleUrl("com.ibm.team.build.web", "graphics/result/icons/error_obj_24.png");
		else if (statusClass == "warning-status")
			imgSrc = dojo.moduleUrl("com.ibm.team.build.web", "graphics/result/icons/warn_obj_24.png");
		else if (statusClass == "incomplete-status")
			imgSrc = this.result.buildResult.buildState == "CANCELED" ?
							dojo.moduleUrl("com.ibm.team.build.web", "graphics/result/icons/build_result_obj_28.png") :
							dojo.moduleUrl("com.ibm.team.build.web", "graphics/result/icons/abanblds_obj_28.png");
		this._statusImage.src = imgSrc;
		this._statusArea.style.display = "block";
	},
	
	updateToolbar: function() {
	    var state = this.result && this.result.buildResult && this.result.buildResult.buildState;
	    if (state == this.prevState) {
	    	return;
	    }
	    if (this.extraButton) {
	    	this.metanav.toolbar.removeItem(this.extraButton);
	    	delete this.extraButton;
	    }
	    if (state == "NOT_STARTED" || state == "IN_PROGRESS") {
			this.extraButton = this.metanav.toolbar.addButton(this, dojo.hitch(this,this.stopBuild), {
				imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/eclipse/stop.gif"),
				label: (state == "NOT_STARTED" ? messages.cancelBuild : messages.abandonBuild),
				index: 3
			});
		} else {
			this.extraButton = this.metanav.toolbar.addButton(this, dojo.hitch(this,this.deleteBuild), {
				imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/eclipse/delete_edit.gif"),
				label: messages.deleteBuild,
				index: 3
			});
		}
		this.prevState = state;
	},
	
	populateOverview: function() {
		
		this.tagsInput.value = this.result.buildResult.tags;
		this.connect(this.tags,"onkeyup",this._inputChangeListener);
		
		//create deletion allowed.		
		this.deletionAllowed.checked =  this.result.buildResult.deleteAllowed;
		this.connect(this.deletionAllowed,"onclick",this._inputChangeListener);
		new jazz.ui.HoverPopup({around: this.control, html: messages.deletionAllowedMessage, persist: false, openDelay: 1000});
		
		if (this.result.buildResult.buildState == "NOT_STARTED" || this.result.buildResult.buildState == "CANCELED") {
			this.tagsInput.disabled = true;
			this.deletionAllowed.disabled = true;
		} else {
			this.tagsInput.disabled = false;
			this.deletionAllowed.disabled = false;
		}
		
		this.requestor.innerHTML = "";
		if (this._isScheduledBuild()) {
			this.requestor.appendChild(document.createTextNode(messages.scheduled));
			dojo.addClass(this.requestor,"none-text");
		} else if (this.result.requestor) {
			this.requestor.appendChild(document.createTextNode(this.result.requestor.name));
			dojo.removeClass(this.requestor,"none-text");
		}
		
		this.definition.innerHTML = "";
		this.history.innerHTML = "";
		if (this.result.buildDefinition) {
			var a = document.createElement("a");
			a.href = BuildUtil.getDefinitionHref(this.result.buildDefinition.itemId);
			a.appendChild(document.createTextNode(this.result.buildDefinition.id));
			this.definition.appendChild(a);
			dojo.removeClass(this.definition,"none-text");
			a = document.createElement("a");
			a.href = BuildUtil.getDefinitionHref(this.result.buildDefinition.itemId);
			a.appendChild(document.createTextNode(dojo.string.substitute(messages.historyLabel,[this.result.buildCount])));
			this.history.appendChild(a);
			dojo.removeClass(this.history,"none-text");
		} else {
			this.definition.appendChild(document.createTextNode(messages.notAvailable));
			dojo.addClass(this.definition,"none-text");
			this.history.appendChild(document.createTextNode(messages.notAvailable));
			dojo.addClass(this.history,"none-text");
		}
		
		this.engine.innerHTML = "";
		if (this.result.buildEngine) {
			this.engine.appendChild(document.createTextNode(this.result.buildEngine.id));
			dojo.removeClass(this.engine,"none-text");
		} else {
			this.engine.appendChild(document.createTextNode(messages.notAvailable));
			dojo.addClass(this.engine,"none-text");
		}
		
		var conts = this.result.contributions;
		this.contributions.innerHTML = "";
		this._buildContributionLink(conts.downloads.length, messages.downloadLabel, messages.downloadLabelPlural, BuildConstants.RESULT_TAB_DOWNLOADS);
		this._buildContributionLink(conts.links.length, messages.linksLabel, messages.linksLabelPlural, BuildConstants.RESULT_TAB_LINKS);
		this._buildContributionLink(conts.logs.length, messages.logLabel, messages.logLabelPlural, BuildConstants.RESULT_TAB_LOGS);
		this._buildContributionLink(conts.reported.length, messages.reportedLabel, messages.reportedLabelPlural, BuildConstants.RESULT_TAB_WORKITEMS);
		this._buildContributionLink(conts.included.length, messages.includedLabel, messages.includedLabelPlural, BuildConstants.RESULT_TAB_WORKITEMS);
		this._updateSummaries();
	},
	
	_updateSummaries: function() {
		this._junitSummaryNode = null;
		if (this._hasCompileContributions) {
			var callback = dojo.hitch(this, this._handleCompileSummaryCallback);
			var args = {  resultItemId: this.resultId,
								callback: callback,
					      		errback: callback };
			BuildClient.getCompilationSummary(args);
		}
		if (this._hasJUnitContributions) {
			callback = dojo.hitch(this, this._handleJUnitSummaryCallback);
			args = {  resultItemId: this.resultId,
						callback: callback,
			      		errback: callback };
			BuildClient.getJUnitSummary(args);	
		}	
	},
	
	_isScheduledBuild: function() {
		return this.result && this.result.scheduledBuild;
	},
	
	_buildContributionLink: function(count, sLabel, pLabel, tabId) {
		if (!count)
			return;
		var div = document.createElement("div");
		div.className = "link";
		var a = document.createElement("a");
		a.href = BuildUtil.getResultHref(this.resultId, tabId);
		this.connect(a,"onclick",dojo.hitch(this,function(e) {
			this.tabContainer.selectTab(tabId);
		}));
		if (count == 1) {
			a.innerHTML = sLabel;
		} else {
			a.innerHTML = dojo.string.substitute(pLabel,[count]);
		}
		div.appendChild(a);
		this.contributions.appendChild(div);
	},
	
	sortContributions: function() {
		var contributions = this.result.contributions;
		var downloads = [];
		var links = [];
		var logs = [];
		var reported = [];
		var included = [];
		for (var i = 0; i < contributions.length; i++) {
			switch (contributions[i].contribution.extendedContributionTypeId) {
				case "com.ibm.team.build.common.model.IBuildResultContribution.link":
					contributions[i].linkClass = "externalLink";
					links.push(contributions[i]);
					break;
				case "com.ibm.team.build.common.model.IBuildResultContribution.log":
					contributions[i].linkClass = contributions[i].contribution.contributionStatus == "ERROR" ? "errorLog" : "log";
					logs.push(contributions[i]);
					break;
				case "com.ibm.team.build.common.model.IBuildResultContribution.artifact":
					contributions[i].linkClass = "artifact";
					downloads.push(contributions[i]);
					break;
				case "com.ibm.team.build.linktype.reportedWorkItems":
					contributions[i].linkClass = "workitem";
					reported.push(contributions[i]);
					break;
				case "com.ibm.team.build.linktype.includedWorkItems":
					contributions[i].linkClass = "workitem";
					included.push(contributions[i]);
					break;
				case "com.ibm.team.build.common.model.IJUnitTestSuiteContribution":
					this._hasJUnitContributions = true;
					break;
				case "com.ibm.team.build.common.model.ICompileContribution":
					this._hasCompileContributions = true;
					break;
			}
		}
		var compare = dojo.hitch(this, this._compareContributions);
		downloads.sort(compare);
		links.sort(compare);
		logs.sort(compare);
		reported.sort(compare);
		included.sort(compare);
		this.result.contributions = {
			downloads: downloads,
			links: links,
			logs: logs,
			reported: reported,
			included: included
		};
	},
	
	_populateEmptySection: function(parent, message) {
		parent.innerHTML = "";
		var div = document.createElement("div");
		div.className = "none-text message";
		div.innerHTML = message;
		parent.appendChild(div);
	},
	
	_populateCompilationsSection: function(parentTab, compilations){
		this.compileSummarySection.innerHTML = "";
		this.compileDetailsContent.innerHTML = "";
		var table = new Table(COMPILATION_TABLE_SPECS);
		var container = this.compileSummarySection;
		var bodyContainer = table.tbody;
		var _columns = [];
		var rowElements = new Array(compilations.length);
		for( var i = 0; i < compilations.length; i++){
			var rowArgs = {parentPage: this,
					container: bodyContainer, 
					parentRowElement: null,
					_resultItemId: this.resultId, 
					_rowData: compilations[i]			
			};
			rowElements[i] = new CompilationRowElementViewer(rowArgs);
			var rowType = compilations[i][Constants.REPOSITORY_TYPE];
			var columnTypes = TypeUtil.getColumnTypes(rowType);			
			for (var j = 0; j < columnTypes.length; j++) {
				var cellArgs = {rowContainer: rowElements[i],
								cellType: columnTypes[j] };
				_columns[j] = new CompilationCellElementViewer(cellArgs);
				if (j === 0 && !rowElements[i].hasDetails())
					rowElements[i]._twistie = _columns[j].addTwistie();
				rowElements[i].domNode.appendChild(_columns[j].domNode);
			}
			bodyContainer.appendChild(rowElements[i].domNode);
		}
		container.appendChild(table.domNode);
	},
	
	getCompileDetailsContainer: function(){
		return this.compileDetailsContent;
	},
	_populateJUnitSection: function(parentTab, junits){
		this.junitSummarySection.innerHTML = "";
		this.junitDetailsContent.innerHTML = "";
		var table = new Table(JUNIT_TABLE_SPECS);
		var container = this.junitSummarySection;
		var bodyContainer = table.tbody;
		var _columns = [];
		var rowElements = new Array(junits.length);
		for( var i = 0; i < junits.length; i++){
			var rowArgs = { parentPage: this,
					container: bodyContainer, 
					parentRowElement: null,
					_resultItemId: this.resultId, 
					_rowData: junits[i] 			
			};
			rowElements[i] = new JUnitRowElementViewer(rowArgs);
			var rowType = junits[i][Constants.REPOSITORY_TYPE];
			var columnTypes = TypeUtil.getColumnTypes(rowType);
			for (var j = 0; j < columnTypes.length; j++) {
				var cellArgs = {rowContainer: rowElements[i],
								cellType: columnTypes[j] };
				_columns[j] = new JUnitCellElementViewer(cellArgs);
				if (j === 0 && !rowElements[i].hasDetails())
					rowElements[i]._twistie = _columns[j].addTwistie();
				rowElements[i].domNode.appendChild(_columns[j].domNode);
			}
			bodyContainer.appendChild(rowElements[i].domNode);
		}
		container.appendChild(table.domNode);
	},
	
	getJUnitDetailsContainer: function(){
		return this.junitDetailsContent;
	},
	
	_populateSection: function(parent, contributions, groupByComponent, tabular, useResourceLinks) {
		parent.innerHTML = "";
		var lastComponent;
		var container = groupByComponent ? null : parent;
		var children = [];
		for (var i = 0; i < contributions.length; i++) {
			var c = contributions[i];
			if (!c || !c.contribution)
				continue;
			var co = c.contribution;
			if (groupByComponent && (!container || (lastComponent != co.componentName))) {
				container = document.createElement("div");
				container.className = "section";
				var divClass = "section-heading" + (co.componentName ? "" : " none-text");
				container.innerHTML = "<div class='" + divClass + "'>"+(co.componentName ? jazz.util.html.escape(co.componentName) : messages.noComponent)+"</div>";
				children.push(container);
				if (tabular) {
					var table = new Table(TABLE_SPECS);
					container.appendChild(table.domNode);
					container = table.tbody;
				} else {
					dojo.addClass(container, "link-list");
				}
				lastComponent = co.componentName;
			}
			var args = {linkClass: c.linkClass};
			if (c.contentUri)
				args.url = net.jazz.ajax._contextRoot + "/" + c.contentUri;
			var properties = this._mapProperties(co.extendedContributionProperties);
			if (!args.url && properties["com.ibm.team.build.common.model.IBuildResultContribution.url"]) {
				args.url = properties["com.ibm.team.build.common.model.IBuildResultContribution.url"];
			}
			if (properties.imageUrl)
				args.imageUrl = properties.imageUrl;
			if (tabular) {
				args.label = jazz.util.html.escape(properties["com.ibm.team.build.common.model.IBuildResultContribution.fileName"] || this._getFileNameFromUrl(args.url));
				var tr = document.createElement("tr");
				var td = document.createElement("td");
				dojo.attr(td,"width","45%");
				td.className = "fileName";
				
				var link = document.createElement("div");
				link.className = "link";
				this._createDownloadLink(args, link, useResourceLinks);
				td.appendChild(link);			
				tr.appendChild(td);		
				
				var descTd = document.createElement("td");
				descTd.className="description";
				if(co.label){
					descTd.innerHTML=jazz.util.html.escape(co.label);
				}	
				tr.appendChild(descTd);
	
				var contentLength = null;
				if(co.extendedContributionData)
					contentLength = co.extendedContributionData.contentLength;
				if (!contentLength)
					contentLength = properties["com.ibm.team.build.common.model.IBuildResultContribution.fileSize"];
				var sizeTd = document.createElement("td");
				sizeTd.className = "fileSize";
				sizeTd.innerHTML = this._formatLength(contentLength);
				tr.appendChild(sizeTd);
				container.appendChild(tr);
			} else {
				args.label = jazz.util.html.escape(co.label);
				var link = document.createElement("div");
				link.className = "link";
				this._createDownloadLink(args, link, useResourceLinks);
				container.appendChild(link);
			}
		}
		for (var i = 0; i < children.length; i++)
			parent.appendChild(children[i]);
	},
	
	_getFileNameFromUrl: function(url){
		var start = url.lastIndexOf("/");
		var namePart = url.substring(start+1);
		return namePart;
	},
	
	_createDownloadLink: function(parts, parent, useResourceLinks){
		var html = ["<span class='icon ", parts.linkClass, "' "];
		if (parts.imageUrl) {
			html.push("style='background-image: url(");
			html.push(parts.imageUrl);
			html.push(");' ");
		}
		html.push("><a href='");
		html.push(parts.url);
		html.push("'>");
		var labelStr = parts.label;
		var wbrInsertedLabel = ResultUtil.insertWordBreaks(labelStr, ["."], true);
		html.push(wbrInsertedLabel);
		html.push("</a></span>");
		parent.innerHTML = html.join("");
		if (useResourceLinks)
			new jazz.app.ResourceLink({retainLinkText: true}, parent.firstChild.firstChild);
	},

	_formatLength: function(length) {
		if (!length)
			return messages.na;
		length = parseInt(length);
        if (length == -1) {
            return messages.na; 
        }
        if (length < 1024) {
            return dojo.string.substitute(messages.bytes, [length]);
        }
        if (length < (1024*1024)) {
            return dojo.string.substitute(messages.kbytes, [Math.floor(length / 1024.0)]);
        }
        return dojo.string.substitute(messages.mbytes, [Math.floor(length / (1024.0 * 1024.0))]);
    },

    _mapProperties: function(set) {
    	if (!set) return set;
    	var map = {};
		for (var i = 0; i < set.length; i++) {
			map[set[i].name] = set[i].value;
		}
		return map;
    },
	
	_compareContributions: function(c1, c2) {
		var result = this._compareStrings(c1.contribution.componentName, c2.contribution.componentName);
		if (result)
			return result;
		return this._compareStrings(c1.contribution.label, c2.contribution.label);
	},
	
	_compareStrings: function(s1, s2) {
		if (s1 && !s2)
			return -1;
		if (!s1)
			return s2 ? 1 : 0;
		return s1.localeCompare(s2);
	},
	
	_getStatusClass: function(result) {
        if (result.buildResult.buildState == "INCOMPLETE" || result.buildResult.buildState == "CANCELED") {
            return "incomplete-status"; //$NON-NLS-1$
        }
        switch (result.buildResult.buildStatus) {
            case "ERROR":
                return "error-status"; //$NON-NLS-1$
            case "WARNING":
                return "warning-status"; //$NON-NLS-1$
            default:
                return "ok-status"; //$NON-NLS-1$
        }
	},
	
	_getStateText: function(input) {
		switch(input) {
			case "IN_PROGRESS":
				return messages.inProgess;
			case "COMPLETED":
				return messages.completed;
			case "INCOMPLETE":
				return messages.abandoned;
			case "CANCELED":
				return messages.canceled;
			case "NOT_STARTED":
				return messages.pending;
		}
		return messages.unknown;
	},
	
	// unregister handler in destroy.
	destroy: function(){
		if(this.navigateAwayHandle){
			PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
			this.navigateAwayHandle = null;
		}
		
		if(this.onBeforeUnloadHandle){
			PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
			this.onBeforeUnloadHandle = null;
		}	
		delete pickerDialogInfo;
		this.inherited(arguments);
	}

});

})();
