/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.viewlets.web.ui.internal.PlansViewlet");

dojo.require("com.ibm.team.apt.viewlets.web.client.internal.PlansClient");
dojo.require("com.ibm.team.apt.web.client.util");
dojo.require("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar");

dojo.require("com.ibm.team.dashboard.web.ui.PagedTableViewlet");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.data.dom");
dojo.require("dojo.date.stamp");

dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.apt.viewlets.web.ui.internal", "PlansMessages");

(function() {

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var PagedTableViewlet = com.ibm.team.dashboard.web.ui.PagedTableViewlet;
var PlansClient = com.ibm.team.apt.viewlets.web.client.internal.PlansClient;
var PlanUtil = com.ibm.team.apt.web.client.util;
var ProgressBar = com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var replaceChildren = dojox.data.dom.replaceChildren;

var messages = dojo.i18n.getLocalization("com.ibm.team.apt.viewlets.web.ui.internal", "PlansMessages");
	
dojo.declare("com.ibm.team.apt.viewlets.web.ui.internal.PlansViewlet",
	PagedTableViewlet, {
	
	contentClass: "com-ibm-team-apt-viewlets-web-ui-internal-PlansViewlet",
	
	init: function() {
		this.inherited(arguments);
		// cannot use the default line if there is no scope item (e.g. no project area selected)
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA && this.getScopeItem()) {
			var defaultLineId = this.getPreference("defaultLine");
			if (defaultLineId != -1) {
				this._defaultLine = -1;
				this._initDefaultLine(defaultLineId);
			}
		}
		this._updateCurrentIteration();
		this._updateTitle();
		this._linePrefOptions = new Object();
	},
	
	_initDefaultLine: function(id) {
		var serviceObject = {
			self: this,
			success: function(dto) {
				if (dto && dto.itemId) 
					this.self._defaultLine = dto.itemId;
				else {
					// set defaultLine to -2 as a flag to indicate that it's invalid
					this.self._defaultLine = -2;
					if (this.self._currentIterationTimout) {
						clearTimeout(this.self._currentIterationTimout);
						delete this.self._currentIterationTimout;
					}
					if (this.self._fetchTimout) {
						clearTimeout(this.self._fetchTimout);
						delete this.self._fetchTimout;
					}
					this.self.getSite().refresh();
					this.self.getSite().setLoading(false);
				}
			},
			failure: function(error) {
				this.self.getSite().error(error);
			}
		}
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		// cannot get here if there's no scope item, no need to null guard
		PlansClient.getDevelopmentLineById(id, this.getScopeItem().itemId, srh, this.getServiceTracker());
	},
	
	refresh: function(){
		delete this._totalResults;
		// cache the project for team scoped viewlets so we can calculate the title url immediately on hover
		if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA && this.getScopeItem()) {
			PlansClient.getProjectAreaByTeam(this.getScopeItem().itemId);
		}
		this.inherited(arguments);
	},
	
	fetch: function (/* ServiceResponseHandler */ srh, /* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details) {
		// handle Auto(none) scope
		if (!this.getScopeItem()) {
			this._totalResults = 0;
			this._totalResultsChanged();
			this.getSite().setSecondaryTitle("");
			srh.doCallback();
			return;
		}
		var serviceObject = {
			self: this,
			success: function(dto) {
				if (dto && dto.totalPlans > 0) {
					if (this.self._totalResults != dto.totalPlans) {
						this.self._totalResults = dto.totalPlans;
						this.self._totalResultsChanged();
					}
					if (details) {
						var count = dto.results.length;
						for (var i = 0; i < dto.results.length; i++) {
							var serviceObj = {
								result: dto.results[i],
								success: function(iteration) {
									this.result.iteration = iteration;
									count--;
									if (!count)
										srh.doCallback({entries: dto.results, details: dto.results});
								},
								failure: function(error) {
									srh.doErrorCallback(error);
								}
							}
							var srh2 = new ServiceResponseHandler(serviceObj, "success", "failure");
							PlansClient.getIteration(dto.results[i].iteration, srh2, this.self.getServiceTracker());
						}
					} else {
						srh.doCallback({entries: dto.results});
					}
				} else {
					this.self._totalResults = 0;
					this.self._totalResultsChanged();
					srh.doCallback();
				}
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		};
		var params = {
			currentOnly: this.getPreference("active"),
			maxResults: count,
			startIndex: startIndex,
			includeTeamLoad: true
		};
		if (this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA) {
			params.projectAreaId = this.getScopeItem().itemId; // cannot get here if there's no scope item, no need to null guard
			params.relationshipType = PlanUtil.RELATED_DEVLINE;
			params.relatedItemId = this.getPreference("line");
			if (params.relatedItemId == -1)
				params.relatedItemId = this._defaultLine;
			if (params.relatedItemId == -2) {
				// if defaultLine is -2, the defaultLine preference was invalid
				return;
			}
			else if (params.relatedItemId == -1) {
				// if defaultLine is -1, then one has been specified, but is not back from the server yet
				// postpone the response
				this._fetchTimout = setTimeout(dojo.hitch(this, this.fetch, srh, startIndex, count, entries, details), 500);
				return;
			}
		} else if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA) {
			params.relationshipType = this.getScopeSubteams() ? PlanUtil.RELATED_TEAMHIERARCHY : PlanUtil.RELATED_TEAM;
			params.relatedItemId = this.getScopeItem().itemId; // cannot get here if there's no scope item, no need to null guard
		} else {
			params.relationshipType = PlanUtil.RELATED_CONTRIBUTOR;
			params.relatedItemId = this.getScopeItem().itemId; // cannot get here if there's no scope item, no need to null guard
		}
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		PlansClient.searchIterationPlans(params, srh2, this.getServiceTracker());
	},
	
	getResultCount: function() {
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA && (!this._defaultLine || this._defaultLine == -2))
			return 0;
		if (this.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && !this.getScopeItem())
			return 0;
		return (this._totalResults || this._totalResults == 0) ? this._totalResults : -1;
	},
	
	getNoResultsMessage: function() {
		if (!this.getScopeItem()) {
			return this._createNoScopeMsg();
		}
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA && !this._defaultLine) {
			if (!this._notConfiguredMessage) {
				this._notConfiguredMessage = document.createElement("span");
				var links = I18nUtil.addMessage(this._notConfiguredMessage, messages.notConfigured);
				this.connect(links[0], "onclick", this._toggleSetting);
			}
			return this._notConfiguredMessage;
		}
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA && this._defaultLine == -2) {
			if (!this._invalidDefaultMessage) {
				this._invalidDefaultMessage = document.createElement("span");
				var links = I18nUtil.addMessage(this._invalidDefaultMessage, messages.invalidDefault);
				this.connect(links[0], "onclick", this._toggleSetting);
			}
			return this._invalidDefaultMessage;
		}
		if (this._invalidTeam && this.getScope() == DashboardConstants.SCOPE_TEAM_AREA) {
			if (!this._invalidTeamMessage) {
				this._invalidTeamMessage = document.createElement("span");
				var links = I18nUtil.addMessage(this._invalidTeamMessage, messages.invalidTeam);
				this.connect(links[0], "onclick", this._toggleSetting);
			}
			return this._invalidTeamMessage;
		}
		return document.createTextNode(messages.noResults);
	},
	
	_toggleSetting: function (event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},
	
	getItemsPerPage: function() {
		return this.getPreference("items");
	},
	
	_totalResultsChanged: function() {
		this.updateTotalPageFooter();
		this.getSite().setSecondaryTitle(this._invalidTeam ? "" : "(" + this._totalResults + ")");
	},
	
	/* Node[] */ renderEntry: function(/* object */ plan, /* object */ planDetails, /* boolean */ zoomed) {
		var columns = new Array();
		
		// use resource urls to avoid required knownledge of project area
		var href = this._getServerUriRoot() + "/resource/itemOid/com.ibm.team.apt.IterationPlanRecord/" + plan.itemId;
		
		var nameCell = document.createElement("td");
		nameCell.className = "nameCell";
		var nameLink = document.createElement("a");
		nameLink.href = href;
		nameLink.innerHTML = jazz.util.html.escape(plan.label);
		nameLink.style.paddingRight = "0.5em";
		nameLink.style.zoom = "1";
		nameCell.appendChild(nameLink);
		columns.push(nameCell);
		
		var progressCell = document.createElement("td");
		progressCell.className = "progressCell";
		
		var progressWrapper = document.createElement("div");
		progressWrapper.className = "progressWrapper";
		
		if (zoomed) {
			var startDate = document.createElement("div");
			startDate.className = "startDate";
			var instant = {
				getDate: function() { return new Date(dojo.date.stamp.fromISOString(plan.iteration.startDate))}
			};
			var text = plan.iteration.startDate == null ? messages.noStartDate : PlanUtil.formatDate(instant);
			startDate.appendChild(document.createTextNode(text));
			progressWrapper.appendChild(startDate);
			
			var endDate = document.createElement("div");
			endDate.className = "endDate";
			instant = {
				getDate: function() { return new Date(dojo.date.stamp.fromISOString(plan.iteration.endDate))}
			};
			text = plan.iteration.endDate == null ? messages.noEndDate : PlanUtil.formatDate(instant);
			endDate.appendChild(document.createTextNode(text));
			progressWrapper.appendChild(endDate);
		}
        
        var element;
        var labelProvider;
        var loadBar;
        
        PlanUtil.runWithCondition(dojo.hitch(this, function() {
			var options= {};
			options.parameters= {};
            if(plan.teamLoad.complexityAttribute) {
                options.parameters["complexityLabel"]= plan.teamLoad.complexityAttribute.label;
                options.parameters["complexityUnit"]= plan.teamLoad.complexityAttribute.unit;
            }
			
			element= new com.ibm.team.apt.client.DTOBasedProgressInformation(-1, -1, plan.teamLoad);
			var labelProviderCtor= dojo.getObject(plan.teamLoad.labelProviderImplementation);
			labelProvider= new labelProviderCtor(options);
			var constructorArgs= ProgressBar.constructorArgs(element, labelProvider);
			loadBar= new ProgressBar(constructorArgs);
        
			loadBar.domNode.style.width = "";
			progressWrapper.appendChild(loadBar.domNode);
			
			if (zoomed) {
	            var progressNumbers= document.createElement("div");
				progressNumbers.className = "progressNumbers";
	
				var hours = document.createElement("div");
				hours.className = "hours";
				hours.innerHTML = labelProvider.getProgressText(element);
				progressNumbers.appendChild(hours);
				
				var quality = document.createElement("div");
				quality.className = "quality";
				quality.innerHTML = labelProvider.getQualityText(element);
				progressNumbers.appendChild(quality);
				
				progressWrapper.appendChild(progressNumbers);
			}
		}), PlansClient.getScriptsLoadedCondition());
		
		progressCell.appendChild(progressWrapper);
		columns.push(progressCell);
		
		return columns;
	},
	
	renderDetails: function () {
		var cell = document.createElement("td");
		cell.colSpan = 2;
		cell.className = "divider";
		cell.innerHTML = "<div />";
		return [cell];
	},
	
	_updateCurrentIteration: function() {
		this._invalidTeam = false;
		if (!this.getScopeItem() || this.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR) {
			if (this.heading)
				this.heading.style.display = "none";
			return;
		}
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA && (!this._defaultLine|| this._defaultLine == -2))
			return;
		var line = this.getPreference("line");
		if (line == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA) {
			line = this._defaultLine;
			if (line == -1) {
				this._currentIterationTimout = setTimeout(dojo.hitch(this, this._updateCurrentIteration), 500);
				return;
			}
		}
		if (!this._currentIteration) {
			this.heading = document.createElement("div");
			this.heading.className = "globalHeading";
			this.heading.appendChild(document.createTextNode(messages.currentIteration));
			this._currentIteration = document.createElement("strong");
			this.heading.appendChild(this._currentIteration);
			this.setGlobalHeading(this.heading);
		} else {
			this.heading.style.display = "";
		}
		this._currentIteration.innerHTML = messages.loading;
		var serviceObject = {
			self: this,
			success: function(iteration) {
				this.self._currentIteration.innerHTML = jazz.util.html.escape(iteration.name);
			},
			failure: function(error) {
				this.self._invalidTeam = true;
				this.self.heading.style.display = "none";
				this.self.update();
			}
		}
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		// cannot get here if there's no scope item, no need to null guard
		PlansClient.getCurrentIteration(line == -1 ? "" : line, line == -1 ? this.getScopeItem().itemId : "", srh, this.getServiceTracker());
	},
	
	getTitleHyperlinkURL: function() {
		if (this.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR || !this.getScopeItem()) {
			return null;
		}
		var uriRoot = this._getProjectUriRoot();
		if (!uriRoot)
			return null;
		if (this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA) {
			var relationshipType = PlanUtil.RELATED_DEVLINE;
			var relatedItemId = this.getPreference("line");
			if (relatedItemId == -1)
				relatedItemId = this._defaultLine;
			if (!relatedItemId || relatedItemId == -2 || relatedItemId == -1)
				return uriRoot + "#action=jazz.viewPage&id=com.ibm.team.apt";
		} else if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA) {
			var relationshipType = this.getScopeSubteams() ? PlanUtil.RELATED_TEAMHIERARCHY : PlanUtil.RELATED_TEAM;
			var relatedItemId = this.getScopeItem().itemId; // cannot get here if there's no scope item, no need to null guard
		}
		return uriRoot + PlanUtil.getSearchURI(PlanUtil.createCustomQueryDescriptor(relationshipType, relatedItemId, this.getPreference("active") ? "true" : null));
	},
	
	_getProjectUriRoot: function() {
		var projectName;
		if (this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA) {
			projectName = this.getScopeItem().name;
		} else if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA) {
			project = PlansClient.getProjectAreaByTeam(this.getScopeItem().itemId);
			if (project)
				projectName = project.name;
		}
		if (!projectName)
			return null;
		return this._getServerUriRoot() + "/web/projects/" + encodeURIComponent(projectName) + (djConfig.isDebug ? "?debug=true" : "");
	},
	
	_getServerUriRoot: function() {
		return net.jazz.ajax._dataServerRoot || net.jazz.ajax._contextRoot;
	},
	
	settingChanging: function(/* string */ settingId) {
		var result = new Array();
		if (settingId == "scope" || settingId == "scopeItem")
			result.push("line");
		return result;
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var updateTitle = newSettings.userTitle != oldSettings.userTitle;
		if (oldSettings.preferences.line != newSettings.preferences.line ||
				newSettings.scope == DashboardConstants.SCOPE_CONTRIBUTOR)
			this._updateCurrentIteration();
		if (oldSettings.scopeItemId != newSettings.scopeItemId ||
				oldSettings.scopeSubteams != newSettings.scopeSubteams ||
				oldSettings.preferences.active != newSettings.preferences.active ||
				oldSettings.preferences.line != newSettings.preferences.line) {
			this.reset(true);
			updateTitle = true;
		}
		else if (oldSettings.preferences.items != newSettings.preferences.items) {
			this.reset(false);
		}
		if (updateTitle)
			this._updateTitle();
	},
	
	getPreferenceOptions: function(/* string */ preferenceId, /* object */ currentSettings) {
		if (preferenceId == "line") {
			if (!currentSettings.scopeItemId || currentSettings.scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
				return {options: [{label: messages.notApplicable, value: -1}]};
			}
			if (this._linePrefOptions[currentSettings.scopeItemId] && this._linePrefOptions[currentSettings.scopeItemId] != "fetching") {
				var result = dojo.clone(this._linePrefOptions[currentSettings.scopeItemId]);
				if (this.getPreference("line") == -1 && currentSettings.scope == DashboardConstants.SCOPE_PROJECT_AREA && this._defaultLine && this._defaultLine != -2)
					result.computedDefault = this._defaultLine;
				return result;
			}
			if (!this._linePrefOptions[currentSettings.scopeItemId]) {
				this._linePrefOptions[currentSettings.scopeItemId] = "fetching";
				var serviceObject = {
					self: this,
					success: function(dtos) {
						var options = new Array();
						for (var i = 0; i < dtos.length; i++) {
							options.push({label: dtos[i].name, value: dtos[i].itemId});
						}
						this.self._linePrefOptions[currentSettings.scopeItemId] = {options: options};
					},
					failure: function(error) {
						this.self._linePrefOptions[currentSettings.scopeItemId] = {options: [{label: "None", value: -2}]};
					}
				}
				var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
				PlansClient.getDevelopmentLines(currentSettings.scopeItemId, currentSettings.scope, srh, this.getServiceTracker());
			}
			return null;
		}
	},
	
	hasZoomableContent: function() {
		return this.getResultCount() > 0;
	},
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var links = I18nUtil.addMessage(editSpan, messages.notScopedEdit);
			msgSpan.appendChild(editSpan);
			this.connect(links[0], "onclick", this._toggleSetting);
		}
		return msgSpan;
	},

	_updateTitle: function() {
		var scopeItem = this.getScopeItem();
		if (!scopeItem || !scopeItem.name)
			var title = messages.titleDefault;
		else if (this.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR) {
			var title = dojo.string.substitute(this.getPreference("active") ? messages.titleUserCurrent : messages.titleUser, [scopeItem.name]);
		} else {
			var title = dojo.string.substitute(this.getPreference("active") ? messages.titleOtherCurrent : messages.titleOther, [scopeItem.name]);
		}
		this.getSite().setTitle(title);
	}
});

})();
