/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportParamSingleSelect");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.reports.web.ui.internal.ListBox");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParam");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {
	
	var ListBox = com.ibm.team.reports.web.ui.internal.ListBox;
	
    dojo.declare("com.ibm.team.reports.web.ui.internal.ReportParamSingleSelect", [com.ibm.team.reports.web.ui.internal.ReportParam], {
        templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportParamSingleSelect.html"),
		
		constructor: function() {
			this._addedValues = 0;
			this._selectedValue;
			this._values = [];
			this._duplicates = [];
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			
			this._localizedLabels = [];
			this._localizedLabels[this.statics.MY_TEAM_AREAS] = this.messages.myTeamAreas;
			this._localizedLabels[this.statics.CURRENT_ITERATION] = this.messages.currentIteration;
			this._localizedLabels[this.statics.CURRENT_CONTRIBUTOR] = this.messages.currentContributor;
		},
		
		postCreate: function() {
			this.inherited(arguments);
			var sortBy = this.parameterDTO.sort;
			var sortDirection = null;
			var sortField = "label";
			if (sortBy == this.statics.SORT_BY_LABEL_ASC || sortBy == this.statics.SORT_BY_VALUE_ASC) {
				sortDirection = "asc";
			}
			if (sortBy == this.statics.SORT_BY_LABEL_DESC || sortBy == this.statics.SORT_BY_VALUE_DESC) {
				sortDirection = "desc";
			}
			if (sortBy == this.statics.SORT_BY_VALUE_ASC || sortBy == this.statics.SORT_BY_VALUE_DESC) {
				sortField = "value";
			}
			this._listBox = new ListBox({
				height: "157px",
				width: "250px",
				margin: "0px 5px 0px 5px",
				sortField: sortField,
				sortDirection: sortDirection,
				callbackSingleClick: dojo.hitch(this, this._onSelect),
				allowMultipleSelections: false,
				allowDeSelection: false
			});
			this._listBoxDiv.appendChild(this._listBox.domNode);
			this._listBox.addEntry({
				label: this.messages.loading,
				value: this.messages.loading,
				id: "_loading_"
			});
		},
		
		init: function(reportParameterDTO) {
			this.inherited(arguments);
			this._addNoSelectionOption();
		},
		
		postInit: function() {
			this._listBox.removeEntry({
				id: "_loading_"
			});
		},
		
		cascadeClear: function() {
			this._clear();
			if (this.childReportParam) {
				this.childReportParam.cascadeClear();
			}
		},
		
		appendDefaultValues: function (reportParameterDTO, isLast) {
			if (!reportParameterDTO.labels) {
				return;
			}
			this._updateSelect(reportParameterDTO.values, reportParameterDTO.labels, reportParameterDTO);
			this._defaultValues[this._defaultValues.length] = [reportParameterDTO, reportParameterDTO.values, reportParameterDTO.labels];
		},
		
		loadDefaults: function(reportParameterDTOs) {
			this._clear();
			if (reportParameterDTOs == null) {
				for (var i = 0; i < this._defaultValues.length; i++) {
					var reportParameterDTO = this._defaultValues[i][0];
					this._updateSelect(this._defaultValues[i][1],this._defaultValues[i][2],this._defaultValues[i][0]);
				}
			} else {
				for (var i = 0, len = reportParameterDTOs.length; i < len; i++) {
					var reportParameterDTO = reportParameterDTOs[i];
					this._updateSelect(reportParameterDTO.values,reportParameterDTO.labels,reportParameterDTO);
				}
			}
		},
		
		addValues: function(reportParameterDTOs) {
			this._addedValues++;
			for (var i = 0; i < reportParameterDTOs.length; i++) {
				var reportParameterDTO = reportParameterDTOs[i];
				this._updateSelect( reportParameterDTO.values, reportParameterDTO.labels, reportParameterDTO);
				if (this.childReportParam) {
					this.childReportParam.addValues(reportParameterDTO.children);
				}
			}
		},
			
		removeAllAndAdd: function(reportParameterDTOs) {
			this._clear();
			for (var i = 0; i < reportParameterDTOs.length; i++) {
				this._updateSelect(reportParameterDTOs[i].values,reportParameterDTOs[i].labels,reportParameterDTOs[i]);
			}
		},
		
		setValues: function(values) {
			var id = this._uuid + "_" + values;
			var entry = this._listBox.getEntry(id);
			if (entry) {
				this._listBox.removeAllSelections();
				this._listBox.setSelected({id: id}, true);
				this._onSelect(entry, false);	
			}
		},
			
		setDefaultValues: function() {
			if (this.parameterDTO.defaultValue && this.parameterDTO.defaultValue.length > 0) {
				var id = this._uuid + "_'" + this.parameterDTO.defaultValue + "'";
				var entry = this._listBox.getEntry(id);
				if (entry) {
					this._listBox.setSelected({id: id}, true);
					this._onSelect(entry, false);	
				}
			} else {
				// Select no selection
				var id = "'{No Selection}'";
				var entry = this._listBox.getEntry(id);
				if (entry) {
					this._listBox.setSelected(entry, true);
					this._onSelect(entry, false);
				}
			}
		},
		
		getSelectedValues: function() {
			return [this._selectedValue];
		},
		
		_addNoSelectionOption: function() {
			if (this.parameterDTO.required) {
			} else {
				// Add no selection option
				var entry = {
					id: this._uuid + "_'{No Selection}'",
					value: "'{No Selection}'",
					label: this.messages.noSelection
				};
				this._listBox.addEntryToStart(entry);
			}
		},
		
		_onSelect: function(entry, fireListener) {
			var value = entry.value;
			var label = entry.label;
			var fireDirtyListener = true;
			if (fireListener != undefined && fireListener == false) {
				fireDirtyListener = false;
			}
			
			this.selectedTitle.innerHTML = label;
			this._selectedValue = value;
			
			if (value == "'{No Selection}'") {
				if (this.childReportParam == null) {
					return;
				}
				if (this._addedValues == 0) {
					this.childReportParam.loadDefaults();
				} else {
					var reportParameterDTOs = [];
					var entries = this._listBox.getAllEntries();
					for (var i = 0; i < entries.length; i++) {
						if (entries[i].value == "'{No Selection}'") {
							continue;
						}
						var optId = entries[i].id;
						var reportParameterDTO = this._reportParameterDTOs[optId];
						for (var j = 0; j < reportParameterDTO.children.length; j++) {
							reportParameterDTOs.push(reportParameterDTO.children[j]);
						}
						if (this._duplicates[optId]) {
							for (var k in this._duplicates[optId]) {
								var dupReportParameterDTO = this._duplicates[optId][k]
								for (var l = 0; l < dupReportParameterDTO.children.length; l++) {
									reportParameterDTOs.push(dupReportParameterDTO.children[l]);
								}
							}
						}
					}
					this.childReportParam.loadDefaults(reportParameterDTOs);
				}
			} else if (this.childReportParam != null) {
				var selectedReportParameterDTO = this._reportParameterDTOs[entry.id];
				var values = selectedReportParameterDTO.values;
				if (values) {
					for (var i = 0, len = values.length; i < len; i++) {
						if (value == "'"+values[i]+"'") {
							var dtos = [];
							var duplicates = this._duplicates[this._uuid + "_" + value];
							if (selectedReportParameterDTO.children) {
								dtos.push(selectedReportParameterDTO.children[i]);
							}
							for (var j in duplicates) {
								var dto = duplicates[j];
								if (!dto.children) {
									continue;
								}
								for (var k = 0; k < dto.values.length; ++k) {
									if (value == "'" + dto.values[k] + "'") {
										dtos.push(dto.children[k]);
									}
								}
							}
							this.childReportParam.removeAllAndAdd(dtos);
							break;
						}
					}
				}
			}
			if (fireDirtyListener) {
				this.onDirty();
			}			
		},
		
		_updateSelect: function(values, labels, reportParameterDTO) {
			var entries = [];
			for (var i = 0; i < labels.length; i++) {
				var id = this._uuid + "_'" + values[i] + "'";
				this._values.push(values[i]);
				if (this._listBox.getEntry(id) == null) {
					var label = labels[i];
					var value = values[i];
					if (value == this.statics.MY_TEAM_AREAS ||
					    value == this.statics.CURRENT_ITERATION ||
					    value == this.statics.CURRENT_CONTRIBUTOR) {
						label = this._localizedLabels[value];
					}
					entries.push({
						label: label,
						value: "'" + value + "'",
						id: id,
						type: 0
					});
					this._reportParameterDTOs[id] = reportParameterDTO;
				} else {
					if (!this._duplicates[id]) {
						this._duplicates[id] = [];
					}
					this._duplicates[id].push(reportParameterDTO);
				}
			}
			this._listBox.addEntries(entries);
		},
		
		_clear: function() {
			this._listBox.removeAllEntries();
			this._reportParameterDTOs = [];
			this._values.length = 0;
			this._duplicates = [];
			this._addedValues = 0;
			this._addNoSelectionOption();
		},

		remoteError: function(error) {
			alert(error.message);
		}
		
	});

})();
