/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportViewer");

dojo.require("dojo.i18n");
dojo.require("dijit.Dialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.FilteringSelect");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");

dojo.require("com.ibm.team.reports.web.ui.internal.SaveAsDialog");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamFactory");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamGroup");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.StringUtils");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParametersPanel");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.require("jazz.ui.internal.SimpleToolbar");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ReportParamFactory = com.ibm.team.reports.web.ui.internal.ReportParamFactory;
	var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
	var StringUtils = com.ibm.team.reports.web.ui.internal.utils.StringUtils;
	var MAX_MESSAGE_LENGTH = 500;
	
	dojo.declare("com.ibm.team.reports.web.ui.internal.ReportViewer", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportViewer.html"),
		
		widgetsInTemplate: true,
		projectAreaUUID: "", /* Required: If not operating within the web ui */
		queryUUID: "", /* Required: If no reportUUID is set */
		reportUUID: "", /* Required: If no query UUID is set */
		showArchived: false, /* Optional: Set if you wish to display archived results */
		getCached: false, /* Optional: Set if you wish to display cached results */
		controller: null, /* Optional: Set by web ui */
		urlPrefix: net.jazz.ajax._contextRoot + "/", /* Optional */
		
		constructor: function(props) {
			dojo.mixin(this, props);
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			if (!this.parameterName || !this.parameterValue) {
				/*
				 * If we have not been passed in any parameter names or values, we must check if they
				 * are provided in the url.  Since we do not know beforehand the names of all potential
				 * parameters, we map each of the url params to 'this' to be matched with the parameters
				 * retrieved from the server for this query (or report).
				 */
				this.parameterName = [];
				this.parameterValue = [];
				window.location.search.replace(new RegExp("([^?=&]+)(=([^&]*))?", "g"), dojo.hitch(this, function($0, $1, $2, $3) {
					var s = decodeURIComponent($3);
					this[$1] = s;
				}));
			}
			this.reportParams = [];
			this.inEditMode = false;
			this.enableParams = false;
			this.enableExport = false;
			this.titleHidden = false;
			this.reportTitle = "";
			this.rcpCall = false;
			this.isDirty = false;
			this.isObsolete = false;
		},
		
		postCreate: function() {
			// Create top name field
			this._nameField = document.createElement("input");
			this._nameField.type = "text";
			this._nameField.size = 40;
			this._nameField.value = "";
			this._titleSection.appendChild(this._nameField);
			var self = this;
			dojo.connect(this._nameField, "onkeypress", this, function(arg) {
				self.setDirty(true);
			});
			dojo.connect(this._nameField, "onChange", this, function(arg) {
				self.setDirty(true);
			});
			
			this.reportParamsPanel.setDirtyListener(this);
			this._toolbar = new jazz.ui.internal.SimpleToolbar({});
			this._toolbarDiv.appendChild(this._toolbar.domNode);

			this._toolbar.addButton(this, this._showExportDialog, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/export.gif"), label: this.messages.exportTitle});
			this._toolbar.addButton(this, this._handleSetAsDefault, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/default-report.gif"), label: this.messages.setAsDefaultReport});
			
			this._button = document.createElement("div");
			this._button.className = "jazz-ui-SimpleToolbar-button";
			this._button.tabIndex = "0";
			this._button.title = this.messages.parametersTooltip;
			
			var img = document.createElement("img");
			img.src = dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/edit.gif");
			img.alt = this.messages.parametersTooltip;
			this._button.appendChild(img);
			dojo.connect(this._button, "onclick", this, this._handleEdit);			
			this._toolbar.addWidget({domNode: this._button});
			if (location.href.match(/_ajax-modules/)) {
				// Widget embedded in the RCP context
				this._loadRCPParameters();
				this.rcpCall = true;
			}

			// Set the project area uuid if it was not provided in the constructor
			if (!this.rcpCall && this.projectAreaUUID == "") {
				this.projectAreaUUID = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId;
			}
			
			if (this.controller) {
				this.showArchived = this.controller.getIncludeArchived();
			}
			
			// Set loading message
			if (this.controller) {
				this.controller.startOperation(this.messages.loadingReport);
			}
			if (!this.rcpCall) {
				if (!this.navigateAwayHandle) {
					this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
				}
				if (!this.onBeforeUnloadHandle){
					this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.navigateAwayEvent), /*priority*/ 10000);	
				}
			}
			if (this.reportUUID.length > 0) {
				// Disable saving of report templates
				this._saveButton.disabled = true;
				this._initReport();
			}
			else {
				this._initQuery();
			}
			
			if (!this.rcpCall && this.projectAreaUUID != com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId) {
				var responseHandler = new ServiceResponseHandler(this, "_setProjectAreaMismatchMessage", "_remoteError");
				com.ibm.team.process.web.client.internal.ProcessClient.getProjectAreaByUUID({uuid: this.projectAreaUUID}, responseHandler);
			}
		},
		
		_showExportDialog: function() {
			if (!this.enableExport) {
				alert(this.messages.exportOperationNotSupported);
				return;
			}
			this._exportDialog.show();
		},
		
		_loadRCPParameters: function() {
			// Widget parameters are passed via URL path segments after the Ajax 
			// module ID
			// e.g.
			// https://localhost:9443/jazz/_ajax-modules/com.ibm.team.reports.web.reportViewer/_fa8m8B_GEd6-xKd-yus_YQ/_j4YOsCOvEd6HYJh3DaxtXw/undef/false/false

			var segments = location.href.split(/\//);
			var ajaxModuleID = "com.ibm.team.reports.web.reportViewer";
			var index = 0;
			var notSpecified = "undef";
			var expectedParamCount = 5;
			
			// Find first parameter index (after Ajax module ID)
			for (index = 0; index < segments.length; index++) {
				if (segments[index] === ajaxModuleID) {
					index++;
					break;
				}
			}
			
			// Validate parameters
			var actualParamCount = segments.length - index;
			if (actualParamCount < expectedParamCount) {
				// Serious error
				var errorMessage = "Failed to load RCP parameters. Number of expected (" + 
									expectedParamCount + 
									") versus actual (" + 
									actualParamCount + 
									") parameters did not match.";
				this._logError(new Error(errorMessage));
				alert(errorMessage);
			}
			
			// Parse parameters
				
            // 0: projectAreaUUID
			this.projectAreaUUID = decodeURIComponent(segments[index]); 
			index++;

            // 1: queryUUID
			var myQueryUUID = decodeURIComponent(segments[index]);
			if (myQueryUUID !== notSpecified) {
				this.queryUUID = myQueryUUID;
			}
			index++;

            // 2: reportUUID
			var myReportUUID = decodeURIComponent(segments[index]);
			if (myReportUUID !== notSpecified) {
            	this.reportUUID = myReportUUID;
			}
			index++;

           	// 3: showArchived
           	this.showArchived = segments[index];
			index++;

           	// 4: getCached
			this.getCached = segments[index];
		},
		
		_logError: function(errorObject) {
			
			var className = "";
			if (this.declaredClass) {
				className = this.declaredClass + ": ";
			} 
			
			var lineNumber = "";
			if (errorObject.lineNumber) {
				lineNumber = "(line " + errorObject.lineNumber + "): ";
			} 
			
			var errorMessage = "[ERROR]: " +
								className +
								lineNumber +
								errorObject.message;
								
			console.error(errorMessage);
		},		
		
		_initReport: function() {
			// Get the report descriptor that is used to select parameters
			var args = {
				projectAreaUUID: this.projectAreaUUID,
				reportUUID: this.reportUUID
			};
			var srh = new ServiceResponseHandler(this, "_handleReportDescriptor", "_remoteError");
			ReportsClient.getReportDescriptor(srh, args);
		},
		
		_initQuery: function() {
			this._isQuery = true;
			// Get the query descriptor that is used to select parameters
			var args = {
				queryUUID: this.queryUUID
			};
			var srh = new ServiceResponseHandler(this, "_handleQueryDescriptor", "_remoteError");
			ReportsClient.getQueryDescriptor(srh, args);
		},
		
		_handleReportDescriptor: function(reportDescriptor) {
			this._initReportEngineParameters(reportDescriptor.engine);
			if (this.queryUUID.length > 0) {
				var args = {
					queryUUID: this.queryUUID
				};
				var srh = new ServiceResponseHandler(this, "_handleReportQueryDescriptor", "_remoteError");
				ReportsClient.getQueryDescriptor(srh, args);
			}
			else {
				if (!this.titleHidden) {
					this._setReportTitle(reportDescriptor.name);
				}
				var args = {
					projectAreaUUID: this.projectAreaUUID,
					reportUUID: this.reportUUID
				};
				var srh = new ServiceResponseHandler(this, "_handleParameters", "_remoteError");
				ReportsClient.getParameters(srh, args);
			}
		},
		
		_handleReportQueryDescriptor: function(queryDescriptor) {
			if (!this.titleHidden) {
				this._setReportTitle(queryDescriptor.name);
			}
			// Fetch the parameters
			if (queryDescriptor.parameterNames) {
				this.parameterName = queryDescriptor.parameterNames;
				this.parameterValue = queryDescriptor.parameterValues;
			}
			this.queryUUID = "";
			this._nameField.value = queryDescriptor.name;
			var args = {
				projectAreaUUID: this.projectAreaUUID,
				reportUUID: this.reportUUID
			};
			var srh = new ServiceResponseHandler(this, "_handleParameters", "_remoteError");
			ReportsClient.getParameters(srh, args);
		},
		
		_handleQueryDescriptor: function(queryDescriptor) {
			this.reportUUID = queryDescriptor.reportUUID;
			this._initReportEngineParameters(queryDescriptor.engine);
			if (!this.titleHidden) {
				this._setReportTitle(queryDescriptor.name);
			}
			// Fetch the parameters
			if (queryDescriptor.parameterNames) {
				this.parameterName = queryDescriptor.parameterNames;
				this.parameterValue = queryDescriptor.parameterValues;
			}
			this._nameField.value = queryDescriptor.name;
			var args = {
				queryUUID: this.queryUUID
			};
			var srh = new ServiceResponseHandler(this, "_handleParameters", "_remoteError");
			ReportsClient.getParameters(srh, args);
		},
		
		_initReportEngineParameters: function(descriptor) {
			this.titleHidden = descriptor.titleHidden;
			for (var i = 0; i < this._exportSelect.options.length; i++) {
  				this._exportSelect.options[i] = null;
			}
			if (!descriptor.exportValues || descriptor.exportValues.length == 0) {
				// Do nothing
			}
			else {
				this.enableExport = true;
				var index = 0;
				for (var i = 0; i < descriptor.exportValues.length; i++) {
					var value = descriptor.exportValues[i];
					var label = value;
					if (descriptor.exportLabels != null && descriptor.exportLabels.length > i) {
						label = descriptor.exportLabels[i];
					}
					if (value != null) {
						var option = document.createElement("option");
						option.value = value;
						option.text = label;
						this._exportSelect.options[index] = option;
						index++;
					}
				}
			}
		},
		
		destroy: function() {
			if (!this.rcpCall) {
				if (this.navigateAwayHandle) {
					PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				}
				if (this.onBeforeUnloadHandle){
					PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);	
				}
			}
			this._exportDialog.destroyRecursive();
		},
		
		isDirty: function() {
			return this.isDirty;
		},
		
		setDirty: function(dirty) {
			if (dirty == this.isDirty) return;
			this.isDirty = dirty;
			this._setReportTitle(this.reportTitle);
		},
		
		_handleRun: function(event) {
			dojo.stopEvent(event);
			this.renderedReportDiv.innerHTML = this.messages.loading;
			this._updateReport();
		},
		
		_handleRefresh: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (this.inEditMode) {
				this._handleEdit();
			}
			for (i in this.reportParams) {
				var reportParam = this.reportParams[i];
				reportParam.destroy();
			}
			this.reportParams.length = 0;
			this._removeChildren(this.reportParamsPanel.domNode);
			if (this.reportUUID.length > 0) {
				this._initReport();
			}
			else {
				this._initQuery();
			}
		},
		
		_handleCancel: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			for (i in this.reportParams) {
				var reportParam = this.reportParams[i];
				reportParam.setDefaultValues();
			}
		},
		
		_handleEdit: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (!this.enableParams) {
				return;
			}
			if (this.inEditMode) {
				this.inEditMode = false;
				this._button.className = "jazz-ui-SimpleToolbar-button";
				this._editDiv.className = "display-none";
			}
			else {
				this.inEditMode = true;
				this._button.className = "jazz-ui-SimpleToolbar-button-selected";
				this._editDiv.className = "editPanel";
			}
		},
				
		_handleParameters: function(baseParamsArray) {
			if (!baseParamsArray || baseParamsArray.length == 0) {
				baseParamsArray = [];
			}
			else {
				this.enableParams = true;
			}
			
			if (!this._isQuery) {
				dojo.forEach(baseParamsArray, dojo.hitch(this, function(param) {
					if (param._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterGroupDTO") {
						for (var i = 0; i < param.parameters.length; i++) {
							var parameterDTO = param.parameters[i];
							if (this[parameterDTO.name]) {
								this.parameterName[this.parameterName.length] = parameterDTO.name;
								this.parameterValue[this.parameterValue.length] = this[parameterDTO.name];
							}
						}
					}
					else if (param._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterDTO") {
						var parameterDTO = param;
						if (this[parameterDTO.name]) {
							this.parameterName[this.parameterName.length] = parameterDTO.name;
							this.parameterValue[this.parameterValue.length] = this[parameterDTO.name];
						}
					}
				}));
			}
			
			var missingQueryParams = false;
			
			dojo.forEach(baseParamsArray, dojo.hitch(this, function(param) {
				var args = {
					projectAreaUUID: this.projectAreaUUID,
					queryUUID: this.queryUUID,
					reportUUID: this.reportUUID,
					showArchived: this.showArchived
				};
				if (param._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterGroupDTO") {
					var reportParamGroup = new com.ibm.team.reports.web.ui.internal.ReportParamGroup({
						parameterGroupDTO: param,
						urlPrefix: this.urlPrefix,
						parameterName: this.parameterName,
						parameterValue: this.parameterValue,
						dirtyListener: this
					});
					this.reportParamsPanel.appendParamGroup(reportParamGroup);
					reportParamGroup.appendReportParameters(this.reportParams);
					var srh = new ServiceResponseHandler(reportParamGroup, "initGroupValues", "remoteError");
					args.groupName = param.name;
					ReportsClient.getParameterGroupValues(srh, args);
					
					dojo.forEach(param.parameters, dojo.hitch(this, function(singleParam) {
						if (singleParam.required && 
						    (!singleParam.defaultValue || 
						     singleParam.defaultValue.length == 0 || 
						     param.defaultValue == "''")) {
							var found = -1;
							if (this.parameterName.length == 0) {
								missingQueryParams = true;
							} else {
								for (var i = 0; i < this.parameterName.length; i++) {
									if (singleParam.name == this.parameterName[i]) {
										found = i;
										break;
									}
									if (found != -1 && 
									    (found >= this.parameterValue.length || 
									     !this.parameterValue[found] || 
									     this.parameterValue[found].length == 0 ||
									     this.parameterValue[found] == "''")) {
										missingQueryParams = true;
									}
								}
							}
						}
					}));
				} else if (param._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterDTO") {
					var reportParam = ReportParamFactory.create(param, this.urlPrefix);
					if (!param.concealed) {
						this.reportParamsPanel.appendParam(reportParam);
					}
					this.reportParams.push(reportParam);
					var self = this;
					var obj = {
						success: function(reportParameterDTO) {
							reportParam.init(reportParameterDTO);
							if (self.parameterName.length > 0) {
								for (var i = 0; i < self.parameterName.length; i++) {
									if (self.parameterName[i] == reportParam.parameterDTO.name) {
										reportParam.setValues(self.parameterValue[i]);
										break;
									}
								}
							}
							reportParam.postInit();
						},
						error: function(error) {
							alert(error.message);
						}
					}
					var srh = new ServiceResponseHandler(obj, "success", "error");
					args.parameter = param.name;
					ReportsClient.getParameterValues(srh, args);
					
					if (param.required && 
					    (!param.defaultValue || 
					     param.defaultValue.length == 0 || 
					     param.defaultValue == "''")) {
						var found = -1;
						if (this.parameterName.length == 0) {
							missingQueryParams = true;
						}
						else {
							for (var i = 0; i < this.parameterName.length; i++) {
								if (param.name == this.parameterName[i]) {
									found = i;
									break;
								}
							}
							if (found != -1 && 
							    (found >= this.parameterValue.length || 
							     !this.parameterValue[found] || 
							     this.parameterValue[found].length == 0 ||
							     this.parameterValue[found] == "''")) {
								missingQueryParams = true;
							}
						}
					}
				}
			}));
			if (this.controller) {
				this.controller.finishOperation();
			}
			if (missingQueryParams) {
				this._removeChildren(this.renderedReportDiv);
				this._handleEdit();
			} else if (this.parameterName) {
				this._updateReport(this.parameterName, this.parameterValue);
			}
		},
		
		_displayChart: function(chartHTML) {
			if (dojo.isIE) {
				// Insert innerHTML directly. That, coupled with defer tags
				// on the scripts, causes new Javascript to be executed properly.
				this.renderedReportDiv.innerHTML = chartHTML;
			} else {
				// Create a new node and appendChild so javascript gets executed.
				var newNode = document.createElement('div');
				newNode.innerHTML = chartHTML;
				this._removeChildren(this.renderedReportDiv);
				this.renderedReportDiv.appendChild(newNode);
			}
		},
		
		_callRestService: function(methodName, args, scopeObj, successMethodName, failureMethodName) {
			var srh = new com.ibm.team.repository.web.transport.ServiceResponseHandler(scopeObj, successMethodName, failureMethodName);
			var serviceRequest = new com.ibm.team.repository.web.transport.ServiceRequest("com.ibm.team.reports.common.internal.service.IReportRestService", methodName, args);
			com.ibm.team.repository.web.transport.TeamServerClient.invokeService(serviceRequest, srh);
		},
		
		_updateReport: function(parameterNames, parameterValues) {
			var selections = null;
			if (!parameterNames && !parameterValues) {
				selections = this._getParameterSelections();
				if (!selections) {
					return;
				}
			}
			else {
				selections = {};
				selections.parameterNames = parameterNames;
				selections.parameterValues = parameterValues;
			}
			var args = {
				projectAreaUUID: this.projectAreaUUID,
				embeddable: true,
				parameterName: selections.parameterNames,
				parameterValue: selections.parameterValues,
				urlPrefix: this.urlPrefix,
				showArchived: this.showArchived,
				getCached: this.getCached
			};
			var restServiceName;
			if (this.reportUUID.length > 0) {
				restServiceName = "postRenderReport"
				args.reportUUID = this.reportUUID;
				args.queryUUID = this.queryUUID;
			}
			else if (this.queryUUID.length > 0) {
				restServiceName = "postRenderQuery"
				args.queryUUID = this.queryUUID;
			}
			else {
				return;
			}
			var callback = {
				success: dojo.hitch(this, this._displayChart),
				error: dojo.hitch(this, this._remoteError)
			}
			this._callRestService(restServiceName, args, callback, "success", "error");
			
			if (this.inEditMode) {
				// collapse the parameters section			
				this._handleEdit();
			}
		},
		
		_postSave: function() {
			this.setDirty(false);
			this._setReportTitle(this._nameField.value);
			if (this.controller) {
				this.controller.refreshNavigator();
			}
			else {
				// Assume we are in the RCP ui
				window.location = "http://com.ibm.team.reports/new_query/";
			}
		},
		
		_handleSetAsDefault: function() {
			var self = this;
			var obj = {
				success: function(args) {
					alert(self.messages.nowDefaultReport);
				},
				error: function(error) {
					alert(dojo.string.substitute(self.messages.handleSetAsDefaultError, [error.message]));
				}
			};
			this._callRestService("postSetDefaultQueryDescriptor", {
				queryUUID: this.queryUUID
			}, obj, "success", "error");
		},
		
	    _handleNavigate: function(urlObj) {
			this.isObsolete = true;
			return false;
        },
        
        navigateAwayEvent: function() {
        	this.isObsolete = true;
            return false;
        },
		
		_getParameterSelections: function() {
			var result = {
				parameterNames: [],
				parameterValues: []
			};
			var parameterNames = [];
			var parameterValues = [];
			for (var i = 0, len = this.reportParams.length; i < len; i++) {
				var reportParam = this.reportParams[i];
				result.parameterNames[i] = reportParam.parameterDTO.name;
				var vals = reportParam.getSelectedValues();
				var noSelection = (vals == null || vals.length == 0 || vals[0] == undefined);
				if (reportParam.parameterDTO.required && noSelection) {
					var label = reportParam.parameterDTO.name;
					if (reportParam.parameterDTO.displayName && reportParam.parameterDTO.displayName.length > 0) {
						label = reportParam.parameterDTO.displayName;
					}
					this._remoteError({
						message: dojo.string.substitute(this.messages.missingReqParam, [label])
					});
					return null;
				}
				if (noSelection) {
					result.parameterValues[i] = "";
				} else {
					result.parameterValues[i] = vals.join(","); 
				}
			}
			return result;
		},
		
		_handleExport: function() {
			if (!this.enableExport) {
				return;
			}
			var selections = this._getParameterSelections();
			if (selections.parameterNames.length > 0) {
				var args = {
					queryUUID: this.queryUUID,
					reportUUID: this.reportUUID,
					parameterName: selections.parameterNames,
					parameterValue: selections.parameterValues
				};
				var srh = new ServiceResponseHandler(this, "_handleSaveQueryParameters", "_remoteError");
				ReportsClient.postSaveQueryParameters(srh, args);
			}
			else {
				var fmt = this._exportSelect.options[this._exportSelect.selectedIndex].value;
				var url = this.urlPrefix + "service/com.ibm.team.reports.service.internal.IReportViewerService";
				url += "?__format=" + fmt;
				url += "&__reportUUID=" + this.reportUUID;
				url += "&__queryUUID=" + this.queryUUID;
				document.location.href = url;
			}
		},
		
		_handleSaveQueryParameters: function(result) {
			var fmt = this._exportSelect.options[this._exportSelect.selectedIndex].value;
			var url = this.urlPrefix + "service/com.ibm.team.reports.service.internal.IReportViewerService";
			url += "?__format=" + fmt;
			url += "&__queryUUID=" + result;
			document.location.href = url;
		},
		
		_handleSave: function() {
			var selections = this._getParameterSelections();
			if (!selections) {
				return;
			}
			var obj = {
				success: dojo.hitch(this, function(args) {
					this._postSave();
				}),
				error: this._remoteError
			};
			var args = {
				projectAreaUUID: this.projectAreaUUID,
				queryUUID: this.queryUUID,
				reportUUID: this.reportUUID,
				folderUUID: "" /* Not moving the report */,
				name: this._nameField.value,
				parameterName: selections.parameterNames,
				parameterValue: selections.parameterValues
			};
			this._callRestService("postSaveQueryDescriptor", args, obj, "success", "error");
		},
		
		_handleSaveAs: function() {
			var onSubmit = dojo.hitch(this, function(saveName) {
				var selections = this._getParameterSelections();
				if (!selections) {
					return;
				}
				var args = {
					name: saveName,
					projectAreaUUID: this.projectAreaUUID,
					reportUUID: this.reportUUID,
					parameterName: selections.parameterNames,
					parameterValue: selections.parameterValues
				};
				var obj = {
					success: dojo.hitch(this, function(args) {
						this._postSave();
					}),
					error: this._remoteError
				};
				this._callRestService("postSaveQueryDescriptor", args, obj, "success", "error");
			});
			var saveAsDialog = new com.ibm.team.reports.web.ui.internal.SaveAsDialog({
				onSubmit: onSubmit
			});
			saveAsDialog.open();
		},
		
		_setReportTitle: function(title) {
			if (this.isDirty) {
				this.reportTitleSpan.innerHTML = title + "*";
			} else {
				this.reportTitleSpan.innerHTML = title;
			}
			this.reportTitle = title;
		},
		
		_remoteError: function(error) {
			if (this.isObsolete) {
				return;
			}
			this._removeChildren(this.renderedReportDiv);
			alert(error.message);
		},
		
		_removeChildren: function(element) {
			while (element.firstChild) {
				element.removeChild(element.firstChild);
			}
		},
		
		_setProjectAreaMismatchMessage: function(projectArea) {
			this.setHeaderMessage({
				message: dojo.string.substitute(this.messages.msgProjectAreaIsDifferent, [projectArea.name, com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().name]),
				details: this.messages.msgProjectAreaIsDifferentDetails,
				showOnlyMessage: true
			});		
		},
		
		setHeaderMessage: function(message) {
			this.clearValidationMessage();
			if (!message) {
				return;
			}
			
            if (!message.details) {
                message.details = "";
            }
            
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            } else {
            	if (message.showOnlyMessage == true) {
            		messageText = message.message
            	} else {
                	messageText = message.message + " " + message.details;            		
            	}
                if (messageText.length > MAX_MESSAGE_LENGTH) {
                    messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
                }
            }
            messageText = messageText.replace(/\.\.\.\./, "...");
            
            this.validationMessage.__jazzMessage = message;
            
			// We only support Info
			this.validationMessageSpan.className = "validationMessageInfo";

            this.validationMessage.title = message.details || message.message;
            dojox.data.dom.replaceChildren(this.validationMessage, document.createTextNode(messageText));
		},
		
		_handleHeaderMessageClick: function(e) {
			e.preventDefault();
			if (!e || !e.target || !e.target.__jazzMessage) return;

			var message = e.target.__jazzMessage;
			var text = message.message;
			if (message.details) {
				text += "\n\n";
				text += message.details;
			}
			alert(text);
		},
		
		clearValidationMessage: function(force) {
			this.validationMessageSpan.className = "validationMessageEmpty";
		}		
		
	});
	
})();
