/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore", dojo.data.ItemFileWriteStore, { //$NON-NLS-1$
		/*
		 * Constructs the PermissionsScopeStore instance
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 *    editorContext: the editor context tree is being used in... a value in ProcessConstants.EditorContext
		 *    processAreaItemId: the item id of the process area to load the timeline model for
		 *    errorContext: the object to invoke the error call back on
		 *    errorCallback: the function to call if an error occurs loading the model
		 *  }
		 */
		constructor: function(args){
			// A little dangerous here...setting the private value 
			// that is normally taken from the args in the constructor
			// for ItemFileReadStore.  Don't want to force the client
			// to have to pass that in since it's pre-determined for this
			// model
			this._jsonData = {
				identifier: 'id', //$NON-NLS-1$
				label: 'name', //$NON-NLS-1$
				items: []
			};
			
			this._editorContext = args.editorContext;
			this._processAreaItemId = args.processAreaItemId;
			this._errorContext = args.errorContext;
			this._errorCallback = args.errorCallback;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._initialize();
		},
		
		_initialize: function(){
			this._addProjectRootItem();
			this._addTeamRootItem();
			this._addIterationTypesRootItem();
			this._addTimelinesRootItem();
			this._addIterationsRootItem();
			this.save();
			
			var srh = new ServiceResponseHandler(this, "_populateIterationTypesData", "_handleIterationTypesFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getIterationTypes({
				processAreaItemId: this._processAreaItemId,
				processAreaContext: this._editorContext
			}, srh);
			
			if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
				srh = new ServiceResponseHandler(this, "_populateDevelopmentLinesData", "_handleDevelopmentLinesFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getProjectDevelopmentLines({
					uuid: this._processAreaItemId
				}, srh);
			} else {
				srh = new ServiceResponseHandler(this, "_populateIterationsData", "_handleIterationsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getTeamAreaIterations({
					uuid: this._processAreaItemId
				}, srh);
				
			}
			
		},
		
		_populateDevelopmentLinesData: function(data){
			var self = this;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.DEV_LINES,
				onItem: function(item, request){
					dojo.forEach(data, function(developmentLine){
						var newChildItem = self.newItem({
							name: developmentLine.label,
							id: developmentLine.itemId,
							stateId: developmentLine.stateId,
							parent: item.id,
							context: ProcessConstants.PermissionContext.TEAM,
							contextId: developmentLine.id,
							enabledContext: ProcessConstants.PermissionContext.DEV_LINE,
							overrideAllowed: true,
							projectTimeline: developmentLine.projectTimeline,
							startDate: dojo.date.stamp.fromISOString(developmentLine.startDate),
							endDate: dojo.date.stamp.fromISOString(developmentLine.endDate)
						}, {
							parent: item,
							attribute: "items" //$NON-NLS-1$
						});
						
					});
				}
			});
			
			dojo.forEach(data, function(developmentLine){
				var srh = new ServiceResponseHandler(self, "_populateIterationData", "_handleIterationsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getIterations({
					uuid: developmentLine.itemId
				}, srh);
			});
			
		},
		
		_populateIterationsData: function(data){
			var self = this;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.ITERATIONS,
				onItem: function(item, request){
					dojo.forEach(data, function(iteration){
						var newChildItem = self.newItem({
							name: iteration.label,
							id: iteration.itemId,
							stateId: iteration.stateId,
							parent: item.id,
							context: ProcessConstants.PermissionContext.TEAM,
							contextId: iteration.id,
							enabledContext: ProcessConstants.PermissionContext.ITERATION,
							overrideAllowed: true,
							current: iteration.current,
							completed: iteration.completed,
							leaf: iteration.leaf,
							startDate: dojo.date.stamp.fromISOString(iteration.startDate),
							endDate: dojo.date.stamp.fromISOString(iteration.endDate),
							iterationTypeItemId: iteration.iterationTypeItemId ? iteration.iterationTypeItemId : ProcessConstants.ITERATION_TYPE_NONE,
							hasDeliverable: iteration.hasDeliverable
						}, {
							parent: item,
							attribute: "items" //$NON-NLS-1$
						});
						
					});
				}
			});
			
			dojo.forEach(data, function(iteration){
				if (iteration.iterations) {
					this._populateIterationData(iteration.iterations);
				}
			}, this);
			
			this.save();
			
		},
		
		_populateIterationData: function(iterations){
			if (iterations) {
				var self = this;
				for (var i = 0; i < iterations.length; i++) {
					var iteration = iterations[i];
					this.fetchItemByIdentity({
						identity: iteration.parentItemId,
						onItem: function(item, request){
							var newChildItem = self.newItem({
								name: iteration.label,
								id: iteration.itemId,
								stateId: iteration.stateId,
								parent: iteration.parentItemId,
								context: ProcessConstants.PermissionContext.TEAM,
								contextId: item.contextId + ProcessConstants.PERMISSION_PATH_SEPARATOR + iteration.id,
								enabledContext: ProcessConstants.PermissionContext.ITERATION,
								overrideAllowed: true,
								current: iteration.current,
								completed: iteration.completed,
								leaf: iteration.leaf,
								startDate: dojo.date.stamp.fromISOString(iteration.startDate),
								endDate: dojo.date.stamp.fromISOString(iteration.endDate),
								iterationTypeItemId: iteration.iterationTypeItemId ? iteration.iterationTypeItemId : ProcessConstants.ITERATION_TYPE_NONE,
								hasDeliverable: iteration.hasDeliverable
							}, {
								parent: item,
								attribute: "items" //$NON-NLS-1$
							});
						}
					});
					this._populateIterationData(iteration.iterations);
				}
				this.save();
			}
		},
		
		_populateIterationTypesData: function(data){
			var self = this;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.ITERATION_TYPES,
				onItem: function(item, request){
					var items = {};
					var i = 0;
					dojo.forEach(data, function(iterationType){
						var newChildItem = self.newItem({
							name: iterationType[ProcessConstants.IterationTypeDTO.LABEL],
							id: iterationType[ProcessConstants.IterationTypeDTO.ITEM_ID],
							parent: item[ProcessConstants.IterationTypeDTO.ID],
							context: ProcessConstants.PermissionContext.TEAM,
							contextId: iterationType.id,
							enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE,
							overrideAllowed: true
						}, {
							parent: item,
							attribute: "items" //$NON-NLS-1$
						});
						items[i] = newChildItem;
						i++;
					});
				}
			});
			this.save();
		},
		
		_handleDevelopmentLinesFetchError: function(error){
			this._errorCallback.call(this._errorContext, error, ProcessConstants.PermissionsScopeContext.DEV_LINES);
		},
		
		_handleIterationTypesFetchError: function(error){
			this._errorCallback.call(this._errorContext, error, ProcessConstants.PermissionsScopeContext.ITERATION_TYPES);
		},
		
		_handleIterationsFetchError: function(error){
			this._errorCallback.call(this._errorContext, error, ProcessConstants.PermissionsScopeContext.ITERATIONS);
		},
		
		_addProjectRootItem: function(){
			if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
				this.newItem({
					id: ProcessConstants.PermissionsScopeContext.PROJECT,
					name: this.messages.PermissionsScopeStore_ProjectConfigurationNodeName,
					context: ProcessConstants.PermissionContext.PROJECT,
					contextId: "", //$NON-NLS-1$
					enabledContext: ProcessConstants.PermissionContext.PROJECT,
					overrideAllowed: false
				});
			}
		},
		
		_addTeamRootItem: function(){
			var teamItem = {
				id: ProcessConstants.PermissionsScopeContext.TEAM,
				name: this.messages.PermissionsScopeStore_TeamConfigurationNodeName,
				context: ProcessConstants.PermissionContext.TEAM,
				contextId: "", //$NON-NLS-1$
				enabledContext: ProcessConstants.PermissionContext.TEAM,
				overrideAllowed: false
			};
			if (this._editorContext === ProcessConstants.EditorContext.TEAM) {
				teamItem.name = this.messages.PermissionsScopeStore_PermissionsNodeName;
				teamItem.overrideAllowed = true;
			}
			this.newItem(teamItem);
		},
		
		_addIterationTypesRootItem: function(){
			this.newItem({
				id: ProcessConstants.PermissionsScopeContext.ITERATION_TYPES,
				name: this.messages.PermissionsScopeStore_IterationTypesNodeName
			});
		},
		
		_addTimelinesRootItem: function(){
			if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
				this.newItem({
					id: ProcessConstants.PermissionsScopeContext.DEV_LINES,
					name: this.messages.PermissionsScopeStore_TimelinesNodeName
				});
			}
		},
		
		_addIterationsRootItem: function(){
			if (this._editorContext === ProcessConstants.EditorContext.TEAM) {
				this.newItem({
					id: ProcessConstants.PermissionsScopeContext.ITERATIONS,
					name: this.messages.PermissionsScopeStore_IterationsNodeName
				});
			}
		},
		
		isTimeline: function(item){
			var context = this.getValue(item, "enabledContext"); //$NON-NLS-1$
			if (context === ProcessConstants.PermissionContext.DEV_LINE) {
				return true;
			}
			return false;
		},
		
		isProjectTimeline: function(item){
			return this.getValue(item, "projectTimeline"); //$NON-NLS-1$
		},
		
		isIteration: function(item){
			var context = this.getValue(item, "enabledContext"); //$NON-NLS-1$
			if (context === ProcessConstants.PermissionContext.ITERATION) {
				return true;
			}
			return false;
		},
		
		isCurrentIteration: function(item){
			return this.getValue(item, "current") === true && this.getValue(item, "leaf") === true; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		getTimelineForIteration: function(iteration){
			var self = this;
			var timeline;
			this.fetchItemByIdentity({
				identity: iteration.parent,
				onItem: function(item, request){
					if (self.isTimeline(item)) {
						timeline = item;
					} else {
						timeline = self.getTimelineForIteration(item);
					}
				}
			});
			return timeline;
			
		},
		
		getParent: function(item){
			var parent;
			this.fetchItemByIdentity({
				identity: item.parent,
				onItem: function(item, request){
					parent = item;
				}
			});
			return parent;
		},
		
		deleteItem: function(item){
			// Get the children of the item
			var children = this.getValues(item, "items"); //$NON-NLS-1$
			// Now get rid of the parent first
			// to prevent flickering in listeners of the delete
			this.inherited(arguments);
			
			// Now destroy the children recursively
			if (children) {
				dojo.forEach(children, function(child){
					this.deleteItem(child);
				}, this);
			}
		}
		
		
	});
	
})();
