﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

import flash.external.ExternalInterface;
 
class ChartShape extends ChartObject implements IChartShape {

	public var id:String;
	
	public var outlineColour:Number;
	
	public var fillColour:Number;
	
	public var animate:Boolean;

	public var tooltip:Tooltip;

	public var dataValue:Number;
	
	public var dataName:String;
	
	private static var _instanceCount:Number = 0;

	
	public function ChartShape(outlineColour:Number, fillColour:Number, name:String, value:Number, animate:Boolean) {
		super();
		
		this.tooltip = new Tooltip(name, String(value));
		this.outlineColour = outlineColour;
		this.fillColour = fillColour;
		this.animate = animate;
		this.dataValue = value;
		this.dataName = name;
		
		ChartShape._instanceCount++;
		this.id = "shape_mc_" + ChartShape._instanceCount;		
	}	

	public function doRollOver():Void {
		this.mc._alpha = 60;
		this.tooltip.allowMove = false;
		this.tooltip.show();		
	}
	
	public function doRollOut():Void{
		this.mc._alpha = 100;
		this.tooltip.allowMove = true;
		this.tooltip.hide();
	}
	
	public function getLabel():String {
		return this.getName();
	}	

	public function doDrillIn():Void {

		var model:ChartModel = ChartEngine.getInstance().getModel();

		if (model.onClickHandler) {
			var label:String = this.getLabel();
			var result:Object = ExternalInterface.call(model.onClickHandler, model.chartID, label);
			if (!result && model.isDebug) {
				ChartEngine.getInstance().getChartArea().getDebugArea().debug("ERROR: Drill-in failed [" + label + "]");
			}
		} else {
			if (model.isDebug) {
				ChartEngine.getInstance().getChartArea().getDebugArea().debug("ERROR: Load vars failed");
			}
		}
	}			
	
	public function getFillColour():Number {
		return this.fillColour;
	}
	
	public function getOutlineColour():Number {
		return this.outlineColour;
	}

	public function getValue():Number {
		return this.dataValue;
	}

	public function getName():String {
		return this.dataName;
	}
	
	public function destroy():Void {
		
		delete this.id;
		delete this.outlineColour;
		delete this.fillColour;
		delete this.animate;
		delete this.dataValue;
		delete this.dataName;
		
		// Tooltip
		if (this.tooltip) {
			this.tooltip.destroy();
			delete this.tooltip;
		}
		
		super.destroy();
	}		
	
}