﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

class Legend extends LayoutObject {
	
	private static var _LEGEND_ENTRY_SPACING:Number = 6;
	
	private static var _LEFT_MARGIN:Number = 20;
	
	private var _shapes:Array;
	
	private var _legendEntries:Array;
	
	private var _maxLabelWidth:Number;
	
	
	public function Legend() {
		super();
		
		this._maxLabelWidth = 0;
	}
	
	public function draw(parentMC:MovieClip, shapes:Array):Void {
		
		this._shapes = shapes;
		this._legendEntries = new Array();
		this.mc = 
			parentMC.createEmptyMovieClip(
				"legend", 
				parentMC.getNextHighestDepth()
			);

		var labelWidth:Number = this._getLabelWidth();
		var nextY:Number = this.startY;
		var maxY:Number = this.startY + this.height;

		for (var index:Number = 0; index < this._shapes.length; index++) {
			// Prevent legend entry clipping
			// This can happen when the number of legend entries exceeds the legend height
			if ((nextY + LegendEntry.EDGE_LENGTH) > maxY) {
				break;
			}
			
			var shape:IChartShape = this._shapes[index];
			this._legendEntries[index] = new LegendEntry(shape);
			
			this._legendEntries[index].draw(this.mc,
											this.startX + Legend._LEFT_MARGIN, 
											nextY, 
											labelWidth);
			
			nextY = nextY + Legend._LEGEND_ENTRY_SPACING + LegendEntry.EDGE_LENGTH;
			
		}
	}	
	
	public function getPreferredWidth():Number {
		// Note: Left out ChartArea.MARGIN
		// Tighter fit results
		var width:Number = Legend._LEFT_MARGIN +
						   LegendEntry.EDGE_LENGTH +
						   LegendEntry.LABEL_OFFSET_X +
						   this._getMaxLabelWidth();
		
		return width;
	}
	
	private function _getMaxLabelWidth():Number {

		if (this._maxLabelWidth > 0) {
			return this._maxLabelWidth;
		}
		
		var model:ChartModel = ChartEngine.getInstance().getModel();
		this._maxLabelWidth = model.getMaxParameterNameWidth(LegendEntry.TEXT_FORMAT);
		
		return this._maxLabelWidth;
	}
	
	private function _getLabelWidth():Number {
		// Note: Left out ChartArea.MARGIN 
		// Tighter fit results
		var width:Number = this.getWidth() -
						   Legend._LEFT_MARGIN -
						   LegendEntry.EDGE_LENGTH -
						   LegendEntry.LABEL_OFFSET_X;
		
		return width;		
	}
	
	public function getPreferredHeight():Number {
		var model:ChartModel = ChartEngine.getInstance().getModel();
		var swatchCount:Number = model.parameterCount;
		var height:Number = (swatchCount * LegendEntry.EDGE_LENGTH) + 
							((swatchCount - 1) * Legend._LEGEND_ENTRY_SPACING);
							
		return height;
	}	
	
	public function destroy():Void {
		// Max label width
		delete this._maxLabelWidth;
		// Shapes		
		if (this._shapes) {
			for (var index:Number = 0; index < this._shapes.length; index++) {
				if (this._shapes[index]) {
					this._shapes[index].destroy();
					delete this._shapes[index];
				}			
			}
			delete this._shapes;			
		}
		// Legend entries		
		if (this._legendEntries) {
			for (var index:Number = 0; index < this._legendEntries.length; index++) {
				if (this._legendEntries[index]) {
					this._legendEntries[index].destroy();
					delete this._legendEntries[index];
				}			
			}
			delete this._legendEntries;
		}
		
		super.destroy();
	}		
	
}