﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
class VerticalBarGrid extends Grid {

	public static var MIN_H_TICK_SPACING:Number = 15;

	// Margin accounts for the top-half of the biggest Y-axis label on the top horizontal tick (label is vertically centered)
	public static var H_TICK_MARGIN_Y:Number = YAxis.LABEL_FORMAT.getTextExtent("0").textFieldHeight / 2;	
	
	public static var H_TICK_MARGIN_X:Number = 6;	

	
	public function VerticalBarGrid(width:Number, height:Number, startX:Number, startY:Number) {
		super();
		
		this._calculatePartitions(width, 
								  height, 
								  startX, 
								  startY);
	}

	private function _calculatePartitions(width:Number, height:Number, startX:Number, startY:Number):Void {

		var model:ChartModel = ChartEngine.getInstance().getModel();		
		var graphAreaHeight:Number = height - VerticalBarGrid.H_TICK_MARGIN_Y;
		var yAxisSectionCount:Number = Math.floor(graphAreaHeight / VerticalBarGrid.MIN_H_TICK_SPACING);
		
		// Calculate the maximum number of sections available (assume tightest tick spacing) given the available height
		var result:Object = this.calculateIncrement(yAxisSectionCount, 
													model.maxValue, 
													graphAreaHeight, 
													VerticalBarGrid.MIN_H_TICK_SPACING);
		
		var yAxisMaxLabelWidth:Number = YAxis.LABEL_FORMAT.getTextExtent(String(result.maxValue)).width;
		var graphAreaWidth:Number = width - yAxisMaxLabelWidth - VerticalBarGrid.H_TICK_MARGIN_X - Axis.TICK_LENGTH;
		
		// Set data
		this.initializeGridData(width, 
								height, 
								result.maxValue, 
								startX, 
								startY,
								0,
								yAxisMaxLabelWidth,
								graphAreaWidth, 
								graphAreaHeight, 
								result.spacing, 		// Y-Axis
								result.increment, 		// Y-Axis
								result.sectionCount, 	// Y-Axis
								0, 						// X-Axis: Not used
								0,						// X-Axis: Not used
								0);						// X-Axis: Not used
		
	}	
	
	public function initializeGridData(width:Number, height:Number, max:Number, startX:Number, startY:Number, xAxisMaxLabelWidth:Number,
									   yAxisMaxLabelWidth:Number, graphAreaWidth:Number, graphAreaHeight:Number, 
									   yAxisSpacing:Number, yAxisIncrement:Number, yAxisSectionCount:Number, xAxisSpacing:Number, 
									   xAxisIncrement:Number, xAxisSectionCount:Number):Void {

		super.initializeGridData(width, height, max, startX, startY, xAxisMaxLabelWidth, yAxisMaxLabelWidth, graphAreaWidth, 
					   		     graphAreaHeight, yAxisSpacing, yAxisIncrement, yAxisSectionCount, xAxisSpacing, 
   		      					 xAxisIncrement, xAxisSectionCount);
		
		// TODO: REVISIT: Should h tick offset vars be in grid data?
		this.gridData.hTickOffsetX = startX + VerticalBarGrid.H_TICK_MARGIN_X + yAxisMaxLabelWidth;
		this.gridData.hTickOffsetY = startY + VerticalBarGrid.H_TICK_MARGIN_Y;
		
		this.gridData.xZeroPosition = this.gridData.hTickOffsetX + Axis.TICK_LENGTH;		
		this.gridData.yZeroPosition = this.gridData.hTickOffsetY + (yAxisSpacing * yAxisSectionCount);
		
		// TEST
		//Tester.dumpGridData(this.gridData);
	}	
	
	public function draw(parentMC:MovieClip):Void {
		super.draw(parentMC);
		
		// X-Axis line
		this.xAxis = new VerticalBarXAxis();
		this.xAxis.draw(this.mc, 
						this.gridData);

		// Y-Axis line
		this.yAxis = new VerticalBarYAxis();
		this.yAxis.draw(this.mc, 
						this.gridData);
		
	}	
	
	public function destroy():Void {
		super.destroy();
	}		
	
}
