/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.IFrameInnerContainer");

dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.i18n");

dojo.require("jazz.app.internal.ProxyRequestInterceptor");
dojo.require("jazz.app.proxy");
dojo.require("jazz.core.loader");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "IFrameInnerContainerMessages");

(function() {

var ScopePreferenceProvider = com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "IFrameInnerContainerMessages");

/*
 * The container widget inside the iframe that contains the desired widget.
 * Should only be used by IFrameContainer.
 * 
 * CAUTION: Must remain backward compatible for cross-version interoperability.
 * Dashboard framework must check for existence of new methods added after 0.7.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.util.IFrameInnerContainer",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-util-IFrameInnerContainer\"><div class=\"loading\">" + messages.loading + "</div></div>",
	
	active: false,
	args: null,
	onBustOut: null,
	onBustIn: null,
	onLoad: null,
	onResize: null,
	
	_serviceRoot: null,
	_webUIRoot: null,
	_encodeProxiedURI: null,
	_isAutoResize: false,
	_bustOutCounter: 0,
	
	postCreate: function() {
		// notify parent that container is created
		if (frameElement.onCreate) {
			frameElement.onCreate(this);
		}
	},
	
	destroy: function() {
		this.setActive(false);
		this.inherited(arguments);
	},
	
	/*
	 * Returns the scope preference provider class (constructor) for the
	 * viewlet in this iframe.
	 */
	getScopePreferenceProviderClass: function() {
		return ScopePreferenceProvider;
	},
	
	init: function(/* object */ args) {
		this.args = args.args;
		this.onBustOut = args.onBustOut;
		this.onBustIn = args.onBustIn;
		this.onLoad = args.onLoad;
		this.onResize = args.onResize;
		this._serviceRoot = args.serviceRoot;
		this._webUIRoot = args.webUIRoot;
		this._encodeProxiedURI = args.encodeProxiedURI;
		
		// config data server root
		if (this._serviceRoot) {
			dojo.setObject("net.jazz.ajax._dataServerRoot", this._serviceRoot);
		}
		
		// lazy update link targets to parent window
		this.connect(document.body, "onmouseover", this._onMouseOver);
		
		// set initial active/inactive
		this.setActive(args.active);
		
		// get the class name from the url path
		var path = location.pathname + "";
		var index = path.lastIndexOf("/");
		var className = path.substring(index + 1);
		
		// replace proxy URI building to work in our proxied iframe
		jazz.app.proxy.encodeProxiedURI = this._encodeProxiedURI;
		jazz.app.internal.ProxyRequestInterceptor.setEncodeProxiedURIFunction(this._encodeProxiedURI);
		
		// load the widget
		jazz.core.loader.load_async(className, dojo.hitch(this, function() {			
			// has to be done after code is loaded
			this._applyWorkarounds();
			
			// create and append widget
			var clazz = dojo.getObject(className);
			var widget = new clazz(this.args);
			this.domNode.replaceChild(widget.domNode, this.domNode.firstChild);
			if (this.onLoad) {
				this.onLoad(widget);
			}
		}));
	},
	
	/*
	 * Sets whether or not the container is currently active (visible to the
	 * user on the current page).
	 */
	setActive: function(/* boolean */ value) {
		if (value != this.active) {
			// resize only while active
			this._setAutoResize(value);
			this.active = value;
		}
	},
	
	_setAutoResize: function(/* boolean */ value) {
		if (value != this._isAutoResize) {
			if (value == true) {
				this._onResize(true);
				this._onResizeHandle = this.connect(window, "onresize", this._onResize);
				this._onResizeInterval = setInterval(dojo.hitch(this, this._onResize), 500);
			}
			else {
				this.disconnect(this._onResizeHandle);
				clearInterval(this._onResizeInterval);
				delete this._onResizeInterval;
			}
			this._isAutoResize = value;
		}
	},
	
	_onBustOut: function() {
		// use ref counting to account for hovers in dialogs, vice versa
		if (this._bustOutCounter++ == 0) {
			this._setAutoResize(false);
			
			// delegate to outer container
			if (this.onBustOut) {
				this.onBustOut();
			}
		}
	},

	_onBustIn: function() {
		// use ref counting to account for hovers in dialogs, vice versa
		if (this._bustOutCounter-- == 1) {
			// delegate to outer container
			if (this.onBustIn) {
				this.onBustIn();
			}
			this._setAutoResize(true);
		}
	},
	
	_onMouseOver: function(/* Event */ event) {
		if (event) {
			var node = event.target;
			if (node && node.nodeName == "A" && !node.target) {
				// open links in parent window
				node.target = "_parent";
			}
		}
	},
	
	_onResize: function(/* boolean */ force) {
		var coords = dojo.coords(this.domNode);
		if (force || coords.h != this._h || coords.y != this._y) {
			this._h = coords.h;
			this._y = coords.y;
			if (this.onResize) {
				this.onResize(coords.h + coords.y * 2);
			}
		}
	},
	
	_applyWorkarounds: function() {
		// attach bust-out listeners
		var Dialog = dojo.getObject("jazz.ui.Dialog");
		if (Dialog) {
			this.connect(Dialog.prototype, "postCreate", this._onBustOut);
			this.connect(Dialog.prototype, "destroy", this._onBustIn);
		}
		var MasterPopup = dojo.getObject("jazz.ui.internal._MasterPopup");
		if (MasterPopup) {
			this.connect(MasterPopup.prototype, "hide", this._onBustIn);
			var self = this;
			var showFunc = MasterPopup.prototype.show;
			MasterPopup.prototype.show = function() {
				// need to preemptively bust out for hover popups
				self._onBustOut();
				showFunc.apply(this, arguments);
			};
		}
		
		// set context root to full URI root of the remote server Web UI
		// for links, etc.
		if (this._webUIRoot) {
			dojo.setObject("net.jazz.ajax._contextRoot", this._webUIRoot);
		}

		var ResourceLink = dojo.getObject("jazz.app.ResourceLink");
		if (ResourceLink) {
			ResourceLink.setLazyFetch(true);
		}
	}
});

})();
