/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.utils.SimpleTabContainer");

/*
 * Imports 
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.build.web.ui.internal.utils.SimpleTab");

(function() {

/*
 * Aliases 
 */
var SimpleTab = com.ibm.team.build.web.ui.internal.utils.SimpleTab;

/*
 * Class 
 */
dojo.declare("com.ibm.team.build.web.ui.internal.utils.SimpleTabContainer",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	callbackTabSwitch: null,
	tabOrder: null, // an array of tabIds, can be populated on construction to control the order of the tabs regardless of when they are added

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.build.web", "ui/internal/templates/utils/SimpleTabContainer.html"),
	
	/*
	 * Static Variable Container
	 */
	_static: function() {}, 	

	// Public Methods //------------------------------------------------------->

	/*
 	 * @param Object args
 	 * 
 	 * @return 
 	 */
	constructor: function(args) {
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		this._initializeStaticVariables();
		this._initializeInstanceVariables();
		this._initializeWidget();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		// Call Back Tab Switch
		// Do not delete (do not own)
		
		// Tabs
		if (this._tabHolder) {
			for (var tabID in this._tabHolder) {
				if (this._tabHolder[tabID]) {
					this._tabHolder[tabID].destroy();
					delete this._tabHolder[tabID];
				}
			}
			delete this._tabHolder;
		}

		// Selected Tab
		// No need to delete (selected tab stored in tabHolder)

		this.inherited(arguments);
	},

	/*
 	 * @param String tabID
 	 * @param String title
 	 * @param Function callbackGetNode
 	 * 
 	 * @return void
 	 */		
	addTab: function(tabID, title, callbackGetNode, href) {
		// Create new tab
		var tabWidget = new SimpleTab({
			tabID: tabID,
			label: title,
			callbackGetNode: callbackGetNode,
			callbackOnClick: dojo.hitch(this, this._handleEventOnClickTab),
			linkHref: href
		});
		// Add tab to container
		var refNode = this._getTabInsertRefNode(tabID);
		if (refNode)
			this._tabArea.insertBefore(tabWidget.domNode, refNode)
		else
			this._tabArea.appendChild(tabWidget.domNode);
		// Store tab
		this._tabHolder[tabID] = tabWidget;
	},
	
	getSelection: function() {
		return this._selectedTab ? this._selectedTab.tabID : null;
	},
	
	hasTab: function(tabID) {
		return this._tabHolder[tabID] ? true : false;
	},

	/*
 	 * @param String tabID
 	 * 
 	 * @return void
 	 */		
	selectTab: function(tabID) {
		
		if ((tabID in this._tabHolder) === false) {
			return;
		}
		
		this._fireEventTabSwitch(tabID);
		
		var previousTab = this._selectedTab;
		
		if (previousTab && 
			(previousTab.tabID === tabID)) {
			// Same tab
			// Nothing to do
			return;
		} else if (previousTab) {
			// Deselect previous tab

			previousTab.deselect();
		}

		var currentTab = this._tabHolder[tabID];
		
		// Select current tab
		currentTab.select();
		
		// Update selection
		this._selectedTab = currentTab;
	},

	/*
 	 * @param String tabID
 	 * 
 	 * @return Element
 	 */		
	getContentNode: function(tabID) {
		if ((tabID in this._tabHolder) === false) {
			return null;
		}
		return this._tabHolder[tabID].contentNode;
	},

	// Private Methods //------------------------------------------------------>

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeStaticVariables: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeInstanceVariables: function() {
		this._tabHolder = new Object();
		this._selectedTab = null;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeWidget: function() {
	},
	
	/*
 	 * @param SimpleTab tab
 	 * 
 	 * @return void
 	 */		
	_handleEventOnClickTab: function(tab) {
		
		if (this._selectedTab &&
			this._selectedTab.tabID === tab.tabID) {
			// Same tab
			// Nothing to do
			return;
		}
		
		this.selectTab(tab.tabID);
	},	

	/*
 	 * @param String newTabID
 	 * 
 	 * @return void
 	 */		
	_fireEventTabSwitch: function(newTabID) {

		if (!this.callbackTabSwitch) {
			return;
		}
		
		var oldTabID = null;
		if (this._selectedTab) {
			oldTabID = this._selectedTab.tabID;
		}
		
		this.callbackTabSwitch(newTabID, oldTabID);
	},
	
	_getTabInsertRefNode: function(tabID) {
		if (!this.tabOrder)
			return null;
		for (var i = 0; i < this.tabOrder.length; i++) {
			if (!foundThisTab) {
				var foundThisTab = (tabID === this.tabOrder[i]);
				continue;
			}
			if (this._tabHolder[this.tabOrder[i]])
				return this._tabHolder[this.tabOrder[i]]._tab;
		}
		return null;
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_noop: function() {
	}		
	
});

})();
