/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.utils.WorkspaceSelectionDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.ModeledTree");
dojo.require("jazz.ui.tree.IconItem");
dojo.require("jazz.util.html");

dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function(){

var ScmClient = com.ibm.team.scm.web.client.internal.ScmClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

dojo.declare("com.ibm.team.build.web.ui.internal.utils.WorkspaceSelectionDialog",[dijit._Widget, dijit._Templated], {
	templateString: '<div style="padding: 1em 1em 1em 1em;">' +
						'<div dojoAttachPoint="_content"></div>' +
						'<div style="padding: 1em 0em; float: right;">' +
							'<button dojoAttachPoint="_okCmd" disabled="true">${messages.ok}</button>' +
							'<button dojoAttachPoint="_cancelCmd">${messages.cancel}</button>' +
						'</div>' +
					' </div>',
	
	constructor: function(args) {
		this.messages = messages;
	},
	
	postCreate: function() {
		this.contentWidget = new com.ibm.team.build.web.ui.internal.utils.WorkspaceSelectionDialogContent({
			onLoad: dojo.hitch(this,this._onLoad),
			selection: this.selection
		});
		this._content.appendChild(this.contentWidget.domNode);
		this.dialog = new jazz.ui.Dialog({primaryTitle: messages.selectWorkspace, contentNode: this.domNode});
		this.connect(this._okCmd,"onclick",dojo.hitch(this,this._onOk));
		this.connect(this._cancelCmd,"onclick",dojo.hitch(this.dialog,this.dialog.close));
	},
	
	onOk: function(selection) {
		
	},
	
	_onOk: function() {
		this.onOk(this.contentWidget.getSelection());
		this.dialog.close();
	},
	
	_onLoad: function() {
		if (this.contentWidget.tree.getChildren().length > 0)
			this._okCmd.disabled = false;
		this.contentWidget.tree.onDblClick = dojo.hitch(this,this._onOk);
	}
});

dojo.declare("com.ibm.team.build.web.ui.internal.utils.WorkspaceSelectionDialogContent",[dijit._Widget, dijit._Templated], {
	templateString: "<div class='com-ibm-team-build-web-ui-internal-utils-WorkspaceSelectionDialogContent' style='color: #999999'>${messages.loading}</div>",
	
	constructor: function(args) {
		this.messages = messages;
	},
	
	postCreate: function() {
		var serviceObject = {
			self: this,
			success: function(response) {
				this.self._createTree(response.items);
				if (this.self.onLoad)
					this.self.onLoad();
			},
			failure: function(error) {
				console.error(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		ScmClient.getWorkspaces(srh, {owner: "itemOid/com.ibm.team.repository.Contributor/" + getAuthenticatedContributor().itemId});
	},
	
	getSelection: function() {
		return this.tree.getSelection();
	},
	
	_createTree: function(items) {
		var model = [];
		if (items && items.length > 0) {
			for (var i = 0; i < items.length; i++) {
				var workspace = items[i].workspace;
				if (workspace.isStream)
					continue;
				model.push({label: jazz.util.html.escape(workspace.name), iconClass: "workspaceIcon", itemId: workspace.itemId});
			}
		}
		this.tree = new jazz.ui.ModeledTree({model: model, createTreeItem: function(obj) {
			var result = new jazz.ui.tree.IconItem(obj);
			result.getId = function() {
				return this.itemId;
			}
			return result;
		}});
		if (this.selection) {
			var item = this.tree.getItem(this.selection);
			if (item)
				this.tree.setSelection(item);
		}
		if (this.tree.getSelection().length == 0 && this.tree.getChildren().length > 0)
			this.tree.setSelection(this.tree.getChildren()[0]);
		this.domNode.innerHTML = "";
		this.domNode.style.color = "";
		this.domNode.appendChild(this.tree.domNode);
	}
});

})();
