/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
/**
 * @author bjiang
 */
dojo.provide("com.ibm.team.scm.web.ui.internal.view.browse.FileFolderContentView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");
dojo.require("dojo.io.iframe");
dojo.require("dijit.form.CheckBox");
dojo.require("jazz.util.ResizeNotifier"); //$NON-NLS-1$
dojo.require("jazz.ui.HoverPopup");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("jazz.util.date");
dojo.require("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function() { // module container

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ScmUtil					= com.ibm.team.scm.web.client.util;
var ScmClient				= com.ibm.team.scm.web.client.internal.ScmClient;
var	PlatformUI 				= net.jazz.ajax.ui.PlatformUI;


/* defineWidget */ dojo.declare(
	"com.ibm.team.scm.web.ui.internal.view.browse.FileFolderContentView",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/view/browse/templates/FileFolderContentView.html"),	
		
		// object initializer
		constructor: function(args) {
			this.streamWorkspaceView = args.parentView;
			this.pageController = this.streamWorkspaceView.controller;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
			
			this.versionableDTO = null;
			this._twistieImages = {};
			this._inlineEditors = {};
		},
		
		postCreate: function() {
//			this.pageController = this.parentController;
		},
		
		destroy: function() {
		},
		
		showWidget: function(subElements, args) {			
			this.versionableDTO = subElements;
			this.isComponent = args.isComponent;
			this.refresh();
		},
		
		reload: function() {
			if(!this.versionableDTO)
				return;
				 
			this.retrieveVersionable(this.versionableDTO.workspace.itemId, 
								this.versionableDTO.component.itemId, 
								this.versionableDTO.path, 
								dojo.hitch(this, this.refresh), 
								this);
		},
				
		refresh: function() {	
			if(!this.versionableDTO)
				return;			
			dojox.data.dom.removeChildren(this._headerRowLine);
			dojox.data.dom.removeChildren(this._headerRow);
			dojox.data.dom.removeChildren(this._tableBody);
			dojox.data.dom.removeChildren(this._fileName);
			dojox.data.dom.removeChildren(this._fileBody);
			this._fileContent.style.visibility = 'hidden'; 
				
			if(this.versionableDTO.versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.filesystem.FileItem") {
				//Display the file content
				this._fileContent.style.visibility = 'visible'; 

				if (this._hasTextContentType(this.versionableDTO.versionable)) {					
					var headers = {};
					headers["accept-charset"] = "utf-8";
					
					var self = this;
					var xhr = dojo.xhrGet({
						url:"../../service/com.ibm.team.filesystem.common.IFileContentService/content/" + this.versionableDTO.versionable.itemId + "/" + this.versionableDTO.versionable.stateId,
						headers: headers,
						load: function (content) {										
							self._displayFileContent(content, self._fileBody);				
						},
		                error: function (content) {
		                    console.error('Error: ', content);
		                }
						});
				}
				else if (this._isWebImage(this.versionableDTO.versionable)) {
					var src = ScmUtil.getFileDownloadURI(this.versionableDTO.workspace.itemId, this.versionableDTO.component.itemId, this.versionableDTO.versionable.name, this.versionableDTO.versionable.itemId, this.versionableDTO.versionable.stateId);
					var tr = document.createElement("tr");
					var td = document.createElement("td");
					td.className = "folderContent_td";
					var img = document.createElement("img");
					img.src = src;
					img.title = this.versionableDTO.versionable.name;
					td.appendChild(img);
					tr.appendChild(td);
					this._fileBody.appendChild(tr);
				}
				else {
					var text = document.createTextNode(this.messages.StreamWorkspaceView_0);
					var tr = document.createElement("tr");
					var td = document.createElement("td");
					td.className = "folderContent_td";
					td.appendChild(text);
					tr.appendChild(td);
					this._fileBody.appendChild(tr);
				}
			}	
			else {  // component or folder
				if(this.versionableDTO.children) {
					this._displaySubElements(this.versionableDTO);
				} else {
					//Display an empty directory
					this._displayEmptyDirectory(this.versionableDTO.versionable.name, this.isComponent);
				}
			}		
		},
		
		/**
		 * Display an empty directory.
		 * 
		 * @param {Object} path
		 */
		_displayEmptyDirectory: function(path, isComponent) {
			this._fileContent.style.visibility = 'visible'; 
			var text = document.createTextNode(path);
			this._fileName.appendChild(text);
			
			var tr= document.createElement("tr");
			var td= document.createElement("td");
			td.className = "folderContent_td";
			var div = document.createElement("div");
			div.innerHTML= isComponent ? this.messages.FilesFoldersView_ComponentEmptyMessage: this.messages.StreamWorkspaceView_45;
			td.appendChild(div);
			tr.appendChild(td);
			this._fileBody.appendChild(tr);
		},
		
		/**
		 * Parse and display the content of a file.
		 * 
		 * @param {Object} encodedContent
		 * @param {Object} tbody
		 */
		_displayFileContent: function(encodedContent, tbody) {
			var count = 0;
			var lines = [];
			var j = 0;
			var cr = 0;
			var lf = 0;
			var crlf = 0;
			while (count < encodedContent.length) {
				var tr = document.createElement("tr");
				var th = document.createElement("th");
				var id = j+1;
				th.id = "line" + id;
				th.innerHTML = id;
				tr.appendChild(th);
				var td = document.createElement("td");
				var div = document.createElement("div");
				div.className = "com-ibm-team-scm-file-contentViewer";
				//td.setAttribute("width", "100%");
				
				if (cr != -1) {
					cr = encodedContent.indexOf("\r", count);
				}
				if (lf != -1) {
					lf = encodedContent.indexOf("\n", count);
				}
				if (crlf != -1) {
					crlf = encodedContent.indexOf("\r\n", count);
				}
				
				if (cr != -1 || lf != -1 || crlf != -1) {
					if (crlf != -1 && crlf <= cr && crlf < lf) {
						lines[j] = encodedContent.slice(count, crlf);
						count = crlf + 2;
					}
					else if(cr!=-1 && ((lf != -1 && cr < lf) || lf == -1)) {
						lines[j] = encodedContent.slice(count, cr);
						count = cr + 1;
					}
					else if(lf!=-1) {
						lines[j] = encodedContent.slice(count, lf);
						count = lf + 1;
					}

					if (lines[j].length > 0) {
						var text = document.createTextNode(lines[j]);
						div.appendChild(text);
						if(count>=encodedContent.length) { //End of file reached (ending with a line separator)
							var divLast = document.createElement("div");
							divLast.innerHTML = " ";
						}
					}
					else {
						div.innerHTML = " ";
					}
					j++;
				}
				else { //End of file reached
					lines[j] = encodedContent.slice(count, encodedContent.length);
					count = encodedContent.length;
					if (lines[j].length > 0) {
						var text = document.createTextNode(lines[j]);
						div.appendChild(text);
					}
					else {
						div.innerHTML = " ";
					}
				}

				td.appendChild(div);
				tr.appendChild(td);
				tbody.appendChild(tr);
				if(divLast) {
					var tr = document.createElement("tr");
					var th = document.createElement("th");
					var id = j+1;
					th.id = "line" + id;
					th.innerHTML = id;
					tr.appendChild(th);
					var td = document.createElement("td");
					divLast.className = "com-ibm-team-scm-file-contentViewer";
					td.setAttribute("width", "100%");
					td.appendChild(divLast);
					tr.appendChild(td);
					tbody.appendChild(tr);
				}
			}
			
			this._updateFileContentHeaderRow(id);
		},
		/**
		 * Display the contents of a component or folder.
		 * 
		 * @param {Object} subElements
		 */
		_displaySubElements: function(subElements) {
			var workspaceId = subElements.workspace.itemId;
			var componentId = subElements.component.itemId;
			var isStream = subElements.workspace.stream;
			
			var headers = [{displayText: this.messages.FileFolderProp_Name, className: "fileName"},
							{displayText: this.messages.FileFolderProp_Size, className: "fileSize"},
							{displayText: this.messages.FileFolderProp_LastMod, className: "lastModified"},
							{displayText: this.messages.FileFolderProp_ModBy, className: "modifiedBy"}];
			if(isStream) {
				headers.push({displayText: this.messages.FileFolderProp_Actions, className: "actions"});
			}
							
			var self = this;				
			dojo.forEach(headers, function(col){
				self._addHeaderColumn(col.displayText, col.className);
			});			
			
			var children;
			if(subElements.versionablePlusDTO)
				children = subElements.versionablePlusDTO.childrenPlus;
			else
				children = subElements.children;
				
			dojo.forEach(children, function(x) {
				
				var versionable;
				if(subElements.versionablePlusDTO)
				 	versionable = x.versionable;
				else
					versionable = x;
				
				// file/folder name column
				var isFolder = versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.scm.Folder";
				var isSymlink = versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.filesystem.SymbolicLink";
				
				var lockedBy = x.lockedBy;
				var isLocked = lockedBy ? true : false;
				var contributor = ScmUtil.getLoggedInContributor();
				
				var tr= document.createElement("tr");
				dojo.addClass(tr, "visibleRow");
				var nameTD = document.createElement("td");
				nameTD.className = "folderContent_td";
				nameTD.isSymLink = isSymlink;
				nameTD.isFolder = isFolder;
				nameTD.fileName = versionable.name;
				nameTD.isLocked = isLocked;
				nameTD.lockedBy = lockedBy;
				var img = ScmUtil.getDecoratedResourceIcon({
					isSymbolicLink: isSymlink,
					isFolder: isFolder,
					fileName: versionable.name,
					isLocked: isLocked,
					lockedBy: lockedBy
				});

				dojo.addClass(img, "icon");
				nameTD.icon = img;
				
				var hoverInfo = {
					parentNode: nameTD,
					lockedByMe: lockedBy && lockedBy.itemId === contributor.itemId,
					lockedByOtherName: lockedBy ? lockedBy.name : null,
					subElements: subElements,
					versionable: versionable
				};
				nameTD.hoverInfo = hoverInfo;
				self._hookHover(hoverInfo);
				
				var anchor= document.createElement("a");
				//If the selected item is a component.
                var slashName = "/" + encodeURIComponent(versionable.name); 
                var path = (subElements.path =="/")? slashName : subElements.path + slashName;              
                var href = ScmUtil.getBrowseElementURI(subElements.workspace.itemId, subElements.component.itemId, path);
				anchor.href= (href);
//				anchor.title= versionable.name;
				var text= document.createTextNode(versionable.name);
	
				anchor.appendChild(text);
				nameTD.appendChild(img);
				nameTD.appendChild(anchor);
				
				tr.appendChild(nameTD);
				
				// size column
				var sizeTD = document.createElement("td");
				dojo.addClass(sizeTD, "folderContent_td");
				dojo.addClass(sizeTD, "fileSizeColumn");
				sizeTD.style.paddingTop = "5px";
				if (isFolder || isSymlink) {
					sizeTD.innerHTML = "&nbsp;";
				}
				else {
					var size = versionable.content.size;
					sizeTD.innerHTML = ScmUtil.getSizeDisplay(size);
				}
				tr.appendChild(sizeTD);
				
				// "Last Modified" time column
				var lastModTD = document.createElement("td");
				dojo.addClass(lastModTD, "folderContent_td");
				lastModTD.style.paddingTop = "5px";
				var value = null;
				if (!isFolder) {
					value = ScmUtil.getVersionablePropertyValue(versionable, "fileTimestamp");
					if (value) {
						var modTime = new Date();
						modTime.setTime(value);
						lastModTD.innerHTML = jazz.util.date.formatDif(modTime, true, true);						
					}
				}
				if(!value)
					lastModTD.innerHTML = "&nbsp;";
				tr.appendChild(lastModTD);
				
				// "Modified By" column
				var modByTD = document.createElement("td");
				modByTD.className = "folderContent_td";
				if(isFolder)
					modByTD.innerHTML = "&nbsp";
				else
					modByTD.appendChild(ScmUtil.createContributorElement(x.modifiedBy));
				tr.appendChild(modByTD);
				
				if (isStream) {
					// "Actions" column
					var actionsTD = document.createElement("TD");
					actionsTD.className = "folderContent_td";
					dojo.addClass(actionsTD, "twistie");
					tr.appendChild(actionsTD);
					
					actionsTD._itemId = versionable.itemId;					
					actionsTD.itemId = versionable.itemId;
					actionsTD.workspaceId = workspaceId;
					actionsTD.componentId = componentId;
					actionsTD.path = subElements.path == "/" ? slashName : subElements.path + slashName;
					actionsTD.isFolder = isFolder;
					actionsTD.isLocked = isLocked;
					
					if (!isFolder) {
						// lock and download action
						var lockDownloadImage = self._createHoverActionImage(self.messages.StreamWorkspaceView_68, "lockdownload_co");
						actionsTD.appendChild(lockDownloadImage);
						self.connect(lockDownloadImage, "onclick", function(e){
							dojo.stopEvent(e);
							if (!actionsTD.versionableDTO) {
								self.retrieveVersionable(actionsTD.workspaceId,
														 actionsTD.componentId,
														 actionsTD.path,
														 dojo.hitch(self, self._handleFileLockDownloadAction, actionsTD),
														 actionsTD);
							}
							else
								self._handleLockAndDownloadClick(actionsTD.versionableDTO, actionsTD);
						});
												
						// download action						
//						var downloadImage = self._createHoverActionImage(self.messages.StreamWorkspaceView_55, "download_co");
						var anchor = document.createElement("a");
						anchor.href = ScmUtil.getFileDownloadURI(workspaceId, componentId, versionable.name, versionable.itemId, null);
						var downloadImage = document.createElement("img");
						downloadImage.src = ScmUtil.getIconSrc("download_co");
						downloadImage.title = self.messages.StreamWorkspaceView_55;
						anchor.appendChild(downloadImage);
						anchor.style.visibility = "hidden";
						actionsTD.appendChild(anchor);
						
						// upload action
						var uploadImage = self._createHoverActionImage(self.messages.StreamWorkspaceView_60, "upload_co");
						uploadImage.mode = "SINGLE_FILE_UPLOAD";
						actionsTD.appendChild(uploadImage);
						self.connect(uploadImage, "onclick", function(e){
							dojo.stopEvent(e);
							if (!uploadImage._isExpanded) {
								self._showEditorTwistie(uploadImage, /* open */ true);
								self._openInlineEditor(actionsTD, uploadImage);
							}
						});
						
						// lock action
						var lockImage = self._createHoverActionImage(self.messages.StreamWorkspaceView_64, "locks", isLocked);						
						actionsTD.appendChild(lockImage);	
						actionsTD.lock = lockImage;	
						actionsTD.name = nameTD;							
						
						self.connect(lockImage, "onclick", function(e){
							dojo.stopEvent(e);
							if(actionsTD.isLocked)
								return;
							self._showEditorTwistie(lockImage, /* open */ false);
							if (!actionsTD.versionableDTO) {
								self.retrieveVersionable(actionsTD.workspaceId,
														 actionsTD.componentId,
														 actionsTD.path,
														 dojo.hitch(self, self._handleFileLockAction, actionsTD),
														 actionsTD);
							}
							else
								self._handleFileLockClick(actionsTD.versionableDTO, actionsTD);
						});
						
						// unlock action
						var unlockImage = self._createHoverActionImage(self.messages.StreamWorkspaceView_65, "unlock", !isLocked);
						actionsTD.appendChild(unlockImage);
						actionsTD.unlock = unlockImage;
						
						self.connect(unlockImage, "onclick", function(e){
							dojo.stopEvent(e);
							if(!actionsTD.isLocked)
								return;
							self._showEditorTwistie(unlockImage, /* open */ false);
							if (!actionsTD.versionableDTO) {
								self.retrieveVersionable(actionsTD.workspaceId,
														 actionsTD.componentId,
														 actionsTD.path,
														 dojo.hitch(self, self._handleFileUnlockAction, actionsTD),
														 actionsTD);
							}							
							else
								self._handleFileUnlockClick(actionsTD.versionableDTO, actionsTD);
						});
					}
					
					// delete action
					var deleteImage = self._createHoverActionImage(self.messages.StreamWorkspaceView_61, "delete_edit");
					deleteImage.mode = isFolder ? "SINGLE_FOLDER_DELETE" : "SINGLE_FILE_DELETE";
					actionsTD.appendChild(deleteImage);
					self.connect(deleteImage, "onclick", function(e){
						dojo.stopEvent(e);
						if (!deleteImage._isExpanded) {
							self._showEditorTwistie(deleteImage, /* open */ true);
							self._openInlineEditor(actionsTD, deleteImage);
						}
						else {
						//						self._showEditorTwistie(deleteImage, /* close */ false );
						//						var widget = this._inlineEditors["InlineFileFolderEditor-" + actionsTD._itemId];
						//						if(widget) {
						//							self.hidePreviewEditor(widget.parentParms);
						//						}
						}
					});
					
					
					self.connect(actionsTD.parentNode, "onmouseover", function(e){
						dojo.forEach(actionsTD.childNodes, function(image) {
							image.style.visibility = "";
						});
					});
					self.connect(actionsTD.parentNode, "onmouseout", function(e){
						dojo.forEach(actionsTD.childNodes, function(image) {
							image.style.visibility = "hidden";
						});
					});
				}
				
				self._tableBody.appendChild(tr);
			});	
		},
		/**
		 * Display extra information about the given resource
		 * Lock status / Image thumbnail for web images
		 * Info arguments: 
		 * 1. parentNode - html dom node that will be our parent via the parentNode.hover attribute
		 * 2. lockedByMe - true if locked by me, false otherwise
		 * 3. lockedByOtherName - name of the person who owns the lock, null if locked by me or not locked
		 *    (resource is not locked if lockedByMe and lockedByOther are undefined)
		 * 4. subElements - used to show image thumbnail
		 * 5. versionable - used to show image thumbnail
		 * 
		 * Hook a hover on the given parent node or does nothing if there is
		 * no information to display in the hover.
		 */
		_hookHover: function(info) {
			// Clear any existing hover
			if (info.parentNode.hover) {
				info.parentNode.hover.destroy();
				info.parentNode.hover = null;
			}
			// Check if a new one is needed
			var webImage = this._isWebImage(info.versionable);
			if (!webImage && !info.lockedByMe && !info.lockedByOtherName) {
				return;
			}
			var self = this;
			var hover = new jazz.ui.HoverPopup({ 
				around: [ info.parentNode ],
				_info: info,
				createContents: function () {
					var result = document.createElement("div");
					// Lock info
					if (info.lockedByMe) {
						result.appendChild(document.createTextNode(self.messages.LockedByMeLabel));
					} else if (info.lockedByOtherName) {
						result.appendChild(document.createTextNode(dojo.string.substitute(self.messages.LockedByLabel, [info.lockedByOtherName])));
					}
					var subElements = info.subElements;
					var versionable = info.versionable;
					// Image thumbnail
					if (webImage) {
						if (info.lockedByMe || info.lockedByOtherName) {
							result.appendChild(document.createElement("br"));
						}
						var src = ScmUtil.getFileDownloadURI(subElements.workspace.itemId, subElements.component.itemId, versionable.name, versionable.itemId, versionable.stateId);
						var img = document.createElement("img");
						img.className = "com-ibm-team-scm-FilesFoldersView-Hover";
						img.src = src;
						img.title = subElements.versionable.name;
						result.appendChild(img);
					}					
					return result;
				},
				getInfo: function () {
					return this._info;
				},
				orientation: "L"}); // tooltip to show locked by info.
			info.parentNode.hover = hover;
		},
		_addHeaderColumn: function(displayText, className) {
			// Add top white line trim
			var headerColumnLine = document.createElement("th");
			dojo.addClass(headerColumnLine, "headerColumnLine");
			dojo.addClass(headerColumnLine, className);
			headerColumnLine.innerHTML = "&nbsp;";
			this._headerRowLine.appendChild(headerColumnLine);

			var th= document.createElement("th");
			dojo.addClass(th, "headerColumnUnsorted");
			dojo.addClass(th, className);		
			th.appendChild(document.createTextNode(displayText));			
			this._headerRow.appendChild(th);
		},
		
		_createHoverActionImage: function(actionName, imageName, isDisabled) {
			var twistieImage = document.createElement("img");
			if(!isDisabled){				
				twistieImage.src = ScmUtil.getIconSrc(imageName);
//				dojo.addClass(twistieImage, "twistie");
			}
			else {
				twistieImage.src = ScmUtil.getIconSrc(imageName);
				dojo.removeClass(twistieImage, "twistie");
			}
			twistieImage.title = actionName;
			twistieImage.name = imageName;
			twistieImage.style.visibility = "hidden";
			twistieImage._isExpanded = false;
			return twistieImage;
		},
		
		_showEditorTwistie: function(twistieImage, isOpen) {
 				if(isOpen) {
//						twistieImage.src = ScmUtil.getIconSrc("delete_edit");
						twistieImage._isExpanded = true;
						twistieImage.style.visibility = "";
					} else {
//						twistieImage.src = ScmUtil.getIconSrc("delete_edit"); //need different icons for open and closed state.
						twistieImage._isExpanded = false;
						twistieImage.style.visibility = "hidden";
					}			
		},
		_openInlineEditor: function(col, actionImage) {
			var itemId = col._itemId;
			var row = col.parentNode;
			dojo.addClass(row, "double-row");
			
			var widget = this._inlineEditors[col._itemId + actionImage.mode];
			if(widget && widget.domNode) {
				widget.column.row.style.display="";
				return;					
			} 
			var newRow = row.parentNode.insertRow(row.rowIndex-1);
			var newCol = document.createElement("td");
			newCol.className = "folderContent_td";
			newCol.colSpan = row.childNodes.length;
			col.parentRow = row;
			col.row = newRow;
			col.action = actionImage;
			var self = this;
			var editor = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor(
				{
					mode : actionImage.mode,
					workspaceId: col.workspaceId,
					componentId: col.componentId,
					path: col.path,
					pageController: self.pageController,
					onResourceEditorSaved: dojo.hitch(this, this.reload),
					onResourceEditorClose: function () {
						self.hideInlineEditor(col);
					}
				}
			);
			editor.startup();
			
			this._inlineEditors[itemId + actionImage.mode] = editor; //cache		
			newCol.appendChild(editor.domNode);
			newRow.appendChild(newCol); 
		},
		
		hideInlineEditor: function(col) {
			col.row.style.display="none";
			dojo.removeClass(col.parentRow, "double-row");
			this._showEditorTwistie(col.action, false);
		},
		
		deleteItem: function(col) {
			var editorRow = col.row;
			if(editorRow && editorRow.parentNode)
				this._deleteTableRow(editorRow.parentNode, editorRow.rowIndex);
				
			var itemRow = col.parentRow;
			if(itemRow && itemRow.parentNode)
				this._deleteTableRow(itemRow.parentNode, itemRow.rowIndex);
				
			this._inlineEditors["InlineFileFolderEditor-" + col._itemId] = null;
		},
		
		_deleteTableRow: function(table, rowIndexValue) {
			var rows = table.rows;
			for(var i in rows) {
				if(rows[i].rowIndex == rowIndexValue) {
					table.deleteRow(i);
					return;
				}
			}
		},
		
		_hasTextContentType: function(versionable) {
			var contentType = ScmUtil.getVersionablePropertyValue(versionable, "contentType");
			return contentType && ScmUtil.isKnownTextContentType(contentType);
		},
		_isWebImage: function(versionable) {
			var ext = ScmUtil.getExtension(versionable.name).toLowerCase();
			return ext === "gif" || ext === "jpg" || ext === "jpeg" || ext == "png";
		},
		
		_handleFileLockClick: function(versionableDTO, col) {
			var d;
			PlatformUI.getWorkbench().showMessage(this.messages.StreamWorkspaceView_66);
			var self = this;
			var args = {
				operation : [ "acquire" ],
				streamId : versionableDTO.workspace.itemId,
				componentId : versionableDTO.component.itemId,
				versionableNamespace : [ "com.ibm.team.filesystem" ],
				versionableItemType : [ "FileItem" ],
				versionableItemId : [ versionableDTO.versionable.itemId ]
			};
			d = new dojo.Deferred();
			ScmClient.postUpdateLocks(new ServiceResponseHandler(
			{
				handler : function () {
					PlatformUI.getWorkbench().hideMessageArea();
					if (self.pageController) {
						self.pageController.onUpdateLocks(args);
					}
					// TODO: toggle lock and unlock action states
					if(col) {
						col.lock.src = ScmUtil.getIconSrc(col.lock.name);
						col.unlock.src = ScmUtil.getIconSrc(col.unlock.name);
						col.isLocked = true;
						self._updateFileLockInfo(col.name, true);
					}
					d.callback(true);
				},
				error : function (error) {
					if (error.message !== '') {
						alert(error.message);
					}
					PlatformUI.getWorkbench().hideMessageArea();
				}
			},
			'handler',
			'error'
			),
			args
			);
			return d;
		},
		_handleFileUnlockClick: function(versionableDTO, col) {
			PlatformUI.getWorkbench().showMessage(this.messages.StreamWorkspaceView_67);
			var self = this;
			var args = {
				operation : [ "release" ],
				streamId : [ versionableDTO.workspace.itemId ],
				componentId : [ versionableDTO.component.itemId ],
				versionableNamespace : [ "com.ibm.team.filesystem" ],
				versionableItemType : [ "FileItem" ],
				versionableItemId : [ versionableDTO.versionable.itemId ]
			};
			ScmClient.postUpdateLocks(new ServiceResponseHandler(
			{
				handler : function () {
					PlatformUI.getWorkbench().hideMessageArea();
					if (self.pageController) {
						self.pageController.onUpdateLocks(args);
					}					
					// TODO: toggle lock and unlock action states
					if(col) {
						col.lock.src = ScmUtil.getIconSrc(col.lock.name);
						col.unlock.src = ScmUtil.getIconSrc(col.unlock.name);
						col.isLocked = false;
						self._updateFileLockInfo(col.name, false);
					}
				},
				error : function (error) {
					if (error.message !== '') {
						alert(error.message);
					}
					PlatformUI.getWorkbench().hideMessageArea();
				}
			},
			'handler',
			'error'
			),
			args
			);			
		},
		/**
		 * args: an object with the following attributes
		 * 
		 * workspaceId, componentId, versionable (required)
		 */
        _handleLockAndDownloadClick: function(versionableDTO, col){
			this._handleFileLockClick(versionableDTO, col)
			.addCallback(dojo.hitch(this, function () {
				var href = ScmUtil.getFileDownloadURI(versionableDTO.workspace.itemId, versionableDTO.component.itemId, versionableDTO.versionable.name, versionableDTO.versionable.itemId, null);
				dojo.io.iframe.send({
					url : href,
					timeout : 2000 
				});
			}));
        },		
		/*
		 * Update the name column icon and hover info when lock/unlock by ME.
		 */
		_updateFileLockInfo: function(nameCol, isLocked){
			// Update name column icon
			nameCol.isLocked = isLocked;
			nameCol.lockedBy = null;
			var img = ScmUtil.getDecoratedResourceIcon({
				isSymbolicLink: nameCol.isSymlink,
				isFolder: nameCol.isFolder,
				fileName: nameCol.fileName,
				isLocked: isLocked
			});

			dojo.addClass(img, "icon");
			nameCol.removeChild(nameCol.icon);
			nameCol.icon = img;
			nameCol.insertBefore(img, nameCol.firstChild);
			
			// Update hover
			if (nameCol.hoverInfo) {
				nameCol.hoverInfo.lockedByMe = isLocked;
				nameCol.hoverInfo.lockedByOtherName = null;
				this._hookHover(nameCol.hoverInfo);
			}
		},
		
		_handleFileLockAction: function(col) {
			checkNotNull(col.versionableDTO, "versionableDTO");
			this._handleFileLockClick(col.versionableDTO, col);
		},
		
		_handleFileUnlockAction: function(col) {
			checkNotNull(col.versionableDTO, "versionableDTO");
			this._handleFileUnlockClick(col.versionableDTO, col);			
		},
		
		_handleFileLockDownloadAction: function(col) {
			checkNotNull(col.versionableDTO, "versionableDTO");
			this._handleLockAndDownloadClick(col.versionableDTO, col);
		},
		
		retrieveVersionable: function(workspaceId, componentId, path, successCallback, callbackParamObj) {
			checkNotNull(workspaceId, "workspaceId");
			checkNotNull(componentId, "componentId");
			checkNotNull(path, "path");
			
			var that = this;
			
			PlatformUI.getWorkbench().showMessage(this.messages.FetchingItem);
			var serviceObject = {
				success: function (versionablePlusDTO) {	
					PlatformUI.getWorkbench().hideMessageArea();
					
					if (versionablePlusDTO && versionablePlusDTO.versionableDTO) {
						var versionableDTO = versionablePlusDTO.versionableDTO;
						versionablePlusDTO.versionableDTO = null;
						versionableDTO.versionablePlusDTO = versionablePlusDTO;
						
						//cache this versionableDTO
						callbackParamObj.versionableDTO = versionableDTO;
						successCallback();
					}
				},
				error: function (error) { 
					PlatformUI.getWorkbench().hideMessageArea();
					if (that.pageController) {
						if (error.message !== "") {
							that.pageController._handleError(dojo.string.substitute(that.messages.ScmPageController_12, [ error.message, error.status ]));
						}
						else {
							that.pageController._handleError(that.messages.ScmPageController_13);
						}
					}
				}
			};							
						
			ScmClient.getVersionable(new ServiceResponseHandler(serviceObject, "success", "error"), 
									{workspace: ScmClient.WORKSPACE_PREFIX + workspaceId, component: ScmClient.COMPONENT_PREFIX + componentId}, 
									true, 
									path);
		},
		
		/**
		 * On the header row: number of lines, size, content type, line delimiter and encoding
		 * Show in tooltip: last modified by who at when
		 * @param {Object} numOfLines
		 */
		_updateFileContentHeaderRow: function(numOfLines) {
				
				// work item 96295, add number of lines info. in title. 
				var numOfLines = numOfLines > 1 ? dojo.string.substitute(this.messages.FileNumOfLines, [numOfLines]) : dojo.string.substitute(this.messages.FileNumOfLine, [numOfLines]);
				var span = document.createElement("span");
				span.innerHTML = numOfLines;
				dojo.addClass(span, "item-name");
				this._fileName.appendChild(span);
				
				var versionable = this.versionableDTO.versionable;
				// size
				if(versionable.content && versionable.content.size) {
					var span = document.createElement("span");
					span.innerHTML = ScmUtil.getSizeDisplay(versionable.content.size);
					dojo.addClass(span, "item-name");
					this._fileName.appendChild(span);
				}
				// content type
				var value = ScmUtil.getVersionablePropertyValue(versionable, "contentType");
				if (value) {
					var span = document.createElement("span");
					var ct = dojo.string.substitute(this.messages.FileFolderPropName_ContentType, [ value ]);
					span.appendChild(document.createTextNode(ct));
					dojo.addClass(span, "item-name");
					this._fileName.appendChild(span);
				}
				// line delimiter
				value = versionable.content.lineDelimiterSetting;
				if (value >= 0 && value <= 4) {
					if(value == ScmUtil.LINE_DELIMITER_NONE)
						value = this.messages.UploadItem_cbLdNoneBinary;
					else if(value == ScmUtil.LINE_DELIMITER_LF)
						value = this.messages.UploadItem_cbLdLFUnix;	
					else if(value == ScmUtil.LINE_DELIMITER_CR)
						value = this.messages.UploadItem_cbLdCRMac;
					else if(value == ScmUtil.LINE_DELIMITER_CRLF)
						value = this.messages.UploadItem_cbLdCRLFWindows;
					else if(value == ScmUtil.LINE_DELIMITER_PLATFORM)
						value = this.messages.UploadItem_cbPlatform;
						
					var span = document.createElement("span");
					span.innerHTML = dojo.string.substitute(this.messages.FileFolderPropName_LineDelim, [ value ]);
					dojo.addClass(span, "item-name");
					this._fileName.appendChild(span);
				}
				// encoding
				if(versionable.content && versionable.content.characterEncoding) {
					var span = document.createElement("span");
					var encoding = dojo.string.substitute(this.messages.FileFolderPropName_Encoding, [ versionable.content.characterEncoding ]);
					span.appendChild(document.createTextNode(encoding));
					dojo.addClass(span, "item-name");
					this._fileName.appendChild(span);
				}
				
				value = ScmUtil.getVersionablePropertyValue(versionable, "fileTimestamp");
				if (value) {
					var modTime = new Date();
					modTime.setTime(value);
					var time = jazz.util.date.formatDif(modTime, true, true);
					this._fileName.title = dojo.string.substitute(this.messages.FilesFoldersView_LastModifiedBy_Who_When, [this.versionableDTO.creator.name, time]);
				}				
		}
		
	}
);

})();
