/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ResolutionAttributePart");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");

(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ResolutionAttributePart",
	[com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget, com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart],
	{
        constructor: function() {
			this.attrMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
		},
		postCreate: function()  {
			if (this.isNew()) {
				this._possibleValues = [];
			}
			if(!this.attributeId) this.attributeId = "internalResolution";
			if(!this.attributeValue) this.attributeValue = this.workingCopy.getValue({
				path: "attributes.internalResolution"
			});
			this.workingCopy.addListener({
				path: ["attributes", "internalState"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyStateChange"
			});
			this.workingCopy.addListener({
				path: ["attributes", "workflowAction"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyActionChange"
			});
					
			this.workingCopy.addListener({
				path: ["attributes", "internalResolution"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyResolutionChange"
			});
			this.workingCopy.addListener({
				path: ["workItemSpec"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkItemSpecChange"
			});
				
			this._updateResolutions(this._createUpdateResolutionArgs());
			
			this.inherited(arguments);
			// handle the working copy changing			
			this._workItemChangedHandle = dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_CHANGED, dojo.hitch(this, this._handleWorkItemChanged));
			
			this._checkIfEnabled();
		}, 
		destroy: function() {
			dojo.unsubscribe(this._workItemChangedHandle);
			this.inherited(arguments);
		},
		getDefaultLabel: function() {
			return this.attrMessages.labelResolution;
		},
		_handleWorkItemChanged: function(workingCopy) {
			if(workingCopy !== this.workingCopy) return;
			this._handleWorkingCopyResolutionChange();
			this._checkIfEnabled();
		},
		_handleWorkingCopyStateChange: function(parms) {
			var newState= parms.attributeValue.id || parms.attributeValue;
			this._updateResolutions( { status: newState }, /* updateWorkingCopy */ true);
			this._checkIfEnabled();
		},
		_handleWorkingCopyActionChange: function(parms) {
			var newAction= parms.attributeValue.id || parms.attributeValue;
			this._updateResolutions( { actionId: newAction },  /* updateWorkingCopy */ true );
			this._checkIfEnabled();
		},
		// The resolution is only enabled if a workflow action has  been specified
		_checkIfEnabled: function() {
			var action = this.workingCopy.getValue( {
				path: ["attributes", "workflowAction"] } );
			if (action && this._possibleValues && this._possibleValues.length > 0) {
				this._isAlwaysDisabled = false;
				this._select.disabled = false;
			} else {
				this._isAlwaysDisabled = true;
				this._select.disabled = true;				
			}
		},
		_handleWorkingCopyResolutionChange: function() {
		   var valueId = this.workingCopy.getValue({
				path: "attributes.internalResolution.id"
			});
			this.setValue(valueId, /* noNotify */ true); 
		},
		_handleWorkItemSpecChange: function(parms) {
			this._updateResolutions(this._createUpdateResolutionArgs());
		},
		_isAddingValueToEmptyPossibleValues: function() {
			return false;
		},
		_getEmptyValueSet: function() {
			var values = [];
			var fakeItem = {
				iconUrl: false,
				id: "1",
				itemType: "smallString",
				label: this._getEmptyValueLabel()
			}
			values.push(fakeItem);
			return values;
		},
		_getEmptyValueLabel: function() {
			return this.attrMessages.optionNotResolved
		},
		_createUpdateResolutionArgs: function() {
				var actionId= this.workingCopy.getValue({path: "attributes.workflowAction.id"});
				var statusId= this.workingCopy.getValue({path: "attributes.internalState.id"});
				
				if (actionId) {
					return { actionId: actionId };
				}
				return { status: statusId };
		},
		/*
		 * Update the set of values bound to the 'Resolution' control, based on
		 * the work item bound to the editor and the current value of the
		 * 'State' control.
		 */
		_updateResolutions: function(attributeArgs, updateWorkingCopy) {
			if(this.isNew()) {
				this._bindValuesToSelect([], this._select, this._image);
				return; // no values
			} 
			
			if(!(attributeArgs.actionId || attributeArgs.status)) return; // not initialized yet
			var self= this;
			var serviceObj= {
				success: function(values) {
					values = values || [];
					self._possibleValues = values;
					if (values.length == 0) {
						self._isAlwaysDisabled = true;
					}
					else {
						self._checkIfEnabled();
					}
					
					/*
					 * If the field is disabled and there is no "current" resolution value
					 * then show an empty option list
					 */
					if(self._isAlwaysDisabled && self.workingCopy.getValue({path: ["attributes" , self.attributeId]}) == null) {
						values = []; // show no possible values
					}
					
					self._bindValuesToSelect(values, self._select, self._image);
					if(self._lastValue) {
						self.setValue(self._lastValue, true, false);
					}
				if(updateWorkingCopy) {
					var currentValue = self._getCurrentSelectProps(self._select);
					if(currentValue) {
						self.setAttributeValue(currentValue, /* set option */ false);
					}		
				}

//					self.updateValidationMessage(/* wasResolutionChanged */ !self._isAutomaticallySettingResolutionToDuplicate);
				},
				failure: function(errorObj) {
					console.debug(errorObj.message);
//					self.parentController.handleError(errorObj, "A problem occurred while trying to fetch resolutions.");
				}
			};
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
			var args =  {itemId: this.workItem.itemId, projectAreaItemId: this.workingCopy.getProjectAreaItemId()};
			if (attributeArgs.actionId) {
				args.actionId= attributeArgs.actionId;
			} else if (attributeArgs.status) {
				args.status= attributeArgs.status;
			}
			if(this._priorArgs) {
				var isEqual = true;
				for(var key in this._priorArgs) {
					if(this._priorArgs[key] != args[key]) {
						isEqual=false;
						break;
					}
				}
				if(isEqual) {
					for(var key in args) {
						if(this._priorArgs[key] != args[key]) {
							isEqual=false;
							break;
						}
					}		
				}
				if(isEqual) return; // nothing to do;
			}
			this._priorArgs = dojo.clone(args);
			WorkItemClient.getResolutions(srh, args);
		}
	}
);

})();
