/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.HoverPopup");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function(){
	
var STYLE_VALID= "validation-ok"
var STYLE_INVALID= "validation-failed"

var HoverPopup= jazz.ui.HoverPopup;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/layout/templates/TextWidget.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		this.presentation= args.presentation;
		this.selection= args.selection;
		
		var converters= new Object();
		converters["integer"]= function(value) {
			if(!isNaN(value)) 
  				return parseInt(test) == value;
			return false
		};
		
		converters["long"]= converters["integer"];
		
		converters["float"]= function(value) {
			if(!isNaN(value)) 
  				return parseFloat(test) == value;
			return false
		};
		
		if(args.converter) {
			this._hasCustomConverter = true;
			this._converter = args.converter;
			this._converterParms = args.converterParms;
		} else {
			this._converter= converters[this.presentation.attribute.attributeTypeId];
		}
	},
	
	postCreate: function() {
//		<input class="text-control" type="text" dojoattachpoint="_textInput" dojoattachevent="onchange: valueChanged, onkeyup:doValidate"></input>
		this._textInput = document.createElement("INPUT");
		this._textInput.type = this.isPassword === true ? "PASSWORD" : "TEXT";
		dojo.addClass(this._textInput, STYLE_VALID);
		dojo.addClass(this._textInput, "text-control");
		this.connect(this._textInput, "onkeyup", this.doValidate);
		this.connect(this._textInput, "onchange", this.valueChanged);
		
		this._textDiv.appendChild(this._textInput);
		
		if (this._converter) {
			
			var tooltipMessage= this.messages.titleEnterNumber;
			if(this._hasCustomConverter)
				tooltipMessage = this._converterParms.toolTip;
				
			this._tooltip = new HoverPopup({
				around: [this._textInput],
				html: tooltipMessage,
				orientation: "L",
				programmatic: true
			});
		}
		
		if (this.selection)
			this._setValues(this.selection);
	},
	destroy: function() {
		if(this._tooltip) {
			this._tooltip.destroy();
		}
		this.inherited(arguments);
	},
	/* String[] */ getValues: function() {
		var selection= new Array();
		var rawValues= this._getRawValues();
		for (var i= 0; i < rawValues.length; i++) {
			var value= this._convert(rawValues[i]);
			if (value)
				selection.push(value);
		}
		if(rawValues.length == 0){
			this._convert("");
		}
		return selection;
	},
	valueChanged: function(e) {
		if(this.parentWidget && this.onValueChanged) {
			this.parentWidget[this.onValueChanged](e);
		}
	},
	doValidate: function(e) {
		this.valueChanged(e);
		var inputNode= this._textInput;
		if (this._validateInput()) {
			dojo.disconnect(this._onBlur_handle);
			dojo.removeClass(inputNode, STYLE_INVALID);
			dojo.addClass(inputNode, STYLE_VALID);
			if (this._tooltip)
				this._tooltip.close();
		} else {
			dojo.removeClass(inputNode, STYLE_VALID);
			dojo.addClass(inputNode, STYLE_INVALID);
			if (this._tooltip) {
				if(this._hasCustomConverter) {
					var _self = this;
					
					this.connect(inputNode, "onblur", function(e) {
							_self._validateOnBlur(e);
						})				
				}
				if (!this._tooltip._isOpen)
					this._tooltip.open();
			}
		}
		if(this.valueChangedListener && this.valueChangedFunction && dojo.isFunction(this.valueChangedListener[this.valueChangedFunction])) {
			this.valueChangedListener[this.valueChangedFunction]( this._textInput.value);
		}
	},
	
	_validateOnBlur: function(e) {
		dojo.disconnect(this._onBlur_handle);
		dojo.removeClass(this._textInput, STYLE_INVALID);
		dojo.addClass(this._textInput, STYLE_VALID);
		this._tooltip.close(); 
		this._textInput.value = this._converter(this._textInput.value,  this._converterParms,  /* get valid */ true);

	},
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_convert: function(value) {
		if (value.length > 0 && this._converter) {
			return this._converter(value, this._converterParms);
		} else {
			if(this._hasCustomConverter) {
				return this._converter("", this._converterParms);
			}
		}
		return value;
	},
	
	_validateInput: function() {
		if (this._converter) {
			var values= this._getRawValues();
			for (var i= 0; i < values.length; i++) {
				var converted= this._convert(values[i])
				if(converted && converted.isCustom) {
					if(!converted.isValid) {
						return false;
					}
				} else {
					if (isNaN(converted) || (values[i] != converted))
						return false;			
				}
			}
		}
		return true;	
	},
	
	/* String[] */ _getRawValues: function() {
		var selection= new Array();
		var stringInput= this._textInput.value;
		var parts= stringInput.split(",");
		for (var i= 0; i < parts.length; i++) {
			var trimmed= dojo.trim(parts[i]);
			selection.push(trimmed);
		}
		return selection;
	},
	
	_setValues: function(/* UIItemDTO[] */ selection) {
		var list= new Array();
		for (var i= 0; i < selection.length; i++) {
			list.push(selection[i].id);
		}
		this._textInput.value= list.join(", ");
	}
});
})();
