/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.TimeElementViewerItem"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItem"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.ImageClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemMenuAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.MoveIntoGroupAction"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var doc= document;
var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var ViewModelViewerItem				= com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItem;
var ImageClient						= com.ibm.team.apt.web.client.internal.ImageClient;

var MoveIntoGroupAction= com.ibm.team.apt.web.ui.internal.view.plan.actions.MoveIntoGroupAction;
var NewWorkItemMenuAction= com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemMenuAction;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.TimeElementViewerItem", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItem], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	__groupElement: null,
	__openCount: 0,
	__closedCount: 0,
	
	__groupContent: null,

	constructor: function(entry, parentItem, plannedItemsTab, viewer) {
		this.__groupElement= entry.getElement();
	},

	// ---- ViewModelViewerItem implementation -----------------------------------------------------------------------------

	_getActions: function() {
		if (!this.__actions) {
			this.__actions= this._createActions([ MoveIntoGroupAction, NewWorkItemMenuAction ]);
		}
		return this.__actions;
	},

	_createContainer: function() {
		var result= doc.createElement("tr"); //$NON-NLS-1$
		dojo.addClass(result, "group"); //$NON-NLS-1$
		return result;
	},

	_createContent: function() {
		var localDoc= doc;

		this.__calculateOpenCloseCount();

		var actionCell=localDoc.createElement("td"); //$NON-NLS-1$
		dojo.addClass(actionCell, "timeGroupTd"); //$NON-NLS-1$
		dojo.addClass(actionCell, "actionBarCell"); //$NON-NLS-1$
		this.domNode.appendChild(actionCell);
		
		var actionContent=localDoc.createElement("div"); //$NON-NLS-1$
		dojo.addClass(actionContent, "content"); //$NON-NLS-1$
		actionCell.appendChild(actionContent);
		
		var actionBar= this._createActionBar();
		actionContent.appendChild(actionBar);

		var cell= localDoc.createElement("td"); //$NON-NLS-1$
		cell.colSpan= 100;
		dojo.addClass(cell, "timeGroupTd"); //$NON-NLS-1$

		this.__groupContent= localDoc.createElement("div"); //$NON-NLS-1$
		dojo.addClass(this.__groupContent, "content"); //$NON-NLS-1$
		dojo.addClass(this.__groupContent, "label"); //$NON-NLS-1$


		var groupTextContainer= localDoc.createElement("span"); //$NON-NLS-1$
		dojo.addClass(groupTextContainer, "labelContainer"); //$NON-NLS-1$

		var titleElement= localDoc.createElement("span"); //$NON-NLS-1$
		dojo.addClass(titleElement, "title"); //$NON-NLS-1$
		titleElement.appendChild(localDoc.createTextNode(this.__groupElement.getLabel()));
		groupTextContainer.appendChild(titleElement);

		var countElement= localDoc.createElement("span"); //$NON-NLS-1$
		dojo.addClass(countElement, "counts"); //$NON-NLS-1$
		var countText= null;
		if (this.__openCount > 0) {
			countText= bind(MSGS.TIMEWIDGET_COUNT_OPEN, [this.__openCount]);
		} else {
			countText= bind(MSGS.TIMEWIDGET_COUNT_CLOSED, [this.__closedCount]);
		}
		countElement.appendChild(localDoc.createTextNode(countText));
		groupTextContainer.appendChild(countElement);
		
		this.__groupContent.appendChild(groupTextContainer);
		cell.appendChild(this.__groupContent);

		this.domNode.appendChild(cell);

		this._applyTreeLayout(this.domNode);		
	},


	_getTreeLayoutElement: function() {
		return this.__groupContent;
	},

	_getTreeLevel: function() {
		return this._parentItem._getTreeLevel();
	},

	_getChildTreeLevel: function(type) {
		return this._parentItem._getChildTreeLevel(type);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__calculateOpenCloseCount: function() {
		var openCount= 0;
		var closedCount= 0;
		this._entry.getModel().readModel(dojo.hitch(this, function(readAccessor) {
			readAccessor.accept(function(entry){
				if (entry.getElement() instanceof com.ibm.team.apt.client.PlanItem) {
					if (entry.getElement().isResolved()) {
						closedCount++;
					} else {
						openCount++;
					}
				}
				return true;
			}, this._entry);
		}));
		
		this.__openCount= openCount;
		this.__closedCount= closedCount;
	},

	__sentinel: null // terminates this class definition
});

})();
