/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.query.QueryResultPageControl");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSectionContainer");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
dojo.require("com.ibm.team.workitem.web.cache.internal.OperationLogConsole");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.ToolbarWidget");

(function() {

var checkNotNull 		= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative 	= net.jazz.ajax.toolbox.lang.checkNonNegative;
var checkNotEmpty 		= net.jazz.ajax.toolbox.lang.checkNotEmpty;

var ImageClient			= com.ibm.team.workitem.web.client.internal.ImageClient;
var WorkItemUtil 		= com.ibm.team.workitem.web.client.util;
var BULK_OPERATIONS_WIDGET_ID = "queryresult/builoperationswidget";
/*
 * Constants
 */
var PAGE_SIZE_INTERVALS = [10, 25, 50, 75, 100, 250, 500];

var BUTTON_CONTEXT = "com.ibm.team.workitem.web";

/**
 * A meta-area widget that allows the user to manipulate query results by
 * navigating between query result pages and by performing textual filters on a
 * query page.
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.query.QueryResultPageControl",
	[dijit._Widget, dijit._Templated, com.ibm.team.workitem.web.cache.internal.OperationLogConsole],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/query/templates/QueryResultPageControl.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/query/templates/Query.css"),
		
		// populate the template with data, connect events, and show initial
		// sub-tab
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		}, 
		postCreate: function() {
			checkNotNull(this.query, "this.query");
			checkNotNull(this.queryResultsSummaryPage, "this.queryResultsSummaryPage");
			checkNotNull(this.queryResultsWidget, "this.queryResultsWidget");
			checkNotNull(this.parentView, "this.parentView");
			
			if(this.isBottomControl) {
				this.domNode.className = "queryResultPageControlBottom";
				this.topRow.style.display = "none";
			}

			this._bulkOperationResultDiv = document.createElement("div");
			var noResultDiv = document.createElement("DIV");
			noResultDiv.style.color = "#CCC";
			noResultDiv.appendChild(document.createTextNode(this.messages.labelNoResults));
			this._bulkOperationResultDiv.appendChild(noResultDiv);

			this._stopOperation();
			var links= this.titleContainer.getElementsByTagName("a");
			var name= this.query.name ? this.query.name : this.messages.titleNewQuery;
			name = WorkItemUtil.plainToHTML(name)
			var shortName = (name.length> 55) ? name.substring(0,52) + "..." : name;
			this.titleContainer.innerHTML = shortName;
			this.titleContainer.title =name;
			
			if (this._getResultCount() > 0) {
				this._showPagingControls();
			}
			this._toolbar = new com.ibm.team.workitem.web.ui.internal.view.layout.ToolbarWidget({});
			this._simpleToolbarContainer.appendChild(this._toolbar.domNode);
			var self = this;			
			var editQueryButton = this._toolbar.addTwoStateButton(this,
				function(event) {
					if(!editQueryButton.isEnabled) return;
					window.location.assign(WorkItemUtil.getEditQueryUri(self.query.itemId));
				},
				 {isEnabled: (this.query.itemId && !this.query.isNew && this.query.isEditable && window.location.href.indexOf("action=newQuery")== -1), 
				 	imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, "ui/internal/view/layout/images/edit-current-query.gif"), 
				 	label: this.messages.titleEditCurrentQuery,
				 	alt:this.messages.buttonEdit });

			this._editMultipleButton = this._toolbar.addTwoStateButton(this,
				function(event) {
					dojo.stopEvent(event);
					if(!self._editMultipleButton.isEnabled) return;
					self._handleTableEditClick()
				},
				 {isEnabled: false, // eanbled below 
				 	imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, "ui/internal/view/layout/images/edit-multiple-wi.gif"), 
				 	label: this.messages.titleShowHideBulkOperations,
				 	alt: this.messages.buttonEditTable });

			this._setTableEditImageState();

			this._exportButton = this._toolbar.addTwoStateButton(this,
				function(event) {
					dojo.stopEvent(event);
					if(!self._exportButton.isEnabled) return;
					window.open(self.query.csvExportLink, "_blank"); // Open in a new window
				},
				 {isEnabled: (this.query.csvExportLink != null), 
				 	imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, "ui/internal/view/query/images/q-result-export-table.gif"), 
				 	label: this.messages.titleExportCSV,
				 	alt: this.messages.titleExportCSV });
						

			// REFRESH
			if(this.queryUrl) {
				this.refreshUrl = WorkItemUtil.reuniquifyUrl(this.queryUrl);
			} else {
				this.refreshUrl = WorkItemUtil.reuniquifyUrl(WorkItemUtil.getQueryUri(this.query), true);
			}
			
	
			this._refreshButton = this._toolbar.addTwoStateButton(this,
				function(event) {
					dojo.stopEvent(event);
					if(!self._refreshButton.isEnabled) return;
					self._handleRefreshClick()
				},
				 {isEnabled: true, 
				 	imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, "ui/internal/view/editor/images/refresh.gif"), 
				 	label: this.messages.titleReRunQuery,
				 	alt: this.messages.titleReRunQuery });
				
			this._toolbar.addSeparator();			

			this._printButton = this._toolbar.addTwoStateButton(this,
				function(event) {
					dojo.stopEvent(event);
					if(!self._printButton.isEnabled) return;
					window.open(this.query.htmlExportLink, "_blank");
				},
				 {isEnabled: true, 
				 	imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, "ui/internal/view/query/images/q-result-print.gif"), 
				 	label:  this.messages.titleViewFullReport,
				 	alt:  this.messages.titleViewFullReport });	
			
			this._resetFilter();
			
			this._bulkOperationsMomento = com.ibm.team.workitem.web.cache.internal.Cache.getItem(BULK_OPERATIONS_WIDGET_ID);
			if( !this.isBottomControl && this._bulkOperationsMomento) {
				this._operationSection = this._bulkOperationsMomento.operationSection;
				this.setTableEditMode(this._bulkOperationsMomento.isShown);
			}
		},
		destroy: function() {

			this._stopOperation();

			if(this._toolbar) this._toolbar.destroy();
			this.inherited(arguments);
		},
		_handleTableEditClick: function() {
			this.logClearMessages();
			this.setTableEditMode(!this.query._isTableEditModeSelected);
		},
		refresh: function() {
			this._handleRefreshClick();
		},
		_handleRefreshClick: function(e) {
			this.getQueryResultsWidget().refreshLastQuery(this.query);
		},
		setRunButtonEnabled: function() {
				if(this._bulkOperationsMomento && this._bulkOperationsMomento.operationSection) {
					this._bulkOperationsMomento.operationSection.setRunButtonEnabled();	
				}
		},
		showBulkOperations: function(isVisible){
			if(!this._bulkContainer && !isVisible) return;
			if(isVisible) {
				this.getQueryResultsWidget()._hideAllEditors();				
			}

			if (!this._bulkOperationsMomento) {
				this._bulkOperationsMomento = com.ibm.team.workitem.web.cache.internal.Cache.getItem(BULK_OPERATIONS_WIDGET_ID);
			} 
			if(this._bulkOperationsMomento) {
				    if(this._bulkOperationsMomento.operationSection) {
						 this._bulkOperationsMomento.operationSection.queryResultPageControl = this;
					}
					this._attributeTree = this._bulkOperationsMomento.attributeTree;
					this._bulkContainer = this._bulkOperationsMomento.container;
					this._bulkOperationsMomento.isShown = isVisible;
					this.setRunButtonEnabled();
			}
			this.getQueryResultsWidget().showCheckboxes(isVisible, this._attributeTree);
			
			if (this._bulkContainer && this._bulkOperationEditorDiv ) {
				var displayValue = isVisible ? "" : "none";
				this.bulkOperationRow.style.display = displayValue;
				return;
			}
			if(this._hasShownBulkOperations ) return; // clicked while loading
			this._hasShownBulkOperations = true; // avoid double clickng on the icon while loading
						
			this._bulkOperationEditorDiv = document.createElement("div");
			var loadingDiv = document.createElement("DIV");
			loadingDiv.style.color = "#CCC";
			loadingDiv.appendChild(document.createTextNode(this.messages.busyLoading));
			this._bulkOperationEditorDiv.appendChild(loadingDiv);
			dojox.data.dom.replaceChildren(this._operationViewletArea,this._bulkOperationEditorDiv);

			this.bulkOperationRow.style.display = "";
 			if (!this.isBottomControl) {
				if (!this._bulkContainer) {
					var srh = new com.ibm.team.repository.web.transport.ServiceResponseHandler(this, "_setAttributeTree", "_handleAttributeError");
					com.ibm.team.workitem.web.client.internal.QueryClient.getAttributeTree(srh);
				}
				else {
					dojox.data.dom.replaceChildren(this._bulkOperationEditorDiv, this._bulkContainer.domNode);
				}
			}
		},
		/*
		 * OperationLog functions
		 */
		logSetVisible: function(isVisible) {
			this.inherited(arguments);
		},
		logClearMessages: function() {
			dojo.removeClass(this._errorDiv, "errorMessageArea");
			dojo.removeClass(this._errorDiv, "errorMessageAreaWarning");
			dojo.removeClass(this._errorDiv, "errorMessageAreaInfo");
			dojo.addClass(this._errorDiv, "errorMessageArea-disabled");
			dojox.data.dom.replaceChildren(this._errorMessageAnchor, document.createTextNode(" "));
			dojox.data.dom.removeChildren(this._bulkOperationResultDiv);
			this.inherited(arguments);
		},
		logAppendMessageElement: function(messageNode) { 
			this._bulkOperationResultDiv.appendChild(messageNode);
			this.inherited(arguments);
		},
		logStop: function(showTerminatingMessage) {
			this._stopOperation(showTerminatingMessage);
		},
		logSetStopButtonEnabled: function(isEnabled) {
			if(this._bulkContainer) 	this._bulkContainer.setStopButtonEnabled(isEnabled);
		},
		logMessage: function(parms /* message */) {
			console.debug(parms.message);
			this.inherited(arguments);
		},
		logTerminatingMessage: function(parms /* message, details */) {
			this.logStop(/* show Terminating message */ false);
			
			this._currentTerminatingMessage = parms;

			if (parms.severity == 0) {
				    var msg = parms.message;
					dojo.removeClass(this._errorDiv, "errorMessageArea");
					dojo.removeClass(this._errorDiv, "errorMessageAreaWarning");
					dojo.addClass(this._errorDiv, "errorMessageAreaInfo");
					dojo.removeClass(this._errorDiv, "errorMessageArea-disabled")			
			}
			else {
				if(parms.details) {
					var msg = dojo.string.substitute(this.messages.errorTerminatingStep, [parms.message || "", parms.details ]);									
				} else {
					var msg = dojo.string.substitute(this.messages.errorTerminatingStepWithoutDetails, [parms.message || "" ]);				
				}
 				console.debug(msg);
				if (parms.severity == 4) {
					dojo.removeClass(this._errorDiv, "errorMessageArea");
					dojo.removeClass(this._errorDiv, "errorMessageAreaInfo");
					dojo.addClass(this._errorDiv, "errorMessageAreaWarning");
					dojo.removeClass(this._errorDiv, "errorMessageArea-disabled")
				}
				else {
					dojo.addClass(this._errorDiv, "errorMessageArea");
					dojo.removeClass(this._errorDiv, "errorMessageAreaInfo");
					dojo.removeClass(this._errorDiv, "errorMessageAreaWarning");
					dojo.removeClass(this._errorDiv, "errorMessageArea-disabled")
				}
			}
			dojox.data.dom.replaceChildren(this._errorMessageAnchor, document.createTextNode(msg));			
		},
		/*
		 * Busy messages
		 */
		_stopOperation: function(showTerminatingMessage) {
			dojo.publish(com.ibm.team.workitem.web.ui.internal.page.EventTopics.STOP_BACKGROUND_OPERATIONS, [ showTerminatingMessage]);
		},
		_handleAttributeError: function(error) {
			alert(error.message);
		},
		addBulkOperationAttribute: function( attributeId) {
			if(this._bulkContainer) {
				this._bulkContainer.sectionWidget.addBulkOperationAttribute(attributeId);
			}
		},
		_setAttributeTree: function(tree) {
			this._attributeTree= tree;
			this._createBulkOperationPart(this._bulkOperationEditorDiv);
		},
		_createBulkOperationPart: function(node){
			if (this._bulkContainer) {
//95375			com.ibm.team.workitem.web.cache.internal.Cache.decrementUseCount(BULK_OPERATIONS_WIDGET_ID);
// decrement use count will do a destroy this._bulkContainer.destroy();
				this._bulkContainer = null;
			}
			
			var opSection = new com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSection({
				stopButton: this.stopButton,
				queryResultPageControl: this,
				attributes: this._attributeTree,
				term: this.query.expression
			});
			var container = new com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSectionContainer({
				sectionWidget: opSection
			});
			this._operationSection = opSection;
			dojox.data.dom.replaceChildren(node, container.domNode);
			
			this._bulkContainer = container;
			this._bulkOperationsMomento = {
				operationSection: this._operationSection,
				attributeTree: this._attributeTree,
				container: container,
				isShown: true,
				destroy: function() {
					this.container ? this.container.destroy() : null;
				}
			} 
			com.ibm.team.workitem.web.cache.internal.Cache.addItem(BULK_OPERATIONS_WIDGET_ID, this._bulkOperationsMomento);
			com.ibm.team.workitem.web.cache.internal.Cache.incrementUseCount(BULK_OPERATIONS_WIDGET_ID);

		},

		setTableEditMode: function(isTableEditModeSelected ) {
			this.query._isTableEditModeSelected = isTableEditModeSelected == true;
			/*
			 * Save all open work items before going into edit mode
			 */
			if(this.query._isTableEditModeSelected) {
				 var operationManager = new com.ibm.team.workitem.web.cache.internal.MultipleWorkItemOperation({
					log: this
				});
				operationManager.saveOpenWorkItems({
					ignoreIsExecuting: true,
					onSaveAllSuccess: dojo.hitch(this, this.saveAllWorkItemsSuccess),
					onSaveAllError: dojo.hitch(this,this.saveAllWorkItemsError)
					});
			} else {
				this._setTableEditImageState();
				this.showBulkOperations(this.query._isTableEditModeSelected);
			}
		},
		saveAllWorkItemsError: function() {
			this.setTableEditMode(false);
		},
		saveAllWorkItemsSuccess: function() {
			   
			this._setTableEditImageState();	
		
			// Show bulk operations window
			this.showBulkOperations(this.query._isTableEditModeSelected);	
		
		},
		_setTableEditImageState:	function() {		
			// TODO: Disable when no results
			this._editMultipleButton.setEnabled(true);
		},
		_createLink: function(url, style, tooltip) {
			var link= document.createElement("a");
			link.title= tooltip;
			dojo.addClass(link, style);
			if (url) {
				link.href= url;
			} else {
				dojo.addClass(link, "disabledIcon");
			}
			return link;
		},
		handleRowSelectionChanged: function() {
			this._operationSection.handleRowSelectionChanged();
		},
		_createSeparator: function() {
			var div= document.createElement("div");
			dojo.addClass(div, "separator");
			return div;
		},

		//
		// event handlers related to the filter control
		//
		getQueryResultsWidget: function() {
			return this.parentView._getQueryResultsWidget();
		}, 
		_onSetFilterValue: function(e) {
				this.getQueryResultsWidget().filter(this.filterControl.value);
		},
		
		_onFilterFocus: function(e) {
			if (this.filterControl.__jazzIsReset) {
				this.filterControl.value= "";
				dojo.removeClass(this.filterControlWrapper,"disabledText");
			}
			this.filterControl.__jazzIsReset= false;
		},
		
		_onFilterBlur: function(e) {
			var filterVal= dojo.trim(this.filterControl.value);
			if (filterVal.length === 0) {
				this._resetFilter();
			} else {
				this.filterControl.value= filterVal;
			}
		},
		setFilter: function(filterVal) {
				this.filterControl.value= filterVal;
				dojo.removeClass(this.filterControlWrapper,"disabledText");
				this.filterControl.__jazzIsReset= false;
				if(filterVal == "") {
					this._resetFilter();
				}
		},		
		_onClearFilterClick: function(e) {
			e.preventDefault();
			this._resetFilter();
		},
			
		_resetFilterControl: function() {
			if(this.filterControl.value =="") {
				this.getQueryResultsWidget().filter("");
				dojo.addClass(this.filterControlWrapper,"disabledText");
				this.filterControl.value= this.messages.labelTypeFilterText;
				this.filterControl.__jazzIsReset= true;
			}
		},	
		_resetFilter: function() {
			// unfilter results
			this.getQueryResultsWidget().filter("");
			dojo.addClass(this.filterControlWrapper,"disabledText");
			this.filterControl.value= this.messages.labelTypeFilterText;
			this.filterControl.__jazzIsReset= true;
		},
		
		//
		// logic for displaying the paging controls
		//
		
		_showPagingControls: function() {
			
			// show items per page drop-down
			if (!this.isBottomControl) {
				this.itemsPerPageDiv.appendChild(this._getItemsPerPageElement());
			}
			
			// show previous page text or link
			this.pageNumberDiv.appendChild(this._getPreviousElement());
			
			// show count info
			var pageInfoSpan= document.createElement("span");
			
			pageInfoSpan.className= "pageInfoSpan";
			var endIndex = Math.min(this._getMaxPageIndex(), this._getResultCount());
			var actual = this._getActualCountInfo();
			var resultCount = this._getResultCount() + "";
			if(endIndex == 1) {
				if(actual) {
					pageInfoSpan.innerHTML = dojo.string.substitute(this.messages.labelPage1OfXWithActual, 
						[ resultCount, (actual + "") ]);
				} else {
					pageInfoSpan.innerHTML = dojo.string.substitute(this.messages.labelPage1OfX, 
						[ resultCount ]);
				}
			} else {
				var startIndex = this._getStartIndex() + "";
				var pageIndex = Math.min(this._getMaxPageIndex(), resultCount)  + "";
				if(actual) {
					pageInfoSpan.innerHTML = dojo.string.substitute(this.messages.labelPageXToYOfZWithActual, 
						[ startIndex, pageIndex, resultCount, (actual+"") ]);
				} else {
					pageInfoSpan.innerHTML = dojo.string.substitute(this.messages.labelPageXToYOfZ, 
						[ startIndex, pageIndex, resultCount ]);
				}
			}
			this.pageNumberDiv.appendChild(pageInfoSpan);
			
			// show next page link or text
			this.pageNumberDiv.appendChild(this._getNextElement());
			
			// filter only appears at top
			if (this.isBottomControl) {
				this.filterControlWrapper.style.display = "none";
			}
		},
		
		_getActualCountInfo: function() {
			var count = this._getResultCount();
			var actual = this._getTotal();
			if (count < actual) {
				return actual;
			}
			return null;
		},
		
		/*
		 * Creates and returns the dom node of the page size drop down with
		 * label div.
		 */
		_getItemsPerPageElement: function() {
			var div = document.createElement("div");
			div.className = "itemsPerPage";
			
			var select = document.createElement("select");
			var count = this._getResultCount();
			
			// populate the drop-down
			var optionIndex = 0;
			var defaultSize = WorkItemUtil._getDefaultQueryPageSize();
			var currentSize = this.queryItemsPerPage;
			if (!currentSize) {
				currentSize = count > defaultSize ? defaultSize : "all";
			}
			for (var i=0;i<PAGE_SIZE_INTERVALS.length;i++) {
				var pageSize = PAGE_SIZE_INTERVALS[i];
				if (count > pageSize) {
					var option = new Option("" + pageSize, "" + pageSize);
					select.options[optionIndex++] = option;
					option.selected = (pageSize == currentSize);
				}
				else break;
			}
			// show all not supported for large queries
			if (count <= PAGE_SIZE_INTERVALS[PAGE_SIZE_INTERVALS.length - 1]) {
				var option = new Option(this.messages.labelShowAll, "all");
				select.options[optionIndex] = option;
				option.selected = (currentSize == "all");
			}
			
			// disable drop-down if only one choice
			if (count <= PAGE_SIZE_INTERVALS[0]) {
				select.disabled = true;
				dojo.addClass(select, "disabled");
			}
			
			// register event handler
			select.onchange = dojo.hitch(this, function() {
				var itemsPerPage = select.options[select.selectedIndex].value;
				location.href = WorkItemUtil.getQueryUri(this.query, 1, this.queryOrderBy, this.queryOrderDirection, itemsPerPage, true);
			});
			
			div.appendChild(select);
			var labelSpan = document.createElement("span");
			labelSpan.className = "itemsPerPageText";
			labelSpan.innerHTML = this.messages.labelItemsPerPage;
			div.appendChild(labelSpan);
			return div;
		},
		
		_getPreviousElement: function() {
			return (this._getStartIndex() > 1) ?
				this._getLinkElement(WorkItemUtil.getQueryUri(this.query, Math.max(this._getStartIndex() - this.getPageSize()), this.queryOrderBy, this.queryOrderDirection, this.queryItemsPerPage, true), this.messages.labelPrevious) :
				this._getDisabledTextElement(this.messages.labelPrevious);
		},
		
		_getNextElement: function() {
			return (this._getMaxPageIndex() < this._getResultCount()) ?
				this._getLinkElement(WorkItemUtil.getQueryUri(this.query, this._getMaxPageIndex() + 1,  this.queryOrderBy, this.queryOrderDirection, this.queryItemsPerPage, true), this.messages.labelNext) :
				this._getDisabledTextElement(this.messages.labelNext);
		},
		
		_getDisabledTextElement: function(str) {
			var span= document.createElement("span");
			span.className= "disabledText";
			span.appendChild(document.createTextNode(str));
			return span;
		},
		
		_getLinkElement: function(href, text) {
			checkNotEmpty(text, "text");
			checkNotEmpty(href, "href");
			
			var linkElement = document.createElement("a");
			linkElement.href = href;
			linkElement.appendChild(document.createTextNode(text));
			
			return linkElement;
		},
		
		_getStartIndex: function() {
			checkNotNull(this.queryResultsSummaryPage.startIndex, "this.queryResultsSummaryPage.startIndex");
			
			return Number(this.queryResultsSummaryPage.startIndex) + 1;
		},
		
		_getResultCount: function() {
			checkNotNull(this.queryResultsSummaryPage.limit, "this.queryResultsSummaryPage.limit");
			
			return Number(this.queryResultsSummaryPage.limit);
		},
		
		_getTotal: function() {
			var total = new Number(this.queryResultsSummaryPage.totalCount);
			var estimatedTotal = new Number(this.queryResultsSummaryPage.estimatedTotal);
			
			if (estimatedTotal > total)
				return estimatedTotal;
			
			return total;
		},
		
		_getMaxPageIndex: function() {
			if (this.queryItemsPerPage == "all") {
				return this._getResultCount();
			}
			return this._getStartIndex() + this.getPageSize() - 1;
		},
		
		getPageSize: function() {
			if (this.queryItemsPerPage && this.queryItemsPerPage != "all") {
				return Number(this.queryItemsPerPage);
			}
			return WorkItemUtil._getDefaultQueryPageSize();
		},
		_handleErrorMessageClick: function(e) {
			dojo.stopEvent(e);
			if (this._currentTerminatingMessage) {
				var msg = this._currentTerminatingMessage.details;
				if (msg) 
					alert(msg);
			}
		}
	});

})(); 