/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressInformation");

(function() {

var HOUR= 60 * 60 * 1000;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressInformation", null, {

	_workHoursDone: 0,
	_workHoursLeft: 0,
	_realTimeDone: -1,
	_realTimeLeft: -1,
	_openCount: 0,
	_closeCount: 0,
	_estimatedCount: 0,

	constructor: function(progressItemDTO) {
		if (progressItemDTO != null) {
			this._workHoursDone= progressItemDTO.workHoursDone;
			this._workHoursLeft= progressItemDTO.workHoursLeft;
			this._realTimeDone= progressItemDTO.realTimeDone;
			this._realTimeLeft= progressItemDTO.realTimeLeft;
			this._openCount= progressItemDTO.openCount;
			this._closeCount= progressItemDTO.closeCount;
			this._estimatedCount= progressItemDTO.estimatedCount;
		}
	},
	
	// ---- api ------------------------------------------------------------------------------------------------------------

	setWorkHoursDone: function(workHoursDone) {
		this._workHoursDone= workHoursDone;
	},
	
	setWorkHoursLeft: function(workHoursLeft) {
		this._workHoursLeft= workHoursLeft;
	},
	
	setRealTimeDone: function(realTimeDone) {
		this._realTimeDone= realTimeDone;
	},
	
	setRealTimeLeft: function(realTimeLeft) {
		this._realTimeLeft= realTimeLeft;
	},
	
	setOpenCount: function(openCount) {
		this._openCount= openCount;
	},
	
	setCloseCount: function(closeCount) {
		this._closeCount= closeCount;
	},
	
	setEstimatedCount: function(estimatedCount) {
		this._estimatedCount= estimatedCount;
	},

	// ---- delta ----------------------------------------------------------------------------------------------------------
	
	deltaWorkHoursDone: function(delta) {
		this._workHoursDone= this._saveDelta(this._workHoursDone, delta);
	},
	
	deltaWorkHoursLeft: function(delta) {
		this._workHoursLeft= this._saveDelta(this._workHoursLeft, delta);
	},
	
	deltaRealTimeDone: function(delta) {
		this._realTimeDone= this._saveDelta(this._realTimeDone, delta);
	},
	
	deltaRealTimeLeft: function(delta) {
		this._realTimeLeft= this._saveDelta(this._realTimeLeft, delta);
	},
	
	deltaOpenCount: function(delta) {
		this._openCount= this._saveDelta(this._openCount, delta);
	},
	
	deltaCloseCount: function(delta) {
		this._closeCount= this._saveDelta(this._closeCount, delta);
	},
	
	deltaEstimateCount: function(delta) {
		this._estimatedCount= this._saveDelta(this._estimatedCount, delta);
	},
	
	delta: function(progressItemDTO) {
		this.deltaCloseCount(progressItemDTO.closeCount);
		this.deltaOpenCount(progressItemDTO.openCount);
		this.deltaWorkHoursDone(progressItemDTO.workHoursDone);
		this.deltaWorkHoursLeft(progressItemDTO.workHoursLeft);
		this.deltaEstimateCount(progressItemDTO.estimatedCount);
	},

	// ---- query api ------------------------------------------------------------------------------------------------------

	isAllWorkDone: function() {
		return this._closeCount > 0 && this._openCount == 0;
	},

	isRealTimeSpecified: function() {
		return this._realTimeDone > 0 || this._realTimeLeft > 0;
	},
	
	getWorkHoursDone: function() {
		return Math.round(this._workHoursDone / HOUR);
	},

	getWorkHoursLeft: function() {
		return Math.round(this._workHoursLeft / HOUR);
	},
	
	getWorkHours: function() {
		return Math.round((this._workHoursDone + this._workHoursLeft) / HOUR);
	},

	getRealTimeDone: function() {
		return Math.round(this._realTimeDone / HOUR);
	},

	getRealTimeLeft: function() {
		return Math.round(this._realTimeLeft / HOUR);
	},

	getRealTime: function() {
		return Math.round((this._realTimeDone + this._realTimeLeft) / HOUR);
	},

	getOpenCount: function() {
		return this._openCount;
	},

	getCloseCount: function() {
		return this._closeCount;
	},

	getCount: function() {
		return this._openCount + this._closeCount;
	},
	
	getEstimatedCount: function() {
		return this._estimatedCount;
	},

	getWorkHoursDoneExpected: function() {
		// getWorkHours() * getRealTimeProgress()
		return Math.round(((this._workHoursDone + this._workHoursLeft) * this.getRealTimeProgress()) / HOUR);
	},
	
	getWorkHoursDoneDelta: function() {
		// workHoursDone() - workHoursDoneExpected()
		return Math.round((this._workHoursDone - ((this._workHoursDone + this._workHoursLeft) * this.getRealTimeProgress())) / HOUR);
	},

	getWorkHourProgress: function() {
		var divisor= this._workHoursDone + this._workHoursLeft;
		return divisor <= 0 ? 0 : this._workHoursDone / divisor;
	},

	getRealTimeProgress: function() {
		var divisor= this._realTimeDone + this._realTimeLeft;
		return divisor <= 0 ? 0 : this._realTimeDone / divisor;
	},

	getClosedRatio: function() {
		if (this.getCount() == 0) {
			return 100;
		}
		return Math.round((this._closeCount / this.getCount()) * 100);
	},

	getQualityOfPlanning: function() {
		if (this._openCount == 0) {
			return this._closeCount == 0 ? 0 : 100;
		}
		return Math.round((this._estimatedCount / this._openCount) * 100);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_saveDelta: function(value, delta) {
		return Math.max(0, value + delta);
	},

	__sentinel: null // terminates this class definition
});

})();
