/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
 
dojo.provide("com.ibm.rational.cm.web.ui.page.Preference");

dojo.require("dojo.i18n");
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("com.ibm.rational.cm.web._base");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "cqPreference");

(function(){

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var cq = com.ibm.rational.cm.web;
var executeFunc = cq.executeFunc;
var BASE_URL = "cquser.cq";
var messages, _init, _processing, _queue, _data;

dojo.declare(
	"com.ibm.rational.cm.web.ui.page.Preference",
	null,
{
	constructor: function(){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "cqPreference");
		_data = null;
		_init = false;
		_processing = false;
		_queue = [];
	},

	/*
	 * Fetch user preferences from server
	 */
	_load: function(){
		if(_processing){ return; } // don't load again if it is already loading
		
		var self = this;
		var workbench = PlatformUI.getWorkbench();
		cq.xhrGet({
			url: BASE_URL + "?action=getPreference",
			onBegin: function(){
				dojo.forEach(_queue, function(obj){
					executeFunc(obj.onBegin);
				});
				if(workbench){
					workbench.showMessage(messages.LoadPref);
				}		
				_processing = true;
			},
			load: function(response, ioArgs){
				// check if response is valid
				if(!response){
					alert(messages.ErrLoad);
					return;
				}
			
				_data = response;
				_init = true;

				// process onLoad handler
				dojo.forEach(_queue, function(obj){
					executeFunc(obj.onLoad);
				});
			},
			error: function(response, ioArgs){
				dojo.forEach(_queue, function(obj){
					executeFunc(obj.onError);
				});
				alert(messages.ErrLoad);
			},
			handle: function(response, ioArgs){
				if(workbench){
					workbench.hideMessageArea();
				}
				_queue = [];
				_processing = false;
			},
			onCancel: function(){
				dojo.forEach(_queue, function(obj){
					executeFunc(obj.onCancel);
				});
				_queue = [];
			},
			handleError: true,
			preventCache: true
		});
	},
	
	/*
	 * Save user preference data to server
	 */
	_save: function(callbackObj){
		if(callbackObj && callbackObj.noSave == true){ return; } // don't save if noSave=true
		
		var msg = (callbackObj && callbackObj.statusMessage) ?
			callbackObj.statusMessage : messages.SavePref;

		var workbench = PlatformUI.getWorkbench();
		cq.xhrPost({
			url: BASE_URL,
			content: {
				action: "setPreference",
				prefData: dojo.toJson(_data)
			},
			onBegin: function(){
				if(workbench){
					workbench.showMessage(msg);
				}	
			},
			load: function(response, ioArgs){
				if(!response || response.status != true){
					throw new Error(messages.ErrSave);
				}
				if(callbackObj){
					executeFunc(callbackObj.onSaved);
				}
			},
			error: function(response, ioArgs){
				if(callbackObj){
					executeFunc(callbackObj.onError);
				}
				alert(response);
			},
			handle: function(response, ioArgs){
				if(workbench){
					workbench.hideMessageArea();
				}
			},
			handleError: true
		});
	},
	
	getValue: function(category, key){
		var prefData = _data;
		if(prefData && prefData[category]){
			if(prefData[category][key]){
				return prefData[category][key].value;
			}
		}
		return null;
	},
	
	setValue: function(category, key, value, callbackObj){
		if(!_data[category]){
			_data[category] = {};
		}
		_data[category][key] = { value: value };
		
		this._save(callbackObj);
	},
	
	/*
	 * Processes callback function after preference has been loaded
	 * If preference is already loaded, processes callback function immediately
	 * argument is a JavaScript object in the following form:
	 * {
	 * 	onBegin: function(){}	gets called right before calling getPreference
	 * 	onLoad: function(){}	gets called after preference has been successfully loaded
	 * 	onError: function(){}	gets called when an error occurred while loading preferences
	 * 	onCancel: function(){}	gets called when user presses [Cancel] in the login dialog
	 * 	forceLoad: boolean		if true, loads preference even if it has already been loaded
	 * }
	 */
	process: function(/* Object */ args){
		if(_init && args.forceLoad != true){
			executeFunc(args.onBegin);
			executeFunc(args.onLoad);
		}else{
			_queue.push(args);
			this._load();
		}
	}
});

})();

com.ibm.rational.cm.preference = new com.ibm.rational.cm.web.ui.page.Preference();