/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

/*
 * This Grid widget includes a methods that overrides dojox.Grid as workarounds of dojox.Grid's bugs
 * This widget should be removed once dojo fixes these bugs
 */
dojo.provide("com.ibm.rational.cm.web.ui.view.common.Grid");

dojo.require("dojox.grid.Grid");
dojo.require("com.ibm.rational.cm.web._base");

(function(){

var cq = com.ibm.rational.cm.web;
var cqNS = "com.ibm.rational.cm.web.ui";

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.Grid",
	dojox.Grid,
{
	minimumWidth: 50, // minimum width in pixels
	defaultAutoWidth: 150, // default width for "auto" columns in pixels

	// RATLC01050483 : Cannot navigate through records within resultset using keyboard
	// Also submitted defect to Community Source
	// https://cs.opensource.ibm.com/tracker/index.php?func=detail&aid=76353&group_id=2078&atid=5124
	onKeyDown: function(e){
		if(e.altKey || e.ctrlKey || e.metaKey ){ return; }
		var keys = dojo.keys;
		var edit = this.edit;
		var focus = this.focus;
		switch(e.keyCode){
			case keys.ESCAPE:
				edit.cancel();
				break;
			case keys.ENTER:
				if(!e.shiftKey){
					var isEditing = edit.isEditing();
					edit.apply();
					if(!isEditing){
						edit.setEditCell(focus.cell, focus.rowIndex);
					}
				}
				break;
			case keys.TAB:
				focus[e.shiftKey ? 'previousKey' : 'nextKey'](e);
				break;
			case keys.LEFT_ARROW:
				if(!edit.isEditing()){
					dojo.stopEvent(e);
					focus.move(0, -1);
				}
				break;
			case keys.RIGHT_ARROW:
				if(!edit.isEditing()){
					focus.move(0, 1);
				}
				break;
			case keys.UP_ARROW:
				if(!edit.isEditing() && focus.rowIndex != 0){
					dojo.stopEvent(e);
					focus.move(-1, 0);
				}else if(focus.rowIndex == 0){
					dojo.stopEvent(e);
				}
				break;
			case keys.DOWN_ARROW:
				if(!edit.isEditing() && focus.rowIndex+1 != this.model.count){
					dojo.stopEvent(e);
					focus.move(1, 0);
				}
				break;
		}
	},
	
	// RATLC01050489 : The right-clicked row of resultset should be selected color
	onCellContextMenu: function(e){
		var selection  = this.selection;
		var index = e.rowIndex;
		if(selection.isSelected(index)){
			selection.beginUpdate();
			selection.addToSelection(index);
			selection.endUpdate();
		}else if(!e.ctrlKey){
			selection.beginUpdate();
			selection.select(index);
			selection.endUpdate();
		}
		this._rightClickInfo = { cell: e.cell, rowIndex: index };
	},
	
	getRightClickedInfo: function(){
		return this._rightClickInfo;
	},
/*
	// RATLC01226897 : [IE] Script error occurs when logging-out in IE
	createView: function(inClass){
		var view = this.inherited(arguments);

		view.doscroll = function(inEvent){
			if(!this.headerNode || !this.headerNode.parentNode || !this.scrollboxNode || !this.scrollboxNode.parentNode){ return; }
			this.headerNode.scrollLeft = this.scrollboxNode.scrollLeft;
			var top = this.scrollboxNode.scrollTop;
			if(top != this.lastTop){
				this.grid.scrollTo(top);
			}
		};
		
		view.setScrollTop = function(inTop){
			this.lastTop = inTop;
			if(this.scrollboxNode && this.scrollboxNode.parentNode){
				this.scrollboxNode.scrollTop = inTop;
			}
			return inTop;
		};
		return view;
	},
*/	
	// Override dojox.VirtualGrid#doApplyCellEdit so that it notifies triggers if cell values changes by edit
	// (doApplyCellEdit is called even if values are not changed and onclick)
	doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		var oldVal = this.model.getDatum(inRowIndex, inFieldIndex);
		// fix cross-site scripting risk for grid editors
		if(dojo.isString(inValue)){
			inValue = cq.sanitizeString(inValue);
		}
		this.inherited(arguments);
		if(inValue != oldVal){
			this.onEditChanged();
		}
	},
	
	// a stub function that can be connected
	onEditChanged: function(){},

	getColumnWidth: function(colIndex){
		var cell = this.getCell(colIndex);
		if(!cell){ return -1; }
		var node = cell.getNode(0);
		if(!node){ return -1; }
		var w = dojo.contentBox(node).w;
		return w;
	},
	
	setStructure: function(inStructure){
	
		// RATLC01362177 : Submit_Date does't display locale time format in query on chinese locale
		// for handling DateTime column
		var model = this.model;
		var convertDateTime = function(inRowIndex){		
			var index = this.fieldIndex; // RATLC01258679 : "Submit_date" column has no value when columns are customized
			if(index == -1){ return ""; }
			return cq.grid.convertDateTime(model.getDatum(inRowIndex, index));
		};
		
		dojo.forEach(inStructure, function(view){
			dojo.forEach(view.cells, function(cell){
				dojo.forEach(cell, function(item){
					// RATLC01262708 : Falcon FVT: CQ Properties never shows for Folder & other CQ Navigator items [only in IE]
					// when property view preference is changed from hidden -> show, script error occurs because 'item' is undefined
					if(item && item.dataType == "DateTime" && !item.get){
						item.get = convertDateTime;
					}
				});
			});
		});
		
		this.inherited(arguments);
	},
	
/*
	resize: function(){
		this.inherited(arguments);
		
		var views = this.views.views;
		if(views.length === 0){ return; }

		// RATLC01228100 : Height of grid header which includes Japanese characters is too long in FF
		var h = 0;
		dojo.forEach(views, function(inView){
			h = Math.max(dojo.contentBox(inView.headerContentNode).h, h);
		});
		if(this.headerNode){
			this.headerNode.style.height = h + "px";
		}
//		this.resizeHeight(); // there's no such method in dojo 1.2.0
	},
*/	
	// override dojox.Grid in order to update previous selection after sort
	// RATLC01270995 : IHD: Selections are not properly tracked when sorting by column in the query results view
	sort: function(){
		// cache previously selected rows
		var selectedPropName = "__cqSelected";
		var sel = this.selection.getSelected();
		var remaining = sel.length;
		dojo.forEach(sel, function(i){
			this.model.data[i][selectedPropName] = true;
		}, this);
		
		this.inherited(arguments);

		// is there a more easier way to get row number for selected rows after sort, rather than setting __cqSelected property?? 
		this.selection.unselectAll();
		for(var i = 0, len = this.model.data.length; (i < len && remaining > 0); i++){
			var row = this.model.data[i];
			if(row[selectedPropName]){
				this.selection.addToSelection(i);
				row[selectedPropName] = null;
				remaining--;
			}
		}
	},

	postCreate: function(){
		dojo.addClass(this.domNode, "com-ibm-rcm-Grid");
		
		// RATLC01065039 : Popup after running query
		// override to add null check of inRowNodes[0].parentNode
		this.views.normalizeRowNodeHeights = function(inRowNodes){
			var h = 0;
			for(var i=0, n, o; (n=inRowNodes[i]); i++){
				h = Math.max(h, (n.firstChild.clientHeight)||(n.firstChild.offsetHeight));
			}
			h = (h >= 0 ? h : 0);

			var hpx = h + 'px';
			for(var i=0, n; (n=inRowNodes[i]); i++){
				if(n.firstChild.clientHeight!=h){
					n.firstChild.style.height = hpx;
				}
			}
			// querying the height here seems to help scroller measure the page on IE
			if(inRowNodes && inRowNodes[0] && inRowNodes[0].parentNode){
				inRowNodes[0].parentNode.offsetHeight;
			}		
		};
	
		var defaultAutoWidth = this.defaultAutoWidth;
		var minimumWidth = this.minimumWidth;
		this.connect(this.views, "addView", dojo.hitch(this.views, function(inView){
		
			// RATLC01058931 : extreme waste of screen space in query list
			inView.resizeWidth = function(){
				// this points to view
				if(this.flexCells){
					var cw = parseInt(this.getContentWidth()); // displayed width of grid

					// if grid content is very wide so that horizontal scrollbar is shown, and the last column is width="auto",
					// actual displayed width of last column is 0. Therefore, set to defaultAutoWidth.
					var ths = this.headerContentNode.getElementsByTagName("TH");
					var len = ths.length;
					if(len > 0 && ths[ths.length - 1].style.width == "auto"){
						
						// calculate total width excluding last column
						var totalWidth = 0;
						dojo.forEach(ths, function(th, idx){
							if(idx < len - 1){
								totalWidth += dojo.marginBox(th).w;
							}
						});
							
						if(cw < totalWidth + minimumWidth){
							cw = totalWidth  + defaultAutoWidth;
						}
					}

					this.contentWidth = cw + "px";
					this.headerContentNode.firstChild.style.width = this.contentWidth;
				}
				var w = this.scrollboxNode.offsetWidth - this.getScrollbarWidth();
				w = Math.max(w, this.getColumnsWidth()) + 'px';
				with(this.contentNode){
					style.width = '';
					offsetWidth;
					style.width = w;
				}
			}
		}));
	
		this.inherited(arguments);
		
		// RATLC01221845 : [Usability] Context menu disappears quickly when right clicking on a selected record of resultset
//		this.focus.styleRow = function(){};
	}
});

// RATLC01060653 : CQBeta7.1.2r3: [MUSTFIX] can't copy text out of a grid - (or copy to clipboard via pencil icon?)
com.ibm.rational.cm.web.createGridCopyMenu = function(obj){
	dojo["requireLocalization"](cqNS, "common");
	dojo["requireLocalization"](cqNS, "ResultSetMessages");
	
	dojo["require"]("dojo.i18n");
	dojo["require"]("dijit.Menu");
	return new dijit.MenuItem({
		label: dojo.i18n.getLocalization(cqNS, "common").lblCopy,
		iconClass: "MenuIcon_Copy",
		onClick: function(){
			var grid = null;
			if(dojo.isFunction(obj)){
				grid = obj();
			}else{
				grid = obj;
			}
			if(!grid || !(grid instanceof com.ibm.rational.cm.web.ui.view.common.Grid)){ return; }
			var info = grid.getRightClickedInfo();
          	var inRowIndex = info.rowIndex;
			var colIndex = info.cell.index;
			var str = grid.model.getDatum(inRowIndex, colIndex);
			var unsupportedMsg = dojo.i18n.getLocalization(cqNS,"cqResultSetTable").unsupportedBrowser;
			if(dojo.isIE){
				var ret = dojo.global.clipboardData.setData("Text", str);
				if(!ret){ // if false, user did not allow scripts to access clipboard in IE7
					alert(unsupportedMsg);
				}
			}else{
				try{
					netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
					var clip = Components.classes['@mozilla.org/widget/clipboard;1'].createInstance(Components.interfaces.nsIClipboard);
					var trans = Components.classes['@mozilla.org/widget/transferable;1'].createInstance(Components.interfaces.nsITransferable);
					trans.addDataFlavor('text/unicode');
					var ss = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString); 
					ss.data = str; 
					trans.setTransferData("text/unicode", ss, str.length*2); 
					clip.setData(trans, null, Components.interfaces.nsIClipboard.kGlobalClipboard);
				}catch(e){
					alert(unsupportedMsg);
				}
			}
		}
	});
}

com.ibm.rational.cm.web.grid = {
	// converts ISO date string to date in client locale
	convertDateTime: function(/* String */ value){
		if(!value || value.length === 0){ return ""; }
		dojo["require"]("dojo.date.stamp");
		var d = dojo.date.stamp.fromISOString(value);
		if(!d){ return "" } // when loading, value is "..." and d is null
		dojo["require"]("dojo.date.locale");
		return dojo.date.locale.format(d, {
			formatLength: "long"
		});
	}
}

})();