/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007,2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
dojo.provide("com.ibm.rational.cm.web.ui.view.record.CqRecord");
dojo.provide("com.ibm.rational.cm.web.ui.view.record.CqListControl"); // For CqChangeSet only
dojo.provide("com.ibm.rational.cm.web.ui.view.record.CqFormControl"); // For CqDateTimeTextBox.js
dojo.provide("com.ibm.rational.cm.web.ui.view.record.TextBoxResizeMixin"); // For CqDateTimeTextBox.js

dojo.require("dojo.i18n");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.Menu");
dojo.require("dijit.Toolbar");
dojo.require("dijit.Tooltip");
dojo.require("dijit.Dialog");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.SplitContainer");
dojo.require("dijit.form._FormWidget");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.NumberTextBox");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");
dojo.require("com.ibm.rational.cm.web.ui.view.record.CqCustomizeContent");
dojo.requireLocalization("com.ibm.rational.cm.web.ui","cqRecordStrings");
dojo.requireLocalization("com.ibm.rational.cm.web.ui","common");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("dojo.cookie");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.ui.view.common.DoubleListBox");
dojo.require("com.ibm.rational.cm.web.ui.utils.RecordChooser");

// the following line might make IE confused...
//var shellService;	// for getI18NString. initialized in ShellService.js

(function(){

var cq = com.ibm.rational.cm.web;
var cqNS = "com.ibm.rational.cm.web.ui";
var cqLoginController = com.ibm.rational.cm.cqLoginController;
var RecordDetailRESTServiceURL = "cqartifactdetails.cq";
var workbench;
var COOKIE_NAME_FOR_UCMWARNING = "cqbridge.cqRecord.dontShowUCMWarningDialog";
var dontShowUCMWarningDialog = dojo.cookie.isSupported() ? dojo.cookie(COOKIE_NAME_FOR_UCMWARNING) : false;

/* 
 * @description: com.ibm.rational.cm.web.ui.view.record.CqRecord is a dojo widget extension for CQForm.
 */	
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqRecord",
	[dijit.layout.BorderContainer, dijit._Templated],
	{
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/record/templates/CqRecord.html"),
		
		stableLocation: "",
		isUcmEnabled: false,
		isCCSupported: false,
		cqFormLayout:null,	//	cqFormLayout, would store the JSON representation of CQForm associated for the selected record's recordType.
		cqRecordData:null,	//	cqRecordData, would have all the data values for the selected record.
		cqState: "VIEW",	// "VIEW","MODIFY", "SUBMIT or "BATCH". default is "VIEW"

		gutters: false,		//  override property of BorderContainer
		xScaleFactor:1.0,	//	xScale factor		
		yScaleFactor:1.0,	//	yScale factor	
		closable:true,		//	closable
		custom: false,		//  custom view or not
		
		_loadedTabs: null,				//  parts of tabs are loaded (valid only non-custom case)
		_loadedPartially: false,		//  parts of fields are loaded 
		_modifiedList:null,				//	list of modified fields after clearModifiedList()
		// RATLC01477352
		_modifiedTimestampList: null,	//	list of last modified timpstamp of modified fields
		_allModified:null,				//	list of all modified fields for template
		_controls: null,				//  array of {tabName, dataUpdated(true, false or undefined), controls=[widget], invalidControls=[invalid]}
		_hiddenList: null,				//  [tabName+tabIndex][fieldSource::itemType::name]

		formMain:null,
		_formContainer: null,			//  Main Form Container (tabContainer or table)
		_tabIndex: 0,					//  Keep the previous tab index
		_dynamicActions:null,			//  "Save" and "Cancel"
										//  "Change State"["Resolve", "Postpone", ...],"Actions"["Delete","Duplicate"] ...  
		_nowLoading: false,
		_loadingImage: null,
		_messages: null,
		_customizeDialog: null,
		
		_resources: null,
		/*
		 * 
		 */
		postMixInProperties:function(){
			var getLoc = dojo.i18n.getLocalization;
			this.commonI18nStrings = getLoc(cqNS, "common", this.lang);
			this._messages = getLoc(cqNS, "cqRecordStrings", this.lang);
			this.inherited(arguments);
			workbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
		},
		
		/*
		 * Get the display name of the record type 
		 */
		getRecordTypeDisplayName: function(){
			if(this.cqRecordData){
				var userfriendlyid = this.cqRecordData.RecordId;
				var leftmark = userfriendlyid.indexOf(':');
				var rightmark = userfriendlyid.indexOf('/');
				return userfriendlyid.substring(leftmark+1, rightmark);
			}
			return "";
		},
		 
		/*
		 * Retrieves the display CqRecordId
		 */
		getDisplayId:function(){
			if(this.cqRecordData){
				if(this.cqState=="BATCH"){
					return this._messages.titleForBatchUpdate;
				}
				if (this.cqState=="SUBMIT" && this.cqRecordData.IsStateless) {
					return this._messages.titleForStatelessRecord;
				}
				if (this.cqRecordData.DisplayName) {
					return this.cqRecordData.DisplayName;
				}
				var userfriendlyid = this.cqRecordData.RecordId;
				if (userfriendlyid) {
					var leftmark = userfriendlyid.indexOf('/');
					var rightmark = userfriendlyid.lastIndexOf('@');
					var id = userfriendlyid.substring(leftmark+1, rightmark);
					return id;
				}
			}
			return "";
		},
		
		/*
		 * Update Tab Title
		 */
		_updateTitle:function() {
			var newTitle = this.getDisplayId();
			if(newTitle == ""){
				if(this._nowLoading) {
					newTitle = this._messages.loading;
				}else{
					newTitle = this._messages.defaultTitle;
				}
			} 
			else {
				if(this._tasksView){
					var titleForTasks = newTitle;
					var headline = this.findFieldDataFor("Headline");
					if(headline){
						titleForTasks = newTitle + ": " + headline.CurrentValue;
					}
					this._tasksView.setRecordTitle(titleForTasks);
				}
			}
//			console.log("CqRecord._updateTitle is called. old title=[" + this.title + "], new title=[" + newTitle + "]");
			if(this.title != newTitle){
				this.title = newTitle;
				if (this.controlButton) {
					setTimeout(dojo.hitch(this, function(){
						var mainTabButton = this.controlButton;
						if(mainTabButton && mainTabButton.domNode){
							// setLabel in Button.js set display="none" to avoid Mozilla bug. 
							// But continuous calls in short time cause problem.
							mainTabButton.domNode.style.display = "";
							var f = dojo.hitch(mainTabButton, mainTabButton.setAbbreviatedLabel || mainTabButton.setLabel);
							f(com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(this.title)); // tab title
						}
					}), 0);
				}
				var t = this.getRecordTypeDisplayName();
				if (t) {
					if (this.title) {
						t = t + "/" + this.title;
					} 
				} else if (this.title) {
					t = this.title;
				}
				if (t) {
					t = com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(t);
					this.formTitle.title = t;
dojo.publish("CQ_ITEMLABEL_CHANGED", [{ id:this.stableLocation, title:t }]);
					if(this.cqRecordData && this.cqRecordData.MasteredLocally !== undefined && this.cqRecordData.MasteredLocally !== true) {
						this.formTitle.title = this._messages.remoteMasterd + " " + t;
						var imgUrl = dojo.moduleUrl("com.ibm.rational.cm.web", "ui/images/remot_record.gif");
						t = "<img align='top' src='" + imgUrl + "'>" + t;
					}
					this.formTitle.innerHTML = t; 
				}
			}
			if(this.batchRecordNames){
				this.formTitle.title = this.batchRecordNames; 
			}
		},

		/*
		 * Update Tooltip for Tab
		 * - Find "Headline" field and use it as tooltip text
		 * - Eclipse client does similar thing on Duplicate dialog 
		 */
		_updateTooltip: function() {
			setTimeout(dojo.hitch(this, function(){
				if (!this.domNode || !this.cqRecordData || !this.controlButton) {
					return;
				}
				var label; 
				if(this.batchRecordNames){
					label = this.batchRecordNames; 
				} else {
					var headline = this.findFieldDataFor("Headline");
					if(headline){
						label = cq.sanitizeString(headline.CurrentValue);
					}
				}
				var tooltip = this._tooltip;
				if(label){
					var connectionId = this.controlButton.focusNode.id;
					if(tooltip && tooltip.connectId[0] != connectionId){
						tooltip.destroy();
						tooltip = this._tooltip = null;
					} 
					if(!tooltip){
						tooltip = this._tooltip = new CqRecordTooltip({connectId:[connectionId]});
					}
					tooltip.label = label;
				} else if(tooltip){
					tooltip.destroy();
					delete this._tooltip;
				}
			}), 0);
		},

		/*
		 * Set loading status 
		 */
		setLoading:function(/* boolean */loading) {
			if(this._nowLoading == loading){
				return;
			}
			this._setReadonly(loading);
			if(this._contextMenu && this._contextMenu.domNode){
				loading ? this._contextMenu.unBindDomNode(this.domNode) : this._contextMenu.bindDomNode(this.domNode);
			}
			if(this._resourceButton){
				this._resourceButton.setDisabled(loading);
			}
			dojo.toggleClass(this.domNode, "formLoading", loading);
			if(!loading && dojo.isIE){ // work-around for RATLC01229447 
				var s = this.domNode.style;
				var orgc = s.cursor;
				s.cursor = "default";
				s.cursor = orgc;
			}
			this._nowLoading = loading;
			this._updateTitle();
			this._updateToolbar();
			if(workbench){
				if(loading){
					workbench.showMessage (this._messages.loadingRecord);
				}else{
					workbench.hideMessageArea();
				}
			} else if (this.controlButton) {
				if (this._loadingImage) {
					// accidently created
					this._loadingImage.style.visibility = "hidden";
					delete this._loadingImage;
					this._loadingImage = null;
				}
			} else {
				// Loading image on main container
				var img = this._loadingImage; 
				if (!img) {
					img = dojo.doc.createElement("img");
					img.src = dojo.moduleUrl("com.ibm.rational.cm.web", "ui/images/loading.gif");
					var s = img.style;
					s.position = "absolute";
					s.top = "0px";
					s.left = "0px";
					s.zIndex = "999"; // always top
					this.formMain.domNode.appendChild(img);
					this._loadingImage = img;
				}
				img.style.visibility = loading ? "" : "hidden";
			}
		},
		
		/*
		 * Plugin the construction of the CqRecord Widget
		 */
		_getRightPaneView: function(){
			return null;
		},
				
		postCreate: function(args, frag) {	
			this.inherited(arguments);
			this._tasksView = this._getRightPaneView();
			if(this._tasksView){
				this._rightPane = new CqRecordRightPaneContainer({
					content: this._tasksView.domNode,
					region: "right",
					splitter: "true"
				});
				this._rightPane.domNode.style.width = "150px";
				this.formMain.addChild(this._rightPane);
			}
			this._dynamicActions = [];
			this._resources = [];
			this._modifiedList = {};
			this._modifiedTimestampList = {};
			if (this.cqStatus != "SUBMIT") {
				this._setStableLocationId(this.id);
			}
			this._updateTitle();
			this._updateToolbar();
			// main
			this._buildLayout();
			this.updateRecordData(this.cqRecordData);
			// 
			if(this.batchRecords && this.batchRecords.length > 0){
				cq.xhrPost({
					url:"cqproperties.cq",	
					handleAs: "json-comment-optional",
					content: {
						action:"GetDisplayName", 
						resourceId: this.batchRecords.join(",")
					},
					load: dojo.hitch(this, function(data, ioArgs){
						if(data && data.STATUS=="true" && dojo.isArray(data.data)){
							this.batchRecordNames = data.data.join(",");
						}
					})		
				});
			}
			if(dojo.isFF == 2){
				// FF2 has problem of TABLE rendering. TextOut does same workaround.
				var hack = function(node){
					node.style.opacity = "0.999";
					setTimeout(function(){
						node.style.opacity = "";
					}, 0);
				};
				var fav = this.tool_favorite;
				this.connect(fav, "setDisabled", function(){hack(fav.domNode);});
				var cus = this.tool_custom;
				this.connect(cus, "setDisabled", function(){hack(cus.domNode);});
			}  
		},
		startup: function(){
			this.inherited(arguments);
			this._forEachControl(function(c){
				c.startup();
			});
		},
		
		_setStableLocationId: function(id){
			this.stableLocation = id;
			if(this._tasksView){
				this._tasksView.setRecordId(this.stableLocation);
			}
		},
		/*
		 * Uninitialize
		 */
		uninitialize:function() {
			var tv = this._tasksView;
			if(this.cqState=="CANCEL_SUBMIT" && tv && tv.removeAllLinks){
				tv.removeAllLinks();
			}
			this._destroyControls();
			this._destroyDynamicActions();
			dojo.forEach(["_formContainer","_customizeDialog","formHeader","_rightPane","formMain","formToolbar","_tooltip","tool_favoriteMenu","tool_customMenu"], function(t){
				var target = this[t];
				if(target){
					if(target.domNode) {
						target.destroyRecursive();
					}
					delete this[t];
				}
			}, this);
			this._destroyTitleMenu();
			this._clearMenu(this._contextMenu);
			if(this._popupMenus){
				dojo.forEach(this._popupMenus, function(menu){
					if (menu.domNode) {
						menu.destroyRecursive();
					}
				});
				this._popupMenus = null;
			}
			delete this.cqRecordData;
			delete this.cqFormLayout;
		},
		
		onClose: function(container, page){
			if((this.cqState=="MODIFY"||this.cqState=="SUBMIT"||this.cqState=="BATCH") && this.cqRecordData && this._controls && this._controls.length > 0){
				if (!confirm(this.commonI18nStrings.msg_confirm_close)){
					return false;
				}
				this._fireAction({actionString:"CANCEL"});
			}
			return true;
		},
		
		/*
		 * Toolbar Actions
		 */
		_refresh: function(){
			dojo.publish('CQ_RECORD_REFRESH', [{ recordId:this.id, state:this.cqState }]);			
		},
		_addToFavorites: function() {
			dojo.publish('CQ_RECORD_INTERNAL_FAVORITES', [{ record: this }]);
		},
		_getBookmarkTitle: function(){
			var title = "";
			var recordType = this.getRecordTypeDisplayName();
			if(recordType){
				title += recordType + "/";
			} 
			title += this.getDisplayId();
		},
		_bookmark: function(){
			dojo["require"]("com.ibm.rational.cm.web.ui.utils.Bookmark");
			com.ibm.rational.cm.web.ui.utils.addToBookmark({
				type: "RECORD",
				title: this._getBookmarkTitle(),
				stableLocation: this.id
			});
		},
		_email: function(){
			dojo["require"]("com.ibm.rational.cm.web.ui.utils.Bookmark");
			com.ibm.rational.cm.web.ui.utils.addEmailLink({
				type: "RECORD",
				title: this._getBookmarkTitle(),
				stableLocation: this.id
			});
		},
		_print: function() {
			dojo.publish('CQ_RECORD_PRINT', [{ record:this }]);
		}, 
		_changeViewMode: function(){
			this.setCustom(!this.custom);
		},
		_changeToTabMode: function(){
			this.setCustom(false);
		},
		_changeToFlatMode: function(){
			this.setCustom(true);
		},
		_customizeView: function(evt){
			this.openCustomizeDialog();
		}, 
		/*
		 * publish Event to handle record action
		 */
		_fireAction:function(action){
			var actionString = action.actionString;
			if (!actionString) {
				return;
			}
			var pubArgs = {"view":this,"action":actionString,"recordId":this.id}; // id is recordId
			if(this.batchRecords){
				pubArgs.batchRecords = this.batchRecords;
			}
			if (action.actionId) {
				pubArgs.actionId = action.actionId;
			}
			dojo.publish("CQ_RECORD_ACTION",[pubArgs]);
		},
		
		/*
		 * 
		 */
		_updateToolbar:function() {
			var disabled = (!this._nowLoading && this.cqRecordData && this.cqFormLayout) ? false : true;
			var t_custom = this.tool_custom;
			if (this.cqState == "SUBMIT" || this.cqState == "BATCH") {
				this.tool_refresh.domNode.style.display = "none";
				this.tool_favorite.domNode.style.display = "none";
				this.tool_print.domNode.style.display = "none";
				if (this.cqState == "BATCH"){
					t_custom.domNode.style.display = "none";
					return;
				}
			} else {
				this.tool_refresh.domNode.style.display = (this.cqState == "MODIFY" ? "none" : "");
				this.tool_favorite.domNode.style.display = "";
				this.tool_print.domNode.style.display = "";
				if (this.tool_refresh.disabled != disabled) {
					// always same status so I check tool_refresh.disabled only
					this.tool_refresh.setDisabled(disabled);
					this.tool_favorite.setDisabled(disabled);
					this.tool_addToFavorite.setDisabled(disabled);
					this.tool_bookmark.setDisabled(disabled);
					this.tool_email.setDisabled(disabled);
					this.tool_print.setDisabled(disabled);
				}
			}
			if(this._dynamicActions && this._dynamicActions.length > 0){
				if(this._dynamicActions[0].disabled != disabled){
					// always same status so I check the first one only
					for (var i in this._dynamicActions) {
						var a = this._dynamicActions[i];
						if (a.setDisabled) {
							a.setDisabled(disabled);
						}
					}
				}
			}
//			disabled = disabled || (this.cqState != "VIEW");
			if (t_custom.disabled != disabled) {
				t_custom.setDisabled(disabled);
			}
			dojo.toggleClass(t_custom.iconNode, "cqrecordMenuIcon_tabMode", !this.custom);
			dojo.toggleClass(t_custom.iconNode, "cqrecordMenuIcon_flatMode", this.custom);
			disabled = disabled || !this.custom;
			if (this.tool_customize.disabled != disabled) {
				this.tool_customize.setDisabled(disabled);
			}
		},

		/*
		 * Actions
		 */
		createActions: function(refresh){
			this._createActions(this.formToolbar, refresh);
		},
		_createActions:function(target /* toolbar or menu */, refresh){
			if(this._getActionsXHR){
				this._getActionsXHR.cancel();
				delete this._getActionsXHR; 
			}
			var status = this.cqState;
			var actionType = (status == "MODIFY" || status == "BATCH" || status == "SUBMIT") ? "MODIFY" :
//				(status == "VIEW" && (!this.cqRecordData || this.cqRecordData.MasteredLocally)) ? "VIEW" : null;
				"VIEW"; // always VIEW (RATLC01237443)
			if(!refresh && this._actionsType == actionType){
				return;
			}
			this._destroyDynamicActions();
			// function to create control for action
			var abstractItemCreator = dojo.hitch(this, function(clazz, action, actionId, label, iconClass) {
				iconClass = iconClass || "";
				var menuitem = new clazz({label:label, iconClass: iconClass});
				this.connect(menuitem, "onClick", function() {
					var recType = this.getRecordTypeDisplayName();
					if ((label == "Refresh") && (cqLoginController.getLoginInfo("isReqProEnabled"))
						&& (recType = "Requirement")){
						//  Removed code to construct a cqReqWebInt and call cqReqWebInt handleRequest
						//  if that integration is to be supported this code would need to be put back

						cq.openMessageDialog(this._messages.cqReqProIntegrationUnsupported);
						
						return menuitem;
					}
					this._fireAction({actionString: action,	actionId: actionId});
				});
				return menuitem;
			});				
			// Menu item
			var menuItemCreator = function(action, actionId, label, iconClass) {
				return abstractItemCreator(dijit.MenuItem, action, actionId, label, iconClass);
			}
			// The default item for action execution is Button
			var itemCreator = dojo.hitch(this, function(action, actionId, label, iconClass) {
				return abstractItemCreator(dijit.form.Button, action, actionId, label, iconClass);
			});
			// Default clickable widget for sub menu is DropDownButton
			var subItemClass = dijit.form.DropDownButton;
			// check the taget is toolbar or menu
			var toolbar = target instanceof dijit.Toolbar;
			if(!toolbar){
				// must be menubar or context menu
				// item for action execution is MenuItem
				itemCreator = menuItemCreator;
				// Clickable widget for sub menu is PopupMenuItem
				subItemClass = dijit.PopupMenuItem;
			}

			// create actions
			if (actionType == "MODIFY") {
				// actions for modify/submit mode
				var ret = []; // actions for return
				if (toolbar && status != "BATCH") ret.push(new dijit.ToolbarSeparator());
				var applyButton = itemCreator("APPLY", null, this._messages.submit_action_label); 
				ret.push(applyButton);
				if (toolbar) ret.push(new dijit.ToolbarSeparator());
				var cancelButton = itemCreator("CANCEL", null, this._messages.cancel_action_label);
				ret.push(cancelButton);

				// Add to toolbar or menubar
				for (var i = 0, l = ret.length; i < l; i++) {		
					target.addChild(ret[i] /*, 0 */);
				}
				this._dynamicActions = ret;
				this._actionsType = "MODIFY";
				this._updateToolbar();
			}else if (actionType == "VIEW") {
				var self = this;
				var setActions = function(data){
					if(!target || !target.domNode || !data){
						return;
					}
					var actions = data.actions;
					if (actions) {
						var ret = [];
						var addDropdownMenu = function(actionArray, label, iconClass, defaultAction){
							if(!actionArray || actionArray.length<=0){
								return;
							}
							if (toolbar) ret.push(new dijit.ToolbarSeparator());
							if(actionArray.length==1){
								var a = actionArray[0];
								var button = itemCreator("MODIFY", a.LocationId, a.DisplayName, iconClass);
								ret.push(button);
							}else{				
								var subMenu = new dijit.Menu();
								var subMenuItems = [];
								var defId = defaultAction ? defaultAction.LocationId : null;
								var addDef = false;
								for (var obj in actionArray){
									var a = actionArray[obj];
									if(!a.LocationId) continue;
									if(a.LocationId === defId) {
										addDef = true;
										continue;
									}
									subMenuItems.push(menuItemCreator("MODIFY", a.LocationId, a.DisplayName, ""));
								}
								if(addDef){
									subMenu.addChild(menuItemCreator("MODIFY", defId, defaultAction.DisplayName, ""));
									subMenu.addChild(new dijit.MenuSeparator());
								}
								for(var i=0,l=subMenuItems.length; i<l; i++){
									subMenu.addChild(subMenuItems[i]);
								}
								self._addPopupMenu(subMenu);
								var button = new subItemClass({dropDown: subMenu, popup: subMenu, label:label, iconClass:iconClass});
								ret.push(button);
							}
						};
						// MODIFY menu
						addDropdownMenu(actions.Modify, self._messages.modify_action_label, toolbar ? "" :"cqrecordMenuIcon_modify", data.DefaultAction);
						// ChangeState menu
						addDropdownMenu(actions.changeState, self._messages.changestate_action_label, "", data.DefaultAction);
						// Utilities menu
						if (actions.Delete == "true" || actions.Duplicate == "true" || actions.UnDuplicate == "true" ||
							(actions.recordScriptAlias && actions.recordScriptAlias.length>0)) { 
							if (toolbar) ret.push(new dijit.ToolbarSeparator());
							var utilitiesMenu = new dijit.Menu();
							var a,item;
							if (actions.Delete == "true") {
								a = actions.list.Delete;
								item = menuItemCreator("DELETE", a.LocationId, a.DisplayName, "cqrecordMenuIcon_delete");
								utilitiesMenu.addChild(item);
							}
							if (actions.Duplicate == "true") {
								a = actions.list.Duplicate;
								item = menuItemCreator("DUPLICATE", a.LocationId, a.DisplayName, "cqrecordMenuIcon_duplicate");
								utilitiesMenu.addChild(item);
							}
							if (actions.UnDuplicate == "true") {
								a = actions.list.UnDuplicate;
								item = menuItemCreator("UNDUPLICATE", a.LocationId, a.DisplayName, "");
								utilitiesMenu.addChild(item);
							}
							if (actions.recordScriptAlias) {
								for (var obj in actions.recordScriptAlias){
									a = actions.recordScriptAlias[obj];
									item = menuItemCreator("SCRIPTALIAS", a.LocationId, a.DisplayName, "");
									utilitiesMenu.addChild(item);
								}
							}
							self._addPopupMenu(utilitiesMenu);
							var utilityButton = new subItemClass({dropDown: utilitiesMenu, popup: utilitiesMenu,
																label:self._messages.utilities_action_label});
							ret.push(utilityButton);
						}
						// Add to toolbar or menubar
						for (var i = 0, l = ret.length; i < l; i++) {		
							target.addChild(ret[i] /*, 0 */);
						}
						self._dynamicActions = self._dynamicActions.concat(ret); 
						self._actionsType = "VIEW";
						self._updateToolbar();
						if(self.cqRecordData) self.cqRecordData.actions = data.actions; // override
					}
				};
				var initialActions = this.cqRecordData ? this.cqRecordData.actions : null; 
				if(refresh || !initialActions){
					if(toolbar){
						var loadingIcon = this.loadActIcon;
						if(!loadingIcon){
							loadingIcon = this.loadActIcon = dojo.doc.createElement("td");
							loadingIcon.vAlign = "bottom";
							var imgUrl = dojo.moduleUrl("com.ibm.rational.cm.web", "ui/images/loading16.gif");
							loadingIcon.innerHTML = "<img src='" + imgUrl + "'>";
						}
						target.domNode.parentNode.parentNode.appendChild(loadingIcon);
					}
					this._getActionsXHR = com.ibm.rational.cm.web.ui.view.record.CqRecord.callXHR(cq.xhrGet, {
						url: RecordDetailRESTServiceURL,
						content: {
							action: "GetActions",
							resourceId: this.stableLocation
						},
						preventCache: true,
						handle: function(){
							delete self._getActionsXHR;
							if(loadingIcon && loadingIcon.parentNode){
								loadingIcon.parentNode.removeChild(loadingIcon);
							}
						},
						load: setActions
					}, function(data){
						if(data && data.STATUS == "false" && data.ReasonCode == "not-found"){
							return false; // open warning dialog by another method.
						}
						return true;
					});
				}else{
					setActions(this.cqRecordData);
				}
			}
		},
		/*
		 * 
		 */
		_destroyDynamicActions: function() {
			for (var i=0,l=this._dynamicActions.length; i<l; i++) {
				this._dynamicActions[i].destroy();
			}
			this._dynamicActions = []; // clear
			delete this._actionsType;
		},
		/*
		 * Build the CqRecord
		 */
		_buildLayout:function(){
			this._controls = [];
			var formContainer = this._formContainer;
			var formMain = this.formMain;
			if (formContainer) {
				formMain.removeChild(formContainer);
				if(formContainer.domNode){
					formContainer.destroyRecursive();
				}
				formContainer = null;
			}
			var formLayout = this.cqFormLayout;
			if (formLayout){				
				// main container
				var form = formLayout.mForm;
				if(!form || !form.mFormPage || form.mFormPage.length == 0){
					formContainer = new dijit.layout.ContentPane({region: "center"});
					formContainer.setContent(this._messages.noFormPermitted);
					dojo.addClass(formContainer.domNode, "cqNoForm");
					formMain.addChild(formContainer);					
				}else if(this.custom) {
					formContainer = this._createTableForm(formMain);
				} else {
					formContainer = this._createNormalForm(formMain);
				}
			}
			this._formContainer = formContainer;
		},
		
		/*
		 * 
		 */
		_createNormalForm: function() {
			var tabContainer = new dijit.layout.TabContainer({tabPosition:"top", region: "center", splitter:true});
			var tablist = tabContainer.tablist
			var oldonkeypress = dojo.hitch(tablist, tablist.onkeypress); 
			tablist.onkeypress = function(e){
				if(e.ctrlKey && (e.keyCode == dojo.keys.LEFT_ARROW || e.keyCode == dojo.keys.RIGHT_ARROW)){
					return;
				}
				oldonkeypress(e);
			};			
			// TOOD: this is just work around for scrollbar problem of dojo0.9 on firefox. 
			// Remove when we move to dojo 1.0
			if (dojo.isMozilla) {
				tabContainer.domNode.style.overflow = "hidden";
			}
			//
			var formDefinition = this.cqFormLayout.mForm;
			var formHeight = Math.round(formDefinition.height * this.yScaleFactor) + "px";
			var formWidth = Math.round(formDefinition.width * this.xScaleFactor) + "px";
			if (this._tabIndex >= formDefinition.mFormPage.length) {
				this._tabIndex = 0; // something wrong. reset
			}
			if(dojo.isIE){
				this.formMain.addChild(tabContainer);
			}
			var selectedPage = null;
			for(var i=0,l=formDefinition.mFormPage.length; i<l; i++){
				var formPage = formDefinition.mFormPage[i];	
				// TAB page
				var caption = this._filterCharsForCaption(formPage.caption);
				var tabPage = new dijit.layout.ContentPane({title: caption, tabIndex: i, closable: false });
				var formDomNode = tabPage.domNode;
				dojo.addClass(formDomNode, "cqFormPage");
				formDomNode.style.width = formWidth;
				formDomNode.style.height= formHeight;
				tabContainer.addChild(tabPage);
				// create controls
				var controlsArray = [];
				if (i == this._tabIndex) {
					// create only controls in the previous selected tab
					if(dojo.isIE){
						tabContainer.selectChild(tabPage);
					}else{
						tabPage.selected = true;
					}	
					this._createControls(tabPage.domNode, formPage.mFormItem, tabPage, controlsArray, false);
				}
				this._controls.push({tabName: caption, controls: controlsArray, invalidControls: []});
			}
			this.connect(tabContainer, "selectChild", "_tabSelected");
			if(!dojo.isIE){
				this.formMain.addChild(tabContainer);
			}
			return tabContainer;
		},
		
		_tabSelected: function(page) {
			var index = page.tabIndex;
			var controls = this._controls[index];
			var numc = controls.controls.length; 
			if (numc == 0) {
				this._createControls(page.domNode, this.cqFormLayout.mForm.mFormPage[index].mFormItem, page, controls.controls, false);
			}
			if(this._loadedTabs && !this._loadedTabs[index]){
				this._loadData({tabIndex: index});
			}else if(!controls.dataUpdated) {
				this.updateRecordData(this.cqRecordData, this.cqState, index);
			} else {
				for(var i=0; i<numc; i++){
					var c = controls.controls[i];
					if(c.onVisible) c.onVisible();
				}
			}
			this._tabIndex = index;
			this.layout(); // tab change might cause wrap tabs 
		},
		
		/*
		 * 
		 */
		_createTableForm: function() {
			var container = new dijit.layout.ContentPane({region: "center",splitter:true});
			container.domNode.style.position = "relative"; // important ... same technich with StackContainer 
			if(dojo.isIE){
				this.formMain.addChild(container);
			}
			var tableForm = new com.ibm.rational.cm.web.ui.view.record.TableForm();
			this.connect(tableForm, "onCustom", "openCustomizeDialog");
			// body
			container.setContent(tableForm.domNode);
			var doc = dojo.doc;
			for(var i=0,l=this.cqFormLayout.mForm.mFormPage.length; i<l; i++){
				var formPage = this.cqFormLayout.mForm.mFormPage[i];					
				var caption = this._filterCharsForCaption(formPage.caption);
				var tr = doc.createElement("tr");
				var td = doc.createElement("td");
				if (dojo.isIE) {
					td.innerText = caption;
				} else {
					td.innerHTML = caption;
				}
				dojo.addClass(td, "pageCaption");
				tr.appendChild(td);
				td.colSpan = "2";
				tableForm.containerNode.appendChild(tr);
				// create controls
				var controlsArray = [];
				var hiddenList = this._hiddenList ? this._hiddenList : [];  
				this._createControls(tableForm.containerNode, formPage.mFormItem, null, controlsArray, true, hiddenList[caption+i]);
				if (!tr.nextSibling) {
					// no control is under this category...
					tr.style.display = "none";
				}
				this._controls.push({tabName: caption, controls: controlsArray, invalidControls: []});
			}
			if(!dojo.isIE){
				this.formMain.addChild(container);
			}
			return container;
		},
		
		_classmap: {},
		_initClassmap: function() {
			dojo.mixin(this._classmap, { 
				"0": com.ibm.rational.cm.web.ui.view.record.CqButton, 		// FORM_BUTTON
				"1": com.ibm.rational.cm.web.ui.view.record.CqStaticText,	// FORM_STATICTEXT
				"2": com.ibm.rational.cm.web.ui.view.record.CqFormEdit,		// FORM_EDIT
				"3": com.ibm.rational.cm.web.ui.view.record.CqGroupBox,		// FORM_GROUP
				"4": com.ibm.rational.cm.web.ui.view.record.CqCheckBox,		// FORM_CHECKBOX
				"5": com.ibm.rational.cm.web.ui.view.record.CqOptionGroup,	// FORM_OPTIONGROUP
				"6": com.ibm.rational.cm.web.ui.view.record.CqRadioButton,	// FORM_OPTIONBUTTON
				"7": com.ibm.rational.cm.web.ui.view.record.CqListBox,		// FORM_LISTBOX 
				"8": com.ibm.rational.cm.web.ui.view.record.CqComboBox,		// FORM_COMBOBOX
				"9": com.ibm.rational.cm.web.ui.view.record.CqEditableCombo,	// FORM_DROPCOMBOBOX
				"10":com.ibm.rational.cm.web.ui.view.record.CqFilteringSelect, // FORM_DROPLISTBOX
				"11":null,						// FORM_PICTURE
				"12":com.ibm.rational.cm.web.ui.view.record.CqListControl,	// FORM_LISTCONTROL
				"13":null,						// FORM_ACTIVEX_CONTROL
				"20":com.ibm.rational.cm.web.ui.view.record.CqListControl,	// FORM_HISTORY
				"21":com.ibm.rational.cm.web.ui.view.record.CqAttachment,	// FORM_ATTACHMENT
				"22":com.ibm.rational.cm.web.ui.view.record.CqDuplicateBase,	// FORM_DUPLICATE_BASE
				"23":com.ibm.rational.cm.web.ui.view.record.CqDuplicates,	// FORM_DUPLICATES_DEPENDENT
				"24":null						// FORM_SQUID_CHANGE_SET
			});
		},

		/*
		 * 
		 * return {control, captionNode}
		 */
		_newControl: function(controlInfo, customMode, hiddenList){
			var itemType = controlInfo.itemType;
			if(this.cqState == "BATCH"){
				if(itemType!="2"){
					return null;
				}
				hiddenList = null; // ignore hiddenList
			}
			var props = {
					view: this,
					itemType: itemType,
					fieldSource: controlInfo.fieldSource,
					controlId: controlInfo.name,
					webDependentFields: controlInfo.webDependentFields,
					caption: this._filterCharsForCaption(controlInfo.caption) // used by customize dialog, CqGroup and CqListControl.newResource
					};
			if(controlInfo.isAutoSort === "1"){props.isAutoSort = true;}
			if(!this._classmap["0"]) {
				this._initClassmap();
			}
			var clazz = this._classmap[itemType];
			switch(itemType) {
			case "0": // FORM_BUTTON
				if (controlInfo.preClickHook){
					props.preClickHook = controlInfo.preClickHook;
					props.isPreclickHookEnabledForWeb = controlInfo.isPreclickHookEnabledForWeb; 
				}
				if (controlInfo.postClickHook){
					props.postClickHook = controlInfo.postClickHook;
					props.isPostclickHookEnabledForWeb = controlInfo.isPostclickHookEnabledForWeb;
				} 					
				props.associatedControlName = controlInfo.associatedControlName;
				props.btnType = controlInfo.btnType;
				break;
			case "1": // FORM_STATICTEXT
				props.value = props.caption;
				break;
			case "2": // FORM_EDIT
				if(controlInfo.isPassword == "1") props.isPassword = true;
				if(controlInfo.isMultiLine == "1") props.isMultiLine = true;
				if(controlInfo.isVScrollbar != "0") props.isVScr = true;
				if(controlInfo.isAutoVScroll != "0") props.isAutoVScr = true;
				if(controlInfo.isHScrollbar != "0") props.isHScr = true;
				if(controlInfo.isAutoHScroll != "0") props.isAutoHScr = true;
				if(controlInfo.formDate) props.formDate = controlInfo.formDate; 
				break;
			case "4": // FORM_CHECKBOX
				props.checkedValue = controlInfo.value; 
				props.unCheckedValue = controlInfo.unCheckedValue; 
				break;
			case "6": // FORM_OPTIONBUTTON
				props.name = controlInfo.grpName;
				props.value = controlInfo.value;
				break;
			case "12": // FORM_LISTCONTROL
				if(controlInfo.columnInfo) props.columnInfo = controlInfo.columnInfo;
				break;	
			case "13": // FORM_ACTIVEX_CONTROL, lets see if its a changeset control TODO
				/**
				* ActiveX-type controls on a ClearQuest form have an ID string as part of the form schema.
				* If the ActiveX-type control is a special change set control which was added from the UCM package
				* as part of SQUID (UCM-ClearQuest integration), it will have this special ID string
				* "ClearCase.ActivityChangeSetCtrl.1"
				if (controlInfo.fieldSource == "ClearCase.ActivityChangeSetCtrl.1"){ */
				if (this.isUcmEnabled && this.isCCSupported && controlInfo.name == "ClearCase.ActivityChangeSetCtrl.1"){
					//  Removed code to instantiate/use the CQChangeSet widget.  It will
					//  need to be added at a later time to support UCM
					//
					//  If customMode (listview) dont think we need to warn user.  That  
					//  user may not care about UCM in this case whereas a user who 
					//  clicks on the UCM tab generally does care. 										
					if((!customMode) && (!dontShowUCMWarningDialog)){
 						cq.openMessageDialog(this._messages.cqUCMIntegrationUnsupported);
						dontShowUCMWarningDialog = true;
						dojo.cookie(COOKIE_NAME_FOR_UCMWARNING, dontShowUCMWarningDialog, { expires: 0 });
 					}

				}
				break;
			case "20":// FORM_HISTORY
				props.fieldSource = "history";// special care for "history"
				props["class"] = "formHistory";
				break;
			case "22":// FORM_DUPLICATE_BASE
				props["class"] = "formDuplicateBase";
				break;
			case "23":// FORM_DUPLICATES_DEPENDENT
				props["class"] = "formDuplicateDependent";
				break;
			}
			var control = null;
			// 87297: Adopt 7.1.0.3 fix for RATLC01063612, show form buttons in ListView
			if(customMode && (itemType == "1" || itemType == "3")) {
				control = dojo.mixin(props, {controlInfo:controlInfo, cqInvisible: "always"});
			}else if(customMode && hiddenList && hiddenList[props.fieldSource + "::" + itemType + "::" + props.controlId]) {
				control = dojo.mixin(props, {controlInfo:controlInfo, cqInvisible: "custom"});
			}else if(clazz){
				control = new clazz(props);
			}
			var info = {control:control};
			if (control && control.cqInvisible != "always" && control.cqInvisible != "custom"){
				control.setCqState(this.cqState);
				if(control.callHook && this.cqState != "VIEW"){
					control.callHookHandler = this.connect(control, "callHook", this._callHook); 
				}
				if (control.domNode) {
					var i = this._positionControl(controlInfo, control, customMode);
					if(i.wrapNode){
						control.wrapNode = i.wrapNode;
					}
					if(i.batchActionNode){
						control.batchActionNode = i.batchActionNode;
					}
				}
			}
			return info;
		},
		
		// 87297: Adopt 7.1.0.3 fix for RATLC01063612, layout listcontrol and return new parent node for its children items
		_layoutListControl: function(control){			
			var table = document.createElement("table");
			var tbody = table.appendChild(document.createElement("tbody"));
			
			var td = control.domNode.parentNode;
			td.removeChild(control.domNode);
			td.appendChild(table);
			
			var newtr = tbody.insertRow(0); 
			var lefttd = newtr.insertCell(0);
			var righttd = newtr.insertCell(1);
			lefttd.appendChild(control.domNode);
			
			var table = document.createElement("table");
			var tbody = table.appendChild(document.createElement("tbody"));
			righttd.appendChild(table);
			
			return tbody;
		},
		
		_createControls: function(parentNode, formItems, tabPage, controlArray, customMode, hiddenList){
//			console.log("CqRecord._createControls is called");
			for(var i=0,l=formItems.length; i<l; i++){
				var formItem = formItems[i];
				var info = this._newControl(formItem, customMode, hiddenList);
				
				// 87297: Adopt 7.1.0.3 fix for RATLC01063612, assign new parentNode to ListControl's child items
				var newParentNode = null;
				
				if(info && info.control){
					var control = info.control;
					if(control.domNode){
						parentNode.appendChild(control.wrapNode || control.domNode);
						var cap = control.captionNode; 
						if(cap && cap.notAppended){
							parentNode.appendChild(cap);
							cap.notAppended = false;
						}
						control.startup();
					
						// 87297: Adopt 7.1.0.3 fix for RATLC01063612, reLayout ListControl
						if(customMode && formItem.itemType == "12"){
							newParentNode = this._layoutListControl(info.control);
						}
					}else{
						control.parent = parentNode; // used in _updateControls
					}
					control.tabPage = tabPage; // for _validateByControl
					controlArray.push(control);
				}				
				// sub components
				if (formItem.buttons) {
					// 87297: Adopt 7.1.0.3 fix for RATLC01063612, assign new parentNode to ListControl's child items
					this._createControls(newParentNode || parentNode, formItem.buttons, tabPage, controlArray, customMode, hiddenList);
				}
				if (formItem.optButtons) {
					this._createControls(parentNode, formItem.optButtons, tabPage, controlArray, customMode, hiddenList);
				}
				if (formItem.items) {
					this._createControls(parentNode, formItem.items, tabPage, controlArray, customMode, hiddenList);
				}
			}		
		},
		
		_destroyControls: function() {
			this._forEachControl(function(c){
				if (c.domNode) {
					c.destroyRecursive();
				} 
			});
			this._controls = null;
		},
		
		/*
		 * retrieves the Data type for the given field Source
		 */
		_getControlTypeForField:function(sFieldName){
			for (var objIndex in this.cqRecordData.fields){
				if ( this.cqRecordData.fields[objIndex].FieldName == sFieldName){
					return this.cqRecordData.fields[objIndex].DataType;
				}
			}
			return "";
		},

		/*
		 * TOOD: We should treat & as real mnemonic for keyboard accessibility and usability
		 */
		_filterCharsForCaption:function(sValue){
			if (!sValue || sValue.length == 0) {
				return "";
			}
			return com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(sValue.replace(/&/g,""));
		},
		
		/*
		 * 
		 * return {wrapNode, captionNode}
		 */
		_positionControl:function(mFormItem, control, customMode){
			var xFactor = this.xScaleFactor;
			var yFactor = this.yScaleFactor;
			// Size
			control.setSize(Math.round(mFormItem.width*xFactor), Math.round(mFormItem.height*yFactor));
			// Position
			var ret = {};
			if (customMode) {
				var doc = dojo.doc;
				var tr = doc.createElement("tr");
				// caption
				var td = doc.createElement("td");
				dojo.addClass(td, "fieldCaption");
				tr.appendChild(td);
				var captionNode = this._createCaption(mFormItem, control, customMode);
				if(captionNode){
					td.appendChild(captionNode);
					control.captionNode = captionNode;
				}
				// data
				td = doc.createElement("td");
				dojo.addClass(td, "fieldCell");
				td.appendChild(control.domNode);
				tr.appendChild(td);
				// action for Bacth update
				if(this.cqState == "BATCH"){
					td = doc.createElement("td");
					dojo.addClass(td, "fieldBatchAction");
					var select = doc.createElement("select");
					var msgs = this._messages;
					select.options[0] = new Option(msgs.batchActionReplace, "replace");
					select.options[1] = new Option(msgs.batchActionPrefix, "prefix");
					select.options[2] = new Option(msgs.batchActionAppend, "append");
					td.appendChild(select);				
					tr.appendChild(td);
					ret.batchActionNode = select;
				}
				ret.wrapNode = tr;
			}else{
				var style = control.domNode.style; 
				style.position = "absolute";
				style.top = Math.round(mFormItem.y*yFactor)+"px";
				style.left = Math.round(mFormItem.x*xFactor)+"px";
				var captionNode = this._createCaption(mFormItem, control, customMode);
				if(captionNode){
					captionNode.notAppended = true;
					control.captionNode = captionNode;
				}
			}
			return ret;
		},

		/*
		 * 
		 */
		_createCaption: function(mFormItem, control, customMode){
			if (mFormItem.hasSeparateCaption == "1"){
				var cap = dojo.doc.createElement("label");
				cap.htmlFor = (control.focusNode||control.domNode).id;
				cap.id = "cap_" + control.id; 
				if (!customMode) {
					// dojo.addClass(spanObj, "dijitFormWidgetDisabled");
					var xFactor = this.xScaleFactor;
					var yFactor = this.yScaleFactor;
					var s = cap.style; 
					var capRect = mFormItem.captionRect;
					s.position = "absolute";
					s.left=Math.round(capRect.x*xFactor)+"px";
					s.top=Math.round(capRect.y*yFactor)+"px";
					s.width=Math.round(capRect.width*xFactor)+"px";
					s.height=Math.round(capRect.height*yFactor)+"px";
				}
				dojo.addClass(cap, "captionNode");
				cap.innerHTML=this._filterCharsForCaption(mFormItem.caption);
				return cap;
			}	
			return null;
		},
		
		/*
		 * 
		 */
		setCustom: function(custom) {
			if (this.custom != custom) {
				this.custom = custom;
				if(this._loadedPartially) this._loadData();
				this.updateCQForm(this.cqFormLayout, this.cqState, true);
			}
		},
		
		/*
		 * 
		 */
		setCqState: function(newState){
			if (newState && this.cqState != newState) {
				var isBatch = newState=="BATCH"; 
				if(isBatch || this.cqState == "BATCH"){
					this.showRightPane(!isBatch);
				}
				this.cqState = newState;
				if(isBatch){
					this.custom = true;
				}
			}
		},

		/*
		 * 
		 */
		showRightPane: function(show){
			var rightPane = this._rightPane; 
			if(rightPane){
				var p = this.formMain;
				var added = rightPane.getParent();
				if(show){
					if(!added){
						p.addChild(rightPane);
					}
				}else{
					if(added){
						p.removeChild(rightPane);
					}
				}
			}
		},	
		/*
		 * 
		 */
		updateCQForm: function(formLayout, newState, force) {
//			console.log("CqRecord.updateCQForm is called: state=["+newState+"]");
			if (newState && this.cqState != newState) {
				this.setCqState(newState);
			}
			if(force || this.cqFormLayout != formLayout){
				this.cqFormLayout = formLayout;
				this._buildLayout();
				this.updateRecordData(this.cqRecordData, newState);
			}
		},
		
		/*
		 * 
		 * args: tabIndex or fields
		 */
		_getRecordVersion: function(data){
			var v = this.findFieldDataFor("version", data);
			return v ? v.CurrentValue : null; 
		},
		_loadData: function(args){
			this.setLoading(true);
			args = args || {};
			var state = this.cqState;
			if(state != "VIEW" && state != "SUBMIT"){
				state = "MODIFY";
			}
			var content = dojo.mixin(args, {action: "GetCQRecordDetails", state:state, requestBasicData: "false", resourceId: this.stableLocation});
			com.ibm.rational.cm.web.ui.view.record.CqRecord.callXHR(cq.xhrGet, {
				url: RecordDetailRESTServiceURL,
				preventCache: true,
				content: content,
				load: dojo.hitch(this, function(data, ioArgs){
					if(!isNaN(args.tabIndex) && args.tabIndex >= 0){
						if(data && this._getRecordVersion(data.fields) != this._getRecordVersion()){
							cq.openMessageDialog(this._messages.recordUpdated, null, null, null);
							this._loadedTabs = []; // clear all tab
							this.createActions(true); // re-create actions
						}
						this._loadedTabs[args.tabIndex] = true;
					}else if(args.fields){
					}else{
						// all data has been loaded
						this._loadedPartially = false; 
					}
					this._updateByHook(data.fields);
				}),
				handle: dojo.hitch(this, function(response, ioArgs){
					this.setLoading(false);
				})
			}, dojo.hitch(this, function(data){
				if(data && data.STATUS == "false" && data.ReasonCode == "not-found"){
					var dlg = cq.openMessageDialog(this._messages.recordDeletedSinceViewed);
					this.connect(dlg, "hide", function(){
						this._fireAction({actionString:"CLOSE"});
					});
					return false;
				}
				return true;
			}));
		},
		
		updateRecordData: function(recordData, newState, tabIndex, partial){
//			console.log("CqRecord.updateRecordData is called. data=" + recordData + ",newState=" + newState + ",tabIndex=" + tabIndex);
			if(this.cqRecordData != recordData){
				// initial load
				if(!isNaN(tabIndex) && tabIndex >= 0){
					this._loadedPartially = true;
					this._loadedTabs = [];
					this._loadedTabs[tabIndex] = true;
				}else{
					this._loadedPartially = partial;
					if(partial && newState != "VIEW"){
						this._loadData();
					}
					delete this._loadedTabs;
					tabIndex = undefined;
				}
				this.cqRecordData = recordData;
				var initialLoad = true;
			}
			if (newState && this.cqState != newState) {
				this.setCqState(newState);
			}
			this._updateTitle();
			this._updateTooltip();
			if(!this.cqFormLayout) {
				return;
			}
			if (recordData){
				if (recordData.StableLocation) {
					this._setStableLocationId(recordData.StableLocation);
				}
				var fields = recordData.fields;
				// only at first time
				if (initialLoad) {
					setTimeout(dojo.hitch(this, function(){
						// validate tabs
						this._validateTabs();
						this._needRebuildResMenu = true;
						// restricted mode also allow submit button
						// move the restrict logic into _createActions
						this._destroyDynamicActions();
						this.createActions();
						// update related resources
						this._resources = [];
						this._buildTitleMenu();
						//this._buildContextMenu();
					}),0);
					delete this._dupInfo; // clear
				}
				// update controls after creating action. 
				// Because even if exception occurs in _updateControl, action should be updated... 
				this._updateControls(fields, false, false, false, false, tabIndex);
			}
			this._updateToolbar();
		},
		
		/*
		 * fieldArray must be array of {FieldName, ControlValue, DataType}
		 */
		updateControls: function(fieldArray){
			var updatefields = dojo.filter(fieldArray, function(f){
				var target = this.findFieldDataFor(f.FieldName);
				// return target && target.Requiredness != "READ_ONLY";
				return !target || target.Requiredness != "READ_ONLY"; // RATLC01362020 
			}, this);
			this._updateControls(updatefields, false, true, true, true);
			this._updateModel(updatefields, "FieldName", "ControlValue", false, true);
			var url = RecordDetailRESTServiceURL + "?action=RecordUpdate&resourceId=" + encodeURIComponent(this.stableLocation);
			// RATLC01477352
			// Add an argument to indicate it's template case.
			this._callHook(null, url, null, null, null, true, true); // fire hook
		},
		
		findFieldDataFor: function(fieldName, argFieldArray){
			var fieldArray = argFieldArray || this.cqRecordData.fields; 
			if(fieldName){
				for(var i=0, l1=fieldArray.length; i<l1; i++){
					var fieldData = fieldArray[i];
					var n = fieldData.FieldName || fieldData.Name;
					if(n == fieldName){
						return fieldData;
					}
				}
			}
			return null;
		},
		
		_updateControls: function(fieldArray, requirednessOnly, setDataOnly, separateModel, userUpdate, tabIndex) {
			var hideControl = function(control){
				if(control.domNode){
					(control.wrapNode || control.domNode).style.display = "none";
					if(control.captionNode){
						control.captionNode.style.display = "none";
					}
				}
			};
			var showHideCustomField = function(self, control, fieldData, prevSibling){
				if(!fieldData || fieldData.Requiredness != "MANDATORY"){
					hideControl(control);
					return null;
				}
				if(control.domNode){
					(control.wrapNode || control.domNode).style.display = "";
					if(control.captionNode){
						control.captionNode.style.display = "";
					}
				}else{
					var info = self._newControl(control.controlInfo, true);
					if (!info || !info.control || info.control.cqInvisible) {
						return null;
					}
					if (info.control.domNode) {
						var parentNode = control.parent; // must be set in _createControls
						var nextNode = prevSibling ? (prevSibling.wrapNode || prevSibling.domNode).nextSibling : null;
						var c = info.control; 
						parentNode.insertBefore(c.wrapNode || c.domNode, nextNode);
						if(c.captionNode && c.captionNode.notAppended){
							parentNode.appendChild(c.captionNode);
							c.captionNode.notAppended = false;
						}
						c.startup();
					}
					info.control.cqInvisible = control.cqInvisible; // keep the visibility for customize dialog
					control = info.control;
				}
				return control;
			};
			// loop for each field and update the values.
			var invalidFields = this.cqRecordData ? this.cqRecordData.InvalidFields : null; 
			var isBatch = (this.cqState=="BATCH");
			var prevSibling = null;
			for (var i = 0, l1 = this._controls.length; i < l1;  i++) {
				var controlsInTab = this._controls[i];
				if ((tabIndex === undefined || i == tabIndex) && controlsInTab.controls.length > 0) {
					controlsInTab.dataUpdated = true;
					for (var j = 0, l2 = controlsInTab.controls.length; j < l2;  j++) {
						var control = controlsInTab.controls[j];
						var fieldData = this.findFieldDataFor(control.fieldSource, fieldArray);
						// visibility 
						if(isBatch && fieldData && fieldData.Requiredness == "READ_ONLY"){
							hideControl(control);
							continue;
						}
						if(control.cqInvisible){
							control = showHideCustomField(this, control, fieldData, prevSibling);
							if(!control){
								continue;
							}
							// control may be recreated.
							controlsInTab.controls[j] = control;
						}
						prevSibling = control;
						// connect for hook
						if(this.cqState != "VIEW" && this.cqState != "BATCH" && !control.callHookHandler){
							control.callHookHandler = this.connect(control, "callHook", this._callHook);
						}
						//  set state
						//
						//  For Req-enabled entity disable the RequirementsPro and Refresh
						//  buttons.
						//  Refresh button has pre-click hook named CQI_RefreshReq 
						//  RequisitePro button has pre-click hook named CQI_Select_Req
						var rpdisabledbutton = false;
						if (control.preClickHook){
							var leftmark = control.preClickHook.indexOf('/');
							var rightmark = control.preClickHook.indexOf('@');
							var rwebViewStringMaybe = control.preClickHook.substring(leftmark+1, rightmark);
							if ((rwebViewStringMaybe == "CQI_Refresh_Req") ||
								(rwebViewStringMaybe == "CQI_Select_Req")){
									rpdisabledbutton = true;
							}
						}
						if (rpdisabledbutton){
							control.setCqState("VIEW");
						}else {												
							control.setCqState(this.cqState);
						}
						// update choicelist before calling setFieldData
						if(!requirednessOnly && !userUpdate && control.updateChoiceList){
							// RATLC01051076 
							control.updateChoiceList();
						}
						// set CqInvalid flag. this must be done before setFieldData. RATLC01363642 
						control.setCqInvalid(invalidFields && (dojo.indexOf(invalidFields, control.fieldSource) >= 0));
						// data update
						if(fieldData){
							if(!setDataOnly){
								if(isBatch){
									fieldData.Requiredness = ""; // ignore requiredness
									fieldData.CurrentValue = ""; // ignore value
									delete fieldData.ControlValue; // not set "" to ControlValue, because "" for ControlValue is regarded as updated value in updateModifiedList // RATLC01062296  
								}
								if(control.setDisabled){
									// Always enabled in "VIEW" state to allow user to focus on the control
									control.setDisabled(fieldData.Requiredness == "READ_ONLY");
								}
								if(control.setMandatory){
									control.setMandatory(fieldData.Requiredness == "MANDATORY");
								}
							}
							// set data
							if(!requirednessOnly){
								// visible control must have setField method
								control.setFieldData(fieldData, separateModel);
								control.setHasHelp(fieldData.hasHelpText);
							}
						}
						// revalidate even if "VIEW" state
						// e.g. "MODIFY" -> (revert) -> "VIEW" 
						if(control.validateCtrl){
							control.validateCtrl();
						}
					}
				} else if (!tabIndex){
					// tab index is not specified:
					// it means all tab should be updated, but controls for this tab are not created yet.
					// lazy update is necessary.
					controlsInTab.dataUpdated = false;
				}
			}
		},
		
		_forEachControl: function(func,tabpage){
			var f = dojo.hitch(this, func);
			for (var i = 0, l1 = this._controls.length; i < l1;  i++) {
				var controlsInTab = this._controls[i];
				if(!tabpage || i === tabpage.tabIndex){
					for (var j = 0, l2 = controlsInTab.controls.length; j < l2;  j++) {
						var c = controlsInTab.controls[j];
						f(c);
					}
				}
			}
		},
		
		_setReadonly: function(readonly) {
			if(this._coverDiv){
				var s = this._coverDiv.style;
				if(readonly){
					s.visibility = "";
					s.width = "100%";
					s.height = "100%";
					dojo.style(this._coverDiv, "opacity", "0.2");
				}else{
					s.visibility = "hidden";
				}
			}
		},

		/*
		*
		*/
		_cmpFD: function(f1, f2){
			var ff1 = (f1.FieldName || "").toLowerCase();
			var ff2 = (f2.FieldName || "").toLowerCase();
			return ff1.localeCompare(ff2);
		},
		/*
		* build menus 
		*/
		_buildOpenRelatedMenu: function(parentMenu){
			var subMenu = new dijit.Menu({parentMenu:parentMenu});
			var self = this;
			subMenu.connect(subMenu, "onOpen", function(){
				if(!subMenu.domNode || !self._needRebuildResMenu || self.callingGetReferencedRecords) return;
				subMenu.destroyDescendants();
				subMenu.addChild(new dijit.MenuItem({iconClass: "cqLoadingImg_small", label: self._messages.loading}));
				self.callingGetReferencedRecords = com.ibm.rational.cm.web.ui.view.record.CqRecord.callXHR(cq.xhrGet, {
					url: RecordDetailRESTServiceURL,
					content: {action: "GetReferencedRecords", resourceId: self.stableLocation, state: self.cqState},
					handle:function(){
						// This is a workaround for what seems to be dojo Menu bug
						// without this, error occurs when putting mouse over loading menu, and hovering over other menus after menus are loaded
						// this is caused by Menu trying to set focus to savedFocus in _openMyself
						subMenu.focusFirstChild(); // RATLC01264682
						delete self.callingGetReferencedRecords;
					},
					load: function(data, ioArgs){
						self._needRebuildResMenu = false;
						if(!subMenu.domNode)return; // maybe record view is closed ...
						subMenu.destroyDescendants();
						subMenu.focusedChild = null; // RATLC01264682
						var resources = data.fields || [];
						resources = resources.sort(self._cmpFD);
						for (var i=0, l=resources.length; i < l; i++) {
							var fieldData = resources[i];
							if (fieldData.DataType == "RESOURCE_LIST" && fieldData.CurrentValue) {
								var numSubResources = fieldData.CurrentValue.length;
								if (numSubResources > 0) {
									var subResources = [];
									for (var j = 0; j < numSubResources; j++) {
										var v = fieldData.CurrentValue[j];
										if(v.RecordId){
											subResources.push(v);
										}
									}
									if(subResources.length > 0){
										var subSubMenu = new dijit.Menu({parentMenu:subMenu, _cqSubRes:subResources});
										subSubMenu.connect(subSubMenu, "onOpen", function(){
											var children = this.getChildren();
											if(children && children.length > 0) return; // already added
											for (var k=0,kl=this._cqSubRes.length; k < kl; k++) {
												var r = this._cqSubRes[k];
												this.addChild(com.ibm.rational.cm.web.ui.view.record.CqRecord._createMenuItemForCQResource(r.DisplayName, r.RecordId));
											}
										});
										self._addPopupMenu(subSubMenu);
										subSubMenu.startup();
										var menuItem = new dijit.PopupMenuItem({label:fieldData.FieldName, popup:subSubMenu});
										menuItem.startup();
										subMenu.addChild(menuItem);
									}
								}
							} else if (fieldData.RecordId){
								subMenu.addChild(com.ibm.rational.cm.web.ui.view.record.CqRecord._createMenuItemForCQResource(
																		fieldData.FieldName + " : " + fieldData.CurrentValue,
																		fieldData.RecordId));
							}
						}
						if(!subMenu.hasChildren()){
							subMenu.addChild(new dijit.MenuItem({label: self._messages.noRelatedResources}));
						}
					}});
			});
			subMenu.startup();
			return subMenu;
		},
		
		_addPopupMenu: function(menu){
			if(!this._popupMenus) this._popupMenus = [];
			this._popupMenus.push(menu);
		},
		
		_destroyTitleMenu: function(){
			if(this._resourceButton && this._resourceButton.domNode){
				this._resourceButton.destroyRecursive();
				this._resourceButton = null;
			}
			if (this._titleMenu && this._titleMenu.domNode) {
				this._titleMenu.destroyRecursive();
				this._titleMenu = null;
			}
		},
		
		_buildTitleMenu: function(){
			if(this.cqState=="BATCH"){
				// RATLC01233279: this method is called after action is created. so it is change to re-layout.
				this.layout(); 
				return;
			}
			this._destroyTitleMenu();
			// defer the actual creation for performance
			setTimeout(dojo.hitch(this, function(){
				this._destroyTitleMenu();
				this._titleMenu = new dijit.Menu();
				var resourceMenu = this._buildOpenRelatedMenu(this._titleMenu);
				this._addPopupMenu(resourceMenu);
				var menuItem = new dijit.PopupMenuItem({label:this._messages.openRelatedResource, popup:resourceMenu});
				this._titleMenu.addChild(menuItem);
				// create button 
				this._resourceButton = new dijit.form.DropDownButton({dropDown:this._titleMenu, label:this._messages.openRelatedResource, showLabel: false, iconClass: "cqrecordMenuIcon_openRecord"});
				var s = this._resourceButton.domNode.style;
				s.margin = "0px"; // override dijit.css
				s = this._resourceButton.focusNode.style;
				s.background = "transparent";
				s.border = "none";
				s.padding = "0px";
				this.formTitle.appendChild(this._resourceButton.domNode);
				this._titleMenu.startup();
				this.layout(); // size of this.formTitle might be changed, then need relayout  
			}), 0);
		},
		
		_clearMenu: function(menu){
			if (menu) {
				menu.unBindDomNode(this.domNode); // this is not necessary after at least dojo 1.1
				menu.destroyRecursive();
			}
		},
/*		
		_buildContextMenu: function(){
			this._clearMenu(this._contextMenu);
			var menu = new dijit.Menu({targetNodeIds:[this.domNode]});
			// dynamic actions
			this._createActions(menu);
			// open related resource
			var resourceMenu = this._buildOpenRelatedMenu(menu);
			if(resourceMenu){
				menu.addChild(new dijit.PopupMenuItem({label:this._messages.openRelatedResource, popup:resourceMenu}));
			}
			// Close
			var closemenu = new dijit.MenuItem({label:this._messages.close, iconClass: "cqrecordMenuIcon_close"});
			closemenu.onClick = dojo.hitch(this, function(){
				this._fireAction({actionString:"CLOSE"});
			});
			menu.addChild(closemenu);
			//
			menu.startup();
			this._contextMenu = menu;
		},
*/
		/*
		 * 
		 */
		findControlById:function(id,tabPage) {
			if(id){
				var findInTab = function(tab){
					for (var j = 0, l2 = tab.controls.length; j < l2;  j++) {
						var c = tab.controls[j];
						if (c.controlId == id) {
							return c;
						}
					}
					return null;
				};
				if(tabPage){
					return findInTab(this._controls[tabPage.tabIndex]);
				}else{
					for (var i = 0, l1 = this._controls.length; i < l1;  i++) {
						var c = findInTab(this._controls[i]);
						if(c){
							return c;
						}
					}
				}
			}	
			return null;
		},
		
		findControlsFor:function(fieldName) {
			var controls = [];
			if(fieldName){
				this._forEachControl(function(c){
					if (c.fieldSource == fieldName) {
						controls.push(c);
					}
				});
			}	
			return controls;
		},

		findControlItemsFor:function(fieldName) {
			var items = [];
			var findInControls = function(controls){
				for (var j = 0, l2 = controls.length; j < l2;  j++) {
					var c = controls[j];
					if(c.fieldSource == fieldName) {
						items.push(c);
					}
					if(c.items){
						findInControls(c.items);
					}
				}				
			}
			if (fieldName && this.cqFormLayout) {
				var formPages = this.cqFormLayout.mForm.mFormPage;
				for (var i = 0, l1 = formPages.length; i < l1;  i++) {
					var controls = formPages[i].mFormItem;
					findInControls(controls);
				}
			}
			return items;
		},

		/*
		 * 
		 */
		_validateByControl: function(control) {
			if(!control.isValidField) return;
			var isValid = control.isValidField();
			if(isValid && this.cqRecordData && this.cqRecordData.InvalidFields){
				var i = dojo.indexOf(this.cqRecordData.InvalidFields, control.fieldSource);
				if(i >= 0){
					this.cqRecordData.InvalidFields.splice(i,1);
				}
			}
			if(control.tabPage && (this._formContainer instanceof dijit.layout.TabContainer)){
				var tab = this._controls[control.tabPage.tabIndex];
				if(tab){
					if(this.cqState == "VIEW"){
						tab.invalidControls = []; 
					}else if(isValid){
						this._removeInvalidControl(control, tab);
					}else if(dojo.indexOf(tab.invalidControls, control)<0){
						tab.invalidControls.push(control);
					}
					this._setTabInvalid(control.tabPage, tab.invalidControls.length > 0);
				}
			}
		},
		_removeInvalidControl: function(control, tab){
			if(!this._controls) return;
			if(!tab && control.isValidField && control.tabPage && (this._formContainer instanceof dijit.layout.TabContainer)){
				tab = this._controls[control.tabPage.tabIndex];
			}
			var invalidControls = tab ? tab.invalidControls : null; 
			if(invalidControls && invalidControls.length > 0){
				var i = dojo.indexOf(invalidControls, control);
				if(i >= 0){
					invalidControls.splice(i,1);
				}
			}
		},

		/*
		 * 
		 */
		_validateTabs:function(){
			setTimeout(dojo.hitch(this, function(){
				if(!this.cqFormLayout || !this._formContainer || !this._formContainer.domNode || !(this._formContainer instanceof dijit.layout.TabContainer)){
					return;
				}
				var containItem = function(controls, field){
					for(var i=0,l=controls.length; i<l; i++){
						var c = controls[i];
						if(c.fieldSource == field.FieldName){
							return true;
						}
						if(c.items && containItem(c.items, field)){
							return true;
						}
					}
					return false;
				};
				var invalidFields = this.validateFields();
				var tabPages = this._formContainer.getChildren();
				var tabs = this.cqFormLayout.mForm.mFormPage;
				for(var i=0,l=Math.min(tabs.length, tabPages.length);i<l;i++){
					var controls = tabs[i].mFormItem;
					var invalid = dojo.some(invalidFields, function(f) {
						return containItem(controls, f);
					});
					this._setTabInvalid(tabPages[i], invalid);
				}
				this.layout(); // Tab might have "*" for invalid. It might cause wrap of tab. Then need relayout. 
			}));
		},
		/*
		 * 
		 */
		_setTabInvalid: function(tabPage, invalid){
			if(tabPage.cqInvalid == invalid) return;
			tabPage.cqInvalid = invalid;
			var btn = tabPage.controlButton;
			if(btn){
				btn.staticClass = invalid ? "cqInvalidTab" : null;
				if(btn.domNode){
					dojo.toggleClass(btn.domNode, "cqInvalidTab", invalid);
				}
				if(btn.containerNode){
					btn.containerNode.innerHTML = (invalid ? "<span class='invalidMark'>*</span>" : "" ) + tabPage.title;
				}
			}
		},
		/*
		 * validate requiredness only
		 */
		validateRequires:function(){
			var invalids = [];
			if(this.cqState != "VIEW"){
				invalids = dojo.filter(this.cqRecordData.fields, function(f){
					if(f.Requiredness == "MANDATORY"){
						if(f.ControlValue){
							if(f.ControlValue.length == 0){
								return true;
							}
						}else if(!f.CurrentValue || f.CurrentValue.length == 0){
							return true;
						}
					}
					return false;
				});
			}
			return invalids;			
		},
		/*
		 * validate full
		 */
		validateFields:function(){
			var invalids = [];
			if(this.cqState != "VIEW"){
				if(this.cqRecordData && this.cqRecordData.InvalidFields){
					invalids = dojo.map(this.cqRecordData.InvalidFields, function(f){
						return {FieldName: f};
					});
				}
				dojo.forEach(this._controls, function(tab){
					dojo.forEach(tab.invalidControls, function(c){
						if(c.fieldSource){
							invalids.push({TabName: tab.tabName, FieldName: c.fieldSource});
						}
					});
				});
			}
			return invalids;
		},

		_computeAllWebDepFields: function(webdeps)
		{
			var all = dojo.clone(webdeps);
			while (webdeps.length > 0) {
				var controls = this.findControlItemsFor(webdeps.shift());
				if (!controls){
					continue;
				}
				for (var i=0, l1=controls.length; i<l1; i++){
					var c = controls[i];
					var w = c.webDependentFields;
					if (w && w.length > 0) {
						for (var j=0,l2=w.length; j<l2; j++) {
							var fieldName = w[j].displayName;
							if (dojo.indexOf(all, fieldName)<0){
								all.push(fieldName);
								webdeps.push(fieldName);
							}
						}
					}
				}
			}
			return all;
		},
				
		_callHook: function(source, url, fieldData, newValue, webdeps, stopWhenNoModification, isForTemplate /* RATLC01477352 */, callback /*75755*/) {
			if(this._sessionExpired){
				var msg = this._messages;
				var error = msg.sessionExpired + "\n" + msg.sessionExpiredInstruction + "\n\n" + msg.sessionExpiredTips + "\n" + msg.sessionExpiredTipsBody;
				cq.openMessageDialog(error);
				return;
			}
			if (this._nowLoading) {
				if(!this._hookQueue) this._hookQueue = [];
				this._hookQueue.push(arguments);
				this._coverDiv.style.zIndex = "950"; // cover cqButtons
				return;
			}
			// update controls which bound to same field
			if(fieldData){
				fieldData.ControlValue = newValue;
				var controls = this.findControlsFor(fieldData.FieldName);
				for(var i=0, l=controls.length; i<l; i++){
					if(controls[i]!=source){
						controls[i].setFieldData(fieldData, true);
					}
				}
			}
			// if url is not specified, no more action is necessary
			if(!url){
				this._validateTabs();
				return;
			}
			// call hook
			this.setLoading(true);
			var deps = [];
			if (webdeps) {
				var n = webdeps.length;
				for (var i=0; i<n; i++) {
					deps.push(webdeps[i].displayName);
				}
				deps = this._computeAllWebDepFields(deps);
				var i = dojo.indexOf(deps, source.fieldSource);
				if(i>=0){
					deps.splice(i,1);
				}
			}
			this.updateModifiedList();
/* not necessary because controlValue of fieldData is updated, 
 * so updateModifiedList must have new value ... fix for RATLC01068236
			if (fieldData) {
				this._modifiedList[fieldData.FieldName] = newValue;
			}
*/
			// RATLC01477352 Defect: Conflict error, "Error from RPC server: PROPPATCH failed" when field becomes read-only in Falcon
			// If it's not template case, sort the modified list by last modified timestamp
			// If it's template case, refer 'getModifiedList' method.
			var changed
			if (isForTemplate && isForTemplate == true)
				changed = this.getModifiedList();
			else
				changed = this._getModifiedListForHook();
			
			if(stopWhenNoModification && changed.length == 0){
				this.setLoading(false);
				return;
			}
			var content = {Fields: dojo.toJson(changed), webDependentFields: dojo.toJson(deps), state: this.cqState};
			if(!this.custom && this._loadedTabs){
				var tabIndex = [];
				for(var i=0,l=this._loadedTabs.length; i<l; i++){
					if(this._loadedTabs[i]) tabIndex.push(i);
				}
				if(tabIndex.length > 0) content.tabIndex = dojo.toJson(tabIndex);
			}
			if(content.state === "BATCH"){
				content.state = "MODIFY";
			}
			
			com.ibm.rational.cm.web.ui.view.record.CqRecord.callXHR(cq.xhrPost,{
				url: url,
				content: content,
				load: dojo.hitch(this, function(data, ioArgs){
					this.clearModifiedList();
					this._updateByHook(data.UpdatedFields, data.Requiredness, data.InvalidFields);
					// 75755: Apply 2009B fix (RATLC01280491), pre-action hook synchronization
					if(callback && dojo.isFunction(callback)){
						callback();
					}
				}),
				handle: dojo.hitch(this, function(response, ioArgs){
					this.setLoading(false);
					if(this._hookQueue && this._hookQueue.length > 0){
						var args = this._hookQueue[0];
						this._hookQueue = this._hookQueue.slice(1);
						this._callHook.apply(this, args);
					}else{
						this._coverDiv.style.zIndex = "900"; // back to original (not cover cqButtons)
					}
					this._needRebuildResMenu = true;
				})
			});
		},
		
		_updateModel: function(fields, nameAttr, valueAttr, dontCreate, makeClone){
			var data = this.cqRecordData;
			if (!data) {
				return [];
			}
			var updatedFields = [];
			var n = fields.length;
			var numData = data.fields.length;
			for (var i=0;i<n;i++) {
				var f = fields[i];
				var found = false;
				for (var j=0; j<numData; j++) {
					var fieldData = data.fields[j];
					if (fieldData.FieldName == f[nameAttr]) {
						if(valueAttr){
							fieldData[valueAttr] = f[valueAttr];
						}else{
							dojo.mixin(fieldData, f);
							if(!f.RecordId) delete fieldData.RecordId; // at least, ResourceId should be cleared... 
						}
						updatedFields.push(fieldData);
						found = true;
						break;
					}
				}
				if(!found && !dontCreate){
					// not found. new data. deferred download or template
					if(makeClone) f = dojo.clone(f);
					data.fields.push(f);
					updatedFields.push(f);						
				}
			}
			return updatedFields;
		},

		_updateByHook: function(fields, requiredness, invalidFields) {
			if (invalidFields) {
				this.cqRecordData.InvalidFields = invalidFields;
			}

			// update not only controls but also model, because controls might not be created yet.
			if (fields) {
				var updatedFields = this._updateModel(fields, "FieldName", null);
				// returned updatedFields from _updateModel() are in model, so not specify separateModel parameter
				this._updateControls(updatedFields, false, false);
			}
			
			if (requiredness) {
				this._updateModel(requiredness, "Name", "Requiredness", true);
				// not necessary to specify separateModel parameter if set requiredness only
				this._updateControls(requiredness, true);
			}
			
			this._validateTabs();
		},
		
		// RATLC01477352
		// In the hook case, sort the modified list by last modified timestamp of the field
		_getModifiedListForHook: function(){
			var modList = this._modifiedList;
			var timestampList = this._modifiedTimestampList;
			var	changed = [];
			for(var item in modList){
				changed.push({FieldName: item, FieldValue: modList[item]});
			}
			changed.sort(function(a, b){
				var result = timestampList[a.FieldName] - timestampList[b.FieldName];
				if (isNaN(result))
					return 0;
				return result;
			});
			return changed;
		},
		
		/*
		 * Format of response data is [{FieldName, FieldValue},...] 
		 * which matches RecordUpdate and RecordCommit API input format.
		 * The data is sorted by field dependency. 
		 */
		getModifiedList: function() {
			var modList = this._modifiedList;
			var	changed = [];
			for(var item in modList){
				changed.push({FieldName: item, FieldValue: modList[item]});
			}
			// Need re-order fields array by dependency
			// e.g. If field A's webDependentFields has field B, A must be put before B in the array.
			var groupFields = dojo.hitch(this, function(fields, withDeps, noDeps){
				dojo.forEach(fields, function(f){
					var hasDeps = false;
					var controls = this.findControlItemsFor(f.FieldName) || [];
					for(var i=0,l=controls.length; i<l; i++){
						var webDeps = controls[i].webDependentFields || [];
						hasDeps = dojo.some(webDeps, function(dep){
							return dojo.some(fields, function(f2){
								return dep.displayName === f2.FieldName;
							});
						});
						if(hasDeps)	break;
					}
					(hasDeps ? withDeps : noDeps).push(f);
				}, this);
			});
			var orderFields = function(fields){
				var newFieldArray = [];
				var remains = fields;
				while(remains.length > 0){
					var withDeps = [];
					var noDeps = [];
					groupFields(remains, withDeps, noDeps);
					if(noDeps.length == 0){
						// unexpected ... Maybe circulate dependency
						noDeps = withDeps; withDeps = [];
					}
					newFieldArray = noDeps.concat(newFieldArray);
					remains = withDeps;
				}
				return newFieldArray;
			};
			changed = orderFields(changed);
			return changed;
		},
		
		updateModifiedList:function(){
			var recordData = this.cqRecordData;
			// check if we have record data returned.
			if (recordData){
				// loop for each field and update the values.
				var numData = recordData.fields.length;
				var isBatch = (this.cqState=="BATCH");
				for (var k = 0; k < numData; k++) {
					var fieldData = recordData.fields[k];
					var fieldName = fieldData.FieldName;
					var controlValue = fieldData.ControlValue;
					if (controlValue || controlValue === "") {
						var isDirty = false; 
						var dataType = fieldData.DataType;
						var isListData = dojo.isArray(controlValue); 
						if(dataType=="MULTILINE_STRING" || dataType=="RESOURCE_LIST" || dataType=="ATTACHMENT_LIST"){
							if(!isListData){
								controlValue = [controlValue];
							}
							if(dataType=="RESOURCE_LIST" && controlValue.length > 0 && dojo.isObject(controlValue[0]) && controlValue[0].RecordId){
								// the value must be array of RecordId.
								// if control value is not array of RecordId, conver to the tpe  
								controlValue = dojo.map(controlValue, function(v){
									return v.RecordId;
								});
							}
							isDirty = (dojo.toJson(controlValue) != dojo.toJson(fieldData.CurrentValue));
						}else{
							if(isListData){
								controlValue = controlValue.length>0 ? controlValue[0] : ""; // bad case
							}
							isDirty = (controlValue != fieldData.CurrentValue);
						}
						if (isDirty) {
							if(isBatch){
								var controls = this.findControlsFor(fieldData.FieldName);
								if(controls && controls.length > 0){
									var actionNode = controls[0].batchActionNode;
									this._modifiedList[fieldName] = {
											value: controlValue,
											action: actionNode ? actionNode.value : "replace"
									};
								}
							}else{
								this._modifiedList[fieldName]= controlValue;
							}
							
							// RATLC01477352
							// Fill the list with last modified timestamp
							if (fieldData.lastModified)
								this._modifiedTimestampList[fieldName] = fieldData.lastModified
							
							fieldData.CurrentValue = controlValue; // update data too 
						}
					}
					delete fieldData.ControlValue;
				}
			}
		},
		
		/*
		 * Format of response data is {"fieldname1":"fieldvalue1","fieldname2":"fieldvalue2"...}
		 */
		getAllModified: function(){
			var modList = this._modifiedList || {};
			if(this._allModified){
				dojo.mixin(this._allModified, modList);
			}else{
				this._allModified = dojo.clone(modList);
			}
			return this._allModified;
		},
		
		clearModifiedList: function() {
			if(this.cqState == "SUBMIT"){
				this.getAllModified();
			}
			this._modifiedList = {};
			this._modifiedTimestampList = {};
		},
		
		updateHiddenList: function(hiddenList) {
			this._hiddenList = hiddenList;
			if(this._loadedPartially) this._loadData();
			this.updateCQForm(this.cqFormLayout, this.cqState, true);
		},
		
		getDisplayedControls: function() {
			var f = [];
			this._forEachControl(function(c){
				var fs = c.fieldSource;
				if (!c.cqInvisible && fs && dojo.indexOf(f, fs) < 0) {
					f.push(fs);
				}
			});
			return f;
		},
		
		calcDisplayFields: function(tabindex) {
			var find = function(items, hiddens){
				var visibleItems = [];
				for(var i=0,l=items.length; i<l; i++){
					var item = items[i]; 
					var f = item.itemType == 20 ? "history" : item.fieldSource;
					if(f){
						var found = false;
						if(hiddens){
							for(var h in hiddens){
								if(h.indexOf(f + "::") == 0){ 
									found = true;
									break;
								}
							}
						}
						if(!found){
							visibleItems.push(f);
						}
					}
					if(item.items){
						visibleItems = visibleItems.concat(find(item.items, hiddens));
					}
				}
				return visibleItems;
			};
			var visibleItems = null;
			var form = this.cqFormLayout;
			if(form && this._hiddenList){
				visibleItems = [];
				var page = form.mForm.mFormPage;
				for(var i=0,l=page.length; i<l; i++){
					if(!tabindex || tabindex == i){
						var items = page[i].mFormItem;
						var hiddens = this._hiddenList ? this._hiddenList[this._filterCharsForCaption(page[i].caption)+i] : null; 
						visibleItems = visibleItems.concat(find(items, hiddens));
					}
				}
			}
			return visibleItems;
		},

		openCustomizeDialog: function() {
			if (!this.custom) {
				console.error("customize dialog must be opened when custom view is opened.");
				return;
			}
			if (!this._customizeDialog) {
				this._customizeDialog = new dijit.Dialog({title: this._messages.customize_title});
				this._customizeDialog.execute = dojo.hitch(this, function(formContents){
					// Customize has been done. Update this._hiddenList with the result.
					var hidden = [];
					for (var i = 0, l1 = this._controls.length; i < l1;  i++) {
						var controlsInTab = this._controls[i];
						var hiddenControls = [];
						dojo.forEach(controlsInTab.controls, function(control) {
							var enabled = false;
							dojo.forEach(formContents[controlsInTab.tabName], function(enabledControl) {
								if (enabledControl == control) {
									enabled = true;
								}
							});
							if (!enabled) {
								hiddenControls[control.fieldSource + "::" + control.itemType + "::" + control.controlId] = true; 
							}
						});
						hidden[controlsInTab.tabName + i] = hiddenControls; 
					}
					this.updateHiddenList(hidden);
				});
			}
			var content = new com.ibm.rational.cm.web.ui.view.record.CqCustomizeDialogContent({_controls:this._controls});
			this.connect(content, "onCancel", function(){
				this._customizeDialog.hide();
			});
			this.connect(content, "onSubmit", function(evt){
				this._customizeDialog._onSubmit(evt);
			});
			this._customizeDialog.setContent(content.domNode); // in this, old content will be destroyed.
			this._customizeDialog.show();
		},
		
		_getDuplicateInfo: function(callback){
			if(this._dupInfo){
				callback(this._dupInfo);
				return;
			}
			var resourceId = this.stableLocation;
			if(!resourceId || resourceId.length === 0){
				return;
			}
			var cb = dojo.hitch(this, function(data, ioArgs){
				delete this._getDupInfoXHR;
				delete this._dupInfo;
				if(cq.checkError(data)){
					this._dupInfo = data;
					callback(data);
					return data;
				}
			});
			var xhr = this._getDupInfoXHR; 
			if(!xhr){
				xhr = cq.xhrGet({
					url: RecordDetailRESTServiceURL,
					content: {
						action:"GetDuplicatesInfo",
						resourceId: resourceId
					},
					load: cb,
					preventCache: true
				});
				this._getDupInfoXHR = xhr;
			}else{
				xhr.addCallback(cb);
			}
		},
		showCached: function(){
			if(this._tasksView)
				this._tasksView.refresh();
		}
	} /* end of contents of CqRecord */
);

/* Static methods */
dojo.mixin(com.ibm.rational.cm.web.ui.view.record.CqRecord, {
	escapeHTML: function (str) {
		if(!str)return str;
		return str.replace(/&/gm, "&amp;").replace(/\"/gm, "&quot;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;");
	},
	_createMenuItemForCQResource: function(label, resourceId) {
		if (dojo.isArray(label)) label = label[0];
		if (dojo.isArray(resourceId)) resourceId = resourceId[0];
		return new dijit.MenuItem({
			label:com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(label),
			resourceId: resourceId,
			iconClass:"cqrecordMenuIcon_openRecord",
			onClick: function() {
				var selectedObjs = [{"recordId": this.resourceId}];	
				dojo.publish("CQ_RECORD_SELECTED",[{selectedArtifactIds:selectedObjs,openNewWindow:true}]);	
			}
		});
	},
	callXHR: function(method, args, customChecker){
		dojo.mixin(args, {
			error: function(response, ioArgs){
				if(response.dojoType == "cancel") return;
				var i18nStrs = dojo.i18n.getLocalization(cqNS, "common");
				var message = response.dojoType == "timeout" ?
						dojo.string.substitute(i18nStrs.timeout, [cqGlobal.defaultXHRTimeout]) :																			
						dojo.string.substitute(i18nStrs.server_error, [response]);
				var message2 = djConfig.isDebug && ioArgs && ioArgs.xhr && ioArgs.xhr.responseText ?
						ioArgs.xhr.responseText : null;
				cq.openMessageDialog(message, message2);																
			}
		});
		args.customChecker = customChecker;
		return method(args);
	},
	_cqDateTimeRegExp: new RegExp("\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}$") 
});
		
dojo.declare(
	"CqRecordTooltip",
	dijit.Tooltip,
	{
		// override _onFocus to hide tooltip onMouseOut (RATLC01063599,RATLC01064001)
		_onFocus: function(e){
			// don't show tooltip for focus
		},
		open: function(target){
			this.inherited(arguments);
			if(!this.ttc && dijit._masterTT){
				this.connect(dijit._masterTT.containerNode, "onmouseover", "close");
				this.ttc = true;
			}
		}
	}
);

dojo.declare(
	"CqRecordRightPaneContainer",
	[dijit.layout.ContentPane, dijit._Contained],
	{});
			
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.TableForm",
	[dijit._Widget, dijit._Templated],
	{
		containerNode: null,
		templateString: "<table class='cqCustomTableView'><tbody dojoAttachPoint='containerNode'></tbody></table>"
//		_tableFormTitle: dojo.i18n.getLocalization(cqNS, "cqRecordStrings", this.lang).table_view_title,
//		_customizeButton: dojo.i18n.getLocalization(cqNS, "cqRecordStrings", this.lang).table_view_customize_button,
//		templateString: "<table class='cqCustomTableView'><tbody dojoAttachPoint='containerNode'><tr><td colspan='2'><table cellspacing='0' class='tableFormTitleBar'><tr><td class='tableFormTitle' width='100%'>${_tableFormTitle}</td>" +
//						"<td align='right'><button dojoAttachEvent='onclick:onCustom'>${_customizeButton}</button></td>" +
//						"</td></table></td></tr></tbody></table>",
//		onCustom: function() {	
//		}
	}
);
/*
 * End of CqRecord widget implementation
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqCustomizeDialogContent",
	[com.ibm.rational.cm.web.ui.view.record.CqCustomizeContent],
	{
		_controls: [],
		createBody: function(){
			this.tableContainer.innerHTML = "";
			var doc = dojo.doc;
			var table = doc.createElement("table");
			table.style.width = "100%";
			var tbody = doc.createElement("tbody");
			dojo.forEach(this._controls, function(controlsInTab) {
				var tr = doc.createElement("tr");
				var td = doc.createElement("td");
				dojo.addClass(td, "pageCaption");
				td.style.textAlign = "center";
				td.colSpan = "2";
				td.innerHTML = controlsInTab.tabName;
				tr.appendChild(td);
				tbody.appendChild(tr);
				dojo.forEach(controlsInTab.controls, function(control) {
					if (control.cqInvisible != "always") {
						var tr = doc.createElement("tr");
						var td = doc.createElement("td");
						var prop = {
								name: controlsInTab.tabName,
								value: control,
								checked: (control.cqInvisible != "custom")
						};
						var checkBox = new dijit.form.CheckBox(prop);
						var caption = doc.createElement("label");
						caption.innerHTML = control.caption;
						caption.htmlFor = checkBox.focusNode.id;
						td.appendChild(checkBox.domNode);
						tr.appendChild(td);
						td = doc.createElement("td");
						dojo.addClass(td, "fieldCaption");
						td.appendChild(caption);
						tr.appendChild(td);
						tbody.appendChild(tr);
					}
				});
			});
			table.appendChild(tbody);
			this.tableContainer.appendChild(table);
		}
	}
);

/*
 * -------------------------------------------------------------------------------------------
 * Internal widgets for additional controls for CQForm 
 */
/*
 * Widget for Controls in CqRecord
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqRecordControl",
	null,
	{
		view: null, 	// CqRecord widget
		cqState: "", 	// "VIEW","MODIFY","SUBMIT" or "BATCH". default is "VIEW"
		setSize: function(w, h) {
			this.domNode.style.width=w+"px";
			this.domNode.style.height=h+"px";
		},
		setCqState: function(state) {
			this.cqState = state;
			this.setReadonly(state == "VIEW");
		},
		setCqInvalid: function(invalid) {
			this.cqInvalid = invalid;
		},
		destroyRC: function() {
			if (this._tooltip) {
				this._tooltip.destroy();
				delete this._tooltip;
			}	
		},
		setFieldData: function(fieldData, separateModel) {
			if (fieldData && this.setValue) { 
				this.setValue(fieldData.ControlValue || fieldData.CurrentValue);
			}
		},
		setHasHelp: function(hasHelp){
			var tooltip = this._tooltip;
			if(hasHelp){
				if(!tooltip || !tooltip.domNode){
					// if control gets focus, tooltip doesn't disapper on onblur. this causes bad usability. 
					// so for now, I don't add control itself as target.
					var targets = /* this.id ? [this.id] : */ [];
					var caption = this.captionNode; 
					if(caption && caption.id){
						targets.push(caption.id);
						dojo.addClass(caption, "hasHelp");
					}
					if(targets.length > 0){
						var form = this.view.cqFormLayout; 
						if(!form.helpText) form.helpText = {};
						var label = form.helpText[this.fieldSource];
						this._tooltip = tooltip = new CqRecordTooltip({
							connectId:targets,
							label: this._createTooltipLabel(label || this.view._messages.loading)
						});
						if(!label){
							this.connect(tooltip, "open", function(){
								if(this.gotHelp) return;
								com.ibm.rational.cm.web.ui.view.record.CqRecord.callXHR(cq.xhrGet, {
									url: RecordDetailRESTServiceURL,
									content: {
										action: "GetHelpText",
										resourceId: this.view.stableLocation,
										field: this.fieldSource
									},
									load: dojo.hitch(this, function(data, ioArgs){
										var newLabel = data.HelpText.replace(/\n/gm,"<br>");
										form.helpText[this.fieldSource] = newLabel; // cache
										var tt = this._tooltip; 
										this.gotHelp = true;
										if(tt){
											tt.label = this._createTooltipLabel(newLabel);
											if(tt._connectNode){
												// still opened
												var helpNode = dojo.byId("cqRecordTooltipContent");
												if(helpNode) helpNode.innerHTML = newLabel;
											}
										}
									})
								});
							});
						}
					}
				}
			}else{
				if(this.captionNode){
					dojo.removeClass(this.captionNode, "hasHelp");
				}
				if (tooltip) {
					tooltip.destroy();
					delete this._tooltip;
				}
			}
		},
		_createTooltipLabel: function(label){
			return "<div class='cqRecordTooltipContent' id='cqRecordTooltipContent'>" + label + "</div>";
		},
		setReadonly: function(readonly){
			if (readonly != this.readonly) {
				this.domNode.readOnly = readonly;
				if(this.focusNode){
					this.focusNode.readOnly = readonly;
				}
				dojo.toggleClass(this.styleNode || this.domNode, "cqReadonlyControl", readonly);
				this.readonly = readonly;
			}
		},
		_changeValidState: function(valid){
			var capNode = this.captionNode; 
			if(capNode && capNode.firstChild){
				var hasInvalidMark = (capNode.firstChild.nodeType !== 3); // TEXT_NODE
				if(!valid){
					if(!hasInvalidMark)	capNode.innerHTML = "<span class='invalidMark'>*</span>" + capNode.innerHTML; 
				}else{
					if(hasInvalidMark) capNode.removeChild(capNode.firstChild);
				}
				dojo.toggleClass(capNode, "cqInvalidField", !valid);
			}
			this.view._validateByControl(this);
		}
	}
);
/*
 * Button
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqButton",
	[dijit.form.Button, com.ibm.rational.cm.web.ui.view.record.CqRecordControl],
	{
		postMixInProperties:function(){
			if(!this.label && this.caption){
				this.label = this.caption;
			}
			this.inherited(arguments);
		},
		destroy: function(){
			this.destroyRC();
			this.inherited(arguments);
		},
		postCreate: function(){
			this.attr('disabled', !this.preClickHook && !this.postClickHook && !this.associatedControlName);
			this.inherited(arguments);
			this.focusNode.style.overflow = "hidden"; // RATLC01060882 
			this.domNode.style.zIndex = "901"; // enable even during first _loading
		},
		setReadonly: function(readonly) {
			this.inherited(arguments);
			// RATLC01279665: List view buttons do not follow behavior of list view
			this.attr('disabled', readonly || this.isAssociatedControlDisabled || false);
		},
		setSize: function(w, h) {
			this.inherited(arguments);
			// Defect 78382, change width and height of button, there's different between dojo 1.0 button and 1.2
			w-=6; h-=5;
			this.focusNode.style.width=w+"px";
			this.focusNode.style.height=h+"px";
		},
		onClick: function() {
			var associatedControl = this.associatedControlName ? this.view.findControlById(this.associatedControlName, this.tabPage) : null;
			var selectedReqId;
			var reqWebAction;
			var btLabel = this.label;
			var viewReqFromList = false;
			var viewReqFromReqCQRecord = false;
			
			if (cqLoginController.getLoginInfo("isReqProEnabled")){
				var recType = this.view.getRecordTypeDisplayName();			
			
				//  Did we just click "view" button from a requirement type CQ record?
				if (this.preClickHook){
					//  This is how "View" requirement will look
					//  "cq.hook:Requirement/CQI_Properties_Req@ldapmlbss1/ent"
						var leftmark = this.preClickHook.indexOf('/');
						var rightmark = this.preClickHook.indexOf('@');
						var rwebViewStringMaybe = this.preClickHook.substring(leftmark+1, rightmark);
						if (rwebViewStringMaybe == "CQI_Properties_Req"){
							//  clicked view on a requirement
							if (recType == "Requirement"){
								viewReqFromReqCQRecord = true;
								selectedReqId = this.view.stableLocation;
								reqWebAction = 1;
							}else{
							//  clicked view on req-enabled entity like Defect
								if (associatedControl){
									var fldName = associatedControl._fieldData.FieldName;
									if (fldName ==  "Requirements_List"){
										selectedReqId = associatedControl.getSelectedResource();
										if (selectedReqId){
											reqWebAction = 1;
											viewReqFromList = true;
										}
									}
								}							
							}
						}
				}		
			
				
				if (viewReqFromReqCQRecord || viewReqFromList) {
					var myView = this.view;
					var props = {view: myView, 
						rp_CQ_Record_Identifier: selectedReqId};
						//  removed the code that gets the ReqWebInt widget and calls
						//  handlerequest. If we want to support that integration the
						//  code will need to be put back in here.
						cq.openMessageDialog(myView._messages.cqReqProIntegrationUnsupported);
						
					
					//  The view requirement/refresh logic does it all, we dont  
					//  want to proceed to any hooks when done.
					return true;
				}
			
			}//end of CQ-ReqPro specific logic

			// 75755: Apply 2009B fix (RATLC01280491), pre-action hook synchronization
			// make recordUpdate and postHook as a callback of preClickHook to make sure Pre Action Hook is called first.
			var doRecordUpdateAndPostHook = dojo.hitch(this, function(){
				var postHook = null;
				if (this.postClickHook && this.isPostclickHookEnabledForWeb) {
					postHook = dojo.hitch(this, function(){this._callFireHook(this.postClickHook);});
				}
				if(associatedControl) {
					switch(this.btnType){
					case "4": 
						if(associatedControl.addResource) associatedControl.addResource(postHook);
						break;
					case "5": 
						if(associatedControl.removeSelected) associatedControl.removeSelected(postHook); 
						break;
					case "6": 
						if(associatedControl.newResource) associatedControl.newResource(postHook); 
						break;
					default:
						if(postHook) postHook();
					}
				}else if(postHook){
					postHook();
				}		
			}); 
			
			if (this.preClickHook && this.isPreclickHookEnabledForWeb) {				
				this._callFireHook(this.preClickHook, doRecordUpdateAndPostHook);
			}else{
				doRecordUpdateAndPostHook();
			}			

			return true;
		},
		// 75755, added parameter callback
		_callFireHook: function(hookLocation, callback) { 
			var url = RecordDetailRESTServiceURL + "?action=FireHook&resourceId=" + encodeURIComponent(this.view.stableLocation) + "&hookResourceId=" + encodeURIComponent(hookLocation);
			this.callHook(this, url, /*fieldData*/null, /*newValue*/null, 
				/*webdeps*/null, /*stopWhenNoModification*/null, /*isForTemplate*/false, callback);
		},
		callHook:function(source, url, fieldData, newValue, webdeps, stopWhenNoModification, isForTemplate, callback) {
		},
		setAssociatedControlDisabled: function(disabled){
			this.isAssociatedControlDisabled = disabled;
			this.setDisabled((disabled && !this.preClickHook && !this.postClickHook) || this.readonly); 
		}
	}
);

/*
 * Widget for Static Text
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqStaticText",
	[dijit._Widget, com.ibm.rational.cm.web.ui.view.record.CqRecordControl],
	{
		value: "", // text
		destroy: function(){
			this.destroyRC();
			this.inherited(arguments);			
		},
		postCreate: function() {
			this.domNode.style.whiteSpace = "nowrap";
			this.setValue(this.value);
			this.inherited(arguments);
		},
		setValue: function(value) {
			this.value = value;
			this.domNode.innerHTML = value;
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqDuplicateBase",
	[dijit._Widget, dijit._Templated, com.ibm.rational.cm.web.ui.view.record.CqRecordControl],
	{
		templateString: "<table><tr><td><a dojoAttachPoint='link' dojoAttachEvent='onclick'></a></tr></table>",
		destroy: function(){
			this.destroyRC();
			this.inherited(arguments);			
		},
		postCreate: function() {
			this.domNode.style.whiteSpace = "nowrap";
			this.inherited(arguments);
			this.view._getDuplicateInfo(dojo.hitch(this, this._setInfo));
		},
		_setInfo: function(data){
			if(!this.domNode) return; // already destroyed
			if(data && data.Original){
				this.dupInfo = [{"recordId": data.Original.LocationId}];	
				this.link.innerHTML = data.Original.DisplayName;
			}else{
				this.dupInfo = null;
				this.link.innerHTML = "";
			}
		},
		onclick: function(e){
			if(this.dupInfo){
				dojo.publish("CQ_RECORD_SELECTED",[{selectedArtifactIds:this.dupInfo,openNewWindow:true}]);
			}
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqDuplicates",
	[dijit._Widget, com.ibm.rational.cm.web.ui.view.record.CqRecordControl],
	{
		destroy: function(){
			this.destroyRC();
			this.inherited(arguments);			
		},
		postCreate: function() {
			this.domNode.style.whiteSpace = "nowrap";
			this.inherited(arguments);
			this.view._getDuplicateInfo(dojo.hitch(this, this._setInfo));
		},
		_setInfo: function(data){
			if(!this.domNode) return; // already destroyed
			this.domNode.innerHTML = ""; // clear
			var dups = data ? data.Duplicates : null;
			if(dojo.isArray(dups) && dups.length > 0){
				for(var i=0,l=dups.length; i<l; i++){
					var link = dojo.doc.createElement("a");
					link.innerHTML = dups[i].DisplayName;
					link.recordIds =  [{"recordId": dups[i].LocationId}];
					this.connect(link, "onclick", dojo.hitch(link, function(){
						dojo.publish("CQ_RECORD_SELECTED",[{selectedArtifactIds:this.recordIds,openNewWindow:true}]);
					}));
					this.domNode.appendChild(link);
					this.domNode.appendChild(dojo.doc.createElement("br"));
				}
			}
		}
	}
);

/*
 * Button
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqOptionGroup",
	[dijit._Widget, com.ibm.rational.cm.web.ui.view.record.CqRecordControl],
	{
		_isValid: true,
		destroy: function(){
			this.destroyRC();
			this.inherited(arguments);			
		},
		setMandatory: function(mandatory) {
			this._isMandatory = mandatory;
		},		
		setValue: function(val){
			this._isValid = !(this._isMandatory && !this.readonly && !val);
			this.validateCtrl();
		},
		validateCtrl: function() {
			this._changeValidState(this._isValid);
		}		
	}
);

/*
 * Widget for Group Box
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqGroupBox",
	[dijit._Widget, dijit._Templated, com.ibm.rational.cm.web.ui.view.record.CqRecordControl],
	{
		templateString: "<fieldset class='cqGroupBox' style='width:100%;height:100%;z-index:-1;'><legend dojoAttachPoint='legend'>${caption}</legend><div dojoAttachPoint='containerNode'></div>",
		destroy: function(){
			this.destroyRC();
			this.inherited(arguments);			
		},
		setReadonly: function(){
		}
	}
);

/*
 * Abstract Widget for forms
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.ReadOnlySupport",
	null,
	{
		_setStateClass: function(){
			try {
				this.inherited(arguments);
			} catch (e){
				// not subclass of _FormWidget
			}
			dojo.toggleClass(this.styleNode || this.domNode, "cqReadonlyControl", this.readonly);
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqFormControl",
	[com.ibm.rational.cm.web.ui.view.record.CqRecordControl, com.ibm.rational.cm.web.ui.view.record.ReadOnlySupport],
	{
		_fieldData: null,
		_inSetField: false,
		destroy: function(){
			this.destroyRC();
			this.inherited(arguments);			
		},
		setMandatory: function(mandatory) {
			this._isMandatory = mandatory;
		},
		setFieldData: function(fieldData, separateModel) {
			this._inSetField = true; // setFieldData shouldn't cause RecordUpdate call
			if(separateModel && this._fieldData){
				dojo.mixin(this._fieldData, fieldData);
			}else{
				this._fieldData = fieldData;
			}
			com.ibm.rational.cm.web.ui.view.record.CqFormControl.superclass.setFieldData.apply(this, arguments);
			if((this.cqState == "MODIFY" || this.cqState == "SUBMIT") && this.webDependentFields) {
				this._updateRecordURL = RecordDetailRESTServiceURL + "?action=RecordUpdate&resourceId=" + encodeURIComponent(this.view.stableLocation);
			}
			this._inSetField = false;
		},
		getFieldValue: function() {
			return this.getValue();
		},
		onChange: function(value) {
			//console.debug("CqFormControl.onChange:newvalue="+value);
			if(!this._inSetField){
				// RATLC01477352
				// Add the last modified timestamp to FieldData
				if (this._fieldData /*RATLC01378844*/)
					this._fieldData.lastModified = new Date().getTime();
				
				delete this.cqInvalid; // something has been changed by user -> ignore server status
				if(!this.readonly && !this.disabled && (this._updateRecordURL || this._fieldData) /* RATLC01264197 && this.isValidField() */) {
					this.callHook(this, this._updateRecordURL, this._fieldData, this.getFieldValue(), this.webDependentFields, true);
				}
			}
			this.validateCtrl();
		},
		isValidField: function() {
			if (this.readonly) {
				return true;
			}
			var val = this.getFieldValue();
			if(!val || val.length == 0) {
				return !this._isMandatory;
			}
			if (this.isValid && !this.isValid()) {
				return false;
			}
			if (this.cqInvalid){
				return false;
			}
			return true;
		},
		validateCtrl: function() {
			this._changeValidState(this.isValidField());
		},
		callHook:function(source, url, fieldData, newValue, webdeps, stopWhenNoModification) {
		}
	}
);

/*
 * Widget for FormEdit control
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqFormEdit",
	[dijit._Widget, dijit._Templated, com.ibm.rational.cm.web.ui.view.record.CqRecordControl],
	{
		templateString: "<div class='cqFormEdit' style='border: 1px solid #9B9B9B'></div>",
		_cqControl: null,
		_oldDataType: null,
		_isMandatory: false,
		_recreate: false,
		_w: 0,
		_h: 0,
		setSize:function(w, h){
			this.inherited(arguments);
			if(this._cqControl) {
				this._cqControl.setSize(w, h);
			}
			this._w = w;
			this._h = h;
		},
		destroy: function() {
			this.destroyRC();
			this.inherited(arguments);
			if (this._cqControl) {
				this._cqControl.destroy();
				this._cqControl = null;
			}
		},
		setFieldData: function(fieldData, separateModel) {
			var oldFieldData = this._cqControl ? this._cqControl._fieldData : null;
			var newDataType = fieldData.DataType || this._oldDataType;
			if (!this._cqControl || this._recreate || this._oldDataType != newDataType) {
				this.domNode.style.border = ""; // sub control has border, so this border is not needed any more 
				this._createSubControl(newDataType);	
			}
			if (this._cqControl) {
				if(oldFieldData) this._cqControl._fieldData = oldFieldData;
				this._cqControl.setFieldData(fieldData, separateModel);
			}
	    },
	    setValue: function() {
	    	if (this._cqControl) {
	    		this._cqControl.setValue.apply(this._cqControl, arguments);
	    	}
	    },
	    getValue: function() {
	    	if (this._cqControl) { 
	    		return this._cqControl.getValue();
	    	}
	    	return null;
	    },
	    getFieldValue: function() {
	    	if (this._cqControl) {
	    		if (this._cqControl.getFieldValue) {
	    			return this._cqControl.getFieldValue();
	    		}
	    		return this._cqControl.getValue();
	    	}
	    	return null;
	    },
	    setDisabled: function(disabled) {
	    	if(disabled == this.disabled) return;
	    	this.disabled = disabled;
	    	if(this._cqControl){
	    		// recreate control
		    	this._recreate = true;
				this.setFieldData(this._cqControl._fieldData, false); 
			}
	    	dojo.toggleClass(this.domNode, "cqReadonlyControl", disabled || this.readonly); // share style for readonly 
	    },
	    setReadonly: function(readonly) {
	    	if(readonly == this.readonly) return;
	    	this._recreate = true; // recreate subcontrol when next setFieldData is called
	    	if(this._cqControl){
		    	this._cqControl.setReadonly(readonly);
			}
	    	this.inherited(arguments);
	    },
	    setMandatory: function(mandatory) {
	    	if (this._cqControl) { 
	    		this._cqControl.setMandatory(mandatory);
	    	}
	    	this._isMandatory = mandatory;	    	
	    },
		setCqInvalid: function(invalid) {
	    	if (this._cqControl) {
	    		this._cqControl.setCqInvalid(invalid);
	    	}
	    	this.cqInvalid = invalid;
		},
		validateCtrl: function() {
	    	if (this._cqControl) { 
	    		this._cqControl.validateCtrl();
	    	}
		},
		isValidField: function(){
	    	if (this._cqControl && this._cqControl.isValidField) { 
	    		return this._cqControl.isValidField();
	    	}
			return true;
		},
		_createSubControl: function(dataType){
			var prop = {view: this.view};
			var isMulti = (this.isMultiLine && dataType == "SHORT_STRING") || dataType == "MULTILINE_STRING";
			if(isMulti){
				prop.isVScr = this.isVScr;
				prop.isAutoVScr = this.isAutoVScr;
				prop.isHScr = this.isHScr;
				prop.isAutoHScr = this.isAutoHScr;
			}
			this._recreate = false;
			var formDate = this.formDate;
			var clazz = com.ibm.rational.cm.web.ui.view.record.CqTextBox;
			if(this.isPassword){
				// use CqTextBox 
				prop.type = "password";
			}else if(this.readonly || this.disabled){
				if(isMulti){
					clazz = com.ibm.rational.cm.web.ui.view.record.CqReadonlyTextArea;
				}else if(dataType == "SHORT_STRING"){
					clazz = com.ibm.rational.cm.web.ui.view.record.CqURLTextBox;
				}else if(dataType == "DATE_TIME") {
					clazz = com.ibm.rational.cm.web.ui.view.record.CqReadonlyDateTimeTextBox;
				}	
			}else{
				if (isMulti) {
					clazz = com.ibm.rational.cm.web.ui.view.record.CqTextArea;
				} else if (dataType == "DATE_TIME") {
					cq.loadModuleSync("com.ibm.rational.cm.web.ui.view.record.CqDateTimeTextBox");
					clazz = com.ibm.rational.cm.web.ui.view.record.CqDateTimeTextBox;
					if(formDate){
						var dateCode = formDate.dateCode || "0";
						var timeCode = formDate.timeCode || "0";
						if(dateCode === "0"){
							if(timeCode !== "0"){
								clazz = com.ibm.rational.cm.web.ui.view.record.CqTimeTextBox;
							}
						}else if(timeCode === "0"){
							clazz = com.ibm.rational.cm.web.ui.view.record.CqDateTextBox;
						}
					}
				} else if (dataType == "INTEGER") {
					clazz = com.ibm.rational.cm.web.ui.view.record.CqNumberTextBox;
				} 
			}
			var control = this._cqControl;
			if(typeof control == clazz){
				return;
			}
			if (control) {
				this.view._removeInvalidControl(control);
				control.destroy();
			}
			if(formDate){
				prop.formDate = formDate; 
			}
			control = new clazz(prop);
			this._cqControl = control;
			control.setSize(this._w, this._h);
			control.setReadonly(this.readonly);
			control.setDisabled(this.disabled);
			control.setMandatory(this._isMandatory);
			control.setCqInvalid(this.cqInvalid);
			if(this.captionNode){
				control.captionNode = this.captionNode;
				control.captionNode.htmlFor = (control.focusNode || control.domNode).id;
			}
			control.fieldSource = this.fieldSource;
			control.cqState = this.cqState;
			if(this.tabPage) control.tabPage = this.tabPage;
			this.domNode.appendChild(control.domNode);
			this._oldDataType = dataType;
			this.focusNode = control.focusNode;
			if(control.callHook){
				this.connect(control, "callHook", this._callHook);
			}
			control.startup();
		},
		_callHook: function(source, url, fieldData, newValue, webdeps, stopWhenNoModification) {
			this.callHook(this, url, fieldData, newValue, this.webDependentFields, stopWhenNoModification);
		},
		callHook: function(source, url, fieldData, newValue, webdeps, stopWhenNoModification) {
		}
	}

);

/*
 * Widget for SIMPLE_TEXT
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.TextBoxResizeMixin",
	null,
	{
		setSize: function(w, h){
			// some kind of huck. need remove border width.
			this.domNode.style.width=Math.max(0,(w-2))+"px";
			this.domNode.style.height=Math.max(0,(h-2))+"px";
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqTextBox",
	[dijit.form.TextBox, com.ibm.rational.cm.web.ui.view.record.CqFormControl, com.ibm.rational.cm.web.ui.view.record.TextBoxResizeMixin],
	{
		setValue: function(value){
			if (dojo.isArray(value)) {
				arguments[0] = value.join("\n");
				this.isMulti = true;
			}else if(this.isMulti){
				delete this.isMulti;
			}
			this.inherited(arguments);
		},
		getFieldValue: function(){
			var val = this.inherited(arguments);
			if(this.isMulti && val && val.length > 0){
				val = val.split("\n");
			}
			return val;
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqNumberTextBox",
	[dijit.form.NumberTextBox, com.ibm.rational.cm.web.ui.view.record.CqFormControl, com.ibm.rational.cm.web.ui.view.record.TextBoxResizeMixin],
	{
		trim: true, // trim whitespaces before/after value
		format: function(/*Number*/ value, /*Object*/ constraints){
			return value; // avoid "," is inserted 
		},
		getValue: function(){
			return this.getDisplayedValue(); // get even invalid value
		},
		_onBlur: function(){
			this.setValue(this.getValue(), true); // cause onChange always 
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqReadonlyDateTimeTextBox",
	[dijit.form.TextBox, com.ibm.rational.cm.web.ui.view.record.CqFormControl, com.ibm.rational.cm.web.ui.view.record.TextBoxResizeMixin],
	{
		format: function(/*ISO String*/ value, /*Object*/ constraints){
			if(!value){ return null; }
			dojo["require"]("dojo.date.stamp");
			dojo["require"]("dojo.date.locale");
			var d = value instanceof Date ? value : dojo.date.stamp.fromISOString(value);
			var v = "";
			var formDate = this.formDate || {};
			var dateCode = formDate.dateCode || "1"; // default is short
			var timeCode = formDate.timeCode || "1"; // default is short
			if(dateCode == "0" && timeCode == "0") {
				dateCode = timeCode = "1"; 
			}			
			if(dateCode != "0"){
				v = dojo.date.locale.format(d, {selector: "date", formatLength: dateCode == "1" ? "short" : "long"});
			}
			if(timeCode != "0"){
				v = v + (v.length > 0 ? " " : "") + dojo.date.locale.format(d, {selector: "time", formatLength: timeCode == "1" ? "medium" : "long"});
			}
			return v;
		},
		getValue: function(){
			// for RATLC01064961
			// TextBox#getValue returns parsed value. In this class, it is not appropriate. 
			// this is readonly widget, so it just returns as is of set value.
			return this._lastValue;
		}
	}
);


dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqURLTextBox",
	[dijit.form.TextBox, com.ibm.rational.cm.web.ui.view.record.CqFormControl],
	{
		// 65684: dojo 1.2 migration : onkeyup handler
		templateString:"<table style='display: -moz-inline-stack;' class='dijit dijitReset dijitInlineTable' cellspacing='0' cellpadding='0' id='widget_${id}' name='${name}' dojoAttachEvent='onmouseenter:_onMouse,onmouseleave:_onMouse' waiRole='presentation'><tr class='dijitReset'>" +
						"<td class='dijitReset dijitInputField' width='100%'><input dojoAttachPoint='textbox,focusNode' dojoAttachEvent='onmouseenter:_onMouse,onmouseleave:_onMouse,onfocus:_onMouse,onblur:_onMouse,onkeyup:_onKeyPress,onkeypress:_onKeyPress' autocomplete='off' type='${type}' name='${name}'></td>" +
						"<td class='dijitReset cqMenuURL' width='0%'><div style='width:1px;' dojoAttachPoint='linkMenuNode' class='cqrecordMenuIcon_openURL'></div></td></tr></table>",
		destroy: function(){
			this._clearMenu();
			this.inherited(arguments);
		},
		onChange: function(value){
			this.inherited(arguments);
			this._urls = [];
			if(this.readonly || this.disabled){
				this._urls = CqURLUtil.extractUrls(value);
			}
			this._clearMenu();
			if(this._urls.length > 0){
				this.linkMenuNode.style.width = ""; // important for layouting
				setTimeout(dojo.hitch(this, function(){
					// defer the actual creation for performance
					this._linkMenu = new dijit.Menu();
					// create menu items for each URLs in value 
					dojo.forEach(this._urls, function(url){
						var menuitem = new dijit.MenuItem({label:url});									
						this.connect(menuitem, "onClick", function() {
							window.open(url);
						});
						this._linkMenu.addChild(menuitem);
					}, this);
					// create button 
					this._linkButton = new dijit.form.DropDownButton({dropDown:this._linkMenu, label:this.view._messages.openURL, showLabel: false, iconClass: "cqrecordMenuIcon_openURL"});
					var s = this._linkButton.domNode.style;
					s.margin = "0px"; // override dijit.css
					s = this._linkButton.focusNode.style;
					s.background = "transparent";
					s.border = "none";
					s.padding = "0px";
					this.linkMenuNode.appendChild(this._linkButton.domNode);
				}), 0);
			}else{
				this.linkMenuNode.style.width = "1px"; // important for layouting
			}
		},
		_clearMenu: function(){
			if(this._linkMenu && this._linkMenu.domNode){
				this._linkMenu.destroyRecursive();
				this._linkMenu = null;
			}
			if(this._linkButton && this._linkButton.domNode){
				this._linkButton.destroyRecursive();
				this._linkButton = null;
			}
		}
	}
);

/*
 * Widget for Text Control with MULTILINE_STRING
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqTextAreaCommon",
	null,
	{
		_setScrStyle: function(s, textarea){
			s.overflowY = this.isVScr || this.isAutoVScr ? "auto" : "hidden"; 
			var isHScr = this.isHScr || this.isAutoHScr;
			s.overflowX = isHScr ? "auto" : "hidden";
			if(isHScr && !textarea) s.whiteSpace = "nowrap"; // This causes IE's textarea doesn't work. No effect to FF's textarea
			if(dojo.isIE){
				if(!isHScr) s.wordWrap = "break-word";
			}else{
				// Because FireFox doesn't support WordWrap, need HScrollbar
				// CSS3 has similar property text-wrap, so when FF will support the property, we will use it.
				if(s.overflowY == "auto") s.overflowX = "auto"; 	
			}
		}
	}
);
			
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqTextArea",
	[dijit.form.TextBox, com.ibm.rational.cm.web.ui.view.record.CqFormControl, com.ibm.rational.cm.web.ui.view.record.CqTextAreaCommon],
	{
		templateString: "<textarea dojoAttachPoint='textbox,focusNode' dojoAttachEvent='onmouseenter:_onMouse,onmouseleave:_onMouse,onfocus:_onMouse,onblur:_onMouse'></textarea>",
		postCreate: function() {
			this.inherited(arguments);
			this._setScrStyle(this.domNode.style, true);
		},
		setValue: function(value, priorityChange, formattedValue){
			if (dojo.isArray(value)) {
				var text = value.join("\n");
				com.ibm.rational.cm.web.ui.view.record.CqTextArea.superclass.setValue.call(this, text, priorityChange);
			} else {
				this.inherited(arguments);
			}
		},
		getFieldValue: function() {
			var text = this.getValue();
			if(this._fieldData.DataType != "MULTILINE_STRING"){
				return text;
			}
			if(text && text.length > 0){
				return text.split("\n");
			}
			return [];
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqReadonlyTextArea",
	[dijit.form._FormWidget, com.ibm.rational.cm.web.ui.view.record.CqFormControl, com.ibm.rational.cm.web.ui.view.record.CqTextAreaCommon],
	{
		templateString: "<div dojoAttachPoint='focusNode, content' dojoAttachEvent='onkeypress: _onKeyPress'></div>",
		baseClass: "dijitTextBox",
		postCreate: function(){
			this.inherited(arguments);
			var s = this.content.style;
			this._setScrStyle(s);
			s.width = s.height = "100%";
			// RATLC01377866: the scroll bar doesn't scroll when selecting text in cq.widget.CqReadonlyTextArea
			// Set overflow-y to "scroll", then the scroll bar works ok.
			s.overflowY = "scroll";
		},
		setValue: function(value){
			if (dojo.isArray(value)) {
				value = value.join("\n");
			}
			value = com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(value);
			value = cq.embedLink(value);
			this.content.innerHTML = value.replace(/\n/gm, "<br>").replace(/  /gm, "&nbsp; ").replace(/ &nbsp;/gm, "&nbsp;&nbsp;"); //RATLC01261449
		},
		getFieldValue: function(){
			return null; // no change
		},
		setDisabled: function(){
			// do nothing
		},
		
		// RATLC01261928: Ctrl-A selects entire page when cq.widget.CqReadonlyTextArea has focus.
		// Interception the key press event of 'Ctrl-A' and programmatic select inner text.
		_onKeyPress: function(event){
			// the charCode of key 'A' is 97 
			if(event.ctrlKey && event.charCode == 97){
				this.onSelectAll(event);
			}
		},
		onSelectAll: function(event){
			// stop key press event of 'Ctrl-A'
			dojo.stopEvent(event); 
			
			dojo["require"]("dijit._editor.selection");
			// select inner text
			dijit._editor.selection.selectElementChildren(this.domNode);
		}
	}
);

/*
*
*/
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqLoadingSupport",
	null,
	{
		_loading: false,
		_setLoading: function(loading) {
			this._loading = loading;
			var img = this._loadingImage; 
			if (loading && !img) {
				img = dojo.doc.createElement("img");
				img.src = dojo.moduleUrl("com.ibm.rational.cm.web", "ui/images/loading16.gif");
				var style = img.style; 
				style.position = "absolute";
				style.width = "16px";
				style.height = "16px";
				style.left = "0px";
				style.top = "0px";
				style.zIndex = "999"; // always top
				this._loadingImage = img;
				var targetNode = this.loadingNode || (this.focusNode ? this.focusNode.parentNode : this.domNode); 
				targetNode.appendChild(img);
			}
			if (img) {
				img.style.visibility = loading ? "" : "hidden";
			}
		}		
	}
);
/*
 * Widget for ListControl and History
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqListControl",
	[dijit._Widget, dijit._Templated, com.ibm.rational.cm.web.ui.view.record.CqFormControl, com.ibm.rational.cm.web.ui.view.record.CqLoadingSupport],
	{
		templateString: "<div style='position: relative;overflow:auto'><div dojoAttachPoint='dataTable'></div></div>",
		dataTable: null,
		referencedRecordType: null,
		referencedFriendlyRecordType: null, // WI 84249, Adopt 2009C fix for RATLC01525645
		_width: 100, // default
		_store: null,
		_table: null,
		_selectedData:null,
		_messages: null,
		_isGrid: false,

		postMixInProperties:function(){
			this._messages = this.view._messages;
			// remove fieldPath=="" column
			if(this.columnInfo){
				this.columnInfo = dojo.filter(this.columnInfo, function(c){
					return c.fieldPath; 
				});
			}
			this.inherited(arguments);
		},
		postCreate: function(){
			this.inherited(arguments);
			dojo.addClass(this.domNode, "cqListControl");
		},
		destroy: function(){
			dojo.unsubscribe(this._newRecordSubscription);
			if (this._menu) {
				if(this.domNode){
					this._menu.unBindDomNode(this.domNode); // this is not necessary after at least dojo 1.1
				}
				if (this._menu.domNode) {
					this._menu.destroyRecursive();
				}
				delete this._menu;
			}
			this.inherited(arguments);			
		},
		setSize: function(w, h){
			this._width = w - dojo._getPadBorderExtents(this.domNode).w;
			this.inherited(arguments);
		},
		setDisabled: function(disabled) {
			if(this.disabled != disabled){
		    	this.disabled = disabled;
				var name = this.controlId;
				if(name){
					this.view._forEachControl(function(c){
						if(c.associatedControlName === name && c.setAssociatedControlDisabled){
							c.setAssociatedControlDisabled(disabled);
						}  
					}, this.tabPage);
				}
			}
		},
		setFieldData: function(fieldData, separateModel) {
			// fieldData
			var curData = this._fieldData;
			var curDataType = curData ? curData.DataType : null;
			if(separateModel && curData){
				dojo.mixin(curData, fieldData);
			}else{
				curData = fieldData;
			}
			curData.DataType = curData.DataType || curDataType || "MULTILINE_STRING"; // fallback
			fieldData = this._fieldData = curData;
			// store
			var store = this._createTableStore(fieldData);
			this.setStore(store);
			this.fetchData();
			if (fieldData.DataType == "RESOURCE" || fieldData.DataType == "RESOURCE_LIST") {
				this.referencedRecordType = fieldData.ReferencedRecordType;
				// WI 84249, use userFriendlyLocation when loading referenced Record Template
				this.referencedFriendlyRecordType = fieldData.ReferencedFriendlyRecordType;
				this.setupRowHandlers();
				this._setupMenu();
			}
			// 
			if(!this._loading){
				this.validateCtrl(); // change tab color 
			}
			this._setRecordUpdateURL(); // RATLC01051076 
		},
		getFieldValue: function(args) {
			if(this.disabled){
				return null; // ignored in updateModifiedList
			}
			var store = this._store;
			if (!store) {
				return null;
			}
			var fromStore = args ? args.fromStore: false;
			if (!fromStore && store.isDirty && !store.isDirty()){
				return this._fieldData.ControlValue || this._fieldData.CurrentValue;
			}
			var val;
			store.fetch({
				scope: this,
				onComplete: function(items){
					var dataType = this._fieldData.DataType;
					if(dataType == "RESOURCE_LIST" || dataType == "ATTACHMENT_LIST" || dataType == "MULTILINE_STRING"){
						val = [];
						var func;
						if(dataType == "RESOURCE_LIST"){
							// resource list must be array of RecordId
							// but ListBox create array of displayName (exception)
							func = function(item){
								if(dojo.isString(item)){
									// array of display name (ListBox case) 
									val.push(item);
								}else{	
									val.push(store.getValue(item, "RecordId"));
								}
							};
						}else if(dataType == "ATTACHMENT_LIST"){
							func = function(item){
								var v = {};
								var attrs = store.getAttributes(item);
								dojo.forEach(attrs, function(attr){
									v[attr] = store.getValue(item, attr);
								});
								val.push(v);
							};
						}else{
							func = function(item){
								val.push(store.getValue(item, "line"));
							};
						}
						dojo.forEach(items, func);
 					}else if(items && items.length > 0){
						val = store.getValue(items[0], this._fieldData.FieldName);
					}else if(dataType == "RESOURCE" && this._loading){
						val = this._fieldData.CurrentValue; // avoid validation check while loading
					}else {
						val = "";
					}
				}
			});
			return val;
		},
		/*
		* Controller methods. Buttons associated this ListControl calls these methods.
		*/
		addResource: function(postHook){
		 	if(!this.referencedRecordType){
		 		if(postHook) postHook();
		 		return;
		 	}
//			dojo["require"]("com.ibm.rational.cm.web.ui.utils.RecordChooser");
			com.ibm.rational.cm.openRecordChooser({
				recordType: this.referencedRecordType,
				columnInfo: this.columnInfo,
				onOk: dojo.hitch(this, function(selected){
					if(selected && selected.length > 0){
						var arr = dojo.map(selected, function(sel){
							return { RecordId: sel.recordId };
						});
						// 75755, make sure postHook is called after RecordUpdate
						this._fetchResourceList(arr, false, postHook);
					}
				}),
				multiSelect: true
			});
		},
		removeSelected: function(postHook) {
			if (!this.readonly){
				if(this._isGrid){
					var table = this._table;
					var sel = table && table.selection ? table.selection.getSelected() : null; 
					if(sel && sel.length > 0){
						var selectedRecordId = this._fieldData.DataType == "RESOURCE_LIST" ? this.getSelectedResource(): null;
						table.removeSelectedRows();
						if(selectedRecordId){
							// Resource might be displayed multiple.
							// Remove all of instance if one of them is removed.
							var store = this._store;
							if(store.getFeatures()["dojo.data.api.Write"]){
								store.fetch({onItem: function(item){
									if(selectedRecordId == store.getValue(item, "RecordId")){
										store.deleteItem(item);
									}
								}});
								// Grid is not updated automatically here. Will be update by onChange(). 
								// This seems to be dojo 1.0 bug. This is fixed in 1.0.2 or later...
							}
						}
						this._selectedData = null;
						this._selectionChanged();
						this.onChange();
				 		if(postHook) postHook();
					}
				}else if(this._selectedData) {
					this._store.deleteItem(this._selectedData);
					this.fetchData();
					this.onChange();
			 		if(postHook) postHook();
				}
			}
		},
		newResource: function(postHook) {
			// WI 84249, use userFriendlyLocation when loading referenced Record Template
			var refRecordType = this.referencedFriendlyRecordType || this.referencedRecordType;
			if(!refRecordType){
		 		if(postHook) postHook();
				return;
			}
			var title = this.caption || this.view._messages.defaultTitle;
			if(title.length > 1 && title.charAt(title.length-1)==':'){
				title = title.substring(0,title.length-1);
			}
			title = dojo.string.substitute(this._messages.newRecord, [title]);
			dojo.publish("CQ_RECORD_SUBMIT", [{recordType:refRecordType, referrer:this.id, displayName: title}]);
			if(!this._newRecordSubscription){
			     this._newRecordSubscription = dojo.subscribe("CQ_RECORD_CREATED", this, dojo.hitch(this, function(message){
			     	if(message.referrer != this.id){
			     		return;
			     	}			     	
			     	// 75755, make sure postHook is called after RecordUpdate
			     	this._fetchResourceList([{RecordId:message.id}], false, postHook);
			     }));
			}
		},
		onVisible: function(){
			if(this._isGrid && this._table){
				this._table.update();
			}
		},
		getSelectedResource: function(){
			var s = this._selectedData;
			return s && s.RecordId ? s.RecordId : null; 
		},
		/*
		* Store
		*/
		setStore: function(store){
			this._store = store;
			if(this._isGrid){
				if(!this._table){
					dojo["require"]("com.ibm.rational.cm.web.ui.data.GridModel");
					var model = new com.ibm.rational.cm.web.ui.data.GridModel(null,null,{"store":store});
// work-around for dojo bug
// see http://dojotoolkit.com/forum/dojox-dojox/dojox-grid-support/dojox-grid-data-dojodata-does-not-propogate-removal-item-underl for more detail
					dojo.mixin(model, {
						remove: function(rowNum) {
							var rows = this.getRow(rowNum);
							if(rows){
								var item = rows.__dojo_data_item;
								if (this._canWrite && this.store.isItem(item)) {
									this.store.deleteItem(item);
								}
							}
							dojox.grid.data.Table.prototype.remove.apply(this, arguments);
					  	}
					});
					var structure =[
						{ cells: [ this._store.colData ] }
					];
					this.dataTable.parentNode.style.overflow = "hidden"; // 71428: History grid of CQ Record is empty
					// Since this._table is global variable, we need load Grid synchronously
					cq.loadModuleSync("com.ibm.rational.cm.web.ui.view.common.Grid");
					this._table = new com.ibm.rational.cm.web.ui.view.common.Grid({model:model, structure: structure}, this.dataTable);
				}else{
					this._table.model.setData(store);
				}
			}else{
				if(!this._table){
					dojo["require"]("com.ibm.rational.cm.web.ui.view.record.CqTable");
					this._table = new com.ibm.rational.cm.web.ui.view.record.CqTable({
						isSelectable: true,
						showHeader: (store.colData && store.colData.length > 1)
					}, this.dataTable);
				}
				this._table.setStore(store);
			}
		},
		
		fetchData: function(){
			this._selectedData = null;
			if(this._isGrid){
				this._table.model.requestRows(0, 100);
				this._selectedData = this._getSelectedDataFromGrid(this._table);
			}else{
				this._table.fetchData();
			}
			this._selectionChanged();					
		},
		
		_getSelectedDataFromGrid: function(grid){
			if(grid && grid.selection){
				var sel = grid.selection.selectedIndex;
				if(sel >= 0){
					var row = grid.model.getRow(sel);
					if(row){
						return row.__dojo_data_item; // not good
					}
				}
			}
			return null;
		},
		/*
		 * 
		 */
		_createTableStore: function(fieldData) {
			// use original data
			var originalData = dojo.clone(fieldData.ControlValue || fieldData.CurrentValue || []);
			var d = {items: originalData };
			var columns;
			var w = this._width;
			var pxWidth=function(percentWidth){
				return (w * percentWidth / 100) + "px";
			};
			var msgs = this._messages;
			var escapeHTML = com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML;
			if (fieldData.DataType == "RESOURCE_LIST" || fieldData.DataType == "RESOURCE") {
				var isList = (fieldData.DataType == "RESOURCE_LIST");
				// check data is updated or not
				if(this._store){
					var currentVal = this.getFieldValue({fromStore: true});
					if(dojo.toJson(originalData) == dojo.toJson(currentVal)){
						return this._store;
					}
				}
				//
				var done = false;
				if(dojo.isArray(this.columnInfo) && this.columnInfo.length > 0){
					this._isGrid = true;
					columns = dojo.map(this.columnInfo, function(c){
						return {field:c.fieldPath /* should same as _fetchResourceList() */, name:c.fieldLabel, width:pxWidth(c.widthInPercent)};
					});
					if(this._fetchResourceList(isList ? originalData : fieldData, true)){
						d.items = []; // actual data will be fetched later.
						done = true;
					}
				}	
				if(!done){
					this._isGrid = false;
					if(isList){
						columns = [{field:"DisplayName"}];
					}else{ // RESOURECE
						var data = new Object;
						data[fieldData.FieldName] = dojo.isString(originalData) ? escapeHTML(originalData) : "";
						if (fieldData.RecordId) {
							// for context menu
							data["RecordId"] = fieldData.RecordId;
							columns = [{field:fieldData.FieldName}];
						}
						d = {items: [data] };
					}
				}
			} else if (fieldData.DataType == "ATTACHMENT_LIST") {
				this._isGrid = true;
				columns = [{field:"FileName", name:msgs.attachment_filename, width:pxWidth(20)}, 
							 {field:"FileSize", name:msgs.attachment_filesize, width:pxWidth(15)},
							 {field:"Description", name:msgs.attachment_description, width:pxWidth(35)}];
				dojo.forEach(originalData, function(v){
					if (v["FileName"]) v["FileName"] = escapeHTML(v["FileName"]);
					if (v["Description"]) v["Description"] = escapeHTML(v["Description"]);
				});
			} else if (fieldData.FieldName == "history") {
				this._isGrid = true;
				columns = [{field:"Date", name:msgs.history_date, width:pxWidth(25),
								dataType: "DateTime" // custom property used in CqGrid to convert to date time in client locale
							}, // RATLC01069624 : Falcon RATLC IHD: History table - Dates get sorted as strings (alphabetically) rather than by date
			                 {field:"User", name:msgs.history_user, width:pxWidth(15)},
			                 {field:"Action", name:msgs.history_action, width:pxWidth(15)},
			                 {field:"OldState", name:msgs.history_oldstate, width:pxWidth(15)},
			                 {field:"NewState", name:msgs.history_newstate, width:pxWidth(15)}];
				dojo.forEach(originalData, function(v){
					if(v["User"]) v["User"] = escapeHTML(v["User"]);
					if(v["Action"]) v["Action"] = escapeHTML(v["Action"]);
					if(v["OldState"]) v["OldState"] = escapeHTML(v["OldState"]);
					if(v["NewState"]) v["NewState"] = escapeHTML(v["NewState"]);
				});
			} else if (fieldData.DataType == "MULTILINE_STRING") {
				this._isGrid = false;
				var mod = dojo.map(originalData, function(data){
					return {"line": escapeHTML(data)};
				});
				d = {items: mod };
				columns = [{field:"line"}];
//				dojo.mixin(this._table, {addHeader:function(){}}); // no header
			} else {
				this._isGrid = false;
				var data = new Object;
				data[fieldData.FieldName] = originalData;
				d = {items: [data] };
			}
			var store = (fieldData.FieldName != "history") ?
					new dojo.data.ItemFileWriteStore({data:d}):
					new dojo.data.ItemFileReadStore({data:d});	
			if (columns) {
				store.colData = columns;
			}
			return store;
		},
		
		/*
		 * addData:
		 * Assume Array of {RecordId:id}
		 */
		_addResourceData: function(dataArray, initialFetch) {
			if (!this.readonly || initialFetch){
				if (this._fieldData.DataType == "RESOURCE"){
						this._store.fetch({
							onComplete: dojo.hitch(this, function(items){
								if(items.length > 0){
									this._store.deleteItem(items[0]);
								}
							})
						});
						if (dataArray && dataArray.length > 0) {
							com.ibm.rational.cm.web.ui.view.record.CqRecord.callXHR(cq.xhrPost,{
								url: "cqproperties.cq",
								content: {
									action:"GetDisplayName",
									resourceId: [dataArray[0].RecordId]
								},
								load: dojo.hitch(this, function(data, ioArgs){
									if (data.STATUS == "true" && dojo.isArray(data.data) && data.data.length > 0) {
										dataArray[0][this._fieldData.FieldName] = data.data[0];
									}
								}),
								sync: true
							});
							this._store.newItem(dataArray[0], null);
						}
				}else if(this._fieldData.DataType == "RESOURCE_LIST") {
					dojo.forEach(dataArray, function(item){
						this._store.newItem(item, null);
					}, this);
				}
				this.fetchData();
				if(!this.readonly){
					if(initialFetch){
						this._store.save(); // clear dirty flag
					}else{
						this.onChange();				
					}
				}
			}
		},

		_fetchResourceList: function(res, initialFetch, callback /*75755*/){
			if(!res){
				return false; // fail
			}
			var records;
			if(dojo.isArray(res)){
				if(res.length == 0){
					return true; // success
				}
				records = [];
				dojo.forEach(res, function(d){
					// RESOURCE and RESOURCE_LIST type data has RecordId. [{RecordId: ..., DisplayName, ..., ...}] 			
					// From AddResource dialog, it might cause array of record id. e.g ["cq.repo.cq-record:16777223/33554436@Local/SAMPL", "cq.repo.cq-record:16777223/33554436@Local/SAMPL"]
					// if there are 2 ListControls for a ResourceList data on a form. (RATLC01262800)
					var resId = dojo.isString(d)? d : d.RecordId;
					if(resId) records.push(resId);
				});
				if(records.length == 0) return true;
			}else if(res.DataType == "RESOURCE"){
				var resId = dojo.isString(res)? res : res.RecordId;
				if(!resId) return true;
				records = [resId];
			}else{
				return true; // no record
			}
			var fields = dojo.map(this.columnInfo, function(c){
				return c.fieldPath;
			});
			var url = RecordDetailRESTServiceURL + "?action=GetTableContents";
			this._setLoading(true); 
			var self = this;
			com.ibm.rational.cm.web.ui.view.record.CqRecord.callXHR(cq.xhrPost,{
				url: url,
				content: {
					records: dojo.toJson(records), 
					fields: dojo.toJson(fields)
				},
				preventCache: true,
				load: dojo.hitch(this, function(data, ioArgs){
					if(data.TableContents.length > 0){
						var arr = dojo.map(data.TableContents, function(content, index){
							var d = {};
							for(var i=0,l=fields.length; i<l; i++){
								d[fields[i]]=com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(content[i]); // RATLC01361692
							}
							d["RecordId"] = content[l] || records[index]; // for getFieldValue and context menu. see _setupMenu()
							if(res.RecordId && res.FieldName){
								d[res.FieldName]=res.ControlValue || res.CurrentValue; // For Resource type. see getFieldValue()
							}
							return d;
						});
						
						// RATLC01270867 : loading a REST URI occasionally displays error
						var func = function(){
							// RATLC01264672 : Dates in parent/child control tables are not formatted according to timezone and locale
							if(data.ColumnTypes){
								var hasDate = false;
								var grid = self._table;
								if(grid){
									var layout = grid.structure;
									var len = layout[0].cells[0].length;
									dojo.forEach(data.ColumnTypes, function(type, idx){
										if(
											idx < len && // third column is not displayed in the grid
											type == "DATE_TIME"
										){
											layout[0].cells[0][idx].dataType = "DateTime";
											hasDate = true;
										}
									});
									if(hasDate){
										grid.setStructure(layout);
									}
								}
							}
							self._addResourceData(arr, initialFetch);
							// 75755, make sure postHook is called after recordUpdate.
							if(callback && dojo.isFunction(callback)){
								callback();
							}
						}
						if(this._table){
							func();
						}else{
							setTimeout(func, 0);
						}
					}
				}),
				handle: dojo.hitch(this, function(data, ioArgs){
					this._setLoading(false); 				
				})
			});
			return true; // success
		},

		/*
		* Event handlers on Grid/CqTable
		*/
		openSelected: function() {
			if(this._selectedData && this._selectedData.RecordId){
				var selectedObjs = [{"recordId": this._selectedData.RecordId}];	
				dojo.publish("CQ_RECORD_SELECTED",[{selectedArtifactIds:selectedObjs,openNewWindow:true}]);	
			}
		},
		
		setupRowHandlers: function(){
			var tbl = this._table;
			if(tbl.hasCqRowHandler) return;  // handlers are already attached
			if(this._isGrid){
				this.connect(tbl, "onCellFocus", function(cellIndex, rowIndex){
					tbl.selection.select(rowIndex);
				});
				this.connect(tbl, "onSelectionChanged", function(){
					this._selectedData = this._getSelectedDataFromGrid(tbl);
					this._selectionChanged();
				});
				this.connect(tbl, "onKeyDown", function(e){
					if(e.keyCode == dojo.keys.ENTER){
						this.openSelected();
					}
				});
				this.connect(tbl, "onRowDblClick", "openSelected");
			}else{
				this.connect(tbl, "handleRowClick", function(inIndex, inCtrlKey, inShiftKey){
					var children = tbl.getChildren();
					if(children.length > inIndex && inIndex >= 0){
						this._selectedData = children[inIndex].data;
						this._selectionChanged();
					}
				});
			}
			tbl.hasCqRowHandler = true;
		},
		
		_selectionChanged: function() {
			// do nothing for CqListControl. CqAttachment uses this to enable/disable buttons.
		},
		
		_setupMenu: function() {
			if(!this._menu){
				this._menu = new dijit.Menu({targetNodeIds:[this.domNode]});
				var openItem = new dijit.MenuItem({
					label:dojo.i18n.getLocalization(cqNS, "cqRecordStrings", this.lang).openRecord,
					iconClass:"cqrecordMenuIcon_openRecord",
					onClick: dojo.hitch(this, "openSelected")
				});
				this._menu.addChild(openItem);
				openItem.setDisabled(true);
				var removeMenu = new dijit.MenuItem({
					label:dojo.i18n.getLocalization(cqNS, "cqRecordStrings", this.lang).attachment_remove,
					onClick: dojo.hitch(this, function() {
						this.removeSelected();
					})});
				this._menu.addChild(removeMenu);
				removeMenu.setDisabled(true);
				
				// RATLC01060653 : CQBeta7.1.2r3: [MUSTFIX] can't copy text out of a grid - (or copy to clipboard via pencil icon?)
				if(this._isGrid && this._table){ // RATLC01273043 : error when open a record view with grid before open a query result view
					var self = this;
					cq.loadModule("com.ibm.rational.cm.web.ui.view.common.Grid", function(){
						self._menu.addChild(cq.createGridCopyMenu(this._table));
					});
				}
				
				this.connect(this._menu, "onOpen", function(){
					openItem.setDisabled(!this._selectedData);
					removeMenu.setDisabled(!this._selectedData || this.readonly);
				});
			}
		},
		
		_setRecordUpdateURL: function() {						
			// If this is a resource list control, we need to update the record data with the server
			// to maintain backwards compatibility with previous CQ Web behavior.
			delete this._updateRecordURL;
			if(this._fieldData && 
					(this._fieldData.DataType == "RESOURCE_LIST" || this._fieldData.DataType == "RESOURCE")&& 
					!this.readonly && !this.disabled && 
					(this.cqState == "MODIFY" || this.cqState == "SUBMIT")) {
				this._updateRecordURL =
					RecordDetailRESTServiceURL + "?action=RecordUpdate&resourceId=" +
					encodeURIComponent(this.view.stableLocation);
			}
		}
	}
);

/*
 * Widget for Attachment
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqAttachment",
	com.ibm.rational.cm.web.ui.view.record.CqListControl,
	{
		widgetsInTemplate: true,
		templateString: "<div><table cellspacing='0' cellpadding='0' width='100%'><tr valign='top'>"
						+ "<td width='100%'><div class='tableContainer' dojoAttachPoint='tableContainer' style='width:100%; overflow:auto'><div dojoAttachPoint='dataTable'></div></div></td><td align='center'>"
						+ "<button dojoAttachPoint='_downloadButton' dojoAttachEvent='onClick:_startDownload' dojoType='dijit.form.Button' label='${_messages.attachment_download}'></button><br>"
						+ "<button dojoAttachPoint='_uploadButton' dojoAttachEvent='onClick:_startUpload' dojoType='dijit.form.Button' label='${_messages.attachment_upload}'></button><br>"
						+ "<button dojoAttachPoint='_removeButton' dojoAttachEvent='onClick:_removeAttachment' dojoType='dijit.form.Button' label='${_messages.attachment_remove}'></button>"
						+ "</div></td></tr></table></div>",
		_downloadButton: null,
		_uploadButton: null,
		_removeButton: null,
		_dialog: null,
		postCreate: function(){
			this.inherited(arguments);
			dojo.addClass(this.domNode, "cqAttachment");
			this._downloadButton.setDisabled(true);
			this._uploadButton.setDisabled(true);
			this._removeButton.setDisabled(true);
		},
		setSize:function(w, h){
			this.inherited(arguments);
			this.tableContainer.style.height = h+"px";
		},
		setReadonly: function(readonly) {
			this.inherited(arguments);
//			this._uploadButton.domNode.style.display = readonly ? "none" : "";
//			this._removeButton.domNode.style.display = readonly ? "none" : "";
		},
		setDisabled: function(disabled) {
			this.inherited(arguments);
			var d = disabled || this.readonly || !this.fieldSource;
			this._uploadButton.setDisabled(d);
			this._removeButton.setDisabled(d);
		},
		destroy: function() {
			this.inherited(arguments);
			if (this._dialog && this._dialog.domNode) {
				this._dialog.destroyRecursive();
			}
			if (this._downloadButton.domNode) {
				this._downloadButton.destroyRecursive();
			}
			if (this._uploadButton.domNode) {
				this._uploadButton.destroyRecursive();
			}
			if (this._removeButton.domNode) {
				this._removeButton.destroyRecursive();
			}
		},
		setFieldData: function(fieldData, separateModel) {
			this.inherited(arguments);
			this.setupRowHandlers();
		},
		getFieldValue: function(){
			return null; // record is updated when upload, so don't need to update field value explicitly 
		},
		_selectionChanged: function() {
			var enabled = this._selectedData;
			this._downloadButton.setDisabled(!enabled);
			this._removeButton.setDisabled(!enabled || this.disabled || this.readonly);
		},
		_startDownload: function() {
			if(this._selectedData && this._selectedData.RecordId){
				var url = RecordDetailRESTServiceURL + "?action=GetAttachmentContents&attachmentId=" 
								+ encodeURIComponent(this._selectedData.RecordId)
								+ "&resourceId=" 
								+ encodeURIComponent(this.view.stableLocation);
				window.open(cq.convertUrl(url));
			}
		},
		openSelected: function() {
			this._startDownload();
		},
		_removeAttachment: function() {
			if(!this._selectedData) return;
			//RATLC00998838 - Would like to have a warning When deleting an attachment
			var message = dojo.string.substitute(dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "cqRecordStrings").confirmRemoveAttachment, [this._selectedData.DisplayName]);
			if (confirm(message)) {
				com.ibm.rational.cm.web.ui.view.record.CqRecord.callXHR(cq.xhrGet, {
					url: RecordDetailRESTServiceURL,
					content: {
						action: "RemoveAttachment",
						attachment: this._selectedData.RecordId,
						resourceId: this.view.stableLocation
					},
					preventCache: true,
					load: dojo.hitch(this, function(data, ioArgs){
						this.removeSelected();
					})
				});
			}
		},
		_startUpload: function() {
			var formId = "cqAttachmentUploadForm_" + this.id;
			if (!this._dialog) {
				var msgs = this._messages;
				cq.loadModuleSync("com.ibm.rational.cm.web.ui.view.common.Dialog");
				this._dialog = new com.ibm.rational.cm.web.ui.view.common.Dialog({title:msgs.attachment_dlg_title});
				var url = net.jazz.ajax._contextRoot + "/cqweb/cqattachmentupload.cq?resourceId=" 
					+ encodeURIComponent(this.view.stableLocation) + "&field=" + encodeURIComponent(this.fieldSource);
				var body = "<form action='"+ url + "' id='" + formId + "' method='POST' enctype='multipart/form-data'><table>" +
						"<tr><td><label for='attachment_file'>" + msgs.attachment_dlg_filename + "</label></td><td><input id='attachment_file' type='file' name='attachment' size='80'/></td></tr>" +
						"<tr><td><label for='attachment_description'>" + msgs.attachment_dlg_description + "</label></td><td><input id='attachment_description' name='description' style='width:100%'/></td></tr>" +
						"</table></form>" +
						"<div align='center'><button dojoType='dijit.form.Button' type='submit'>" + msgs.attachment_dlg_ok + 
						"</button><button dojoType='dijit.form.Button' onclick='dijit.byId(\"" + this._dialog.id + "\").hide();'>" + msgs.attachment_dlg_cancel + "</button></div>"; 
				this._dialog.attr("content", body);
			}
			dojo.byId(formId).reset();	// reset file path and file description
			this._dialog.show();
			this._dialog.execute = dojo.hitch(this, function() {
				var form = dojo.byId(formId);
				if(!form || !form.attachment_file) {
					return;
				}
				this._setLoading(true);
				dojo["require"]("dojo.io.iframe");
				dojo.io.iframe.send({
					form: dojo.byId(formId),
					handleAs: "json",
					handle: dojo.hitch(this, function(response, ioArgs){
						this._setLoading(false);
						if(response instanceof Error){
							console.debug("Request FAILED: ", response);
						}else{
//							console.debug("Request complete: ", response);
							var v = response.NewValue;
							if(response.STATUS == "true" && v){
								if (v["FileName"]) v["FileName"] = com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(v["FileName"]);
								if (v["Description"]) v["Description"] = com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(v["Description"]);
								this._store.newItem(v, null);
								this.fetchData();
							}
						}
					})
				});
			});
		}
	}
);
/*
 * Widget for List Box
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqListBox",
	[dijit.form._FormWidget, com.ibm.rational.cm.web.ui.view.record.CqListControl],
	{
		widgetsInTemplate: true,
		templateString: "<div class='cqListBox'><table cellspacing='0' cellpadding='0' width='100%'><tr valign='top'>" +
						"<td width='100%'><div class='tableContainer' dojoAttachPoint='tableContainer' style='overflow:auto'><div dojoAttachPoint='dataTable'></div></div></td>" + 
						"<td valign='top'><div class='editButton' dojoAttachPoint='edit' dojoAttachEvent='onClick:_edit' dojoType='dijit.form.Button' iconClass='editButtonIconClass' label='${_messages.listcontrol_edit}' showLabel='false'></div></td></tr></table></div>",
		baseClass: "cqListBox",
		postCreate: function(){
			this.inherited(arguments);
			dijit.form._FormWidget.prototype.postCreate.apply(this, arguments);
		},
		destroy: function() {
			if (this._editDialog && this._editDialog.domNode) {
				this._editDialog.destroyRecursive();
			}
			if (this.edit && this.edit.domNode) {
				this.edit.destroyRecursive();
			}
			if (this._editList && this.edit.domNode) {
				this._editList.destroyRecursive();
			}
			this.inherited(arguments);
		},
		setSize:function(w, h){
			this.inherited(arguments);
			this.tableContainer.style.height = h+"px";
		},
		_isEditList: function(){
			return this._fieldData && this._fieldData.DataType == "RESOURCE"
		},
		_isInEdit: function(){
			return !this.readonly && !this.disabled;
		},
		setReadonly: function(readonly) {
			this.inherited(arguments);
			var enableEditButton = !readonly && !this._isEditList();
			this.edit.domNode.style.display = enableEditButton ? "" : "none";
		},
		setDisabled: function(disabled) {
			var wasInEdit = this._isInEdit();
			this.inherited(arguments);
			this.edit.setDisabled(disabled);
			var isInEdit = this._isInEdit();
			if(isInEdit != wasInEdit && this._isEditList()){
				this._expandChoiceList(isInEdit);
			}
		},
		setFieldData: function(fieldData, separateModel) {
			this.inherited(arguments);
			this.setupRowHandlers();
			if(this._isEditList()){
				this.edit.domNode.style.display = "none";
				this._expandChoiceList(this._isInEdit());
			}
		},
		updateChoiceList: function(){
			this._needGetChoiceList = true;
		},
		_expandChoiceList: function(expand){
			if(this._table){
				this._table.domNode.style.display = expand ? "none" : "";
			}
			var editList = this._editList;
			if(expand){
				if(!editList){
					editList = this._editList = new com.ibm.rational.cm.web.ui.view.record.CqComboBox({view:this.view, fieldSource:this.fieldSource, showTextBox: false});
					editList.setCqState("MODIFY");
					editList.updateChoiceList();
					editList.setFieldData({}, true);
					this.tableContainer.appendChild(editList._table.domNode);
				}
				editList.setValue(this._fieldData.ControlValue || this._fieldData.CurrentValue);
				editList._table.domNode.style.display = "";
			}else if(editList){
				editList._table.domNode.style.display = "none";
			}
		},
		_onBlur: function() {
			if(this._isEditList() && this._editList){
				this._updateValue(this._editList.getValue());
			}
		},
		_updateValue: function(newValue){
			var fd = this._fieldData;
			var oldValue = fd.ControlValue || fd.CurrentValue;
			if(oldValue != newValue){
				var d = {ControlValue: dojo.clone(newValue), DataType: fd.DataType, FieldName: fd.FieldName}; 
				this.setFieldData(d, true);  
				this.onChange(); // update model
			}
		},
		_edit:function() {
			var getChoiceList = this._needGetChoiceList; 
			var dialog = this._editDialog; 
			if (!dialog && this._fieldData && this._fieldData.FieldName) {
//				dojo["require"]("com.ibm.rational.cm.web.ui.view.common.DoubleListBox");
				this._editDialog = dialog = new com.ibm.rational.cm.web.ui.view.common.DoubleListBoxDialog({
					allowSearch: true,
					allowOrdering: !this.isAutoSort,
					title:dojo.i18n.getLocalization(cqNS, "cqRecordStrings", this.lang).listBoxDialogTitle});
				dialog.execute = dojo.hitch(this, function(formContents) {
					this._updateValue(dialog.getValueList());
				});
				dialog.resize({ w:520, h:350 });
				dialog.startup();
				getChoiceList = true;
			}
			if(getChoiceList){
				var content = {
					action: "GetChoiceList",
					resourceId: this.view.stableLocation,
					fieldName: this._fieldData.FieldName
				}; 
				if(this.isAutoSort){
					content.sort = "true";
				}
				com.ibm.rational.cm.web.ui.view.record.CqRecord.callXHR(cq.xhrGet,{
					url: RecordDetailRESTServiceURL,
					content: content,
					preventCache: true,
					load: dojo.hitch(this, function(data, ioArgs){
						var v = this.getFieldValue();
						if(dojo.isArray(v) && v.length > 0 && 
								// avoid blank entry in value list 
								!(v.length == 1 && v[0] == "")){
							if(dojo.isObject(v[0]) && v[0].DisplayName){
								v = dojo.map(v, function(item){
									return item.DisplayName || "";
								});
							}else{
								v = dojo.clone(v);								
							}
						}else{
							v = [];
						}
						var c = [];
						if(dojo.isArray(data.choices)){
							c = dojo.filter(data.choices, function(d){
								return (!v || dojo.indexOf(v, d) < 0);
							});
						}
						dialog.setAllowUserValue(!this._fieldData || this._fieldData.HasOpenChoiceList || c.length == 0); 
						dialog.setChoiceList(c);
						dialog.setValueList(v);
					})
				});
				delete this._needGetChoiceList;
			}
			dialog.show();
		}
	}
);

/*
 * ChoiceList controls
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqChoiceListControl",
	[com.ibm.rational.cm.web.ui.view.record.CqFormControl,com.ibm.rational.cm.web.ui.view.record.CqLoadingSupport],
	{
		pageSize: 150,
		postMixInProperties: function(){
			this.inherited(arguments);
			this._origStore = this.store;
		},
		setSize:function(w, h){
			this.domNode.style.width = w+"px";
		},
		setFieldData: function(fieldData, separateModel) {
			if (fieldData && this._needChoiceList()){
				if(this.store) this.store.isDate = this._isDateTime(fieldData);
			}else if(this.store && this.store != this._origStore){
				this.store.close();
				this.store = this._origStore;
			}
			this.inherited(arguments);
		},
		setReadonly:function(readonly) {
			this.inherited(arguments);
			if(this.downArrowNode) this.downArrowNode.style.display = readonly ? "none" : "";
		},
		updateChoiceList: function(){
			if(this._needChoiceList()){
				this._updateChoiceList();
			}
		},
		_needChoiceList: function() {
			if ((this.cqState != "MODIFY" && this.cqState != "SUBMIT") || this.readonly || this.disabled) {
				return false;
			}
			return true;
		},
		_setLoading: function(loading) {
			this.inherited(arguments);
			this._setStateClass();
		},
		_setStateClass: function(){
			this.inherited(arguments);
			dojo.toggleClass(this.domNode, "loading", this._loading);
		},
		_isDateTime: function(data){
			var fieldData = data || this._fieldData;
			return (fieldData && fieldData.DataType == "DATE_TIME");
		},
		_updateChoiceList: function(){
			if(this._hasFocus){
				// RATLC01066835
				this._needUpdateStore = true;
				return;
			}
			if(!this.fieldSource) return;
			var url = RecordDetailRESTServiceURL + 
				"?action=GetChoiceList&resourceId=" + 
				encodeURIComponent(this.view.stableLocation) + 
				"&fieldName=" + encodeURIComponent(this.fieldSource) +
				"&cq.preventCache=" + new Date().valueOf(); // same as preventCache in args of cq.xhrGet
			dojo["require"]("com.ibm.rational.cm.web.ui.data.CqChoiceListStore");
			this.store = new com.ibm.rational.cm.web.ui.data.CqChoiceListStore({url:url, sort:this.isAutoSort, isDate:this._isDateTime()});
			delete this._needUpdateStore;
		},
		_onBlur: function(){
			this.inherited(arguments);
			if(this._needUpdateStore) this._updateChoiceList();
		},
		_isValidChoice: function(){ // RATLC01068218
			var val = this.getValue();
			if(val){
				// This method checks only for inputed value
				// Mandatory check should be done outside of this method
				var store = this.store;
				var fieldData = this._fieldData;
				if(store && fieldData && store._loadFinished){
					var dataType = fieldData.DataType;
					// RATLC01259061
					if(fieldData.HasOpenChoiceList === false || dataType == "RESOURCE"){ 
						if(dataType == "DATE_TIME") val = this.format(val); // RATLC01269525
						return store.hasValue(val);
					}
					if(dataType == "INTEGER"){
						return !isNaN(val);
					}
					if(dataType == "DATE_TIME"){
						dojo["require"]("dojo.date.stamp");
						return dojo.date.stamp.fromISOString(val) != null || val.match(com.ibm.rational.cm.web.ui.view.record.CqRecord._cqDateTimeRegExp);
					}
				}
			}
			return true;
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqDateTimeMixin",
	null,
	{
		getValue: function(){
			if(this._isDateTime()){
				return this.parse(this.getDisplayedValue(), this.constraints);
			}
			return this.inherited(arguments);
		},
		format: function(/*String*/ value, /*Object*/ constraints){
			if(value && this._isDateTime()){
				dojo["require"]("dojo.date.stamp");
				var date = dojo.date.stamp.fromISOString(value);
				if(date){
					dojo["require"]("dojo.date.locale");
					return dojo.date.locale.format(date, {formatLength: "medium"});
				}
			}
			return this.inherited(arguments);
		},
		parse: function(/*String*/ value, /*Object*/ constraints){
			if(value && this._isDateTime()){
				dojo["require"]("dojo.date.locale");
				var date = dojo.date.locale.parse(value, {formatLength: "medium"});
				if(date){
					dojo["require"]("dojo.date.stamp");
					return dojo.date.stamp.toISOString(date, {zulu: true});
				}
			}
			return this.inherited(arguments);
		}
	}
);


/*
 * Widget for Editable Combo Box
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqComboCommon",
	[com.ibm.rational.cm.web.ui.view.record.CqChoiceListControl, com.ibm.rational.cm.web.ui.view.record.CqDateTimeMixin],
	{
		postCreate: function() {
			this.inherited(arguments);
			this.focusNode.style.fontFamily = ""; // override ValidationTextBox.postCreate
		},
		_startSearch: function() {
			this.inherited(arguments);
			this._setLoading(this.store ? this.store._loadInProgress : false);
		},
		_openResultList: function() {
			this._setLoading(false);
			this.inherited(arguments);
		},
		onkeypress: function(evt){
			if(this._loading /* RATLC01262030 */ || this.view._nowLoading /* RATLC01066095 */){
				dojo.stopEvent(evt);
			}else{
				this.inherited(arguments);
			}
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqEditableCombo",
	[dijit.form.ComboBox, com.ibm.rational.cm.web.ui.view.record.CqComboCommon],
{
	// 65684: dojo 1.2 migration
	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		if(!srcNodeRef){
			srcNodeRef = dojo.doc.createElement("DIV");
		}
		this.inherited(arguments);
	},
	isValidField: function() {
			return this.inherited(arguments) && this._isValidChoice();
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqFilteringSelect",
	[dijit.form.FilteringSelect, com.ibm.rational.cm.web.ui.view.record.CqComboCommon],
	{
// 65684: dojo 1.2 migration
create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
	if(!srcNodeRef){
		srcNodeRef = dojo.doc.createElement("DIV");
	}
	this.inherited(arguments);
},
		setValue: function(val, priorityChange) {
			if (!this._inSetField // accept any value from non-user input = don't prefetch choicelist 
					&& this._needChoiceList()){
				this.textbox.value = this.format(val) || ""; // set value to textbox even if choicelist is not downloaded yet
				this.inherited(arguments);
			} else if (val !== undefined){
				// By-pass validation
				this._setValue(val, this.format(val), priorityChange);
			}
		},
		setDisplayedValue: function(val){
			if(!this._isMandatory && val === ""){
				this._setValue("", ""); // accept empty text
			}else if(val != this._lastDisplayedValue || val != this.getDisplayedValue()){ // RATLC01268522
				this.inherited(arguments);
			}
		},
		validate: function() {
			// called just after choicelist is downloaded
			if (this._needChoiceList() && this.store !== this._origStore && !this.store._error){
				if(!this._isMandatory && this.getDisplayedValue() === ""){
					this._isvalid=true; // accept empty text
				}
			}else{
				this._isvalid=true; // if API error, no validation. this is work-around for emergency 
			}
			this.inherited(arguments);
			this.validateCtrl(); // revalidate
		},
		_callbackSetLabel: function(/*Array*/ result, /*Object*/ dataObject, /*Boolean, optional*/ priorityChange) {
			// called when item is in downloaded choice list
			this._inSetField = !dataObject; // don't call RecordUpdate by server result
			this.inherited(arguments);
			this._inSetField = false;
		}
	}
);
/*
 * Widget for Check Box
 */

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqCheckBox",
	[dijit.form._FormWidget, com.ibm.rational.cm.web.ui.view.record.CqFormControl],
	{
		checkedValue: "1",
		unCheckedValue: "0",
		templateString:"<span><input dojoAttachPoint='inputNode,focusNode' dojoAttachEvent='onmouseover:_onMouse,onmouseout:_onMouse,onclick:_onClick' type='checkbox' disabled='${disabled}' tabindex='${tabIndex}' value='${value}'><label style='white-space:nowrap;'>${caption}</label></span>",
		baseClass: "cqCheckBox",
		caption: "",

		setSize:function(w, h){
			this.domNode.style.width = w+"px";
		},
		_onClick: function(/*Event*/ e){
			if(this.disabled){ return false; }
			this.onChange();
		},
		setValue:function(val){
			if (val == this.checkedValue) {
				this.focusNode.checked = "true";
			} else {
				this.focusNode.checked = "";
			}
		},
		getValue:function(){
			if (this.focusNode.checked) {
				return this.checkedValue;
			} else {
				return this.unCheckedValue;
			}
		},
		setReadonly:function(readonly){
			this.inherited(arguments);
			this.focusNode.disabled = readonly;
		},
		setDisabled:function(disabled){
			com.ibm.rational.cm.web.ui.view.record.CqCheckBox.superclass.setDisabled.call(this, this.readonly || disabled);
		}
	}
);

/*
 * Widget for Radio Button
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqRadioButton",
	[dijit.form.RadioButton, com.ibm.rational.cm.web.ui.view.record.CqFormControl],
	{
		templateString: "<div><fieldset dojoAttachPoint='stateNode' class=\"dijitReset dijitInline\" waiRole=\"presentation\"\n\t><input\n\t \ttype=\"${type}\" name=\"${name}\"\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\n\t\tdojoAttachPoint=\"inputNode,focusNode\"\n\t \tdojoAttachEvent=\"onmouseover:_onMouse,onmouseout:_onMouse,onclick:_onClick\"\n/></fieldset>${caption}</div>",
		setSize:function(w, h){
			this.domNode.style.width = w+"px";
		},
		setValue: function(val){
			this.setChecked(val == this.value);
			// don't set value. 
		},
		getValue: function(){
			return this.checked ? this.value : null; // ignored by updateModifiedList  
		},
		setReadonly:function(readonly){
			this.inherited(arguments);
			this.setDisabled(readonly);
		},
		setDisabled:function(disabled){
			com.ibm.rational.cm.web.ui.view.record.CqRadioButton.superclass.setDisabled.call(this, this.readonly || disabled);
		}
	}
);

/*
 * Widget for Combo Box
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqComboBox",
	[dijit.form.TextBox, com.ibm.rational.cm.web.ui.view.record.CqChoiceListControl, com.ibm.rational.cm.web.ui.view.record.CqDateTimeMixin],
	{
		// dojo 1.2 migration : onkeyup handler
		templateString:"<div><table style='width:100%;height:100%' cellpadding='0'><tr><td><input class=\"dojoTextBox\" dojoAttachPoint='textbox,focusNode' name=\"${name}\"\n\tdojoAttachEvent='onmouseenter:_onMouse,onmouseleave:_onMouse,onfocus:_onMouse,onblur:_onMouse,onkeyup:_onKeyPress,onkeypress:_onKeyPress'\n\tautocomplete=\"off\" type=\"${type}\"\n\t/></td></tr><tr><td style='width:100%;height:100%;'><div dojoAttachPoint='tableContainer' class='tableContainer' style='width:100%;height:100%;overflow:auto'><div dojoAttachPoint='dataTable'></div></div></td></tr></table></div>",
		baseClass: "cqComboBox",
		setFieldData: function(fieldData, separateModel) {
			this.inherited(arguments);
			this._table.setStore(this.store);
			if(this.store){
				this._setLoading(true);
				this._table.fetchData();
			}else{
				this._table.clear();
			}
		},
		setValue: function(val) {
			this.inherited(arguments);
			if(val && this._table){
				var children = this._table.getChildren();
				if(children){
					for(var i=0,l=children.length; i<l; i++){
						var c = children[i];
						if(c.data.name === val){
							if(this._table.focusedChild != c){
								this._table.selectChild(c);
							}
							break;
						}
					}
				}
			}			
		},
		postCreate: function(){
			this.inherited(arguments);
			dojo["require"]("com.ibm.rational.cm.web.ui.view.record.CqTable");
			this._table = new com.ibm.rational.cm.web.ui.view.record.CqTable({
				isSelectable: true,
				showHeader: false,
				"class": "cqComboTable"
			}, this.dataTable);
			this.loadingNode = this._table.domNode;
			this.connect(this._table, "handleRowClick", function(inIndex, inCtrlKey, inShiftKey){
				var children = this._table.getChildren();
				if(children.length > inIndex && inIndex >= 0){
					var sel = children[inIndex].data;
					this.setValue(sel.name); // "name" property is regarded as value
				}
			});
			this.connect(this._table, "onItemsLoaded", function(){
				this._setLoading(false);
				this.setValue(this.getValue());
			});
			this.connect(this._table, "onGotError", function(){
				this._setLoading(false);
			});
		},
		startup: function(){
			this._started = true;
			this._resetSize();
		},
		setSize:function(w, h){
			this._h = h;
			this.domNode.style.width=w+"px";
			this.domNode.style.height=h+"px";
			// same as TextBoxResizeMixin
			this.textbox.style.width=Math.max(0,(w-2))+"px";
			if(this._started){
				this._resetSize();
			}
		},
		_resetSize: function(){
			if(this._h){
				var textBoxSize = dojo.marginBox(this.textbox);
				if(textBoxSize.h > 0){
					// cellspacing(2)*4
					this.tableContainer.style.height = Math.max(0,(this._h - textBoxSize.h - 8)) + "px";
				}
			}
		},
		setReadonly: function(readonly){
			this.inherited(arguments);
			this.textbox.readOnly = readonly;
		},
		isValidField: function() {
			return this.inherited(arguments) && this._isValidChoice();
		}
	}
);

(function(){
	if(typeof this["CqURLUtil"] == "undefined"){
		this.CqURLUtil = {};
	}
	CqURLUtil.extractUrls = function(text){
		var urls = [];
		this._extractUrls("http://", text, urls); 
		this._extractUrls("ftp://", text, urls); 
		this._extractUrls("https://", text, urls); 
		this._extractUrls("file://", text, urls); 
		this._extractUrls("\\\\", text, urls, true);
		return urls;
	};
	CqURLUtil._extractUrls = function(protocol, text, urls, isUNC){
		var plen = protocol.length;
		var start = text.indexOf(protocol);
		while(start >= 0){
			var url = text.substring(start + plen);
			var urlEnd	= this._findURLEnd(url);
			if(urlEnd < 0){
				break;
			}
			try {
				url = protocol + url.substring(0, urlEnd);
				url = encodeURI(decodeURI(url.replace(/\\/g,'/')));
				if(isUNC){
					url = "file://" + url;
				}
				urls.push(url);
			}catch(e){
				// ignore invalid url
			}
			start = text.indexOf(protocol, start + urlEnd);
		}
	};
	CqURLUtil._findURLEnd = function(text) {
		var len = text.length;
		for (var i=0; i<len; i++){
			var chCode = text.charCodeAt(i);
			//combination of !@! represents a newline
			//if (chCode == 33 && text.charCodeAt(i+1) == 64 && text.charCodeAt(i+2) == 33)
			if(chCode == 171 || chCode == 9 || chCode == 10 || chCode == 13 || chCode == 32 || chCode == 34){
				return i;
			}
			//safety check so that we don't run thru the whole string
			//as the url can be of limited size
			if (i > 2048){
				return -1;
			}
		}
		//it reached end without finding any delimiter
		//so lets return the len as the caller will know
		//search reached end and didn't find the delimiter
		//in which case the whole string becomes part of the url
		return len;
	};
})();

})();