/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.structure.ColumnElement"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.ui.structure.GroupElement"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.structure.RowElement"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.Columns"); //$NON-NLS-1$

dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("com.ibm.team.workitem.web.cache.internal.Cache"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxy"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.SaveRunnable"); //$NON-NLS-1$

(function() {

var GroupElement					= com.ibm.team.apt.ui.structure.GroupElement;
var RowElement						= com.ibm.team.apt.ui.structure.RowElement;
var Columns							= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.Columns;

var PlanningClient		= com.ibm.team.apt.web.client.internal.PlanningClient;
var WorkItemClient		= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkingCopyRegistry	= com.ibm.team.workitem.web.cache.internal.Cache;
var WorkItemProxy		= com.ibm.team.workitem.web.cache.internal.WorkItemProxy;

dojo.declare("com.ibm.team.apt.ui.structure.ColumnElement", GroupElement, { //$NON-NLS-1$

	__row: null,
	__index: null,
	__workflowId: null,
	__id: null,

	constructor: function(row, index, workflowId, id) {
		this.__row= row;
		this.__index= index;
		this.__workflowId= workflowId;
		this.__id= id;
	},

	equals: function(other) {
		if (this === other)
			return true;

		if (this.declaredClass !== other.declaredClass)
			return false;

		return this.__index === other.__index;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getId: function() {
		return this.__id;
	},

	getColumn: function() {
		if (!this.__row.isHeader())
			return null;
		
		if (this.__id == "column.parent") { //$NON-NLS-1$
			return null;
		} else if (this.__workflowId) {
			var plan= this.__row.getParent().getPlanModel();
			var workflow= plan.getWorkflowInfo(this.__workflowId);
			return Columns.stateHeader(workflow, this.__id);
		} else {
			if (this.__id == "1") {//$NON-NLS-1$
				return Columns["todo"]; //$NON-NLS-1$
			} else if (this.__id == "4") {//$NON-NLS-1$
				return Columns["inpr"]; //$NON-NLS-1$
			} else if (this.__id == "2") {//$NON-NLS-1$
				return Columns["done"]; //$NON-NLS-1$
			}
		}
		
		return null;
	},
	
	getStateGroupId: function() {
		return this.__id;
	},
	
	getRow: function() {
		return this.__row;
	},
	
	getLabel: function() {
		return this.__id;
	},
	
	getElementKey: function() {
		return this.__row.getElementKey() + this.getId(); //$NON-NLS-1$
	},
	
	getValue: function() {
		return this.__id;
	},
	
	compareTo: function(other) {
		return this.__index - other.__index;
	},
	
	canAdopt: function(planItem) {
		if (!(planItem instanceof com.ibm.team.apt.client.PlanItem))
			return false;
		
		if (this.__requiresStateChange(planItem) && this.__getAction(planItem) == null)
			return false;
		
		return this.__row.canAdopt(planItem);
	},
	
	adopt: function(planItem) {
		this.__row.adopt(planItem);
		var action= this.__getAction(planItem);
		if (action != null) {
			planItem.setWorkflowAction(action);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__requiresStateChange: function(planItem) {
		if (this.__id == "column.parent") { //$NON-NLS-1$
			return false;
		} else if (this.__workflowId) {
			var state= planItem.getState();
			return state.itemId != this.__id;
		} else {
			var workflowInfo= planItem.getWorkflowInfo();
			if (!workflowInfo)
				return true;
			
			var stateGroup= workflowInfo.getStateGroup(planItem.getState().itemId);
			return stateGroup != this.__id;
		}
	},
	
	__getAction: function(planItem) {
		if (this.__id == "column.parent") { //$NON-NLS-1$
			return null;
		} else if (planItem.isDraftItem()) {
			return null;
		} else if (this.__workflowId) {
			var workflow= planItem.getWorkflowInfo();
			if (!workflow)
				return null;
			
			if (workflow.getId() != this.__workflowId)
				return null;
			
			var actions= workflow.getActions(planItem);
			for ( var i = 0; i < actions.length; i++) {
				var action= actions[i];
				if (action.endState == this.__id)
					return action;
			}
			
			return null;
		} else {
			var workflowInfo= planItem.getWorkflowInfo();
			if (!workflowInfo)
				return null;
			
			return workflowInfo.getAction(planItem, this.__id);
		}
	},
	
	__sentinel: null // terminates this class definition
});

})();
