/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.structure.OutlineEntryItemAccessor"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.PlanningAttributeIdentifier"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.model.EntryUtils"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.model.ViewEntry"); //$NON-NLS-1$

(function() {

var EntryUtils						= com.ibm.team.apt.ui.model.EntryUtils;
var PlanningAttributeIdentifier		= com.ibm.team.apt.client.PlanningAttributeIdentifier;
var ViewEntry						= com.ibm.team.apt.ui.model.ViewEntry;
var PrimaryLocationTag 				= com.ibm.team.apt.ui.model.PrimaryLocationTag.INSTANCE;

dojo.declare("com.ibm.team.apt.ui.structure.OutlineEntryItemAccessor", null, { //$NON-NLS-1$

	__readAccessor: null,
	__complexityAttributeId: null,
	
	constructor: function(readAccessor, complexityAttributeId) {
		this.__readAccessor= readAccessor;
		
		if (complexityAttributeId) {
			this.__complexityAttributeId= new PlanningAttributeIdentifier(complexityAttributeId);
		}
	},

	isPlanItem: function(entry) {
		return EntryUtils.isType(entry, com.ibm.team.apt.client.PlanItem);
	},
	
	isResolved: function(entry) {
		if (EntryUtils.isType(entry, com.ibm.team.apt.client.PlanItem)) {
			return entry.getElement().isResolved();
		}
		return false;
	},
	
	isTopLevelPlanItem: function(entry) {
		if (EntryUtils.isType(entry, com.ibm.team.apt.client.PlanItem)) {
			return entry.getElement().isTopLevelPlanItem();
		}
		return false;
	},
	
	getComplexity: function(entry) {
		if (EntryUtils.isType(entry, com.ibm.team.apt.client.PlanItem)) {
			var value;
			if (this.__complexityAttributeId) {
				try {
					value= entry.getElement().getAttributeValue(this.__complexityAttributeId);
				} catch(e){
					// attribute not defined
				}
			}
			value= this.__workItemEstimatorGetComplexity(value);
			return value;
		}
		return 0;
	},
	
	__workItemEstimatorGetComplexity: function(value){
		if(value === undefined)
			return 0;
		if(value === null)
			return 0;
			
		var number= 0;			
		if(value instanceof Object){
			if(value instanceof Number){
				return value;
			}
			if(value instanceof com.ibm.team.workitem.common.Literal){
				number= Number(value.itemId);
				if(!isNaN(number)){
					return number;
				}
				number= Number(value.label);
				if(!isNaN(number)){
					return number;
				}
			}
		}
		
		number= Number(value);
		if(isNaN(number)){
			return 0;
		} else {
			return number;
		}
	},
	
	isAuxiliaryPlanItem: function(entry) {
		if (EntryUtils.isType(entry, com.ibm.team.apt.client.PlanItem)) {
			return  entry.getElement().isAuxiliaryPlanItem();
		}
		return false;
	},
	
	isPrimaryElement: function(entry) {
		return entry.hasTag(PrimaryLocationTag);
	},
	
	getDuration: function( entry) {
		if (EntryUtils.isType(entry, com.ibm.team.apt.client.PlanItem)) {
			return entry.getElement().getDuration();
		}
		return null;
	},
	
	getChildren: function(entry) {
		return this.__readAccessor.getChildren(entry);
	},
	
	__sentinel: null // terminates this class definition
});

})();
