/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.structure.TimeElement"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.PlanItem"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.internal.Instant"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.structure.GroupElement"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.structure.TimespanDefinition"); //$NON-NLS-1$

(function() {

var Instant							= com.ibm.team.apt.client.internal.Instant;
var PlanElement						= com.ibm.team.apt.client.PlanElement;
var PlanItem						= com.ibm.team.apt.client.PlanItem;
var GroupElement					= com.ibm.team.apt.ui.structure.GroupElement;
var TimeSpanDefinition				= com.ibm.team.apt.ui.structure.TimespanDefinition;

dojo.declare("com.ibm.team.apt.ui.structure.TimeElement", GroupElement, { //$NON-NLS-1$

	__contributor: null,
	__timespanDefinition: null,

	constructor: function(contributor, timespanDefinition) {
		this.__contributor= contributor;
		this.__timespanDefinition= timespanDefinition;
	},

	equals: function(other) {
		if (this === other)
			return true;

		if (this.declaredClass !== other.declaredClass)
			return false;

		return this.__contributor.itemId === other.__contributor.itemId && this.__timespanDefinition.equals(other.__timespanDefinition);
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getId: function() {
		return this.__contributor.itemId + "_" + this.__timespanDefinition.declaredClass; //$NON-NLS-1$
	},

	getLabel: function() {
		return this.__timespanDefinition.label;
	},
	
	getElementKey: function() {
		return this.__contributor.itemId + "_" + this.__timespanDefinition.declaredClass; //$NON-NLS-1$
	},
	
	getValue: function() {
		return this.__timespanDefinition;
	},
	
	compareTo: function(other) {
		if (this.__timespanDefinition == TimeSpanDefinition.UNSCHEDULED || other.__timespanDefinition == TimeSpanDefinition.UNSCHEDULED)
			return (this.__timespanDefinition == TimeSpanDefinition.UNSCHEDULED ? 1 : 0) - (other.__timespanDefinition == TimeSpanDefinition.UNSCHEDULED ? 1 : 0);

		return this.__timespanDefinition.compareTo(other.__timespanDefinition);	
	},
	
	canAdopt: function(planElement, groupEntry) {
		if (!(planElement instanceof PlanItem))
			return false;
		if (this.__timespanDefinition == TimeSpanDefinition.UNSCHEDULED)
			return false;
		if (planElement.getAttributeValue(PlanItem.RESOLVED) != this.__timespanDefinition.isPast())
			return false;
		return true;
	},

	adopt: function(planElement, groupEntry) {
		planElement.setAttributeValue(PlanItem.OWNER, this.__contributor);

		// adopt is called for newly created items, even for the inbox
		if (this.__timespanDefinition != TimeSpanDefinition.UNSCHEDULED) {
			var start= null;
			var end= null;
			groupEntry.getModel().readModel(function(readAccessor) {
				dojo.forEach(readAccessor.getChildren(groupEntry), function(childEntry) {
					
					if (childEntry.getElement() instanceof PlanElement) {
						var scheduledTime =childEntry.getElement().getAttributeValue(PlanItem.SCHEDULED_TIME);
						
						if (scheduledTime != null) {
							if (start == null || scheduledTime.start.before(start))
								start= scheduledTime.start;
							
							if (end == null || scheduledTime.end.after(end))
								end= scheduledTime.end;
						}
					}
				}, this);
			}, this);
			
			if (start == null)
				start= this.__timespanDefinition.getStart();
			
			if (end == null)
				end= this.__timespanDefinition.getEnd();
			
			var targetInstant= new Instant(new Date(start.getMilliseconds() + ((end.getMilliseconds() - start.getMilliseconds()) / 2)));
			var referenceTime= planElement.getPlanModel().getReferenceTime();

			if (targetInstant.before(referenceTime))
				targetInstant= referenceTime;

			
			planElement.schedule(targetInstant);
		}


		return true;
	},


	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

})();
